/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.customize.Menu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.customize.Menu"] = true;
// This is a menu.  It takes a list of sources that each have label attributes to use as display titles.  It publishes an event on a source each time one is clicked.
dojo.provide("com.ibm.customize.Menu");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.fx.easing");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.bundles", "Shelf", null, "ROOT,ar,ca,cs,da,de,el,en,es,fi,fr,he,hr,hu,it,ja,ko,nl,no,pl,pt,pt-br,ro,ru,sk,sl,sv,th,tr,uk,zh,zh-tw");


dojo.declare("com.ibm.customize.Menu",
	[dijit._Widget, dijit._Templated],
	{
		sources: [],									// an array of objects containing labels and datastore hubs for each menu choice
		choices: {},									// a map of label : DOMNode pairs for each choice in the menu
		searchId: "",									// the id of the Container to search within
		menuTitle: "Menu",								// title for the menu
		current: null,									// the menu choice currently selected
		selectedText: "currently selected category",
		unselectedText: "press enter to browse this category",
		publishKeyword: "/epc/customize/changeSource",	// the keyword to publish to add a filter
		templateString: '<div dojoAttachPoint="focusNode"><h3>${menuTitle}</h3><div role="menu" dojoAttachPoint="menuContainer" class="lotusMenu"><ul dojoAttachPoint="list"></ul></div></div>',
		
		constructor: function() {
			this.bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.menuTitle = this.bundle["shelf_menu"];
			this.selectedText = this.bundle["menu_selected_text"] || this.selectedText;
			this.unselectedText = this.bundle["menu_unselected_text"] || this.unselectedText;
		},
	
		postCreate: function(){
			// summary: populates the menu with the given choices , makes sure the first item on the list is selected
			
			//subscribe to when a new source is changed so the highlighting in the menu will match
			dojo.subscribe(this.publishKeyword+this.searchId, this, this.changeHighlighting);
			
			if(this.sources.length > 0) {
				// add the choices to the menu with the first choice selected
				dojo.forEach(this.sources, dojo.hitch(this, function(choice){ this.addLi(choice); }));
				this.select(this.sources[0]);
			}
		},
		
		addLi: function(/*Object*/choice){
			// summary: adds a new element to the list of choices in the menu
			// choice: a member of the choices array to add to the menu
			var li = dojo.create("li");
			var newLink = dojo.create("a",{href:"javascript:void(0)",innerHTML:choice.label});
			newLink.onclick = function(){ return false; /* to prevent onBeforeUnload event from triggering in IE */ };
			li.appendChild(newLink);
			var selectedSpan = dojo.create("span",{innerHTML:this.unselectedText,className:"",className:"ibmPortalHiddenText"});
			newLink.appendChild(selectedSpan);
			// when the menu item is clicked, call the select function on it
			dojo.connect(li, 'onclick', this, function(){ this.select(choice); });
			// add the new item to DOM and the map of choices
			this.list.appendChild(li);
			this.choices[choice.label] = {li:li,span:selectedSpan};
		},
		
		select: function(/*Object*/source){
			// summary: triggered by a menu click,
			// source: the menu choice that was clicked
			
			// publish the event to a specific id so not all instances of the receiving class respond
			dojo.publish(this.publishKeyword+this.searchId, [source]);
		},
		
		changeHighlighting: function(/*String*/source){
			// summary: switches which menu choice is selected
			// source: the menu choice that was chosen
			if(this.current) {
				dojo.removeClass(this.choices[this.current].li, "lotusSelected");
				this.choices[this.current].span.innerHTML = this.unselectedText;
			}
			dojo.addClass(this.choices[source.label].li, "lotusSelected");
			this.choices[source.label].span.innerHTML = this.selectedText;
			this.current = source.label;
		}
});

}
