/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.ItemsLoader"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.ItemsLoader"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.ItemsLoader");

dojo.declare("com.ibm.data.ItemsLoader",
	null,
	{
		store: null,
		query: null,
		cacheContext: null,
		cacheProperty: null,
		preload: true,
		fetchOnReload: false,
		resultSet: null,
		
		constructor: function(/*Object*/ args) {
			dojo.mixin(this, args);
			this._lastModified = (new Date()).getTime();
			if(this.preload) setTimeout(dojo.hitch(this, "_load"), 1);
		},
		filter: function(resultSet) {
			// summary: Filters the resultSet that this ItemsLoader loads, passing the filtered
			//		set to each callback.  Note that this function should be idempotent, meaning 
			// 		that it should be able to run multiple times on its own return value without
			//		changing the return value.  In other words, if the filtering function is
			//		executed once and then executed again on its own return value, the return
			//		value of the second execution should be the same as the first.
			return resultSet;
		},
		_getResultSet: function() {
			// gets the stored result set in this items loader;
			// callbacks may modify this object/array, resulting in changes that are reflected
			// in subsequent callbacks; this also filters the resultSet unless it hasn't been 
			// modified since last being filtered
			if(this._lastModified == this.resultSet._lastModified) {
				return this.resultSet;
			}
			else {
				this.resultSet = this.filter(this.resultSet);
				this._lastModified = this.resultSet._lastModified = (new Date()).getTime();
				if(this.cacheContext && this.cacheProperty) {
					this.cacheContext[this.cacheProperty] = this.resultSet;
				}
				return this.resultSet;
			}
		},
		_load: function() {
			if(!this._def) {
				this._def = new dojo.Deferred();
				if(this.cacheContext && this.cacheProperty) {
					this.resultSet = this.cacheContext[this.cacheProperty];
					if(this.resultSet) {
						this.resultSet.loaded = true;
						this._def.callback();
						return this._def;
					}
				}
				this._fetch();
			}
			
			return this._def;
		},
		_fetch: function() {
			this.store.fetch({
				query: this.query,
				onComplete: function(items){
					items.loaded = true;
					this.resultSet = items;
					this._def.callback();
				},
				onError: function(err) {
					this._def.errback(err);
				},
				scope: this
			});
		},
		addOnLoad: function(/*Function*/ callback) {
			this._load().addCallback(this._wrap(callback));
			return this;
		},
		addOnError: function(/*Function*/ callback) {
			this._load().addErrback(this._wrap(callback));
			return this;
		},
		addOnResult: function(/*Function*/ callback) {
			return this.addOnLoad(this._wrap(callback)).addOnError(this._wrap(callback));
		},
		_wrap: function(f) {
			if(f) return dojo.hitch(this, function() {
				try {
					f(this._getResultSet());
				}
				catch(err) {
					this.raiseError(err);
				}
			})
		},
		raiseError: function(err) {
			console.error(err);
		},
		reload: function() {
			// summary: Reloads this loader to retrieve the items again.
			//		If the previous callbacks haven't been called yet as the 
			//		items haven't been loaded, the previous load request
			//		is cancelled and all existing callbacks that were already 
			// 		added earlier are lost and must be added again if desired.
			if(this._def) this._def.cancel();
			delete this._def;
			if(this.fetchOnReload && this.cacheContext && this.cacheProperty) {
				delete this.cacheContext[this.cacheProperty];
			}
			this._load();
			return this;
		}
	}
);

}
