/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.resolver.NMPocHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.resolver.NMPocHandler"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.resolver.NMPocHandler");

dojo.require("com.ibm.data.resolver.OpenSearchPocHandler");
dojo.require("com.ibm.data.resolver.ResolverQueryRules");
dojo.require("com.ibm.data.resolver.tree.TreeModeledHandler");
dojo.require("com.ibm.data.resolver.tree.PocGlobalTreeModel");

dojo.declare("com.ibm.data.resolver.NMPocHandler",
	[com.ibm.data.resolver.OpenSearchPocHandler, com.ibm.data.resolver.tree.TreeModeledHandler],
	{
		category: "navigation",
		
		_globalTreeModel: null,
		
		namespaces: dojo.mixin({}, 
			com.ibm.data.resolver.OpenSearchPocHandler.prototype.namespaces,
			com.ibm.data.resolver.ResolverQueryRules.namespaces),
		
		constructor: function() {
			this.customAttributes.treeModel.get = function(item, multiValued) {
				if(!this._globalTreeModel) {
					this._globalTreeModel = new com.ibm.data.resolver.tree.PocGlobalTreeModel({
						refItem: item,
						handler: this,
						resolver: this.resolver,
						scheme: this.definition.scheme
					});
				}
				return multiValued ? [this._globalTreeModel] : this._globalTreeModel;
			};
		},
		
		_createTemplates: function() {
			var nsDecl = [];
			for(var prefix in this.namespaces) {
				nsDecl.push('xmlns:', prefix, '="', this.namespaces[prefix], '" ');
			}
			dojo.mixin(this, {
				nodeTemplate: {
					templateString:
						'<?xml version="1.0" encoding="UTF-8"?>'
						+'<atom:feed '
							+nsDecl.join('')
							+'>'
							+'<atom:author>'
								+'<atom:name>IBM WebSphere Portal/6.1</atom:name>'
							+'</atom:author>'
							+'<atom:title>IBM WebSphere Portal Model Feed</atom:title>'
							+'<atom:link href="." rel="self" type="application/atom+xml"/>'
							+'<atom:entry>'
								+'<atom:title>${title}</atom:title>'
								+'<atom:id>${id}</atom:id>'
								+'<atom:content type="application/xml">'
									+'<model:navigation-node>'
										+'<model:title>'
										+'<base:nls-string xml:lang="${locale}">${title}</base:nls-string>'
										+'</model:title>'
									+'</model:navigation-node>'
								+'</atom:content>'
								+'<thr:in-reply-to ext:uri="${parentId}" href="${parentId}" ref="${parentId}" type="application/atom+xml"/>'
							+'</atom:entry>'
						+'</atom:feed>',
					templateProps: {
						id: "",
						title: "",
						locale: dojo.local,
						parentId: ""
					}
				}
			});
		},
		getSaveQuery: function(/*item*/ item) {
			var q = this.inherited(arguments);
			if(this.resolver.isCreated(item)) {
				// merge updates
				// empty representation.. performance-smart
				q.rep = "empty";
			}
			else {
				q.uri = this.getValue(item, "parentId");
			}
			return q;
		},
		fillInProperties: function(newItem, template, keywordArgs, parentInfo) {
			if(keywordArgs.metadata) {
				for(var key in keywordArgs.metadata) {
					keywordArgs["metadata[" + key + "]$"] = keywordArgs.metadata[key];
				}
				delete keywordArgs.metadata;
			}
			if(keywordArgs.friendlyName) {
				keywordArgs["metadata[com.ibm.portal.friendly.name]$"] = keywordArgs.friendlyName;
				delete keywordArgs.friendlyName;
			}
			return this.inherited(arguments);
		},
		getTemplate: function(args, parentInfo) {
			return this.nodeTemplate;
		},
		instantiateTemplate: function(template, args, parentInfo) {
			var xmlDoc = this.inherited(arguments);
			if(!args.parentId) {
				// we don't want the thr:in-reply-to element in the DOM data to send, so remove it
				this.engine.select("./thr:in-reply-to", xmlDoc).destroy();
			}
			return xmlDoc;
		},
		_makeModifiable: function(item) {
			// summary: Modifies the element of this item to become modifiable
			//		for writing back to the server.  Subclasses can override this
			//		to do things like prune an item to only include necessary information
			//		for updates and nothing more.  This is called by backup once after
			//		backing up the original element first.  After this returns,
			//		this item's element may have been modified.  Any queries against
			//		this item should now include logic to check the backup copy as well
			//		if a query doesn't return anything as it could have been deleted by
			//		this function.  Care should be taken to ensure that the query result
			//		specifically detects if nothing was found or if the actual value was
			//		null or an empty string.
			this.engine.select("atom:content/*/* | thr:in-reply-to | atom:link", item.data).destroy();
			
			this.inherited(arguments);
		}
	}
);

(function(){
	var r = com.ibm.data.resolver, 
		nmPoc = r.NMPocHandler, 
		prot = nmPoc.prototype, 
		def = prot.definition,
		atomBase = r.ResolverQueryRules.atom;
		
	def = dojo.mixin({}, def, {
		scheme: "nm"
	});
	
	def.features = dojo.mixin({}, def.features, {
		"TreeModeled": true
	});
	
	prot.definition = def;
	
	dojo.extend(nmPoc, {		
		queryRules: dojo.mixin({}, r.OpenSearchPocHandler.prototype.queryRules, 
					atomBase.threadExt,
					atomBase.treeExt, 
					atomBase.uniqueNameExt, 
					atomBase.L10nExt, 
					atomBase.acExt,  
					atomBase.modelExt,
					atomBase.operationsExt, 
					atomBase.contentExt,
					atomBase.metaExt, {
			contentNode: {selector: "./atom:link[@ext:class='content-node']", field: "@ext:uri", type: "Item"},
			contentNodeId: {selector: "./atom:link[@ext:class='content-node']", field: "@ext:uri"},
			contentNodeType: {selector: "./atom:link[@ext:class='content-node']", field: "@ext:rel"}
		})
	});
	
	var contentNodeSetter = function(data, rule, value){
		var attrs = {
			"ext:class": "content-node", 
			"type": "application/atom+xml"
		};
		attrs[rule.field.substr(1)] = value;
		this.createFromJson(data, {
			name: "atom:link",
			attributes: attrs
		});
		return true;
	};
	
	prot.queryRules.contentNode.setter = contentNodeSetter;
	prot.queryRules.contentNodeId.setter = contentNodeSetter;
	prot.queryRules.contentNodeType.setter = contentNodeSetter;
	
	com.ibm.strategy.Strategy.applyStrategy(nmPoc, new com.ibm.data.resolver.tree.AtomModelTreeStrategy());
	
	com.ibm.data.Resolver.register(def.scheme, nmPoc);
})();

}
