/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.planner.SimplePOPlan"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.planner.SimplePOPlan"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.planner.SimplePOPlan");

// Provides partial-order planning an execution path to achieve
// a desired goal given steps with presteps associated with them.
// This is simple because a real partial-order planner would have
// steps available to it, each with pre-conditions and post-conditions,
// managing the current state of conditions before and after each
// step to reach the desired goal set of conditions.


dojo.declare("com.ibm.planner.SimplePOPlan",
	null,
	{
		constructor: function(/*Array?*/ steps) {
			this.steps = {};
			this.orderedSteps = [];
			this._nctr = Math.floor(Math.random() * 100001);
			if(steps) {
				for(var i=0; i<steps.length; i++) {
					this.insertStep(steps[i]);
				}
			}
		},
		destroy: function() {
			for(var s in this.steps) {
				this.steps[s].destroy();
			}
			delete this.steps;
			delete this.orderedSteps;
		},
		_genName: function() {
			return "s_" + this._nctr++;
		},
		indexOf: function(/*Step*/ step) {
			return dojo.indexOf(this.orderedSteps, step);
		},
		getStep: function(/*String*/ name) {
			return this.steps[name];
		},
		hasStep: function(/*String*/ name) {
			return !!this.steps[name];
		},
		makeStep: function(/*String*/ name, /*Object*/ contents) {
			var step = new com.ibm.planner.SimplePOStep(name, contents);
			this.insertStep(step);
			return step;
		},
		_insStep: function(/*Step*/ step, /*Map*/ map) {
			if(!map[step.name]) {
				var insertIdx = 0;
				for(var i=0, pre; pre = step.preSteps[i], i<step.preSteps.length; i++) {
					var pidx;
					if(map[pre.name]) {
						pidx = this.indexOf(pre);
						if(pidx >= insertIdx) insertIdx = pidx + 1;
					}
					else {
						pidx = this._insStep(pre, map);
						if(pidx <= insertIdx) insertIdx++;
						else insertIdx = pidx + 1;
					}
				}
				this.orderedSteps.splice(insertIdx, 0, step);
				map[step.name] = step;
				
				return insertIdx;
			}
		},
		orderSteps: function() {
			this.orderedSteps = [], map = {};
			for(var name in this.steps) {
				this._insStep(this.steps[name], map);
			}
			return this;
		},
		insertStep: function(/*Step*/ step) {
			if(!step.name) {
				step.name = this._genName();
			}
			if(!this.steps[step.name]) {
				this.steps[step.name] = step;
			}
			return this;
		},
		_reset: function() {
			delete this.groupList;
		},
		replan: function() {
			this._reset();
			this.orderSteps();
			return this;
		},
		_insertIntoGroupList: function(/*Array*/ gList, /*Step*/ step) {
			for(var j=gList.length-1; j>=0; j--) {
				if(!gList[j].hasPreStepOf(step)) {
					if(j > 0) continue;
					else gList[j].addStep(step);
				}
				else if(j < gList.length-1) {
					gList[j+1].addStep(step);
					return;
				}
				else {
					gList.push(new com.ibm.planner.SimplePOStepGroup([step]));
					return;
				}
			}
		},
		size: function() {
			return this.orderedSteps.length;
		},
		getList: function() {
			// summary: Returns an array of steps that satisfy the partial ordering constraints
			//		of the steps in this plan.  It may not be the most optimal ordering, but is
			//		at least consistent in terms of partial ordering.
			return this.orderedSteps;
		},
		filter: function(filterFunc) {
			// summary: Filters the steps in this plan by the filterFunc argument.  Allows a
			//		caller to remove steps from the plan that it knows are irrelevant for its 
			//		needs and allows more efficient processing as well as correct grouping
			//		when creating group lists.
			this.orderedSteps = dojo.filter(this.orderedSteps, filterFunc);
			return this;
		},
		getGroupList: function() {
			// summary: Returns an array of StepGroup objects.  Each group contains steps
			//		which can be (theoretically) executed simultaneously.  The array is
			//		ordered so that all partial ordering constraints are satisfied correctly
			//		but steps that have no ordering constraints between them can be executed
			//		simultaneously.
			// returns: SimplePOStepGroup[] list of grouped steps
			if(!this.groupList) {
				// cache the group list since it can be expensive to create
				var groupList = [], list = this.getList();
				if(list.length>0) groupList.push(new com.ibm.planner.SimplePOStepGroup([list[0]]));
				for(var i=1, st; st=list[i], i<list.length; i++) {
					this._insertIntoGroupList(groupList, st);
				}
				this.groupList = groupList;
			}
			return this.groupList;
		}
	}
);

dojo.declare("com.ibm.planner.SimplePOStep",
	null,
	{
		constructor: function(name, contents) {
			this.name = name;
			this.contents = contents;
			this.preSteps = [];
		},
		destroy: function() {
			delete this.preSteps;
			delete this.contents;
			delete this.name;
		},
		isPreStepOf: function(/*Step*/ step) {
			return step.hasPreStep(this);
		},
		hasPreStep: function(/*Step*/ step) {
			return dojo.indexOf(this.preSteps, step) > -1;
		},
		addPreStep: function(/*Step*/ step) {
			this.preSteps.push(step);
		},
		toString: function() {
			return "<Step '" + this.name + "'>";
		}
	}
);

dojo.declare("com.ibm.planner.SimplePOStepGroup",
	null,
	{
		constructor: function(/*Array?*/ steps) {
			this.steps = steps ? steps.slice() : [];
		},
		destroy: function() {
			delete this.steps;
		},
		getSteps: function() {
			return this.steps;
		},
		addStep: function(/*Step*/ step) {
			this.steps.push(step);
		},
		hasPreStepOf: function(/*Step*/ step) {
			for(var i=0, st; st=this.steps[i], i<this.steps.length; i++) {
				if(st.isPreStepOf(step)) return true;
			}
			return false;
		},
		toString: function() {
			return "<StepGroup (" + this.steps + ")>";
		}
	}
);

}
