/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.backbutton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.backbutton"] = true;
dojo.provide( "com.ibm.portal.aggregation.backbutton" );

dojo.require( "dojo.back" );
dojo.require( "dojox.data.dom" );
dojo.require( "dojo.cookie" );
dojo.require( "com.ibm.portal.navigation.controller");
dojo.require( "com.ibm.portal.EventBroker" );
dojo.require( "com.ibm.portal.aggregation.storage" );
dojo.require( "com.ibm.portal.aggregation.error" );

com.ibm.portal.aggregation.backbutton.CONSTANTS = new function () {
	// summary: Constants for use with the back button support.
	this.storageKeyForSavedState = "testKey";
}

dojo.declare( "com.ibm.portal.aggregation.backbutton.PortalBackButtonHandler",
			  null,
			  {
			  	// summary: Handles all back button events. 
				// description: Subscribes to the state changed topic. Any time the state changes, a new entry is entered into
				// 		the history with the current nav state (TODO: just deltas or full state?). On back button/forward button
				// 		press, the NAVIGATION_CONTROLLER is "initialized" with the new state and the aggregator is called.
				// 		All navigation elements and page elements are responsible for determining whether their state has
				// 		changed from the NavigationalState provided by the NAVIGATION_CONTROLLER and updating accordingly.
				
				constructor: function () {
				  	this._previousState = null;
					this._waitingForPageToRender = false;
					this._isBackButtonStateChange = false;
					this._initialized = false;
					
					this._storageUnit = new com.ibm.portal.aggregation.storage.CookieStorageUnit();
					
				  	//The state is not complete until the page has fully rendered since the portlet
					//state documents aren't known until the portlet fragment request has returned.
					//So we have to ignore state updates for portlets until the page rendering process
					//is complete because we don't want those to be in the back button history. Any 
					//portlet state updates that come in after the page is rendered completely should 
					//be added to the back button history.
					//dojo.subscribe( "/portal/PortalPage/start", this, function () { this._waitingForPageToRender = true; } );
					this._startPageHandle = com.ibm.portal.EVENT_BROKER.startPage.register( this, function () { this._waitingForPageToRender = true; } );
					/*dojo.subscribe( "/portal/NavigationalStateController/portletStateUpdated", 
												this,
												function () {
													if ( !this._waitingForPageToRender ) {
														var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
														this.addToHistory( state );
														this.saveState( state );
													}
												} 
					);*/
					this._fragmentUpdatedHandle = com.ibm.portal.EVENT_BROKER.fragmentUpdated.register( this, 
																		  function () {
																				if ( !this._waitingForPageToRender ) {
																					var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
																					this.addToHistory( state );
																					this.saveState( state );
																				}
																			}
					);
																				
					/*dojo.subscribe( "/portal/PortalPage/complete", 
												this, 
												function () {
													this._waitingForPageToRender = false;
													if ( !this._isBackButtonStateChange) {
														var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
														this.addToHistory( state );
														this.saveState( state );
													}
													else {
														this._fireBackButtonPressedComplete();
													}	
												} 
					);*/
					this._endPageHandle = com.ibm.portal.EVENT_BROKER.endPage.register( this, 
																  function () {
																	this._waitingForPageToRender = false;
																	if ( !this._isBackButtonStateChange) {
																		var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
																		this.addToHistory( state );
																		this.saveState( state );
																	}
																	else {
																		this._isBackButtonStateChange = false;
																	}	
																} 
					);
					this._redirectHandle = com.ibm.portal.EVENT_BROKER.redirect.register( this, function () { this.clearState(); } );
			    },
				setInitialState: function ( /*NavigationalState*/state ) {
					// summary: Sets the initial state for the back button.
					// description Called when the controller/init topic is published so that the initial state
					//		can be recorded.
					ibm.portal.debug.entry( "PortalBackButtonHandler.setInitialState", [ state ] );	
				
					var initialState = dojox.data.dom.innerXML( state.stateDOM );
					this._previousState = initialState;
					dojo.back.setInitialState({
	                	back: this._newBackButtonFunction( initialState ),
	                	changeUrl: false
	             	});	
				 	this._initialized = true;
				 	ibm.portal.debug.exit( "PortalBackButtonHandler.setInitialState" );
				},
				addToHistory: function ( /*NavigationalState*/state ) {
					// summary: Adds the given state to the back button history.
					// description: Subscribes to the state changed	topic and stores the changed state
					//		in the back button history.				
					ibm.portal.debug.entry( "PortalBackButtonHandler.addToHistory" );
					
					if ( !this._initialized ) {
						this.setInitialState( state );	
					}
					else {
						var currentState = dojox.data.dom.innerXML(state.stateDOM);
						
						ibm.portal.debug.text( "Current state: " + currentState  );
						
						if ( currentState != this._previousState ) {
							dojo.back.addToHistory({
								handle: this._newBackButtonFunction( currentState ),
				                changeUrl: false
							});
							this._previousState = currentState;
						}
					}	
					
					ibm.portal.debug.exit( "PortalBackButton.addToHistory" );	
				},
				saveState: function ( /*NavigationalState*/state ) {
					// summary: Stores the current state in the client-side storage so that we can handle a
					//		full-page refresh.
					// description: First attempt to simpy store the xml string as the cookie value. If that value
					//		is too large to be stored as a cookie, attempt to store the serialized url version of 
					//		that state document (not ideal since it requires an additional request to the state
					//		serialization service). If that value is too large to fit in a cookie (probably unlikely), then
					//		report a warning to the user that the navigational state is unable to be maintained on the client.
					ibm.portal.debug.entry( "PortalBackButtonHandler.saveState", [ state ] );
					
					var stateText = dojox.data.dom.innerXML( state.stateDOM );
					var stateKey = this._getStateKey();
					var browserURL = top.location.href;
					var browserURLKey = this._getUrlKey();
					
					if ( this._storageUnit.hasRoom( stateKey, stateText ) ) {
						this._storageUnit.store( stateKey, stateText );
					}
					else {
						ibm.portal.debug.text( "State is too big for the storage unit; serialize state into an url.", "PortalBackButtonHandler.saveState" );
						//Serialize the xml document into an URL and attempt to store that value on the client. 
						var stateMgr = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
						stateMgr.reset( state.stateDOM );
						var serializationMgr = stateMgr.getSerializationManager();
						var me = this;
						serializationMgr.serialize( state.stateDOM, false, function (status, stateDoc, url ) { 
							var _method = "PortalBackButtonHandler.saveState.serializationCallback";
							ibm.portal.debug.entry( _method, [ status, stateDoc, url ] );
							if ( !me._storageUnit.store( stateKey, "s8d:" + url ) ) {
								//The navigational state is simply too large to be stored. Report the warning to the user and move on.
								if ( !me._warningLogged ) {
									com.ibm.portal.aggregation.error.ERROR_HANDLER.reportWarning( "Unable to save the current state!", "Hitting the browser's refresh button now will lose the current navigational state." );
									me._warningLogged = true;
								}
							}
							ibm.portal.debug.exit( _method );
						} );
					}
					
					if ( this._storageUnit.hasRoom( browserURLKey, browserURL ) ) {
						this._storageUnit.store( browserURLKey, browserURL );
					}
					ibm.portal.debug.exit( "PortalBackButtonHandler.saveState" );
				},
				loadState: function () {
					// summary: Loads the current state from the client-side storage if it exists.
					ibm.portal.debug.entry( "PortalBackButtonHandler.loadState" );
					
					var stateText = this._storageUnit.retrieve( this._getStateKey() );
					var stateDocument = null;
					if ( stateText && stateText != "" ) {
						//Check to see if we have a serialized state url as we need to deserialize it before
						//we get anything useful.
						if ( stateText.indexOf( "s8d:" ) == 0 ) {
							ibm.portal.debug.text( "Serialized state discovered! Deserializing..." );
							var stateMgr = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
							var serializationMgr = stateMgr.getSerializationManager();
							var url = stateText.substring( 4 );
							var returnObj = serializationMgr.deserialize( url );
							stateDocument = returnObj["state"];
						}	
						else {
							stateDocument = dojox.data.dom.createDocument( stateText );
						}	
					}
					
					ibm.portal.debug.exit( "PortalBackButtonHandler.loadState", stateDocument );
					return stateDocument;
				},
				clearState: function () {
					this._storageUnit.remove( this._getStateKey() );
				},
				loadUrlFrag: function () {
					var url = this._storageUnit.retrieve( this._getUrlKey() );
					if(!url) url = "";
					return url;
				},
				_newBackButtonFunction: function ( /*String*/state ) { 
					var me = this;
					var handler = function(){
									ibm.portal.debug.text( "backbutton -> " + state );
									me._isBackButtonStateChange = true;
									com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.setStateDocument( dojox.data.dom.createDocument( state ) );
				                }
					return handler;					
				},
				_getStateKey: function () { 
					var sessionID = dojo.cookie( "JSESSIONID" );
					var token = dojo.cookie( "LtpaToken" );
					if(token && token.length > 20) token = token.substr(0, 20);
					var key = window.location.host + sessionID + token;
					//Encode this as if it will be in a url because certain characters are unsafe.
					return encodeURIComponent( key );
				},
				_getUrlKey: function () {
					return this._getStateKey() + "URL";
				}	
			  }  
);

com.ibm.portal.aggregation.backbutton.BACK_BUTTON_HANDLER = new com.ibm.portal.aggregation.backbutton.PortalBackButtonHandler();

}
