/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.event"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.event"] = true;
dojo.provide( "com.ibm.portal.aggregation.event" );

dojo.require( "com.ibm.portal.utilities" );

com.ibm.portal.aggregation.event.normalizeEvent = function ( /*Event?*/e ) {
    // summary: Retrieves and normalizes the event object, hiding the browser differences.
    // description: From the object (or lack thereof) passed into the event handler, gets the
    //      global event object (if IE) and adds the W3C methods and properties to it so that
    //      the same event object can be used in both FF and IE. Also, provides two new properties
    //      "evt.defaultPrevented" and "evt.propagationStopped" which are set when the corresponding
    //      methods are called so that the event handling code can respond accordingly (normally
    //      the browser simply handles these calls with no need for the client to interfere). It is safe
    //      to call this function mulitple times per event. In other words, the event object should 
    //      ALWAYS be passed into this function before using it in an event handler.
    // e: the event object, if none is passed in, this function will try and use the global
    //      IE window.event object.
    // returns: the normalized event object safe for use in IE or FF
    ibm.portal.debug.entry( "normalizeEvent", [e] );
    
    //Check to see if we should be using the global IE event object. If so, we need to
    //wrapper the returnValue and cancelBubble properties in the W3C equivalents, as well
    //as set the target property.
    if ( !e || (!e.preventDefault && !e.stopPropagation) ) {
        e = window.event;
        e.preventDefault = function () {
            this.returnValue = false;
        }
        e.stopPropagation = function () {
            this.cancelBubble = true;
        }
        e.target = window.event.srcElement;             
    }
    
    //Override the preventDefault and stopPropagation methods so that our two new properties
    //get set in addition to the calls to the original functions. The final boolean parameter
    //ensures that this function will only be overridden ONCE if this function is called more
    //than once per event.
    com.ibm.portal.utilities.overwriteProperty( e, "preventDefault", function () { 
        this.defaultPrevented = true;
        com.ibm.portal.utilities.callOverwrittenFunction( this, "preventDefault" );
    }, true );
    com.ibm.portal.utilities.overwriteProperty( e, "stopPropagation", function () { 
        this.propagationStopped = true;
        com.ibm.portal.utilities.callOverwrittenFunction( this, "stopPropagation" );
    }, true );
    
    ibm.portal.debug.exit( "normalizeEvent", [e] );
    return e; // Event
}

dojo.declare( "com.ibm.portal.aggregation.event.EventFilter",
              null,
              {
                // summary: Base interface for a filter. Each implementation must implement
                //      the doFilter function.
                // description: Passes the given event through to give the implementation of
                //      this filter a chance to alter or handle the event in some way. 
                
                doFilter: function (e, eventSource, elementID) {
                    // summary: Filter the given event.
                    // description: Allows this filter to handle or alter the event. If this function
                    //      returns true, processing is stopped and the event is considered "handled".
                    // e: the event object
                    // eventSource: the HTML source element for the event
                    // elementID: 
                    dojo.unimplemented( "com.ibm.portal.aggregation.event.EventFilter.doFilter" );
                }   
              } 
);

dojo.declare( "com.ibm.portal.aggregation.event.PortletEventFilter",
              com.ibm.portal.aggregation.event.EventFilter,
              {
                // summary: Extends the EventFilter interface to provide some portlet specific
                //      capabilities.
                
                triggerPortletUpdate: function ( /*String*/ url, /*String*/portletId ) {
                    // summary: Sends the request for updating a portlet to the given url.
                    // description: Uses the PortletFragmentService to request new portlet markup
                    //      based on the state encoded in the given url.
                    // url: the url to update the portlet with
                    
                    var serviceReq = new com.ibm.portal.services.PortletFragmentService();
                    serviceReq.getPortletInfo( portletId, url );
                },
                isPortalUrl: function ( /*String*/ url ) {
                    // PK72027
                    // summary: Tests the given url to see if it is a portal uri.
                    // description: The given url string is checked to see if it begins with one of
                    //      portal uri strings defined in ibmPortalConfig.
                    // url: the url to test
                    // returns: true if this url matches a portal uri, otherwise false
                    ibm.portal.debug.entry( "PortletEventFilter.isPortalUrl", [ url ] );

                    // if this url is just a query or something to append onto the top.location, then it's a Portal URL
                    // also, if it's of the form http://...., then it'll get picked up later as an external URL and handled there
                    var retVal = false;

                    if(url.length > 0 && url.charAt(0) != "/" && url.indexOf( location.host) == -1) {
                    	retVal = true;
                    }
                    // does the url match one of our portal urls
                    if ( !retVal && ( url.indexOf( ibmPortalConfig["portalPublicURI"] ) >= 0 ) ||
                         ( url.indexOf( ibmPortalConfig["portalProtectedURI"] ) >= 0 ) ) {
                        retVal = true; // Boolean
                    }

                    ibm.portal.debug.exit( "PortletEventFilter.isPortalUrl", retVal );
                    return retVal;

               },
                isFriendlyUrl: function ( /*String*/ url ) {
                	// summary: Tests the given url to see if it is a friendly uri.
                     // description: The given url string is checked to see if it begins with one of
                     //      portal uri strings defined in ibmPortalConfig.
                     // url: the url to test
                     // returns: true if this url matches a portal uri, otherwise false
                     ibm.portal.debug.entry( "PortletEventFilter.isFriendlyUrl", [ url ] );
                     var retVal = false;
                     
                     var regx = new RegExp("(?:" + ibmPortalConfig["portalPublicURI"] + (ibmPortalConfig["isUserLoggedIn"] ? "|" + ibmPortalConfig["portalProtectedURI"] : "") + ")(/[^/]+)(?:/.*)*");
                     var tUrl = top.location.href;
                     
					 var umatch = null;
                     var tmatch = tUrl.match(regx);
                     if(tmatch && tmatch[1]) {
                     	 umatch = url.match(regx);
                     	 if(umatch && umatch[1] && umatch[1] == tmatch[1]) retval = false;
                     	 else {
                     	 	retval = true;
                     	 }
                     }
                    
                     ibm.portal.debug.exit( "PortletEventFilter.isFriendlyUrl", retVal );
                     return retval;
                }
              }  
);

dojo.declare( "com.ibm.portal.aggregation.event.EventFilterChain",
              null,
              {
                // summary: Controls the registration and execution of the filters associated
                //      with this filter chain.
                constructor: function () {
                    this._filters = new Array();
                },
                addFilter: function ( filter ) {
                    // summary: Register a filter at the end of the filter chain.
                    // filter: the filter to append
                    if ( !this._filters ) { this._filters = new Array(); }
                    this._filters.push( filter );
                },
                applyFilters: function (e, eventSource, elementID ) {
                    // summary: Execute the filter chain.
                    // description: Calls doFilter on every filter registered in this filter chain
                    //      until all filters have been called or one of the filters returns true.
                    // e: the event object
                    // returns: true if the event was handled, false if it was not
                    var i = 0;
                    var returnValue = false;
                    while ( i < this._filters.length && !returnValue )  {
                    	try {
                        	returnValue = this._filters[i].doFilter(e, eventSource, elementID);
                    	}
                    	catch(err) {
                    		com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError(err.message, err.description, err.lineNumber);
                    	}
                        i = i + 1;      
                    }
                    
                    return returnValue; //Boolean
                }   
              } 
);

dojo.declare( "com.ibm.portal.aggregation.event.PortletEventHandler",
              null,
              {
                // summary: Examines all clicks that originate inside a portlet body
                //      and handles if appropriate.
                // description: Intercepts all clicks that originate from inside a 
                //      portlet and passes the event through the filter chain. If none of 
                //      filters can handle the event, it is left alone. Otherwise, it is
                //      cancelled and handled.
                constructor: function () {
                    this.onclickFilterChain = new com.ibm.portal.aggregation.event.EventFilterChain();
                    this.targetFilterChain = new com.ibm.portal.aggregation.event.EventFilterChain();
                },
                bubble: function ( /*EventFiltrChain*/filterChain, /*Event*/event, /*HTMLElement*/startElement, /*String*/idOfElemToStopAt ) {
                    // summary: Bubbles an event, executing the FilterChain for each element in the bubble path.
                    // filterChain: the filter chain to execute.
                    // event: the NORMALIZED event object
                    // startElement: the element to start bubbling from
                    // returns: true if any of the filters handles the event, false if none do
                    ibm.portal.debug.entry( "PortletEventHandler.bubble", [ filterChain, event, startElement ] );
                    
                    var retVal = false;
                    var element = startElement;
                    var utilities = com.ibm.portal.utilities;
                    
                    while ( element && !utilities.string.contains( element.getAttribute( "id" ), idOfElemToStopAt ) && !event.propagationStopped ) {
                            ibm.portal.debug.text( "Executing filters for: " + element );
                            //In IE, we have to overwrite any onclick handlers when we insert the markup because IE's event model 
                            //does not support the capture phase of the W3C event model. So before we apply the filters for each
                            //element, we need to check to see if any handlers have been overwritten and then restore the overwritten
                            //value when we're done (for the next click).
                            var overwrittenOnclick = null;
                            //First, restore any overwritten onclick handlers.
                            if ( utilities.getOverwrittenProperty( element, "onclick" ) ) {
                                overwrittenOnclick = utilities.restoreProperty( element, "onclick" );
                            }//If the event gets handled by one of our filters (a filter returns true), we don't want to 
                            //override that value when the next element return false. So once we get a true response,
                            //we just ignore any subsequent return values.
                            if ( !retVal ) {
                                retVal = filterChain.applyFilters(event, element, idOfElemToStopAt);
                            }
                            else {
                                filterChain.applyFilters(event, element, idOfElemToStopAt);
                            }    
                            
                            //Ok Now put it back.
                            if ( overwrittenOnclick ) { utilities.overwriteProperty( element, "onclick", overwrittenOnclick ); }
                            
                            element = element.parentNode;
                            ibm.portal.debug.text( "Next element in bubbling chain is: " + element );
                    }
                    
                    ibm.portal.debug.exit( "PortletEventHandler.bubble", retVal );
                    return retVal;      //Boolean
                },
                handle: function ( e ) {
                    // summary: Handles an event if it originiates from within a portlet.
                    // e: the event object
                    ibm.portal.debug.entry( "PortletEventHandler.handle", [ e ] );
                    
                    // get the event source
                    e = com.ibm.portal.aggregation.event.normalizeEvent( e );
                    var id = com.ibm.portal.utilities.findPortletIdByElement( e.target );
                  
                    ibm.portal.debug.text( "element ID to update: " + id + "; length=" + id.length)
                      
                    //Only want to handle clicks inside a portlet or a portlet toolbar.
                    if ( id.length > 0 ) {
                        //Bubbling simulation...
                        var element = e.target;
                        var activeForm = null;
                        if(element != null) {
					        	activeForm = com.ibm.portal.utilities.findFormByElement(e.target);
					        	if(activeForm) {
									var activeTrigger = { name: e.target.getAttribute( "name" ), value: e.target.getAttribute( "value" ) };
									com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.setActiveSubmitTrigger(activeForm.getAttribute("id"), activeTrigger);
					        	}
						}
                        var val = this.bubble( this.onclickFilterChain, e, element, id );
                        
                        ibm.portal.debug.text( "Default action prevented after bubbling phase? " + e.defaultPrevented );
                        
                        if ( element == null ) {
                            // The parent node of any element in the chain is null, which means the current target probably no
                            // longer exists in the DOM. So, stop propagation and cancel the default action.
                            e.stopPropagation();
                            e.preventDefault();    
                        }
                        else {
                        
                            if ( !e.defaultPrevented ) {
                                ibm.portal.debug.text( "Executing target filters for: " + e.target );
                                val = this.bubble( this.targetFilterChain, e, e.target, id );
                            }
                            
                            //If the event was handled, stop the real bubbling phase from executing. If this is IE, where there is
                            //only a bubbling phase, we've already essentially executed this phase and there's no need for the event
                            //to propagate. In Firefox/W3C compliant browsers, since this function is executed during the capture 
                            //phase of the event handling AND we've already executed any onclick handlers, we want to make sure the
                            //onclick handler is not executed again if propagation is not stopped (i.e. when the event hits the target
                            //element in between the capture/bubble phases). 
                            if ( val || window.event ) {
                                e.stopPropagation();
                            } else if ( !e.propagationStopped ) {
                                ibm.portal.debug.text( 'replacing onclick handler: ' + e.target + "; " + e.target.onclick );
                                com.ibm.portal.utilities.overwriteProperty( e.target, "onclick", function (e) { 
                                                                                                   ibm.portal.debug.text( 'stopping existing onclick' ); 
                                                                                                   e.stopPropagation(); 
                                                                                                   com.ibm.portal.utilities.restoreProperty( this, "onclick" ); 
                                                                                                 } );    
                            }
                        }     
                        
                        if(activeForm) com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.setActiveSubmitTrigger(activeForm.getAttribute("id"), null);   
                    }
        
                    ibm.portal.debug.exit( "PortletEventHandler.handle" );
                }
              }  
);

dojo.declare( "com.ibm.portal.aggregation.event.filters.JSLocationHrefFilter",
              com.ibm.portal.aggregation.event.PortletEventFilter,
              {
                // summary: Filters out any calls to set the location of the window via javascript.
                // description: Filters out and handles events which result in a javascript call to set the window location.
                //      We cannot allow these to execute unmodified since they would completely destroy our client-side
                //      aggregation. This filter checks for an onclick event or an href on the eventSource, parses
                //      out the target URL and loads that URL into the portlet. This filter stops processing if it 
                //      succesfully parses out a target URL and loads it into the portlet.
                doFilter: function ( e, eventSource, elementID ) {
                    // summary: Checks the eventSource element for any calls to set window.location
                    // e: the event object
                    // eventSource: the event source in the DOM
                    // returns: true if we succesfully parse out a target URL and load it into the portlet                  
                    ibm.portal.debug.entry( "JSLocationHrefFilter.doFilter", [ e, eventSource, elementID ] );
                    var targetUrl = "";
                    var jsString = "";
                    
                    if ( eventSource.getAttribute( "onclick" ) ) {
                        //IE departs from W3C DOM spec yet again and returns the function object here
                        jsString = eventSource.getAttribute( "onclick" );
                        
                        ibm.portal.debug.text( "onclick handler is " + jsString );
                        ibm.portal.debug.text( "typeof jsString is " + typeof(jsString ) );
            
                        if ( typeof( jsString ) == "function" ) {
                            ibm.portal.debug.text( "Converting function object into a string..." );
                            jsString = com.ibm.portal.utilities.eventHandlerToString( jsString );
                        }
                    }
                    else {
                        if ( eventSource.href ) {
                            jsString = eventSource.getAttribute( "href" );
                        }
                    }
                    
                    
                    ibm.portal.debug.text( "jsString is " + jsString );
                    
                    if ( jsString != "" ) {
                        targetUrl = this._parseTargetUrl( jsString );
                        
                        if (targetUrl!="")
                        {
                            ibm.portal.debug.text( "target URL is " + targetUrl );

                            var rv = this.isPortalUrl(targetUrl);
                            if (rv) {
                                // stop the event from propagating
                                e.preventDefault();
                                //e.stopPropagation();

                                this.triggerPortletUpdate( targetUrl, elementID );
                            }
                            
                            ibm.portal.debug.exit( "JSLocationHrefFilter.doFilter", [ rv ] );
                            return rv; // Boolean
                        }
                    }
                    
                    ibm.portal.debug.exit( "JSLocationHrefFilter.doFilter", [ false ] );
                    
                    return false; // Boolean    
                },
                
                _parseTargetUrl: function ( jsString ) {
                    // summary: Parse out the url that is being set to one of the location properties of the window.
                    // jsString: the javascript string to parse
                    if (((jsString.indexOf("self.location") != -1) ||
                          (jsString.indexOf("self.location.href") != -1) ||
                          (jsString.indexOf("location.href") != -1) ||
                          (jsString.indexOf("window.location") != -1)) ||
                          (jsString.indexOf("window.location.href") != -1 ) ||
                          (jsString.indexOf("top.location") != -1) ||
                          (jsString.indexOf("top.location.href") != -1) )
                    {
                        // if there's a semi-colon, get rid of everything after it
                        // gets rid of multiple statements in the onclick
                        var semiColonIndex = jsString.indexOf(";");
                        if (semiColonIndex > -1)
                        {
                            jsString.substring(0, semiColonIndex);
                        }
                            
                        // find starting quote char
                        var dblQuoteCharIndex = jsString.indexOf('"');
                        var quoteCharIndex = jsString.indexOf("'");
                    
                        var targetUrl = "";
                                
                        if (dblQuoteCharIndex == -1 && quoteCharIndex == -1)
                        {
                            //couldn't find the quote - just return
                            return;
                        }
                        else if (dblQuoteCharIndex == -1)
                        {
                            //code used single quotes
                            var endIndex = jsString.indexOf("'", quoteCharIndex+1);
                            targetUrl = jsString.substring(quoteCharIndex+1, endIndex);
                        }
                        else
                        {
                            //code used double quotes
                            var endIndex = jsString.indexOf('"', dblQuoteCharIndex+1);
                            targetUrl = jsString.substring(dblQuoteCharIndex+1, endIndex);
                        }
                        
                        return targetUrl; // String
                    }
                    
                    return ""; // String
                }   
              } 
);


dojo.declare( "com.ibm.portal.aggregation.event.filters.OnclickFilter",
              com.ibm.portal.aggregation.event.PortletEventFilter,
              {
                // summary: Handles any onclick events which do not set the window location.
                // description: Executes the onclick function and, based on the result, stops processing 
                //      or continues. If the onclick function returns false, the event is stopped and processing
                //      ended. Otherwise, processing will continue to the next filter.
                
                doFilter: function (e, eventSource, elementID ) {
                    // summary: Check for and execute the onclick event of the event source element.
                    // e: the event object
                    // eventSource: the source element
                    // returns true if the onclick handler indicates the event should be cancelled.
                    ibm.portal.debug.entry( "OnclickFilter.doFilter", [ e, eventSource, elementID ] );
                    
                    ibm.portal.debug.text( "onclick is " + eventSource.onclick );
                    
                    if ( eventSource.onclick ) {
                        var retVal = null; 
                        // this chain is used to keep track of multiple handlers that were
                        // added to this node's onclick event through dojo.connect
                        // the first handler attached is always on eventSource.onclick
                        // the chain only includes every handler after the first one
                        // and only the return value of the first one is kept 
                        var chain = eventSource._onclickChain;
                        if ( window.event ) {
                            retVal = eventSource.onclick();
                            if(chain) {
                            	for(var i = 0; i < chain.length; i++) {
                            		chain[i]();
                            	}
                            }
                        }
                        else {
                            retVal = eventSource.onclick(e);
                            if(chain) {
                            	for(var i = 0; i < chain.length; i++) {
                            		chain[i](e);
                            	}
                            }
                        }
                        
                        ibm.portal.debug.text( "return value from onclick is: " + retVal, "OnclickFilter.doFilter" );
                        
                        //If false, the event is handled.
                        if ( retVal == false ) {  
                            e.preventDefault();
                            //e.stopPropagation();
                            
                            ibm.portal.debug.exit( "OnclickFilter.doFilter", [ true ] );
                            return true;
                        }
                    }
                    
                    //TODO: what to do if the retVal is true???? We've already executed the onclick
                    //but we don't want to stop the event, this will cause the onclick to get executed
                    //again when the event reaches the anchor element.
            
                    ibm.portal.debug.exit( "OnclickFilter.doFilter", [ false ] );
                    return false;
                }   
              }  
);

dojo.declare( "com.ibm.portal.aggregation.event.filters.DefaultActionFilter",
			  com.ibm.portal.aggregation.event.PortletEventFilter,
			  {
			  	// summary: Handles any anchor elements that are clicked on.
                // description: It loads the specified URL into the portlet or, if the href contains javascript, 
                //      executes the javascript.
              
                doFilter: function ( e, eventSource, elementID ) {
                	ibm.portal.debug.entry( "DefaultActionFilter.doFilter", [ e, eventSource, elementID ] );
                	//If the default action was not canceled at some point in handling the event, go
					//ahead and execute our new default action handler.
					var defaultAction = eventSource.defaultAction;
					var retVal = false;
					
					if ( defaultAction ) {
						ibm.portal.debug.text( "Found default action: " + defaultAction );
						if ( !e.defaultPrevented ) {
							defaultAction.apply( eventSource, [ e ] );	
						}	
						
						//Prevent the browser from executing the default action and stop this event
						//from bubbling. The event handlers have already had a chance to handle any
						//original events. The default action is a "browser" action and so shouldn't be
						//handled by anyone except us.
						e.preventDefault();
						e.stopPropagation();
						retVal = true;
					}	
					
					ibm.portal.debug.exit( "DefaultActionFilter.doFilter" );
                }	
			  }
);

dojo.declare( "com.ibm.portal.aggregation.event.filters.AnchorFilter",
              com.ibm.portal.aggregation.event.PortletEventFilter,
              {
                // summary: Handles any anchor elements that are clicked on.
                // description: It loads the specified URL into the portlet or, if the href contains javascript, 
                //      executes the javascript.
              
                doFilter: function ( e, eventSource, elementID ) {
                    // summary: If the eventSource is an anchor, load the href into the portlet.
                    // e: the event object
                    // eventSource: the event source element
                    // returns true if an href value is succesfully loaded into the portlet                 
                    ibm.portal.debug.entry( "AnchorFilter.doFilter", [ e, eventSource, elementID ] );
                    
                    var nodeName = eventSource.nodeName.toLowerCase();
                    if (nodeName == 'a' && e.button == 0) {
                        var url = eventSource.getAttribute('href');
                        
                        ibm.portal.debug.text( "anchor's href is " + url );
                        var target = eventSource.getAttribute('target');
                        
                        //allow the browser to handle this since we need it in a new window.
                        if ( target != null && ( target == "_blank" || target == "_parent" ) ) {
                            return false;
                        } 
                        
                        if (url != null) {
                            
                            if (url.toLowerCase().indexOf("javascript:")==0)
                            {
                                //Stop the event from propagating.
                                //ibm.portal.Utilities.stopEvent(e);
                                e.preventDefault();
                                //e.stopPropagation();
                                //IE url encodes all href values, including javascript href values. This can cause some bizarre results when evaluating 
                                //this js href. So we're going to "decode" any encoded values. WE may need to do more here. For now, this is the only case
                                //we've run into.
                                if ( dojo.isIE ) {
                                    url = url.replace( /%20/g, ' ' );
                                }
                                ibm.portal.debug.text( "Evaluating javascript href: " + url );
                                eval( url );
                                ibm.portal.debug.exit( "AnchorFilter.doFilter", true );
                                return true;
                            }
                            else
                            {
                                var rv = this.isPortalUrl(url);
                                if (rv) {
                                    // stop the event from propagating
                                    e.preventDefault();
                                    //e.stopPropagation();

                                    this.triggerPortletUpdate( url, elementID );
                                }
								ibm.portal.debug.exit( "AnchorFilter.doFilter", rv );
                                return rv;
                            }
                        }
                    }
                    
                    ibm.portal.debug.exit( "AnchorFilter.doFilter", false );
                    return false;   
                }             
              } 
);

com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER = new com.ibm.portal.aggregation.event.PortletEventHandler();

//Regsiter all of our filters. Please note that order IS important! The JSLocationHrefFilter should 
//always be executed before the OnclickFilter and AnchorFilter, since allowing the location to be set 
//completely breaks client side aggregation.
com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.onclickFilterChain.addFilter( new com.ibm.portal.aggregation.event.filters.JSLocationHrefFilter() );
com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.onclickFilterChain.addFilter( new com.ibm.portal.aggregation.event.filters.OnclickFilter() );
com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.targetFilterChain.addFilter( new com.ibm.portal.aggregation.event.filters.AnchorFilter() );
com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.targetFilterChain.addFilter( new com.ibm.portal.aggregation.event.filters.DefaultActionFilter() );

//Register the onclick event handler.
if (document.addEventListener)
{
    document.addEventListener("click", dojo.hitch( com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER, "handle"), true);
}
else if (document.attachEvent)
{
    document.attachEvent("onclick", dojo.hitch( com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER, "handle") );
}

dojo.declare( "com.ibm.portal.aggregation.event.PortletEventDampener",
              null,
              {
                  // summary: Creates a handler that can be used to suppress certain events from
                  //        bubbling. Optionally, the default action for the event can also be 
                  //        cancelled, based on the cancelTest, if one is specified.
                  // description: Event handler that can be attached/added to any DOM element to 
                  //        ensure that the event does not bubble any higher than that DOM
                  //        element. If a cancelTest is specified in the constructor, the boolean
                  //        result of that function execution is used to determine if the default
                  //        action of the event is also cancelled. 
                  constructor: function(/*Function?*/cancelTest) {
                      // summary: Sets the cancelTest if one is specified. 
                      // cancelTest: function to determine wether to cancel the default action for
                      //        an event. a return value of true will cancel the default action. the
                      //        lone parameter will be the event object (regardless of browser).
                      this._cancelDefault = cancelTest;
                      ibm.portal.debug.text( "Cancel test is: " + this._cancelDefault );
                  },
                  dampen: function ( /*Event?*/e) {
                      // summary: Suppresses the event to which this handler is attached.
                      // e: the event object (only expected in Firefox).
                      ibm.portal.debug.entry( "PortletEventDampener.dampen", [ e ] );
                      
                      e = com.ibm.portal.aggregation.event.normalizeEvent(e);
                      ibm.portal.debug.text( "Dampening event: " + e );
                      e.stopPropagation();
                      if ( this._cancelDefault && this._cancelDefault(e) ) {
                          e.preventDefault();
                      }
                      //return false;
                      
                      ibm.portal.debug.exit( "PortletEventDampener.dampen" );
                  }
              }
);


}
