/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.event.DefaultActionManager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.event.DefaultActionManager"] = true;
dojo.provide( "com.ibm.portal.aggregation.event.DefaultActionManager" );

dojo.require( "com.ibm.portal.utilities" );
dojo.require( "com.ibm.portal.aggregation.event" );

dojo.declare( "com.ibm.portal.aggregation.event.DefaultActionManager",
			  null,
			  {
			  	// summary: Allows for "hijacking" an HTML element's default action.
				// description: Most HTML elements that can be interacted with by a user have
				//		default actions. For example, an ANCHOR element's default action is to
				//		follow the href associated with the element. In some cases, we want to 
				//		take complete control of an element (e.g. an INPUT of type SUBMIT). This
				//		manager allows us to override the given event and provide a new handler 
				//		for the default action. The default action is ALWAYS prevented. If the
				//		event handlers don't cancel the default action (i.e. return false), we
				//		call the new default action handler instead of allowing the actual default
				//		action to execute.
				
				setDefaultAction: function ( /*HTMLElement*/element, /*String*/eventName, /*Function*/handler ) {
					// summary: Sets a new default action for the given HTML element and given event name.
					// element: the HTML element we want to take over the default action for
					// eventName: the name of the event we want to control
					// handler: the new default action handler to call instead
					ibm.portal.debug.entry( "DefaultActionManager.setDefaultAction", [ element, eventName, handler ]);
					
					//Set the new default action handler as a property on the object and overwrite the
					//event handler with our new function.
					element.defaultAction = handler;
					//com.ibm.portal.utilities.overwriteProperty( element, eventName, this._newDefaultActionWrapper( eventName ) )
					
					ibm.portal.debug.exit( "DefaultActionManager.setDefaultAction" );
				},
				_newDefaultActionWrapper: function (/*String*/eventName ) {
					// summary: Creates the new handler which wrappers the new default action.
					// eventName: the event name we are overwriting
					// returns: a new Function object
					ibm.portal.debug.entry( "DefaultActionManager._newDefaultActionWrapper" );
					
					var wrapper = function ( /*Event*/evt ) {
						ibm.portal.debug.entry( "NewDefaultAction", [ evt ]);
						
						//Get the normalized event object.
						evt = com.ibm.portal.aggregation.event.normalizeEvent( evt );
						
						//If there was an existing event handler for the given event, we want to restore the original
						//value before passing the event to the event handler. This way the event handler will 
						//automatically handle the original event and call preventDefault in the appropriate
						//places.
						if ( com.ibm.portal.utilities.getOverwrittenProperty(this,eventName) != null ) {
							com.ibm.portal.utilities.restoreProperty( this, eventName );
							com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.handle( evt );
						}
						
						//If the default action was not canceled at some point in handling the event, go
						//ahead and execute our new default action handler.
						if ( !evt.defaultPrevented ) {
							this.defaultAction( evt );	
						}	
						
						//Prevent the browser from executing the default action and stop this event
						//from bubbling. The event handlers have already had a chance to handle any
						//original events. The default action is a "browser" action and so shouldn't be
						//handled by anyone except us.
						evt.preventDefault();
						evt.stopPropagation();
						
						ibm.portal.debug.exit( "NewDefaultAction", [ false ] );
						return false; // Boolean
					}
					
					ibm.portal.debug.exit( "DefaultActionManager._newDefaultActionWrapper", [ wrapper ] );
					return wrapper;	// Function
				}	
			  }
);

com.ibm.portal.aggregation.event.DEFAULT_ACTION_MANAGER = new com.ibm.portal.aggregation.event.DefaultActionManager();

}
