/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval.collectors"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval.collectors"] = true;
dojo.provide( "com.ibm.portal.aggregation.javascript.uneval.collectors" );

dojo.require( "com.ibm.portal.aggregation.javascript.uneval" );

com.ibm.portal.aggregation.javascript.uneval.collectors.COMMON_GLOBAL_VARIABLES = {
	// summary: Tracks any global variables that should be tested for when doing JavaScript
	//		clean up between portlet requests and page switches. This is really only applicable
	//		for IE since we cannot detect global variables in IE. 
	// description: IE does not properly enumerate global javascript variables as properties of the global scope (usually window).
	//		Therefore, we need to have an explicit list of global javascript variables that COULD be created by a portlet and need
	// 		to be cleaned up when the portlet is no longer on the page (or between requests). The primary use case for this would be for
	//		javascript libraries (i.e. JSF, dojo, etc).
	_variableNames: [],
	_variableNamesIndex: {},
	addVariable: function ( /*String*/ varName ) {
		// summary: Add a variable name to the "watched" list.
		// varName: the name of the variable to watch for
		if ( !this.contains( varName ) ) {
			this._variableNames.push( varName );
			this._variableNamesIndex[ varName ] = true;
		}
	},
	contains: function ( /*String*/ varName ) {
		// summary: Indicates whether or not the given variable name is on the "watched" list.
		// varName: the variable name to test for
		// returns: true if the variable name is watched; false if it is not
		return this._variableNamesIndex[ varName ];
	},
	getVariableNames: function () {
		// summary: Returns a list of all "watched" variable names.
		// returns: an array of Strings consisting of the "watched" variable names
		return this._variableNames;
	}
} 

if ( dojo.isIE ) {
	//IBM JSF v3 support
	com.ibm.portal.aggregation.javascript.uneval.collectors.COMMON_GLOBAL_VARIABLES.addVariable( "hX_5" );
}

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector",
		  com.ibm.portal.aggregation.javascript.uneval.Collector,
		  {
			// summary: Collects the enumerated properties in the global scope in most browsers. In IE, collects
			//		any variables in the COMMON_GLOBAL_VARIABLES list that are actually defined.
			constructor: function ( /*Array<String>*/ globalJSVariables ) {
				this._properties = [];
			},
			collect: function () {
				// summary: Collect the necessary information required to clean up a set of properties or objects.
				var _method = this.declaredClass + ".collect";
				ibm.portal.debug.entry( _method );
				
				for ( prop in window ) {
					this._properties[prop] = true;
				}
				var globalVars = com.ibm.portal.aggregation.javascript.uneval.collectors.COMMON_GLOBAL_VARIABLES.getVariableNames();
				var len = globalVars.length;
				for ( var i = 0; i < len; i++ ) {
					ibm.portal.debug.text( "Eval'ing global variable name: " + globalVars[i] );
					//Unfortunately, the only way I can think of to change the string variable name to the actual variable object is 
					//through eval. Since this code will only be executed in IE, we cannot simply test the window[variableName] property.
					//If an error occurs eval'ing the variable name, we will assume that it is undefined.
					try {
						var varType = eval (globalVars[i]);
					} catch ( err ) {
						varType = undefined;
					}
					ibm.portal.debug.text( "varType is: " + varType );
					if ( typeof varType != "undefined" ) {
						ibm.portal.debug.text( "Marking property \"" + globalVars[i] + "\" as defined!" );
						this._properties[globalVars[i]] = true;
					}
				}
				
				ibm.portal.debug.exit( _method );
			},
			getValues: function () {
				// summary: Retrieves the values collected by the collect action.
				// return: implementation specific; a scrubber and a collector usually go hand in hand
				var _method = this.declaredClass + ".getValues";
				ibm.portal.debug.entry( _method );
				ibm.portal.debug.exit( _method, this._properties );
				return this._properties;
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.collectors.DijitCollector",
		  com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector,
		  {
			// summary: Collects any dijits (Dojo widgets) that are currently defined.
			collect: function () {
				var _method = this.declaredClass + ".collect";
				ibm.portal.debug.entry( _method );
				// summary: Collect the necessary information required to clean up a set of properties or objects.
				if(typeof dijit != "undefined" && dijit.registry) {
					var array = this._properties;
					dijit.registry.forEach(function(widget){ ibm.portal.debug.text( "Adding widget: " + widget.id ); array[widget.id] = true; } );
				}
				ibm.portal.debug.exit( _method );
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.collectors.DojoCollector",
		  com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector,
		  {
			// summary: Collects any dojo modules that are currently defined. 
			collect: function () {
				// summary: Collect the necessary information required to clean up a set of properties or objects.
				var _method = this.declaredClass + ".collect";
				ibm.portal.debug.entry( _method );
				var array = this._properties;
				if ( dojo && dojo._modulePrefixes ) {
					for( moduleName in dojo._modulePrefixes ) {
						array[moduleName] = true;
					}
				}
				ibm.portal.debug.exit( _method );
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.collectors.IntervalCollector",
		  com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector,
		  {
			// summary: Collects any intervals that have been set. We need to clean these up when the portlet
			//		goes out of scope since the interval function would no longer be relevant.
			collect: function () {
				// summary: Collect the necessary information required to clean up a set of properties or objects.
				var _method = this.declaredClass + ".collect";
				ibm.portal.debug.entry( _method );
				var utils = com.ibm.portal.utilities;
					
				if ( utils.getOverwrittenProperty( window, "setInterval" ) ) {
					ibm.portal.debug.text( "Restoring interval property." );
					utils.restoreProperty( window, "setInterval" );
				}
				else {	
					ibm.portal.debug.text( "Overwriting interval property." );
					var me = this;
					utils.overwriteProperty( window, "setInterval", function () { return me._setInterval.apply( me, arguments ); } );
				}
				ibm.portal.debug.exit( _method );
			},
			_setInterval: function () {
				ibm.portal.debug.entry( "IntervalCollector._setInterval", arguments );
				
				ibm.portal.debug.text( "window.setInterval is: " + window.setInterval );
				ibm.portal.debug.text( "window[_overwritten_]: " + window["_overwritten_"]);
				ibm.portal.debug.text( "window[_overwritten_][setInterval]: " + window["_overwritten_"]["setInterval"]);
				
				var timeoutFn = com.ibm.portal.utilities.getOverwrittenProperty( window, "setInterval" );
				ibm.portal.debug.text( "Retrieved original overwritten property: " + timeoutFn );
				var pointer = timeoutFn( arguments[0], arguments[1] );
				this._properties.push( pointer );
				
				ibm.portal.debug.exit( "IntervalCollector._setInterval", pointer );
				return pointer;
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.collectors.IBMJSFv3Collector",
		  com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector,
		  {
			// summary: Collects any JSF components that are currently defined. NOTE: Only applicable to 
			//		IBM's JSF v3 implementation.
			collect: function () {
				// summary: Collect the necessary information required to clean up a set of properties or objects.
				var _method = this.declaredClass + ".collect";
				ibm.portal.debug.entry( _method );
	
				this._properties = [];
				
				if(typeof hX_5 != "undefined") {
					var components = hX_5.getComponentsInId( "mainContent" );
					ibm.portal.debug.text( "retrieved components: " + components );
					
					if ( components ) {
						var length = components.length;
						for ( var i = 0; i < length; i++ ) {
							this._properties[ components[i] ] = true;
						}
					}	
				}
				
				ibm.portal.debug.exit( _method );
			}
		  }
);

}
