/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval.scrubbers"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval.scrubbers"] = true;
dojo.provide( "com.ibm.portal.aggregation.javascript.uneval.scrubbers" );

dojo.require( "com.ibm.portal.aggregation.javascript.uneval" );
dojo.require( "com.ibm.portal.aggregation.javascript.uneval.collectors" );

//summary: Provides a list of methods and properties which should never be deleted during cleanup.
//description: In IE, even native methods cannot be enumerated using the global scope. This is a bug in IE or the JScript
//		engine. If we overwrite a native javascript function via window.<method_name> = , then the function becomes declared
//		in the global scope. Since it didn't "exist" in a retrievable form before we took the initial snapshot, the function 
//		appears new to our cleanup code and we attempt to delete it. This list is taken from the MSDN documentation for the window
//		object.	 
com.ibm.portal.aggregation.javascript.uneval.DO_NOT_DELETE = {   "alert": true,
																 "attachEvent": true,
																 "blur": true,
																 "clearInterval": true,
																 "clearTimeout": true,
																 "close": true,
																 "confirm": true,
																 "createPopup": true,
																 "detachEvent": true,
																 "execScript": true,
																 "focus": true,
																 "moveBy": true,
																 "moveTo": true,
																 "navigate": true,
																 "open": true,
																 "postMessage": true,
																 "print": true,
																 "prompt": true,
																 "resizeBy": true,
																 "resizeTo": true,
																 "scroll": true,
																 "scrollTo": true,
																 "setActive": true,
																 "setInterval": true,
																 "setTimeout": true,
																 "showHelp": true,
																 "showModalDialog": true,
																 "showModelessDialog": true,
																 "_overwritten_": true};

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber",
			  com.ibm.portal.aggregation.javascript.uneval.Scrubber,
			  {
				// summary: Manages the WindowPropertyCollectors.
				newCollector: function () {
					// summary: Creates a new collector appropriate for this scrubber.
					// returns: a new instance of a Collector
					return new com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector();
				},
				clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
					// summary: Given two collectors, clean the difference between the two.
					// collector1: the initial state to return to
					// collector2: the final state
					var _method = this.declaredClass + ".clean";
					ibm.portal.debug.entry( _method, [ collector1, collector2 ] );
					
					var undeletables = com.ibm.portal.aggregation.javascript.uneval.DO_NOT_DELETE;
					var propsToDelete = this._getDelta( collector1.getValues(), collector2.getValues(), function ( prop ) { return undeletables[ prop ] != true; } );
					var commonGlobals = com.ibm.portal.aggregation.javascript.uneval.collectors.COMMON_GLOBAL_VARIABLES;
					
					//Delete global variables and functions that are new to this restore point.				
					for ( var i = 0; i < propsToDelete.length; i++ ) {
						var prop = propsToDelete[i];
						ibm.portal.debug.text( "Deleting: " + prop );
						
						//Ideally, we want to delete the property using the javascript delete keyword. If that
						//fails, at the very least, we can null out the property.
						try { 
							if ( commonGlobals.contains( prop ) ) {
								try {
									var varValue = eval( prop );
								} catch ( err ) {
									varValue = null;
								}
								ibm.portal.debug.text( "Eval'd variable value: " + varValue );
								varValue = null;
							} else {
								var success = delete window[prop];
								if(!success) window[prop] = null;
							}	
						}
						catch (err) {	
							ibm.portal.debug.text( "An error occurred while deleting property \"" + prop + "\": " + err.message )
							window[prop] = null;
						}						
					}	
					
					ibm.portal.debug.exit( _method );
				},
				_getDelta: function ( /*Array*/beforeArray, /*Array*/afterArray, /*Function?*/conditional ) {
			  		// summary: Creates an array of all the property names that satisfy the given conditional
			  		// beforeArray: the "before" array
			  		// afterArray: the "after" array
			  		// conditional: a function that can be optionally be used to test additional conditions for inclusion
			  		//		in the delta.
			  		// returns: an array of property names that were introduced by the current portlet executing javascript code
			  		var _method = this.declaredClass + "._getDelta";
					ibm.portal.debug.entry( _method, [ beforeArray, afterArray ] );
			  		
			  		var delta = [];
			  		//In case the optional conditional isn't specified...
			  		if ( !conditional ) { conditional = function () { return true; } }
			  		
			  		for ( var prop in afterArray ) {
			  			if ( beforeArray[prop] != true && conditional( prop ) ) {
			  				delta.push( prop );
			  			}
			  		}
			  		
			  		ibm.portal.debug.exit( _method, delta );
			  		return delta;
			  	}
			  }
);



dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.scrubbers.DijitScrubber",
		  com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber,
		  {
			// summary: Manages the DijitCollectors.
			newCollector: function () {
				// summary: Creates a new collector appropriate for this scrubber.
				// returns: a new instance of a Collector
				return new com.ibm.portal.aggregation.javascript.uneval.collectors.DijitCollector();
			},
			clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
				// summary: Given two collectors, clean the difference between the two.
				// collector1: the initial state to return to
				// collector2: the final state
				var _method = this.declaredClass + ".clean";
				ibm.portal.debug.entry( _method, [ collector1, collector2 ] );
				
				var dijitsToDelete = this._getDelta( collector1.getValues(), collector2.getValues() );
				
				//Call destroy on any widgets created in this restore point.
				for ( var i = 0; i < dijitsToDelete.length; i++ ) {
					ibm.portal.debug.text( "Deleting: " + dijitsToDelete[i] );

					try { 
						var widget = dijit.byId(dijitsToDelete[i]);
						//destroy recursive, may also destroy ones we haven't gotten to yet
						//so we check to ensure the widget exists first in case it was already removed
						if(widget) widget.destroyRecursive();	
					}
					catch (err) {
						dijit.registry.remove(dijitsToDelete[i]);
					}						
				}
				
				ibm.portal.debug.exit( _method );
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.scrubbers.DojoScrubber",
		  com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber,
		  {
			// summary: Manages the DojoCollectors.
			newCollector: function () {
				// summary: Creates a new collector appropriate for this scrubber.
				// returns: a new instance of a Collector
				return new com.ibm.portal.aggregation.javascript.uneval.collectors.DojoCollector();
			},
			clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
				// summary: Given two collectors, clean the difference between the two.
				// collector1: the initial state to return to
				// collector2: the final state
				var _method = this.declaredClass + ".clean";
				ibm.portal.debug.entry( _method, [ collector1, collector2 ] );
				
				var dojoModulesToDelete = this._getDelta( collector1.getValues(), collector2.getValues() );
				
				//Delete any dojo modules loaded via dojo.require in this restore point.
				for ( var i = 0; i < dojoModulesToDelete.length; i++ ) {
					ibm.portal.debug.text( "Deleting Dojo Module: " + dojoModulesToDelete[i] );
					
					//Ideally, we want to delete the property using the javascript delete keyword. If that
					//fails, at the very least, we can null out the property.
					try { 
						//Basically have to undo everything here that dojo.require does...
						var moduleName = dojoModulesToDelete[i];
						var moduleUrl = dojo._getModuleSymbols(moduleName).join("/") + ".js";
						var success1 = delete dojo._modulePrefixes[moduleName];
						var success2 = delete dojo._loadedModules[moduleName];
						var success3 = delete dojo._loadedUrls[moduleUrl];
						
						var j = 0; 
						var success4 = false;
						while ( j < dojo._loadedUrls.length && !success4 ) {
							success4 = (dojo._loadedUrls[j] == moduleUrl );
							if (success4) { 
								delete dojo._loadedUrls[j]; 
							}
							j=j+1; 
						}
						
						if(!success1 && !success2 && !success3 && !success4) {
							dojo._modulePefixes[moduleName] = null;
							dojo._loadedModules[moduleName] = null;
							dojo._loadedUrls[moduleUrl] = null;
						}	
					}
					catch (err) {	
						dojo._modulePrefixes[dojoModulesToDelete[i]] = null;
						dojo._loadedModules[dojoModulesToDelete[i]] = null;
					}	
					
					ibm.portal.debug.exit( _method );
				}			
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.scrubbers.IntervalScrubber",
		  com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber,
		  {
			// summary: Manages the IntervalCollectors.
			newCollector: function () {
				// summary: Creates a new collector appropriate for this scrubber.
				// returns: a new instance of a Collector
				return new com.ibm.portal.aggregation.javascript.uneval.collectors.IntervalCollector();
			},
			clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
				// summary: Given two collectors, clean the difference between the two.
				// collector1: the initial state to return to
				// collector2: the final state
				var _method = this.declaredClass + ".clean";
				ibm.portal.debug.entry( _method, [ collector1, collector2 ] );
				
				var intervalsToDelete = collector1.getValues();
				
				//Clear any intervals set by this restore point.
				for ( i = 0; i < intervalsToDelete.length; i++ ) {
					ibm.portal.debug.text( "Clearing interval: " + intervalsToDelete[i] );
					clearInterval( intervalsToDelete[i] );
				}	
				
				ibm.portal.debug.exit( _method );
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.scrubbers.IBMJSFv3Scrubber",
		  com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber,
		  {
			// summary: Manages the JSFv3Collectors.
			newCollector: function () {
				// summary: Creates a new collector appropriate for this scrubber.
				// returns: a new instance of a Collector
				return new com.ibm.portal.aggregation.javascript.uneval.collectors.IBMJSFv3Collector();
			},
			clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
				// summary: Given two collectors, clean the difference between the two.
				// collector1: the initial state to return to
				// collector2: the final state
				var _method = this.declaredClass + ".clean";
				ibm.portal.debug.entry( _method, [ collector1, collector2 ] );
				
				var compsToDelete = this._getDelta( collector1.getValues(), collector2.getValues() );
				
				//Call destroy on any widgets created in this restore point.
				for ( var i = 0; i < compsToDelete.length; i++ ) {
					ibm.portal.debug.text( "Deleting: " + compsToDelete[i] );

					var comp = hX_5.getComponentById( compsToDelete[i] )
					hX_5.removeComponent( comp );
				}
				
				ibm.portal.debug.exit( _method );
			}
		  }
);

com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS.push( new com.ibm.portal.aggregation.javascript.uneval.scrubbers.IBMJSFv3Scrubber() );
com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS.push( new com.ibm.portal.aggregation.javascript.uneval.scrubbers.DijitScrubber() );
com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS.push( new com.ibm.portal.aggregation.javascript.uneval.scrubbers.DojoScrubber() );
com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS.push( new com.ibm.portal.aggregation.javascript.uneval.scrubbers.IntervalScrubber() );
com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS.push( new com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber() );

}
