/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.theme"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.theme"] = true;
dojo.provide( "com.ibm.portal.aggregation.theme" );

dojo.require( "com.ibm.portal.services.common" );
dojo.require( "com.ibm.portal.utilities.html" );
dojo.require( "com.ibm.aggregation" );
dojo.require("dojo.i18n");

dojo.declare( "com.ibm.portal.aggregation.theme.PortalNavigation",
    com.ibm.aggregation.Navigation,
    {
        // summary: Renders all registered NavigationElements using the current state.
        constructor: function () {
            this._elements = new Array();
        },
        render: function ( /*NavigationalState*/ state ) {
            // summary: Renders the navigation based on the navigational state.
            // state: the current navigational state
            ibm.portal.debug.entry( "PortalNavigation.render", [state]);
            ibm.portal.debug.text( "Navigation has " + this._elements.length + " elements." );
            for ( var i = 0; i < this._elements.length; i++ ) {
                if(this._elements[i].isEnabled()){
                    this._elements[i].render( state );
                }
            }
            ibm.portal.debug.exit( "PortalNavigation.render" );
        },
        addNavigationElement: function (/*NavigationElement*/ elem) {
            this._elements.push( elem );
        }
    } 
);

dojo.declare( "com.ibm.portal.aggregation.theme.NavigationElement",
              null,
              {
                // summary: Describes a generic navigation element.
                // description: Provides a generic abstract navigation element with various helper 
                //      and convenience functions. Determines if the state has changed, if it's 
                //      changed, the url from _getURL is used to send a PortalRestServiceRequest
                //      the result of which is passed into applyXslt. The specified xslt is applied
                //      and the html element is updated with the results. The render function can be
                //      overridden to implement any custom behavior, ignoring the helper functions. 
                
                _xslt: null,
                _htmlElementID: null,
                _resourceBaseURL: dojo.baseUrl + "../com/ibm/portal/aggregation/",        
                render: function ( /*NavigationalState*/ state ) {
                    // summary: Renders the navigation element.
                    // state: the current navigational state.
                    ibm.portal.debug.entry( "NavigationElement.render", [state]);
                    if ( this.hasChanged( state ) ) {
                        this.setLoading();
                        var url = this._getURL( state );
                        if ( url == null ) {
                            this.clearHTMLElement();
                            return; 
                        }
                        var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url );
                        var me = this;
                        var xsltParams = this.getXSLTParams( state );
                        serviceReq.read( 
                            function ( type, xmlData, xhr, args ) { 
                                me.applyXslt( type, xmlData, xhr, args, xsltParams );
                                me.applySelectionPath( state.selectionModel );
                            }
                        );
                    } 
                    else {
                        //The selected node still could have changed.
                        this.applySelectionPath( state.selectionModel );
                    }
                        
                    ibm.portal.debug.exit( "NavigationElement.render" );                    
                },
                setLoading: function () {
                    this.getHTMLElement().innerHTML = "<img src='" + this._resourceBaseURL + "/progress-anim.gif' />&nbsp;Loading...";
                    var str = "<img src='" + this._resourceBaseURL + "progress-anim.gif' />&nbsp;Loading...";
                },
                applyXslt: function ( /*String*/type, /*dojo.dom.Document*/xmlData, /*XMLHttpRequest*/xhr, /*Object?*/args, /*Object*/xsltParams ) {
                    // summary: Applies the associated XSL transformation to the retrieved
                    //      XML data.
                    // description: Clears the html element and updates it with the result of the XSLT.
                    // xmlData: the retrieved XML Data
                    // args: any arguments passed through from the original caller
                    ibm.portal.debug.entry( "NavigationElement.applyXslt", [xmlData, xhr, args, xsltParams ] );
                    ibm.portal.debug.text( "Loading xslt: " + this._xslt );
                    var xslt = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( this._xslt );
                    if (xslt == null) {
                        ibm.portal.debug.text("XSL transformer is null");
                    }
                    this.clearHTMLElement();
                    xslt.transformToRegion( xmlData, xsltParams, this.getHTMLElement(), document );
                    com.ibm.portal.utilities.html.removeNodesOnCondition(this.getHTMLElement(), function(node) {
						if(node && node.nodeType == 3 && node.nodeValue == "\n") {
							return true;
						}
						else return false;
					});
                        
                    ibm.portal.debug.exit( "NavigationElement.applyXslt" );
                },
                isEnabled: function () {
                    // summary: Determines if the element is enabled to be rendered.
                    // description: Indicates if the element is currently active and available to be rendered.
                    //      By default, this just verifies the HTML element is defined on the page, but could
                    //      be extended by navigational elements to control whet

                    return this.getHTMLElement() != null;
                },
                clearHTMLElement: function () {
                    // summary: Clears the markup associated with the navigation element.
                    if ( this.getHTMLElement() ) {
                    	this.getHTMLElement().innerHTML = "";
                    }	
                },
                getHTMLElement: function () {
                    // summary: Returns the markup elements associated with the navigation element.
                    // returns: the HTML element 
                    return dojo.byId( this._htmlElementID ); //HTMLElement
                },
                _getURL: function ( /*NavigationalState*/ state ) {
                    // summary: Returns the url for this service request.
                    // description: Constructs the url to the appropriate content for this element. If this URL is null,
                    //      the HTML element is cleared and it is assumed that this element does not apply to the current
                    //      navigational state.
                    // returns: a URL or NULL if this element should be cleared and doesn't apply to the current state
                    dojo.unimplemented( "com.ibm.portal.aggregation.theme.NavigationElement._getURL" ); 
                },
                applySelectionPath: function ( /*NavigationSelectionModel*/selModel ) {
                    // summary: Sets the selected element for this NavigationElement.
                    // selectedURI: the unique identifier of the selected element.
                    dojo.unimplemented( "com.ibm.portal.aggregation.theme.NavigationElement.applySelectionPath" );  
                },
                hasChanged: function ( /*NavigationalState*/ newState ) {
                    // summary: Determines if the element has changed.
                    // description: Indicates if the element has changed since the last time
                    //      it was rendered.
                    // newState: the state that is being rendered
                    dojo.unimplemented( "com.ibm.portal.aggregation.theme.NavigationElement.hasChanged" );
                },
                getXSLTParams: function ( /*NavigationalState*/ state ) {
                    // summary: Returns any XSLT params necessary for performing this transformation.
                    // state: the current navigational state
                    // returns: A two-dimensional array keeping the XSL parameters, e.g. [["a", "b"], ["x", "y"]]
                }
              } 
);

dojo.declare( "com.ibm.portal.aggregation.theme.ExpandableNavigationElement",
              com.ibm.portal.aggregation.theme.NavigationElement,
              {
                // summary: A navigation element that is expandable.
                
                expand: function ( /*NavigationalState*/state, /*Function?*/callbackFn ) {
                    // summary: Expand the element.
                    // state: the current navigational state
                    // callbackFn: the function to call when the expansion is complete
                    dojo.unimplemented( "com.ibm.portal.aggregation.ExpandableNavigationElement.expand" );
                },
                collapse: function ( /*NavigationalState*/state, /*Function?*/callbackFn ) {
                    // summary: Collapse the element.
                    // state: the current navigational state
                    // callbackFn: the function to call when the collapse is complete
                    dojo.unimplemented( "com.ibm.portal.aggregation.ExpandableNavigationElement.collapse" );
                }

              }  
);

dojo.declare( "com.ibm.portal.aggregation.theme.MainMenu",
              com.ibm.portal.aggregation.theme.NavigationElement,
                // summary: The main menu of the theme.
                // description: The main menu of the theme is constant and always contains the top two
                //      levels of navigation.
                // elemId: the target elem id where the context menu will display
                // rootNode: the rootNode's unique name or object id -- must point to a valid node in the Content Model. If not
                //      provided, defaults to "wps.content.root".
              {     
                constructor: function( /*String*/ elemId, /*String?*/rootNode ) {
                    ibm.portal.debug.entry( "MainMenu.constructor", [ elemId ] );
                
                    this._htmlElementID = elemId;
                    this._menuImgElementId = "wp_main_menu_img";
                    
                    //Set default fallback for the root node.
                    if ( !rootNode ) {
                        rootNode = "wps.content.root";
                    }
                    this._rootNode = rootNode;
                    
                    //create the unique id to reference context menu
                    this._menuId = elemId+"_menuId";
                },
                connected: false,
                render: function(){
                    ibm.portal.debug.entry( "MainMenu.render" );

                    //The menu only has to be connected once.
                    if ( this.connected ) {
                        return;
                    }
                    
                    dojo.connect( this.getHTMLElement(), "onclick", dojo.hitch(this, "displayMainMenu") );
                    this.connected = true;
                    
                    ibm.portal.debug.exit( "MainMenu.render" );
                },
                displayMainMenu: function (){
                    ibm.portal.debug.entry( "MainMenu.displayMainMenu" );
                    
                    var menu = getContextMenu(this._menuId);
                    var menuImgElement = dojo.byId( this._menuImgElementId );
                    
                    //check to see if menu is already created
                    if(menu == null){
                        if(!menuImgElement.oldSrc || menuImgElement.oldSrc == "") {
                            menuImgElement.oldSrc = menuImgElement.getAttribute( "src" );
                        }
                        menuImgElement.setAttribute( "src", this._resourceBaseURL + "/loading.gif" );
                        var url = new com.ibm.portal.services.ContentHandlerURL( "cm:oid:" + this._rootNode, 2, "download", "&mdname=com.ibm.portal.Hidden&mdname=com.ibm.portal.PageIcon&mdname=RenderMode" );
                        var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url );
                        serviceReq.read( dojo.hitch( this, this.loadMenuContents ) );
                    }else{
                        return showContextMenu(this._menuId, this.getHTMLElement());
                    }
                    
                    ibm.portal.debug.exit( "MainMenu.displayMainMenu" );
                },
                
                loadMenuContents: function ( type, xmlData, args ) {
                    //summary: callback function to load the main menu navigation elements and actions, and then display the menu to the user.
                    ibm.portal.debug.entry( "MainMenu.loadMenuContents", [ type, xmlData ] );
                                        
                    //load navigation elements from the xmlData. The XSLT generates javascript to create the menu items, so we eval the results
                    // to execute the javascript
                    var createMenuItemsJs = this.applyXslt( type, xmlData, args );
                    //update the dom with the results
                    eval (createMenuItemsJs);

                    //add global actions to the menu
                    var engineProperties = dojo.i18n.getLocalization("com.ibm.portal", "engine", csaUserLocale);
                    var menu = getContextMenu(this._menuId);
                    // actions differ when logged in
                    if(ibmPortalConfig.isUserLoggedIn){
                        menu.add(new UilMenuItem( engineProperties["link.selfcare"], true, '', "asynchDoFormSubmit('"+this.getSelfcareActionURL()+"');" , null, ibm.portal.ThemeImages.getImagePath('icons/blank.gif'), null, "main-menu-item", "main-menu-item-selected" ));
                        menu.add( new UilMenuItem( engineProperties["link.logout"], true, '', "asynchDoFormSubmit('"+ibmPortalConfig.logoutURL+"')", null, ibm.portal.ThemeImages.getImagePath('icons/blank.gif'), null, "main-menu-item", "main-menu-item-selected" ) );                       
                    }
                    else {
                        menu.add(new UilMenuItem( engineProperties["link.enrollment"], true, '', "asynchDoFormSubmit('"+this.getSelfcareActionURL()+"');" , null, ibm.portal.ThemeImages.getImagePath('icons/blank.gif'), null, "main-menu-item", "main-menu-item-selected" ));
                        menu.add( new UilMenuItem( engineProperties["link.login"], true, '', 'javascript:goToLogin();', null, ibm.portal.ThemeImages.getImagePath('icons/blank.gif'), null, "main-menu-item", "main-menu-item-selected" ) );
                    }
                    
                    var menuImgElement = dojo.byId( this._menuImgElementId );
                    if(menuImgElement.oldSrc && menuImgElement.oldSrc != "") {
                        menuImgElement.setAttribute( "src", menuImgElement.oldSrc);
                    }
                    
                    ibm.portal.debug.exit( "MainMenu.loadMenuContents" );
                    //return the menu to display
                    return showContextMenu(this._menuId, this.getHTMLElement());
                },
                
                applyXslt: function ( type, xmlData, args ) {
                    ibm.portal.debug.entry( "MainMenu.applyXslt", [ type, xmlData ] );
                                        
                    //load the xslt
                    var xslDoc = com.ibm.portal.xslt.loadXsl(ibmPortalConfig["mainMenuXslt"]);
            
                    //apply the xslt
                    if (xslDoc.documentElement === null) {ibm.portal.debug.text("xslDoc is null");}
                    
                    var xsltParams = { menuId: this._menuId };
                    var results = com.ibm.portal.xslt.transform(xmlData, xslDoc, null, xsltParams, true);
                    ibm.portal.debug.exit( "MainMenu.applyXslt", results );
                    return results;
                },
                getSelfcareActionURL: function(){
                    // summary: create and return a url to the selfcare portlet.
                    ibm.portal.debug.entry( "MainMenu.getSelfcareActionURL" );
                    var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                    var currentSelectedNodeOid = state.selectionModel.getSelectedNode().URI;
                    var url = state.newPageURL( "wps.Selfcare", ibmPageActionWindowIDs["wps.Selfcare"] /*portletWindowId*/, "normal" /*portletWindowState*/);
                    var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                    urlObj.addParameter( "ao", "thm" );
                    urlObj.addParameter( "OCN", currentSelectedNodeOid);
                    ibm.portal.debug.exit( "MainMenu.getSelfcareActionURL" );
                    return urlObj.toString();                    
                }   
              }
);

dojo.declare( "com.ibm.portal.aggregation.theme.CrumbTrail",
              com.ibm.portal.aggregation.theme.NavigationElement,
              {
                // summary: The breadcrumb trail reflecting the current selection path.
                constructor: function ( /*int?*/startLevel, /*int?*/stopLevel ) {
                    // summary: Creates a breadcrumb trail which displays the selection path from the
                    //      start navigation level to the stop navigation level. If no stop level is 
                    //      specified, displays all the way to the selected page.
                    // startLevel: OPTIONAL -- defines the navigation level to start the crumbtrail from
                    // stopLevel: OPTIONAL -- defines the navigation level to stop the crumbtrail at. Omitting this parameter
                    //      or passing in -1 indicates that the crumbtrail should go all the way to the current selection.
                    if ( typeof( startLevel ) == "undefined" ) {
                        startLevel = 1;
                    }
                    this._startLevel = startLevel;
                    if ( typeof( stopLevel ) == "undefined" ) {
                        //stop level of -1 indicates no stop level provided.
                        stopLevel = -1;
                    }
                    this._stopLevel = stopLevel;
                },          
                _htmlElementID: "breadcrumbtrail",
                _renderedPath: null,
                render: function( /*NavigationalState*/ state ) {
                    ibm.portal.debug.entry( "CrumbTrail.render", [ state ] );
                    
                    var string = "";
                    var selectionPath = state.selectionModel.selectionPath;
                    
                    if ( selectionPath == this._renderedPath ) {
                        return;
                    }
                    
                    var loopStartIndex = this._startLevel;
                    var loopStopIndex = this._stopLevel;
                    
                    if ( loopStopIndex < 0 || loopStopIndex < loopStartIndex || loopStopIndex > selectionPath.length ) {
                        loopStopIndex = selectionPath.length;
                    } 
                    
                    for ( var i = loopStartIndex; i < loopStopIndex; i++ ) {
                        if ( i > loopStartIndex ) {
                            string += "<span class='wptheme-breadcrumbSeparator'>/</span>";
                        }
                        
                        ibm.portal.debug.text( "Selection Path [" + i + "] = " + selectionPath[i].URI );
                        var node = selectionPath[i];
                        ibm.portal.debug.text( "examining node: " + node.title + "; isLabel? " + node.isLabel );
                        if ( !node.isLabel ) {
                            var href = "javascript:com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.setSelection('" + node.URI + "'); ";
                            string += '<a href="' + href + '" >';
                        }
                        string += node.title;
                        if ( !node.isLabel ) { 
                            string += '</a>';
                        }
                    }

                    var htmlElem = this.getHTMLElement();
                    // might be null, e.g. in case of login link
                    if (htmlElem != null) {
                        htmlElem.innerHTML = string;
                        this._renderedPath = selectionPath;
                    }
                }   
              }
);

dojo.declare( "com.ibm.portal.aggregation.theme.TopLevelLinks",
              com.ibm.portal.aggregation.theme.NavigationElement,
              {
                constructor: function ( /*String*/elementID, /*int*/numToDisplay ) {
                    // summary: Displays the nodes directly underneath content root.
                    // elementID: the html element's id which contains the top level links. The markup will be inserted
                    //      into this element.
                    // numToDisplay: The number of links to display before dumping the rest into the "More..." menu. 
                    this._htmlElementID = elementID;
                    this._numToDisplay = numToDisplay;
                    this.rendered = false;
                },
                _xslt: ibmPortalConfig[ "topLevelLinksXslt" ], 
                _getURL: function ( /*NavigationalState*/ state ) {
                    // summary: Returns the url for this service request.
                    ibm.portal.debug.entry( "TopLevelLinks._getURL", [state] );
                    var url = new com.ibm.portal.services.ContentHandlerURL( "cm:oid:wps.content.root", 2, "download", "&mdname=com.ibm.portal.Hidden&mdname=com.ibm.portal.PageIcon&mdname=RenderMode" );
                    ibm.portal.debug.exit( "TopLevelLinks._getURL", url );
                    return url;
                },
                applySelectionPath: function ( selModel ) {
                    if ( !this.rendered ) {
                        this._buildMoreMenuIfNecessary();
                    }
                    this.rendered = true;
                    
                    var lis = this.getHTMLElement().childNodes[0].childNodes;
                    
                    for ( var i = 0; i < lis.length; i++) {
                        if ( selModel.isNodeInSelectionPath( lis[i].getAttribute( "oid" ) ) ) {
                            var imgs = lis[i].getElementsByTagName( "IMG" );
                            if ( imgs.length > 0 ) {
                                imgs[0].style.display='inline';
                            }   
                            //lis[i].className += " selectedNode";
                            dojo.addClass( lis[i], "wptheme-selectedTopLevelLink" );
                        }
                        else {
                            var imgs = lis[i].getElementsByTagName( "IMG" );
                            if ( imgs.length > 0 ) {
                                imgs[0].style.display='none';
                            }
                            dojo.removeClass( lis[i], "wptheme-selectedTopLevelLink" );
                        }
                    }
                    
                },
                hasChanged: function () {
                    return !this.rendered;
                },
                getXSLTParams: function ( /*NavigationalState*/ state ) {
                    // summary: Returns any XSLT params necessary for performing this transformation.
                    // state: the current navigational state
                    // returns: A two-dimensional array keeping the XSLT params.
                    var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
                    var isRTL = bidiSupport.isRTL;
                    var isFirefoxArabic = isRTL && !dojo.isIE && csaUserLocale == "ar";
                    ibm.portal.debug.text( "numToDisplay: " + this._numToDisplay + "; moreText: " + v3themeProperties["link.more"] + "; themeRoot: " + ibmPortalConfig["themeRootURI"] + "; isRTL: " + isRTL + "; isFirefoxArabic: " + isFirefoxArabic);
                    return {"numToDisplay": this._numToDisplay, "moreText": v3themeProperties["link.more"], "themeRoot": ibmPortalConfig["themeRootURI"], "isRTL": isRTL, "isFirefoxArabic": isFirefoxArabic};
                },
                _buildMoreMenuIfNecessary: function () {
                    var anchor = document.getElementById( "wptheme_more_menu" );
                    if ( anchor ) {
                        var menuID = 'wptheme_more_menu';
                        var isLTR = !bidiSupport.isRTL;
                        var menuBorderStyle="wptheme-more-menu-border";
                        var menuTableStyle="wptheme-more-menu";
                        var emptyMenuText="No actions currently available.";
                        var menuItemStyle = "wptheme-more-menu-item";
                        var menuItemSelectedStyle = "wptheme-more-menu-item-selected";
                        var icon = null;
                        var menu = createContextMenu( menuID, isLTR, null, menuBorderStyle, menuTableStyle, emptyMenuText, null, true );
                        
                        var lis = this.getHTMLElement().childNodes[0].childNodes; 
                        for ( var i = this._numToDisplay + 1; i < lis.length; i++ ) {
                            console.log( "processing: " + lis[i]);
                            var title = lis[i].getElementsByTagName("A")[0].innerHTML;
                            var href = lis[i].getElementsByTagName("A")[0].href;
                            var imgs = lis[i].getElementsByTagName( "IMG" );
                            var icon = null;
                            if ( imgs.length > 0 ) {
                                icon = imgs[0].src;
                            }    
                            
                            menu.add( new UilMenuItem( title, true, '', href, null, icon, null, menuItemStyle, menuItemSelectedStyle ) );
                        }
                        
                        dojo.connect( anchor, "onclick", anchor, function () { showContextMenu( menuID, this ); return false; } );
                    }
                }
          }
);

dojo.declare( "com.ibm.portal.aggregation.theme.ApplicationName",
              com.ibm.portal.aggregation.theme.NavigationElement,
              {
                // summary: The breadcrumb trail reflecting the current selection path.
                constructor: function ( elementID, navLevelOfNodeToDisplay ) {
                    // summary: Displays the current site area's name to the left of the top navigation tabs.
                    // elementID: the html element's ID. the markup will be inserted as a child of this node. 
                    // navLevelOfNodeToDisplay: the navigation level of the node to display the name of
                    this._htmlElementID = elementID;
                    this._navLevel = navLevelOfNodeToDisplay;
                },
                _renderedNode: null,            
                render: function( /*NavigationalState*/ state ) {
                    ibm.portal.debug.entry( "ApplicationName.render", [ state ] );
                    
                    var string = "";
                    var selectionPath = state.selectionModel.selectionPath;
                    
                    var node = selectionPath[ this._navLevel ];
                    
                    if ( node && node.URI == this._renderedNode ) {
                        return;
                    }
                    
                    var htmlElem = this.getHTMLElement();
                    if (htmlElem != null) {
                        htmlElem.innerHTML = node.title;
                        this._renderedPath = node.URI;
                    }
                }   
              }
);

dojo.declare( "com.ibm.portal.aggregation.theme.TopNavigation",
              com.ibm.portal.aggregation.theme.NavigationElement,
              {
                // summary: The top navigation of the theme.
                constructor: function ( /*String*/htmlElementID, /*int?*/startLevel, /*boolean?*/notFirstRow ) {
                    // summary: Creates the controller object for displaying the tabs in the top navigation.
                    // htmlElementID: the id of the html element which will contain the markup.
                    // startLevel: OPTIONAL -- indicates the navigation level in the selection path to pull the 
                    //      page titles from... currently only gathers 1 level of navigation.
                    if ( ibmPortalCurrentThemePolicy && ibmPortalCurrentThemePolicy[ "renderContextMenus" ] ) { 
                    	this._pageMenu = new com.ibm.portal.aggregation.theme.PageMenu();
                    }	
                    this._htmlElementID = htmlElementID;
                    if ( typeof( startLevel ) == "undefined" || startLevel < 1 ) {
                        startLevel = 2;
                    }
                    this._startLevel = startLevel;
                    if ( typeof( notFirstRow ) == "undefined" ) {
                        notFirstRow = false;
                    }
                    this._firstRow = !notFirstRow;
                },
                
                _xslt: ibmPortalConfig["topNavXslt"],
                _previousState: null,
                _getURL: function ( /*NavigationalState*/ state ) {
                    // summary: Returns the url for this service request.
                    ibm.portal.debug.entry( "TopNavigation._getURL", [state] );
                    
                    var node = state.selectionModel.selectionPath[this._startLevel - 1] ;
                    
                    ibm.portal.debug.exit( "TopNavigation._getURL", node.URI );
                    return new com.ibm.portal.services.ContentHandlerURL( node.URI, null, null, "&mdname=RenderMode");  
                },
                applySelectionPath: function ( /*NavigationSelectionModel*/selModel ) {
                    // summary: Sets the selected element for this NavigationElement.
                    // selectedURI: the unique identifier of the selected element.
                    ibm.portal.debug.entry( "TopNavigation.applySelectionPath", [ selModel ] );
                    
                    if ( selModel.selectionPath[this._startLevel] == null ) { return false; } 
                    
                    var selNode = selModel.selectionPath[this._startLevel].URI;
                    var topnavdiv = this.getHTMLElement();
                    var ul = topnavdiv.childNodes[0];
                    
                    dojo.forEach( ul.childNodes, function ( node ) { node.className = "wpsUnSelectedPage"; } );
                    dojo.byId( selNode ).className = "wpsSelectedPage";
                    //get current page displayed
                    var selectedPage = selModel.getSelectedNode();  
                    
                    // check to display the page menu
                    if( (this._pageMenu && selectedPage.URI == selNode) /*&& (dojo.byId("contextMenu_Page") != null)*/ ){
                        var elem = dojo.byId(selNode);
                        this._pageMenu.renderPageMenu(elem, selNode );
                    }
                    
                    ibm.portal.debug.exit( "TopNavigation.applySelectionPath" );    
                },
                hasChanged: function ( /*NavigationalState*/ newState ) {
                    // summary: Determines if the element has changed.
                    // description: Indicates if the element has changed since the last time
                    //      it was rendered.
                    // newState: the state that is being rendered
                    ibm.portal.debug.entry( "TopNavigation.hasChanged", [ newState ] );
                    
                    var retVal = true;
                    var parentLevel = this._startLevel - 1;
                    if ( newState.selectionModel.selectionPath[parentLevel] == null ) {
                        this._previousState = null;
                    }
                    else {
                        ibm.portal.debug.text( "Previous state: " + this._previousState + "; newState = " + newState.selectionModel.selectionPath[parentLevel].URI );
                        
                        if ( this._previousState != null ) {
                            retVal = !( this._previousState == newState.selectionModel.selectionPath[parentLevel].URI );    
                        }
                        
                        if ( retVal ) {
                            this._previousState = newState.selectionModel.selectionPath[parentLevel].URI;
                        }
                    }
                                        
                    ibm.portal.debug.exit ( "TopNavigation.hasChanged", [ retVal ]);
                    return retVal;
                },
                getXSLTParams: function ( /*NavigationalState*/ state, /*Boolean*/addingChildren, /*int*/ navLevel ) {
                    // summary: Returns any XSLT params necessary for performing this transformation.
                    // state: the current navigational state
                    // returns: A two-dimensional array keeping the XSL parameters
                    var params = null;
                    if ( this._firstRow ) {
                        params = {"topNavFirstRow": "true"};
                    }
                    return params;
                }
              } 
);

dojo.declare( "com.ibm.portal.aggregation.theme.SideNavigation",
              com.ibm.portal.aggregation.theme.ExpandableNavigationElement,
              {
                // summary: The side navigation of the theme.
                // description: The side navigation receives expand and collapse notifications
                //      by subscribing to the events published by the navigation controller.
                
                constructor: function ( /*int?*/startLevel ) {
                    // summary: Creates the controller object for the side navigation. 
                    // startLevel: OPTIONAL -- indicates the navigation level in the selection path to pull the 
                    //      page titles from
                    
                    this._expansionHandler = dojo.subscribe( "/portal/NavigationalStateController/expansion", 
                                                this, 
                                                function ( /*Object*/args ) {
                                                    var page = args["pageId"];
                                                    this.expand( page );
                                                }  
                    );
                    this._collapseHandler = dojo.subscribe( "/portal/NavigationalStateController/collapse", 
                                                this, 
                                                function ( /*Object*/args ) {
                                                    ibm.portal.debug.entry( "collapse handler", [args]);
                    								var page = args["pageId"];
                                                    this.collapse( page );
                                                    ibm.portal.debug.exit( "collapse handler" );
                                                }  
                    );
                    this._lastSelected = null;
                    if ( ibmPortalCurrentThemePolicy && ibmPortalCurrentThemePolicy[ "renderContextMenus" ] ) {
                    	this._pageMenu = new com.ibm.portal.aggregation.theme.PageMenu();
                    }	
                    
                    if ( typeof( startLevel ) == "undefined" || startLevel < 1 ) {
                        startLevel = 3;
                    }
                    this._startLevel = startLevel;
                },
                
                _xslt: ibmPortalConfig["sideNavXslt"],
                _htmlElementID: "sideNav",
                _previousState: null,
                _getURL: function ( /*NavigationalState*/ state ) {
                    // summary: Returns the url for this service request.
                    ibm.portal.debug.entry( "SideNavigation._getURL", [ state ]);
                    
                    if ( state.selectionModel.selectionPath.length < this._startLevel ) {
                        ibm.portal.debug.exit( "SideNavigation._getURL", [ null ] );
                        return null;
                    }
                    var topNavLevel = this._startLevel - 1;
                    var topNavNode = state.selectionModel.selectionPath[ topNavLevel ];
                    ibm.portal.debug.text( "top nav node has children? " + topNavNode.hasChildren );
                    if ( !topNavNode.hasChildren ) {
                        ibm.portal.debug.exit( "SideNavigation._getURL", [ null ] );
                        return null;
                    }
                    var node = state.selectionModel.selectionPath[topNavLevel];
                    ibm.portal.debug.exit( "SideNavigation._getURL", node.URI );
                    return new com.ibm.portal.services.ContentHandlerURL( node.URI, null, null, "&mdname=RenderMode" );     
                },
                applySelectionPath: function ( /*NavigationSelectionModel*/selModel, /*int?*/currentLevel ) {
                    // summary: Applies the selection path to this navigation element.
                    // description: Extracts any relevant information from the selection path and uses 
                    //      to set styles, etc in this navigation element.
                    // selModel: the NavigationSelectionModel
                    // currentLevel: when this function is called asynchronously after loading an expanded portion of the side 
                    //      navigation, this parameter indicates the place in the selection path where processing left off.
                    ibm.portal.debug.entry( "SideNavigation.applySelectionPath", [ selModel, currentLevel ] );
                    
                    var startIndex = this._startLevel, selPath = selModel.selectionPath, selIndex;
                    
                    if ( currentLevel ) {
                        selIndex = currentLevel;
                    }                   
                    else {
                        selIndex = startIndex;
                    }
                    
                    //In the case where we have gone from a side nav page to a top nav page, we need to clear out
                    //the old selected side nav item
                    if ( startIndex >= selPath.length && this._lastSelected ) {
                        var lastSelectedElement = this._getSideNavPageElement( this._lastSelected );
                        if ( lastSelectedElement ) {
                        	dojo.removeClass( lastSelectedElement, "wptheme-sideNavSelected" );
                        }	
                        this._lastSelected = null;
                    }
                    
                    for ( ; selIndex < selPath.length; selIndex++ ) {
                        ibm.portal.debug.text( "Examining element " + selIndex + " in the selection path: " + selPath[selIndex].URI );
                        var pageElement = this._getSideNavPageElement( selPath[selIndex].URI );
                        
                        if ( !pageElement ) {
                            // we may only expand nodes being part of the displayed subtree
                            if ( selPath.length-1 > this._startLevel ) {
                                var me = this;
                                this.expand( selPath[selIndex-1].URI, function () { me.applySelectionPath( selModel, selIndex ); } );
                            }
                            break;
                        } else {
                            if ( selIndex == ( selPath.length - 1) ) {
                                ibm.portal.debug.text( "Adding selected class to " + pageElement );
                                dojo.addClass( pageElement, "wptheme-sideNavSelected" );
                                if ( this._lastSelected && this._lastSelected != selPath[selIndex].URI ) {
                                    //Remove the selected class from the previous node.
                                    var lastSelectedElement = this._getSideNavPageElement( this._lastSelected );
                                    if ( lastSelectedElement ) {
                                    	ibm.portal.debug.text( "Removing selected from: " + dojo.byId( this._lastSelected ));
                                    	dojo.removeClass( lastSelectedElement, "wptheme-sideNavSelected" );
                                    }
                                    else {
                                    	this._checkForToggleIcon( selPath[selIndex].URI );
                                    }
                                }   
                                //Store this element as the last selected now.
                                ibm.portal.debug.text( "Setting last selected to: " + selPath[selIndex].URI );
                                this._lastSelected = selPath[selIndex].URI;
                                //get current page displayed                            
                                var selectedPage = selModel.getSelectedNode();  
                                                    
                                // check to display the page menu
                                if( this._pageMenu && (selectedPage.URI == this._lastSelected) ){
                                    var elem = dojo.byId(this._lastSelected).childNodes[0];
                                    this._pageMenu.renderPageMenu(elem, this._lastSelected);
                                }
                            }
                        }
                    } 
                    ibm.portal.debug.exit( "SideNavigation.applySelectionPath" );
                },
                _getSideNavPageElement: function ( uri ) {
                    ibm.portal.debug.entry( "SideNavigation._getSideNavPageElement" , [ uri ]);
                    var retVal = dojo.byId( uri );
                    
                    if ( retVal ) {
                        retVal = retVal.childNodes[0];
                    }
                    
                    ibm.portal.debug.exit( "SideNavigation._getSideNavPageElement", [ retVal ] );
                    return retVal;
                },
                hasChanged: function ( /*NavigationalState*/ newState ) {
                    // summary: Determines if the element has changed.
                    // description: Indicates if the element has changed since the last time
                    //      it was rendered.
                    // newState: the state that is being rendered
                    ibm.portal.debug.entry( "SideNavigation.hasChanged", [ newState ] );
                    
                    var retVal = true;
                    var topNavLevel = this._startLevel - 1;
                    if ( newState.selectionModel.selectionPath[topNavLevel] == null ) {
                        this._previousState = null;
                        this._lastSelected = null;
                    }
                    else {
                        if ( this._previousState != null ) {
                            retVal = !( this._previousState == newState.selectionModel.selectionPath[topNavLevel].URI );    
                        }
                        
                        if ( retVal ) {
                            this._changeOldTopNavLink( this._lastSelected ); // change previous top nav link to reference last selection, order matters
                            this._restoreNewTopNavLink( newState.selectionModel );  // restores the new top nav link if it was changed
                            this._previousState = newState.selectionModel.selectionPath[topNavLevel].URI;
                            this._lastSelected = null;
                        }
                    }   
                    
                    ibm.portal.debug.exit ( "SideNavigation.hasChanged", [ retVal ]);
                    return retVal;
                },
                expand: function ( /*String*/uri, /*Function?*/callbackFn ) {
                    // summary: Expand the element.
                    // uri: the uri of the element to expand
                    // callbackFn: the function to call when the expansion is complete
                    ibm.portal.debug.entry( "SideNavigation.expand", [ uri, callbackFn ] );
                    
                    var node = dojo.byId( uri );
                    if ( node.getElementsByTagName( "UL" ).length > 0 ) {
                        //PK84164: In IE, using dojo.fx.wipeIn/Out leaves the collapsed node in a bad state. Instead,
                    	//simply hide and show using the style.display property.
                    	//com.ibm.portal.utilities.html.show( node.getElementsByTagName("UL")[0] );
                    	node.getElementsByTagName("UL")[0].style.display = "block";
                        this._toggleExpandCollapseIcon( uri );
                        
                        ibm.portal.debug.exit( "SideNavigation.expand" );
                        return;
                    }
                    
                    var url = new com.ibm.portal.services.ContentHandlerURL( uri, 2, "download", "&mdname=RenderMode" );
                    var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url );
                    var me = this;
                    serviceReq.read(
                        function ( type, xmlData, xhr ) {
                            me._addChildren( xmlData, uri );
                            me._toggleExpandCollapseIcon( uri );
                            if ( callbackFn ) {
                                callbackFn();
                            }   
                        }  
                    );
                    
                    ibm.portal.debug.exit( "SideNavigation.expand" );
                },
                _checkForToggleIcon: function ( /*String*/uri ) {
                	ibm.portal.debug.entry( "SideNavigation._checkForToggleIcon", [ uri ] );
                	var liNode = dojo.byId(uri);
                	var aNode = liNode.childNodes[0].childNodes[0];
                                        
                    if ( aNode ) {
                    	var hasChildren = ( liNode.childNodes.length >= 2 && liNode.childNodes[1].tagName && liNode.childNodes[1].tagName.toUpperCase() == "UL" );
                    	ibm.portal.debug.text( "List Item has children? " + hasChildren );
                    	if ( !hasChildren ) {
                    		//Delete the old anchor element.
                    		var node = aNode.parentNode.removeChild( aNode );
                    		delete node;
                    		//Create the spacer gif that exists when a node has no toggle arrow
                    		var dotGifPath = ibm.portal.ThemeImages.dotGifPath;
                    		var imgNode = document.createElement( "IMG" );
                    		imgNode.setAttribute("src", dotGifPath);
                    		imgNode.setAttribute("alt", "");
                    		imgNode.setAttribute("title", "" );
                    		dojo.addClass( imgNode, "wptheme-sideNavIcon" );
                    		dojo.place( imgNode, liNode.childNodes[0], "first" );
                    	}
                    }
                    ibm.portal.debug.entry( "SideNavigation._checkForToggleIcon" );	
                },
                _toggleExpandCollapseIcon: function ( /*String*/uri ) {
                    ibm.portal.debug.entry( "SideNavigation._toggleExpandCollapseIcon", [ uri ] );
                    var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "engine", csaUserLocale);
                    
                    var liNode = dojo.byId(uri);
                    var aNode = liNode.childNodes[0].childNodes[0];
                    var imgNode = aNode.childNodes[0];
                    // change href to collapse
                    //aNode.setAttribute("href","javascript:ibm.portal.Navigation.SideNav.collapse('"+uri+"', 2)");
                    //toggle image
                    if ( imgNode ) {
	                    if ( imgNode.getAttribute("src").indexOf( ibm.portal.ThemeImages.minusGifPath ) >= 0 ) {
	                        imgNode.setAttribute("src", bidiSupport.isRTL ? ibm.portal.ThemeImages.plusGifPathRTL : ibm.portal.ThemeImages.plusGifPath);
	                        imgNode.setAttribute("title", v3themeProperties["link.expand"] );
	                        imgNode.setAttribute("alt", v3themeProperties["link.expand"]);
	                        aNode.setAttribute("href", "javascript:com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.expand('" + uri +"' );" );
	                    }
	                    else {
	                        imgNode.setAttribute("src", ibm.portal.ThemeImages.minusGifPath);
	                        imgNode.setAttribute("title", v3themeProperties["link.collapse"] );
	                        imgNode.setAttribute("alt", v3themeProperties["link.collapse"] );
	                        aNode.setAttribute("href", "javascript:com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.collapse('" + uri +"' );" );
	                    }
                    }
                    else {
                    	ibm.portal.debug.text( "Creating the expand/collapse icon!" );
                    	var anchor = document.createElement( "A" );
                    	imgNode = document.createEleent( "IMG" );
                    	imgNode.setAttribute("src", ibm.portal.ThemeImages.minusGifPath);
	                    imgNode.setAttribute("title", v3themeProperties["link.collapse"] );
	                    imgNode.setAttribute("alt", v3themeProperties["link.collapse"] );
	                    anchor.setAttribute("href", "javascript:com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.collapse('" + uri +"' );" );
	                    dojo.addClass( anchor, "wptheme-sideNavIcon" );
	                    anchor.appendChild( imgNode );
	                    dojo.place( anchor, liNode.childNodes[0], "first" );
                    }  
                    
                    ibm.portal.debug.exit( "SideNavigation._toggleExpandCollapseIcon" );     
                },
                collapse: function ( /*String*/uri, /*Function?*/callbackFn ) {
                    // summary: Collapse the element.
                    // uri: the uri of the element to collapse
                    // callbackFn: the function to call when the collapse is complete
                    ibm.portal.debug.entry( "SideNavigation.collapse", [ uri ] );
                	
                	var liNode = dojo.byId(uri);
                    var selNode = dojo.byId(this._lastSelected);
                    var isDescendant = false;
                    
                    if ( selNode ) {
	                    isDescendant = com.ibm.portal.utilities.html.isDescendantOf( selNode, liNode );
                    }
                    
                    if(liNode.hasChildNodes() && !isDescendant){ 
                        ibm.portal.debug.text("hiding childNode: "+liNode.lastChild);
                        //PK84164: In IE, using dojo.fx.wipeIn/Out leaves the collapsed node in a bad state in this case. Instead,
                    	//simply hide and show using the elem.style.display property.
                    	//com.ibm.portal.utilities.html.hide( liNode.getElementsByTagName("UL")[0] );
                        liNode.getElementsByTagName("UL")[0].style.display = "none";
	                }
	                if(!isDescendant) {
	                    this._toggleExpandCollapseIcon( uri );
	                }
                    
                    ibm.portal.debug.exit( "SideNavigation.collapse" );
                },
                getXSLTParams: function ( /*NavigationalState*/ state, /*Boolean*/addingChildren, /*int*/ navLevel ) {
                    // summary: Returns any XSLT params necessary for performing this transformation.
                    // state: the current navigational state
                    // returns: A two-dimensional array keeping the XSL parameters
                    
                    var plusGifPath = ibm.portal.ThemeImages.plusGifPath;
                    if(bidiSupport.isRTL)
                        plusGifPath = ibm.portal.ThemeImages.plusGifPathRTL; 
                        
                    var minusGifPath = ibm.portal.ThemeImages.minusGifPath;
                    var dotGifPath = ibm.portal.ThemeImages.dotGifPath;
                    var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "engine", csaUserLocale);
                    
                    var level = 1;
                    var rootSideNav = null;
                    if ( addingChildren ) {
                        rootSideNav = "false";
                        level = navLevel;
                    }
                    ibm.portal.debug.text("plusGifPath: " + plusGifPath + "; plusGifAltText: "+v3themeProperties["link.expand"]+"; minusGifPath: " + minusGifPath + "; dotGifPath" + dotGifPath + "; navLevel: " + level + "; rootSideNav: " + rootSideNav );
                    var params = null;
                    if (rootSideNav != null) {
                        params = {"plusGifPath": plusGifPath, "plusGifAltText": v3themeProperties["link.expand"], "minusGifPath": minusGifPath, "dotGifPath": dotGifPath, "navLevel": level, "rootSideNav": rootSideNav};
                    } else {
                        params = {"plusGifPath": plusGifPath, "plusGifAltText": v3themeProperties["link.expand"], "minusGifPath": minusGifPath, "dotGifPath": dotGifPath, "navLevel": level};
                    }
                    return params;
                },
                _addChildren: function ( /*dojo.dom.Document*/xmlData, /*String*/uriToUpdate ) {
                    ibm.portal.debug.entry( "SideNavigation._addChildren", [ xmlData, uriToUpdate ] );

                    ibm.portal.debug.text( "Loading xslt: " + this._xslt );
                    ibm.portal.debug.text( "Applying to: " + dojox.data.dom.innerXML( xmlData ) );
                    var xslt = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( this._xslt );
                    if (xslt == null) {
                        ibm.portal.debug.text("XSL transformer is null");
                    }
                    var node = dojo.byId( uriToUpdate );
                    var navLevel = node.getAttribute("navLevel");
                    navLevel++;
                    var content = xslt.transformToDocument(xmlData, this.getXSLTParams( null, true, navLevel ), true);
                    node.innerHTML += content;
                    ibm.portal.debug.exit( "SideNavigation._addChildren" );
                },
                _changeOldTopNavLink: function ( /*String*/uri ) {
                    var topSelElem = dojo.byId( this._topSelNode );
                    if(topSelElem && topSelElem.childNodes.length > 0 && uri && uri != "") {
                        var topAnchor = topSelElem.childNodes[0];
                        if(!topAnchor.topNodeHref || topAnchor.topNodeHref == "") {
                            topAnchor.topNodeHref = topAnchor.getAttribute("href"); // only save link to the top nav node
                        }
                        topAnchor.setAttribute("href", "javascript:com.ibm.portal." +
                            "navigation.controller.NAVIGATION_CONTROLLER.setSelection('" + uri + "', null, null, '');");
                    }
                },
                _restoreNewTopNavLink: function ( /*NavigationSelectionModel*/selModel ) {
                    this._topSelNode = selModel.selectionPath[2].URI;   // brittle... need agnostic way of getting selected top nav node
                    var topSelElem = dojo.byId( this._topSelNode );
                    if(topSelElem && topSelElem.childNodes.length > 0) {
                        var topAnchor = topSelElem.childNodes[0];
                        if(topAnchor.topNodeHref && topAnchor.topNodeHref != "") {
                            topAnchor.setAttribute("href", topAnchor.topNodeHref); // restore the old link
                        }
                    }
                }
              } 
);

dojo.declare( "com.ibm.portal.aggregation.theme.PageMenu",
              null,
              {
                constructor: function ( ) {
                
                    this._setPageActionWindowIds();
                    this._xslt = ibmPortalConfig["contextMenuXslt"];
                    this._pageMenuArray = new Array();
                    this._id = null;                
                    this._originalPageMenuImgSrc = null;                
                    this._engineProperties = dojo.i18n.getLocalization("com.ibm.portal", "engine", csaUserLocale);
                
                    this._deletePageHandler = dojo.subscribe("/portal/PageMenu/DeletePage/",
                                                this, 
                                                "_doDeletePage");
                                                
                    ibm.portal.debug.exit( "PageMenu.constructor" );
                },
                renderPageMenu: function ( /*HTMLElement*/elem, /*String*/ id, /*String*/ color ) {
                    
                    ibm.portal.debug.entry( "PageMenu.renderPageMenu", [ elem , id, color] );
                        
                    //Clear the menu.
                    this.clearMenu();
                    
                    this._id = id;
                    var menuButtonId = "menu_pageContextMenuButton";
                    var menuImageId = "menu_pageContextMenuButton_img";
                    
                    if(!color){
                        pageMenuIconHoverPath = ibm.portal.ThemeImages.pageMenuIconHoverPath["def"];
                    }else{
                        pageMenuIconHoverPath = ibm.portal.ThemeImages.pageMenuIconHoverPath[color];                    
                    }
                    //Create the button element. We use a button so that the context menu will appear
                    //directly below the image (limitation of the WCL context menu code).
                    var pageMenuButton = com.ibm.portal.utilities.html.createButton( elem,  
                                                                                  "javascript:void(0);",
                                                                                  menuButtonId,
                                                                                  null,
                                                                                  "wptheme-contextMenuIcon" );
                    
                    //Create the image to go on the button.
                    var pageMenuIcon = com.ibm.portal.utilities.html.createImage(pageMenuButton, 
                                                                               ibm.portal.ThemeImages.pageMenuIconPath, 
                                                                               menuImageId, 
                                                                               this._engineProperties["link.page.actions"], 
                                                                               "wptheme-contextMenuIcon" );
                    
                    //Connect all events.
                    var me = this;
                    dojo.connect( pageMenuButton, "onclick", function(){ me["displayPageMenu"](id); });
                    dojo.connect( pageMenuButton, "onmouseover", function () { document.getElementById(menuImageId).src=pageMenuIconHoverPath; });
                    dojo.connect( pageMenuButton, "onmouseout", function () { document.getElementById(menuImageId).src=ibm.portal.ThemeImages.pageMenuIconPath; });             
                    dojo.connect( pageMenuButton, "onfocus", function () { document.getElementById(menuImageId).src=pageMenuIconHoverPath; });
                    
                    ibm.portal.debug.exit( "PageMenu.renderPageMenu" );
                    
                }, 
                displayPageMenu: function (id){
                    
                    var menuId = "contextMenu_Page_" + id;
                    var menu = getContextMenu(menuId);
                    
                    if(menu == null){
                        var domSelNode = dojo.byId(id);
                        var menuUri = domSelNode.getAttribute("menuLink");
                        this.loadPageMenu(menuId, menuUri);
                    }else{
                        var targetImg = document.getElementById( "menu_pageContextMenuButton_img" );
                        return showContextMenu( menuId , targetImg ); 
                    }

                },
                loadPageMenu: function(menuId, menuUri){
                    
                    ibm.portal.debug.entry( "PageMenu.loadPageMenu" );
                    
                    this._originalPageMenuImgSrc = this._getPageMenuImgSrc();
                    this._setPageMenuImgSrc(ibm.portal.ThemeImages.contextLoadingPath);
    
                    var url = new com.ibm.portal.services.ContentHandlerURL( /*String*/menuUri, /*int*/null, /*String*/null, /*String?*/null );
                    var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url );
                    var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                    serviceReq.read( dojo.hitch( this, "applyContextMenuXslt" ), { state: state, me: this, menuId: menuId } );
                    
                },  
                applyContextMenuXslt: function ( /*String*/type, /*dojo.dom.Document*/xmlData, /*XMLHttpRequest*/xhr, /*Array*/callbackargs ) {
                    // summary: Apply the XSLT to get the page action items from menu feed
                    // type: the type of response
                    // xmlData: the xml of the Feed
                    // xhr: the XMLHttpRequest used to retrieve the feed
                    // callbackargs: any arguments passed through to this function
                    
                    ibm.portal.debug.entry( "PageMenu.applyContextMenuXslt", [xmlData,xhr,callbackargs] );

                    var trf = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( ibmPortalConfig["contextMenuXslt"] );
                    if (trf == null) {
                        ibm.portal.debug.text("XSL transformer is null");
                    }
                    
                    var results = trf.transformToDocument(xmlData, {}, true); 
                    eval( results );
                        
                    var me = callbackargs["me"];
                    me._pageMenuArray = menuItems;
                        
                    for(var i = 0; i< me._pageMenuArray.length; i++)                            
                        ibm.portal.debug.text("title: ["+me._pageMenuArray[i].title+"] label: ["+me._pageMenuArray[i].label+"] oncommand: []"+me._pageMenuArray[i].oncommand+"]");
                        
                    var menuId = callbackargs["menuId"];
                    this.createContextMenu(menuId, me);
                    
                },
                createContextMenu: function (menuId, me){
                    
                    ibm.portal.debug.entry( "PageMenu.createContextMenu" );

                    me.createPageMenuURLs(me);
                
                    menu = createContextMenu(menuId, !bidiSupport.isRTL, null, ""/*menuBorderStyle*/, "wptheme-pageContextMenuTable"/*menuTableStyle*/, this._engineProperties["info.emptymenu"]);
                    
                    var contents = me._pageMenuArray;
                    //Cycle through the menu item array.
                    for ( i=0; i < contents.length; i++ ) 
                    {
                       if ( contents[i] && contents[i].length != 0 )
                        {
                            if(contents[i].label != "DeletePage")
                                menu.add( new UilMenuItem( contents[i].title, true, '', "asynchDoFormSubmit('"+contents[i].url+"');", null, /*no icon*/ null, null, 'wptheme-pageContextMenuItem'/*menuItemStyle*/, 'wptheme-pageContextMenuItemHover'/*menuItemSelectedStyle*/) );
                            else
                                menu.add( new UilMenuItem( contents[i].title, true, '', 'dojo.publish("/portal/PageMenu/DeletePage/",[{pageId: "'+me._id+'"}] );', null, /*no icon*/ null, null, 'wptheme-pageContextMenuItem'/*menuItemStyle*/, 'wptheme-pageContextMenuItemHover'/*menuItemSelectedStyle*/) );
                        }
                        
                    } 
                    
                    var targetImg = dojo.byId( "menu_pageContextMenuButton_img");
                    this._setPageMenuImgSrc(this._originalPageMenuImgSrc);
                    
                    return showContextMenu( menuId , targetImg );
                    
                },
                createPageMenuURLs: function(me){   
                    ibm.portal.debug.entry( "PageMenu.createPageMenuURLs" );
                    var nodeOid = me._getSelectionNodeOid(me._id);
                    var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                    var cdoGuid = state.selectionModel.getSelectedNodeApplicationGuid();
                    ibm.portal.debug.text( "Application guid is " + cdoGuid ); 
                    for(var i = 0; i< me._pageMenuArray.length; i++){

                        switch (me._pageMenuArray[i].label){
                                
                            case "CreatePage":
                                var url = state.newPageURL( ibmPageActionIDs.createPage, ibmPageActionWindowIDs[ibmPageActionIDs.createPage] /*portletWindowId*/, "Solo" /*portletWindowState*/);

                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "nId", nodeOid );
                                urlObj.addParameter( "cId", nodeOid );
                                urlObj.addParameter( "init", "init" );
                                urlObj.addParameter( "OCN", nodeOid);
                                urlObj.addParameter( "WCN", nodeOid );
                                urlObj.addParameter( "PAT", "PTQ" );
                                urlObj.addParameter( "PNT", "PNTC" );
                                urlObj.addParameter( "psh", "push" );
                                
                                ibm.portal.debug.text("PageMenu.createPageMenuURLs \"Create Page\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "EditLayout":
                                var url = state.newPageURL( ibmPageActionIDs.editLayout, ibmPageActionWindowIDs[ibmPageActionIDs.editLayout]/*portletWindowId*/, "normal" /*portletWindowState*/ );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "nId", nodeOid );
                                urlObj.addParameter( "cId", nodeOid );
                                urlObj.addParameter( "init", "init" );
                                urlObj.addParameter( "WCN", nodeOid );
                                urlObj.addParameter( "OCN", nodeOid);
                                urlObj.addParameter( "WNN", nodeOid );
                                urlObj.addParameter( "psh", "push" );
                                
                                ibm.portal.debug.text("PageMenu.createPageMenuURLs\"Edit Page Layout\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "EditPageProperties":
                                var selModel = state.selectionModel;
                                
                                var url = state.newPageURL( ibmPageActionIDs.editPageProperties , ibmPageActionWindowIDs[ibmPageActionIDs.editPageProperties] /*portletWindowId*/, "Solo" /*portletWindowState*/);
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "nId", nodeOid );
                                urlObj.addParameter( "cId", nodeOid );
                                urlObj.addParameter( "init", "init" );
                                urlObj.addParameter( "OCN", nodeOid);
                                urlObj.addParameter( "WCN", nodeOid );
                                urlObj.addParameter( "PAT", "PTQE" );
                                urlObj.addParameter( "PAT", "PTE" );
                                if (selModel.getSelectedNode().isLabel) {
                                    urlObj.addParameter( "PNT", "PNTL" );
                                }else{
                                    urlObj.addParameter( "PNT", "PNTC" );
                                }
                                urlObj.addParameter( "psh", "push" );
                                
                                ibm.portal.debug.text("PageMenu.createPageMenuURLs \"Page Properties\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "AssignRoles":
                                var url = state.newPageURL( ibmPageActionIDs.assignRoles, ibmPageActionWindowIDs[ibmPageActionIDs.assignRoles]  /*portletWindowId*/, "Solo" /*portletWindowState*/ );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "nId", nodeOid );
                                urlObj.addParameter( "cId", nodeOid );
                                urlObj.addParameter( "init", "init" );
                                urlObj.addParameter( "WCN", nodeOid );
                                urlObj.addParameter( "OCN", nodeOid);
                                urlObj.addParameter( "psh", "push" );
                                
                                ibm.portal.debug.text("PageMenu.createPageMenuURLs \"Resource Permissions\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "EditApplicationProperties":
                                var url = state.newPageURL( ibmPageActionIDs.editAppProperties, ibmPageActionWindowIDs[ibmPageActionIDs.editAppProperties], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Edit App Properties\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "EditApplicationLayout":
                                var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                                
                                var url = state.newPageURL( ibmPageActionIDs.editAppLayout, ibmPageActionWindowIDs[ibmPageActionIDs.editAppLayout], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Edit App Layout\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "AssignApplicationMember":
                                var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                                
                                var url = state.newPageURL( ibmPageActionIDs.assignAppMembers, ibmPageActionWindowIDs[ibmPageActionIDs.assignAppMembers], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Assign App Members\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "ManageApplicationRoles":
                                var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                                
                                var url = state.newPageURL( ibmPageActionIDs.assignAppRoles, ibmPageActionWindowIDs[ibmPageActionIDs.assignAppRoles], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Assign App Roles\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "ShowApplicationStatusPolicy":
                                var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                                
                                var url = state.newPageURL( ibmPageActionIDs.showAppPolicyStatus, ibmPageActionWindowIDs[ibmPageActionIDs.showAppPolicyStatus], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Show App Policy Status\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "SaveAsTemplate":
                                var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                                
                                var url = state.newPageURL( ibmPageActionIDs.editAppProperties, ibmPageActionWindowIDs[ibmPageActionIDs.editAppProperties], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                urlObj.addParameter( "act", "exp" );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Save App As Template\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                        }
                    }
                    return;                 
                },
                clearMenu: function(){
                                
                    ibm.portal.debug.entry( "PageMenu.clearMenu" ); 
                        
                    var menuButton = dojo.byId("menu_pageContextMenuButton");
                    if(menuButton != null){
                        var parent = menuButton.parentNode;
                        parent.removeChild(menuButton);
                    }
                    
                    var menuDiv = dojo.byId("contextMenu_Page");
                    if(menuDiv != null){
                        var parent = menuDiv.parentNode;
                        parent.removeChild(menuDiv);
                    }
                    
                    this._pageMenuArray = new Array();

                },
                _doDeletePage: function ( /*Object*/ args ) {
                    ibm.portal.debug.entry( "PageMenu._doDeletePage", [ args ] );
                    
                    var pageId = null;
                    
                    // validate input params
                    if(args != null){
                        pageId = args["pageId"];
                    }else{
                        ibm.portal.debug.text("PageMenu_doDeletePage: a required parameter is missing or null.");
                        return false;
                    }
                    
                    // check if the event is for us
                    if (this._id != pageId) {
                        return;
                    }
                    
                    var check = confirm(this._engineProperties["link.deletepage.confirm"]);
                    
                    if(check){
                        var deleteURL = ibmPortalConfig["contentHandlerURI"]+ '?uri='+pageId;
                        ibm.portal.debug.text("makeDeleteRequest: Delete URL is " + deleteURL);
                        
                        //get resource bundle for status messages
                        var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
                        //get parent container to remove deleted node
                        var parentURI = com.ibm.portal.services.NAVIGATION_MODEL.getNode( pageId ).childOf;
                        var parentNavNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode( parentURI );
                        var me = this;
                        
                        var request = new com.ibm.portal.services.PortalRestServiceRequest( { url: deleteURL } );
                        request.remove( function(type, data, xhr, callbackargs) {
                        	var resTxt = data;
                            if (type == "load") {
                                me.removePageDOMItem(pageId);
                                parentNavNode.removeChild(pageId);
                                com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.setSelection(parentURI);
                                
                                ibm.portal.debug.text("PageMenu._doDeletePage: success: "+resTxt);
                                var statusMessage = new com.ibm.portal.aggregation.StatusMessage('info', v3themeProperties["pageDeleteOk"], "");
                                dojo.publish("/portal/status",[{message: statusMessage}]);
                                
                            } else {
                                ibm.portal.debug.text("PageMenu._doDeletePage: failure: "+resTxt);
                                var statusMessage = new com.ibm.portal.aggregation.StatusMessage('error', v3themeProperties["pageDeleteFailed"], resTxt);
                                dojo.publish("/portal/status",[{message: statusMessage}]);
                                dojo.publish("/portal/PortalPage/reload");
                            }
                        });
                    }
                },
                removePageDOMItem: function (pageId){
                    var domItem = dojo.byId(pageId);
                    var domItemParent = domItem.parentNode;
                    
                    if(domItemParent.childNodes.length <= 1){
                        domItemParent.parentNode.removeChild(domItemParent);    
                    }else{
                        domItemParent.removeChild(domItem);
                    }
                },
                applyExtractPortletWindowIdsXslt: function( /*String*/type, /*dojo.dom.Document*/xmlData, /*XMLHttpRequest*/xhr, /*Array*/callbackargs ) {
                    // summary: Apply the XSLT to extract the portlet window id's from the page specified
                    ibm.portal.debug.entry( "PageMenu.applyExtractPortletWindowIdsXslt", [xmlData,xhr,callbackargs] );
                    var pageUnid = callbackargs["page"];
                    var trf = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( ibmPortalConfig["extractPortletWindowIdsXslt"] );
                    if (trf == null) {
                        ibm.portal.debug.text("XSL transformer is null");
                    }
                    var results = trf.transformToDocument(xmlData, {}, true);
                    eval( results );
                        
                    if(controls[0].id != null){
                        ibmPageActionWindowIDs[pageUnid] = com.ibm.portal.utilities.getControlId(controls[0].id);   
                    } else {
                        window.console.log( "Page action control id is null!" );
                    }   
                },
                _setPageActionWindowIds: function (){
                    if(ibmPageActionIDs != null && ibmPageActionWindowIDs == null ){
                        for (var unid in ibmPageActionIDs){
                            var pageNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode(ibmPageActionIDs[unid]);
                            if(pageNode != null){
                                pageNode.getLayoutModel( dojo.hitch( this, "applyExtractPortletWindowIdsXslt" ), {page: ibmPageActionIDs[unid]} );
                            }
                        }
                    }
                },
                _getSelectionNodeOid: function (fullSelectionNodeId){
                    var selectionNodeOid = fullSelectionNodeId.substring(7);
                    return selectionNodeOid;
                },
                _getPageMenuImgSrc: function (){
                    return dojo.byId("menu_pageContextMenuButton_img").src;
                },
                _setPageMenuImgSrc: function (newSrc){
                    dojo.byId("menu_pageContextMenuButton_img").src = newSrc;
                }
                
              }
);

dojo.declare( "com.ibm.portal.aggregation.theme.PaletteURLFactory",
              null,
              {
                _lastSelection: null,
                _lastState: null,
                _pageName: null,
                _portletName: null,
                constructor: function () {
                    com.ibm.portal.EVENT_BROKER.stateChanged.register( this, this.storeLastState );
                },
                storeLastState: function ( /*Object*/args ) { 
                    // summary: Renders the navigation element.
                    // state: the current navigational state.
                    ibm.portal.debug.entry( "PortletPalette.render", [args]);
                    
                    var selectionChanged = false;
                    var currentState = args["state"];
                    if ( this._lastSelection != null ) {
                        if ( this._lastSelection != currentState.getPageSelection() ) {
                            selectionChanged = true;
                        }
                    }
                    
                    this._lastState = currentState;
                    this._lastSelection = currentState.getPageSelection();
                    
                    if ( selectionChanged ) {
                        var palettes = wptheme_InlinePalettes;
                        var contexts = palettes.paletteContextArray;
                        for ( var i = 0; i < contexts.length; i++ ) {
                            if ( contexts[i].selectionDependent && i == palettes.currentIndex && palettes.paletteStatus == 1 ) {
                                palettes.show( i );
                            }       
                        }
                    }       
                    
                    ibm.portal.debug.exit( "PortletPalette.render" );
                },
                createURL: function ( /*PaletteContext*/context ) {
                    // summary: Creates the url to render the portlet palette.
                    // returns: the serialized url pointing to the portlet palette
                    var renderParams = null;
                    if ( context.selectionDependent ) {
                        renderParams = [ {name: "WCN", value: this._lastState.getPageSelection() } ];
                    }
                    
                    var page = null, portlet = null, windowstate = null, portletmode = null, newwindow = null;
                    
                    if ( context.page ) { page = context.page; }
                    if ( context.portlet ) { portlet = context.portlet; }
                    if ( context.portletWindowState ) { windowstate = context.portletWindowState }
                    if ( context.portletMode ) { portletmode = context.portletMode; }
                    if ( context.newWindow ) { newwindow = context.newWindow; } 
                    
                    return this._lastState.newPageURL( page,
                                                       portlet, 
                                                       windowstate, 
                                                       portletmode, 
                                                       renderParams, 
                                                       newwindow); // String
                }   
              } 
);

// summary: Responsible for making sure the mouse cursor reflects the correct state of the
//      page.
com.ibm.portal.aggregation.theme.CURSOR_CONTROLLER = new function () {
    this._portalPageStartHandler = dojo.subscribe( "/portal/PortalPage/start", this, function () { document.body.style.cursor = 'progress'; } );
    this._portalPageCompleteHandler = dojo.subscribe( "/portal/PortalPage/complete", this, function () { document.body.style.cursor = ''; } );
}

}
