/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.collections.PersistentStack"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.collections.PersistentStack"] = true;
dojo.provide( "com.ibm.portal.collections.PersistentStack" );

dojo.require( "dojo.storage.*" );
dojo.require( "dojo.collections.Stack" );
dojo.require( "dojo.dom" );

var com_ibm_portal_collections_UsingStorage = true;

if ( dojo.storage.manager.isInitialized() == false ) {
	window.console.log( "Storage is not initialized. The stack will not be backed up to client-side storage." );
	com_ibm_portal_collections_UsingStorage = false;
}

dojo.declare( "com.ibm.portal.collections.PersistentStack", 
			  dojo.collections.Stack, 
			  function ( /*String?*/ key ) {
			  	// summary: Extends the dojo Stack collection to provide persistence
				// 		in client-side storage.
				// description: Extends the implementation of the dojo Stack collection
				//		to store the stack object in client-side storage anytime the 
				//		stack is changed. 
				// key: String	
				//			(param) the storage key to use, if none is specified a random one
				//			will be generated.
				// storageKey: String
				//			(instance variable) the storage key that will be used to store the stack
				ibm.portal.debug.entry( "PersistentStack.constructor", [ key ] );
				if ( !key ) {
			  		this.storageKey = "com.ibm.portal.Stack" + dojo.dom.getUniqueId();
				}
				else {
					this.storageKey = key;
					//We can only expect an existing stack when the key is provided.
					this._load();
				}
				ibm.portal.debug.exit( "PersistentStack.constructor" );
			  }, 
			  {
			  	clear: function () {
					// summary: Clears the stack of all objects. Also clears
					// 		the stack from client-side storage.
					this._clearStorage();
					com.ibm.portal.collections.PersistentStack.superclass.clear.apply( this );	
				},
				pop: function () { 
					// summary: Pops the top object on the stack and stores
					// 		the resulting stack back into the client-side storage.
					// returns: Object
					//				the top object on the stack
					var retVal = com.ibm.portal.collections.PersistentStack.superclass.pop.apply( this );
					this._store();
					return retVal; //Object
				},
				push: function ( /*Object*/ obj ) {
					// summary: Pushes the given object onto the top of the 
					// 		stack and stores the resulting stack in the client-side
					// 		storage.
					// obj: Object 
					// 			 the object to add to the stack
					var retVal = com.ibm.portal.collections.PersistentStack.superclass.push.apply( this, [ obj ] );
					this._store();
					return retVal; //Object
				},
				_store: function () {
					if ( !com_ibm_portal_collections_UsingStorage ) {
						return;
					}
					var array = com.ibm.portal.collections.PersistentStack.superclass.q;
					dojo.storage.put( this.storageKey, array, this._saveHandler );
				},
				_load: function () {
					ibm.portal.debug.entry( "PersistentStack._load" );
					if ( !com_ibm_portal_collections_UsingStorage ) {
						return;
					}
					var array = dojo.storage.get( this.storageKey );
					if ( array != null ) {
						this._initializeStack( array );	
					}
					ibm.portal.debug.exit( "PersistentStack._load");	
				},
				_getStackArray: function () {
					return com.ibm.portal.collections.PersistentStack.superclass.q;	
				},
				_initializeStack: function ( /*Object[]*/ array ) {
					com.ibm.portal.collections.PersistentStack.superclass.q = array;
					com.ibm.portal.collections.PersistentStack.superclass.count = array.length;	
				},
				_clearStorage: function () {
					dojo.storage.remove( this.storageKey );	
				},
				_saveHandler: function () {} 					
			  }
);

}
