/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.data.PortalTreeModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.data.PortalTreeModel"] = true;
dojo.provide( "com.ibm.portal.data.PortalTreeModel" );

dojo.require( "com.ibm.data.DataTreeModel" );
dojo.require("dojo.i18n");

dojo.declare( "com.ibm.portal.data.PortalTreeModel",
			com.ibm.data.DataTreeModel,
			{
				
				filterHidden: false,
				
				parentAttr: "parent",
				childAttr: "childrenURI",
				
				childrenAttrs: ["children"],
				
				constructor: function() {
					if(this.filterHidden) this.addQueryParams["mdname"] = "com.ibm.portal.Hidden";
					this._notifyOn = true;
					this.connections = [];
					if(this.store.getFeatures()["dojo.data.api.Notification"]) {
						this.connections.concat([
							dojo.connect(this.store, "onNew", this, "_onNew"),
							dojo.connect(this.store, "onSet", this, "_onSet"),
							dojo.connect(this.store, "onDelete", this, "_onDelete")
						]);
					}
				},
				destroy: function() {
				        // summary: destroys this object, releasing connections to the store
				        dojo.forEach(this.connections, dojo.disconnect);
				},
				
				// =======================================================================
				// Methods for traversing hierarchy
				getRoot: function(onItem, onError){
					// summary:
				    //            Calls onItem with the root item for the tree, possibly a fabricated item.
					//            Throws exception on error.
					if(this.root) onItem(this.root);
					else {
						this.store.fetch({
							query: this._fixQuery(this.query),
							onComplete: dojo.hitch(this, function(items) {
								if(items.length == 0) throw new Error("No items returned by query for root node");
								else {
									this.root = items[0];
									onItem(this.root);
								}
							}),
							onError: onError
						});
					}
				},
				_fixQuery: function(/*String|Object*/ query) {
					// summary: Fixes a query to append the addQueryParams query parameters
					//		on this model to the query that's passed as an argument.
					// query: String|Object initial query to manipulate
					// returns: String fixed query after the addQueryParams have been added
					//		to it.
					if(!this.addQueryParams) return query;
					if(!dojo.isString(this.addQueryParams)) {
						this.addQueryParams = dojo.objectToQuery(this.addQueryParams);
					}
					if(dojo.isString(query)) {
						if(query.indexOf("?") > -1) {
							return query + "&" + this.addQueryParams;
						}
						else {
							return query + "?" + this.addQueryParams;
						}
					}
					else {
						return "?" + dojo.objectToQuery(query) + this.addQueryParams;
					}
				},
				mayHaveChildren: function(/*dojo.data.Item*/ item){
				        // summary
				        //            Tells if an item has or may have children.  Implementing logic here
				        //            avoids showing +/- expando icon for nodes that we know don't have children.
				        //            (For efficiency reasons we may not want to check if an element actually
				        //            has children until user clicks the expando node)
				        return item._children ? item._children.length > 0 : !!this.store.getValue(item, this.childAttr);
				},
				getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ onComplete, /*function*/ onError){
				        // summary
				        //           Calls onComplete() with array of child items of given parent item, all loaded.
				        //            Throws exception on error.
				        var store = this.store;
				        var id = store.getIdentity(parentItem);
				        var parentAttr = this.parentAttr;
				        
				        if(!parentItem._children && !this.mayHaveChildren(parentItem)){
				        	parentItem._children = new Array();
				        }
				        if(parentItem._children) {
				        	onComplete(parentItem._children);
				        } 
				        else if((!parentItem._children || parentItem._children.length == 0) && this.mayHaveChildren(parentItem)) {
				        	//parentItem._children
				        	store.fetch({
				        		query: this._fixQuery(store.getValue(parentItem, this.childAttr, "")),
				        		queryOptions: {preventCache: true},
				        		onComplete: dojo.hitch(this, function(items) {
				        			parentItem._children = new Array();
				        			dojo.forEach(items, function(item){
				        				// only push the results whose parent has the same id as this parentItem
				        				if(store.getValue(item, parentAttr) == id) {
				        					if(this.filterHidden) {
				        						// if we're filtering hidden nodes, check the metadata
				        						var isHidden = store.getValue(item, "metadata[com.ibm.portal.Hidden]$");
				        						if(isHidden && (isHidden.toLowerCase() == "yes" || isHidden.toLowerCase() == "true")) {
				        							return;
				        						}
				        					}
				        					
				        					parentItem._children.push(item);
				        				}
				        			}, this);
				        			onComplete(parentItem._children);
				        		}),
				        		onError: function(errorData, request) {
				        			onError(errorData);
				        		}
				        	});
				        } 
				},
				// =======================================================================
				// Inspecting items
				getIdentity: function(/* item */ item){
				        // summary: returns identity for an item
				        return this.store.getIdentity(item);
				},
				getLabel: function(/*dojo.data.Item*/ item){
				        // summary: get the label for an item
				        return this.store.getLabel(item);
				},
				getParent: function(/*dojo.data.Item*/ item) {
					// summary: returns the parent item of this item argument; may return a stub if the parent
					// 		hasn't been loaded yet
					var pId = this.store.getValue(item, "parent");
					if(pId) {
						return this.store.getItem(pId);
					}
					else return null;
				},
				// =======================================================================
				// Write interface
				newItem: function(/* Object */ args, /*Item*/ parent){
				        // summary
				        //            Creates a new item.   See dojo.data.api.Write for details on args.
				        var parentInfo = {parent: parent, attribute: this.childrenAttrs[0]};
				        var item = this.store.newItem(args, parentInfo);
				        this.store.save({onComplete: args.onComplete, onError: args.onError, scope: args.scope});
				        return item;
				},
				pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Item*/ nextSiblingItem, /*Boolean*/ defer){
				        // summary:
				        //            Move an item from one parent item to another or to another position under the same parent.
				        //            Used in drag & drop.
				        // childItem: Item to move.
				        // oldParentItem: Previous parent of item being moved.
				        // newParentItem: New parent of the item being moved.
				        // nextSiblingItem: New next sibling item of this item is.  Can be null
				        //		to indicate that this item is at the end of a list of children.
				        // defer: Boolean flag to persist the change in the store, or defer persisting
				        //		till later.  True does not persist the change.  False persists the change
				        //		immediately.  If the change is not persisted, a call to the save function
				        //		will persist all pending changes in the model.
				        
				        this._notifyOn = false;
				        // turning off notifications while doing multiple updates... that way
				        // we can send one onChildrenChange event to all listeners with the proper
				        // list of children
				        var store = this.store;
				        var childId = store.getIdentity(childItem), 
				        	newParentId = store.getIdentity(newParentItem),
				        	oldParentId = store.getIdentity(oldParentItem),
				        	nextId = nextSiblingItem ? store.getIdentity(nextSiblingItem) : null;
				        if(childId == newParentId || childId == nextId) {
				        	// dropped on ourself... do nothing... as if this is possible somehow
				        	this._notifyOn = true;
				        	return;
				        }
				        else {
				        	
				        	if(oldParentId != newParentId || !nextSiblingItem) {
				        		store.setValue(childItem, "parent", newParentId);
				        	}
				        	
				        	if(nextId) {
				        		store.setValue(childItem, "next", nextId);
				        	}
				        	
							var onError = function(errors){
								dojo.forEach(errors, function(error){
									var message = error.message;
									var responseText = error.responseText;
									if(responseText.indexOf("EJPEK0020E") >= 0){ // error for insufficient permissions on drop
										bundle = dojo.i18n.getLocalization("com.ibm.bundles", "NavWidget");
										message = bundle["nav_invalid_drop_error"];
										responseText = bundle["nav_invalid_drop_msg"];
									}
									var statusMessage = new com.ibm.widgets.StatusMessage( "error", message, responseText );
									dojo.publish("/portal/status",[{message: statusMessage, uid: 'ibmPortalStatusBox'}]);
								});
							}
			        		if(!defer) store.save({onError:onError});
			        		this._notifyOn = true;
			        		
			        		if(oldParentId != newParentId) {
			        			// reparenting involves removing from the old and inserting into the new
				        		this._removeChildItem(oldParentItem, childItem);
				        		this._insertChildItem(newParentItem, childItem, nextSiblingItem, dojo.hitch(this, function(){
				        			this.onTreeChange(childItem, oldParentItem, newParentItem, nextSiblingItem);
				        		}));
				        	}
				        	else {
				        		// if we're not reparenting, then we can only be moving
				        		this._moveChildItem(newParentItem, childItem, nextSiblingItem, dojo.hitch(this, function(){
				        			this.onTreeChange(childItem, oldParentItem, newParentItem, nextSiblingItem);
				        		}));
				        	}
				        }
				},
				onTreeChange: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Item*/ nextSiblingItem) {
					
				},
				_onDelete: function(/* Item */ deletedItem) {
					// summary: Callback handler for the store attached to this tree model to call
					//		when an item is deleted
					if(!this._notifyOn) return;
					if(deletedItem.parentId) {
						var parentItem = this.store.getItem(deletedItem.parentId);
						this._removeChildItem(parentItem, deletedItem, dojo.hitch(this, function(items) {
								this.onTreeChange(deletedItem, parentItem);
						}));
					}
				},
		
				_onNew: function(/* Item */ newItem, /* Object */ parentInfo) {
					// summary: Callback handler for the store attached to this tree model to call
					//		when an item is created 
					if(!this._notifyOn) return;
					if(!parentInfo || !parentInfo.item) return;
					this._insertChildItem(parentInfo.item, newItem, null, dojo.hitch(this, function(){
	        			this.onTreeChange(newItem, null, parentInfo.item);
	        		}));
					return;
					
					if(!parentInfo.item._children) {
						// never been retreived yet
						parentInfo.item._children = [newItem];
					}
					else {
						parentInfo.item._children.push(newItem);
					}
					this.getChildren(parentInfo.item, dojo.hitch(this, function(items) {
							this.onChildrenChange(parentInfo.item, items);
						})
					);
				},
		
				_onSet: function(/* Item */ item, /*String*/ attribute, /*Object | Array*/ oldValue, /*Object | Array*/ newValue) {
					// summary: Callback handler for the store attached to this tree model to call
					//		when an item is modified
					if(!this._notifyOn) return;
					if(attribute != "next" && oldValue == newValue) return;
					if(attribute == "parent") {
						// If the parent has been changed, refresh the old and new parent items
						var oldP = this.store.getItem(oldValue);
						if(this.store.isItemLoaded(oldP)) {
							this._removeChildItem(oldP, item);
						}
						else {
							this.store.loadItem({
								item: oldP,
								onItem: function(oldParent) {
									this._removeChildItem(oldParent, item);
								},
								scope: this
							});
						}
						
						
						var newP = this.store.getItem(newValue);
						if(this.store.isItemLoaded(newP)) {
							this._insertChildItem(newP, item);
						}
						else {
							this.store.loadItem({
								item: newP,
								onItem: function(newParent) {
									this._insertChildItem(newParent, item);
								},
								scope: this
							});
						}
					}
					else if(attribute == "next") {
						// If the item has been reordered within the same parent list of children, refresh
						// its parent
						var parentId = this.store.getValue(item, "parent");
						var parentItem = this.store.getItem(parentId);
						if(this.store.isItemLoaded(parentItem)) {
							this._moveChildItem(parentItem, item, newValue);
						}
						else {
							this.store.loadItem({
								item: parentItem,
								onItem: function(loadedParentItem) {
									this._moveChildItem(loadedParentItem, item, newValue);
								},
								scope: this
							});
						}
					}
					else {
						this.onChange(item);
					}
				},
				
				refreshItemChildren: function(/*Item*/ item, /*Function?*/ modifier) {
					item._children = null;
					var func = modifier ? modifier : function(items) {
							this.onChildrenChange(item, items);
						};
					this.getChildren(item, dojo.hitch(this, func));
				},
				_insertChildItem: function(/*Item*/parentItem, /*Item*/ item, /*Item*/ newNextSibling, /*Function*/callbackfn) {
					this.getChildren(parentItem, dojo.hitch(this, function(items) {
						// refresh the item children with a function that will add the item first
						if(!newNextSibling) {
							items.push(item);
						}
						else {
							var id = this.getIdentity(newNextSibling);
							for(var i = 0; i < items.length; i++) {
								if(this.getIdentity(items[i]) == id) {
									items.splice(i, 0, item);
									break;
								}
							}
						}
						if(callbackfn) callbackfn();
						this.onChildrenChange(parentItem, items);
					}));
				},
				_removeChildItem: function(/*Item*/parentItem, /*Item*/ item, /*Function*/callbackfn) {
					this.getChildren(parentItem, dojo.hitch(this, function(items) {
						var id = this.store.getIdentity(item);
						for(var i = 0; i < items.length; i++) {
							if(this.store.getIdentity(items[i]) == id) {
								items.splice(i,1);	// remove the item from the array
								break;	// break out of the loop since we already found ourselves
							}
						}
						if(callbackfn) callbackfn();
						this.onChildrenChange(parentItem, items);
					}));
				},
				_moveChildItem: function(/*Item*/parentItem, /*Item*/ item, /*String*/ newNextSibling, /*Function*/callbackfn) {
					this.getChildren(parentItem, dojo.hitch(this, function(items) {
						var id = this.store.getIdentity(item);
						var nextIndex = items.length - 1;
						for(var i = 0; i < items.length; i++) {
							if(this.store.getIdentity(items[i]) == id) {
								items.splice(i, 1);	// remove item from index i
								i--;
							}
							else if(newNextSibling && this.store.getIdentity(items[i]) == newNextSibling) {
								// insert item at index i before the existing item at that index without deleting it
								items.splice(i, 0, item);
								i++;
							}
						}
						if(!newNextSibling) {
							// if this is null, append the item to the end of the list
							items.push(item);
						}
						if(callbackfn) callbackfn();
						this.onChildrenChange(parentItem, items);
					}));
				},
				toString: function() {
					return "[TreeModel " + this.declaredClass + "]";
				}
			}
);

}
