/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.debug.NewWindow"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.debug.NewWindow"] = true;
dojo.provide("com.ibm.portal.debug.NewWindow");

dojo.require( "dojo.dnd.common" );
dojo.require( "com.ibm.portal.debug" );

/* create the new window for debug statements */
if ( djConfig["openDebugWindowOnDemand"] == true ) {
	com.ibm.portal.debug._newWindowObj = null;	
}
else {
	com.ibm.portal.debug._newWindowObj = self.open( "about:blank", "_blank", "toolbar=no,location=no,menubar=yes,titlebar=no,scrollbars=yes" );
}

/* track the number of statements we've logged so we can alternate the background colors, making the debug statements easier to read */
com.ibm.portal.debug._statementCount = 0;

com.ibm.portal.debug.NewWindow.startLogging = function () {
	/*
	 * summary: Opens the new browser window which will serve as the debug console, if necessary.
	 * 
	 * description: If the new browser window to be used as the debug console has not been opened
	 * yet, this function will create and open this window.
	 */
	if ( com.ibm.portal.debug._newWindowObj == null ) {
		com.ibm.portal.debug._newWindowObj = self.open( "about:blank", "_blank", "toolbar=no,location=no,menubar=yes,titlebar=no,scrollbars=yes" );
	}	
}

com.ibm.portal.debug.NewWindow.endLogging = function () {
	/*
	 * summary: Closes the new browser window which contains the debugging statements.
	 */
	if ( com.ibm.portal.debug._newWindowObj != null ) {
		com.ibm.portal.debug._newWindowObj.close();
	}
}

com.ibm.portal.debug.NewWindow.isOpen = function () {
	/*
	 * summary: Indicates whether the debug window is currently open.
	 */
	return ( com.ibm.portal.debug._newWindowObj != null );
}

com.ibm.portal.debug.NewWindow.println = function ( /*String*/line ) {
	/*
	 * summary: Prints a string to a new window.
	 *
	 * description: Writes a given string to the new window's document. This allows for debugging statements to be 
	 * tracked in a separate browser window from the browser window in which the code is being executed. This seems 
	 * to be cleaner when having to copy and paste code or save "logs" using the debug console widget or the firebug 
	 * console. To use the new window debugging, simply add the following code to your html file:
	 *
	 *						dojo.requires( "com.ibm.portal.debug.NewWindow" );
	 *
	 * line: String				The text to write to the new window's document.
	 *
	 */
	 
	if ( com.ibm.portal.debug._newWindowObj == null ) return;
	
	try {
		var doc = com.ibm.portal.debug._newWindowObj.document;
		
		//Setting the title initially doesn't seem to work in Firefox. So do it here. 
		//Note: This isn't necessary, it really just makes it easier to find the debug window from many open 
		//	    browser windows
		if ( doc.title != "Debug Console" ) {
			doc.title = "Debug Console";
		}
		
		var console = doc.getElementsByTagName("body")[0] || doc.body;
		
		var unique = dojo.dnd.getUniqueId();
		var unique2 = dojo.dnd.getUniqueId();
			
		var div = doc.createElement("div");
		var div2 = doc.createElement( "div" );
		div.innerHTML = "<a id=\"" + unique + "\" href=\"javascript:void(0);\" onclick=\"document.getElementById('"+ unique2 + "').focus()\" >End</a>";
		div2.appendChild(doc.createTextNode(line))
		div.appendChild( div2 );
		div.innerHTML += "<a id=\"" + unique2 + "\" href=\"javascript:void(0);\" onclick=\"document.getElementById('"+ unique + "').focus()\">Top</a>";		
		
		//Every other statement that is logged, change the background color to make it easier to see where each 
		//statement begins and ends. This is useful when large amounts of information are being logged in a single 
		//statement.
		if ( com.ibm.portal.debug._statementCount % 2 == 0 ) {
			div.style.backgroundColor = "#CCCCCC";
		}
		
		console.appendChild(div);
		
		var hr = doc.createElement( "hr" );
		console.appendChild( hr );
	} catch (e) {
		try{
			// safari needs the output wrapped in an element for some reason
			doc.write("<div>" + line + "</div>");
		}catch(e2){
			window.status = line;
		}
	}
	
	com.ibm.portal.debug._statementCount++;
}

// overwriting the console doesn't always seem to work, so just overwrite the
// ibm.portal.debug.text function instead

// window.console.log = com.ibm.portal.debug.NewWindow.println;
ibm.portal.debug.text = function( str, functionName ) {
	com.ibm.portal.debug.NewWindow.println(str);
}

dojo.addOnUnload( function () { 
	if ( com.ibm.portal.debug._newWindowObj == null || ( com.ibm.portal.debug._newWindowObj && com.ibm.portal.debug._newWindowObj.closed ) ) return; 
	var val = confirm( "Do you want to close the debug window?"); 
	if ( val ) { 
		com.ibm.portal.debug._newWindowObj.close() 
	} 
} ); 

}
