/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.services.NavigationSelectionModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.services.NavigationSelectionModel"] = true;
dojo.provide("com.ibm.portal.services.NavigationSelectionModel");

dojo.require( "com.ibm.portal.services.NavigationModel" );

dojo.declare( "com.ibm.portal.services.NavigationSelectionModel",
			  null,
			  {
			  	constructor: function ( /*String*/pageID ) {
				  	ibm.portal.debug.entry( "NavigationSelectionModel.constructor", [ pageID ] );
				  
				  	this.selectionPath = new Array();
					this._populateParentNodes( pageID );

                    if(this.selectionPath.length<1) {
					    ibm.portal.debug.text( "ctr:this.selectionPath < 1: Wait for page refresh"); 	
                        return this.selectionPath;
                    }                    

					var navNode = this.selectionPath[ this.selectionPath.length - 1 ];
					//var navNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode( pageID );
					var doValidation = navNode.isLabel;
					//Populate any parent nodes if they exist.
					
					//Append this node.
					//this.selectionPath.push( navNode );
					//If the given pageID points to a label, we have
					//to find the actual selected page.
					if ( navNode.isLabel && navNode.hasChildren ) {
						var calcSel = this._calculateSelectedPage( navNode );
						var selected = calcSel.found ? calcSel.found : calcSel.maybe;
						if(selected != null) {
							var selectedNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode( selected );
							var addToPath = new Array ();
							
							if ( selected ) {
								var lastNodeURI = this.selectionPath[ this.selectionPath.length - 1 ].URI;
								ibm.portal.debug.text( "Last entry in selection path: " + lastNodeURI );
								
								var parentURI = selectedNode.childOf;
								var done = ( lastNodeURI == parentURI );
								//We know what the selected node is now. Now we have to walk back up from the
								//selected node and add any nodes that are in between it and the last of the
								//parent nodes.
								while ( !done ) {
									ibm.portal.debug.text( "parentURI = " + parentURI );
									var node = com.ibm.portal.services.NAVIGATION_MODEL.getNode( parentURI );
									addToPath.push( node );
									parentURI = node.childOf;
									ibm.portal.debug.text( "lastNodeURI is " + lastNodeURI + "; parentURI is " + parentURI + "; equal? " + ( lastNodeURI == parentURI ) );
									done = ( lastNodeURI == parentURI );	
								}
								
								var toAdd = addToPath.pop();
							
								while ( toAdd ) {
									this.selectionPath.push( toAdd );
									toAdd = addToPath.pop();
								}
								
								//Only perform validation if the node originally passed into this function is
								//a label.
								if ( doValidation ) {
									var validatedSelection = this._validateSelection( selected );
									
									if ( validatedSelection != selected ) {
										ibm.portal.debug.text( "Selection did not validate! Replacing with the validated selection id." );
										//selected = validatedSelection;
										selectedNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode( validatedSelection );
									}
								}	 
								
								//Finally add the calculated selection.
								this.selectionPath.push( selectedNode );	
							}
						}
					}
					
					ibm.portal.debug.exit( "NavigationSelectionModel.constructor", [this.selectionPath] );				
			    },
			  	getSelectedNode: function () {
					// summary: Get the selected navigation node.
					// returns: the selected NavigationNode
					return this.selectionPath[ this.selectionPath.length - 1 ]; //NavigationNode
				},
				getInheritedProperty: function ( /*String*/property, /*String*/defaultValue ) {
					// summary: Certain properties are inherited down the content/navigation model. This function
					//		walks up the selection path looking for the most recent definition for the given
					//		property. A default value may be specified to return if no explicit value is discovered
					//		during the search.
					// property: the property to search for.
					// defaultValue: the value to return if the property is not found
					ibm.portal.debug.entry( "NavigationSelectionModel.getInheritedProperty", [ property, defaultValue ] );
					
					var retVal = defaultValue;
					var i = this.selectionPath.length - 1;
					var done = false;
					while ( !done && i >= 0 ) {
						var node = this.selectionPath[i];
						if ( node[property] != null ) {
							done = true;
							retVal = node[property];
							ibm.portal.debug.text( "Property \"" + retVal + "\" found for page: " + node.title );
						}
						i--;
					}
					
					ibm.portal.debug.exit( "NavigationSelectionModel.getInheritedProperty", retVal );
					return retVal;	
				},
				getSelectedNodeTheme: function () { 
					// summary: Get the selected node's theme. If no theme is specified on the selection path, the default
					//		theme is assumed.
					// returns: the selected node's theme
					ibm.portal.debug.entry( "NavigationSelectionModel.getSelectedNodeTheme" );
					
					var theme = this.getInheritedProperty( "theme", "default" );

					ibm.portal.debug.exit( "NavigationSelectionModel.getSelectedNodeTheme", [ theme ]);
					return theme;
				},
				getSelectedNodeThemePolicy: function () { 
					// summary: Get the selected node's theme policy. If no theme policy is specified on the selection path, the default
					//		theme policy is assumed.
					// returns: the selected node's theme policy
					ibm.portal.debug.entry( "NavigationSelectionModel.getSelectedNodeThemePolicy" );
					var themePolicy = this.getInheritedProperty( "themePolicy", "default" );
					ibm.portal.debug.exit( "NavigationSelectionModel.getSelectedNodeThemePolicy", [ themePolicy ]);
					return themePolicy;
				},
				getSelectedNodeApplicationGuid: function () {
					// summary: Get the selected node's application guid if it has one. If no application guid is specified on the selection path, NULL
					//		is returned
					// returns: the selected node's theme
					ibm.portal.debug.entry( "NavigationSelectionModel.getSelectedNodeApplicationGuid" );
					var appGuid = this.getInheritedProperty( "applicationGuid", null );
					ibm.portal.debug.exit( "NavigationSelectionModel.getSelectedNodeApplicationGuid", [ appGuid ]);
					return appGuid;	
				},
				isNodeInSelectionPath: function ( /*String*/oid ) {
					ibm.portal.debug.entry( "NavigationSelectionModel.isNodeInSelectionPath", [ oid ] );
					
					var i = 0;
					var found = false;
					
					while ( !found && i < this.selectionPath.length ) {
						
						//console.log( "i = " + i );
						found = (this.selectionPath[i].URI == oid);
						i = i+1;
					}
					
					ibm.portal.debug.exit( "NavigationSelectionModel.isNodeInSelectionPath", found );
					return found;
				},
				_populateParentNodes: function ( /*String*/ uri ) {
					// summary: Populates the parent nodes of the given node.
					// description: Populates the selection path with the parent
					//		nodes of the given node all the way to the root node.
					// navNode: the selected node.
					ibm.portal.debug.entry( "NavigationSelectionModel._populateParentNodes", [ uri ] );
					
					/*var parentURI = navNode.childOf;
					var parentChain = new Array();
					
					while ( parentURI ) {
						//Add the parent node to the end of the chain.
						parentChain.push( parentURI );
						//Retrieve the information about the parent node.
						var parentNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode( parentURI );
						parentURI = parentNode.childOf;
					}*/
					var parentChain = com.ibm.portal.services.NAVIGATION_MODEL.getCompleteAncestorPath( uri );
					
					//We have to reverse the order in which we added the nodes.
					//We can't add to the front of the array using unshift() because
					//it apparently doesn't work properly in IE so we reverse the order
					//in which the parent nodes were added this way.
					//var toAdd = parentChain.pop();
					
					//while ( toAdd ) {
					//	this.selectionPath.push( toAdd );
					//	toAdd = parentChain.pop();
					//}
					this.selectionPath = parentChain;
					
					
					
					ibm.portal.debug.exit( "NavigationSelectionModel._populateParentNodes" );
				},
				
				_calculateSelectedPageDefault: function( /*NavigationNode*/ navNode ) {
					ibm.portal.debug.entry( "NavigationSelectionModel._calculateSelectedPageDefault", [ navNode ] );
					var children = navNode.getChildren();
					var found = null, maybe = null;
					var i = 0;
					
					while ( i < children.length && !found ) {
						if ( !children[i].isLabel ) {
							maybe = children[i].URI;
							if(!children[i].isExternalURL && !children[i].isInternalURL) {
								found = maybe;
							}
						}
						i++;
					}
					
					var j = 0;
					var ret = null;
					while ( j < children.length && !found ) {
						if(children[j].hasChildren) {
							ret = this._calculateSelectedPage( children[j] );
							found = ret.found;
							if(!maybe) maybe = ret.maybe;
						}
						j++;
					}
					
					ibm.portal.debug.exit( "NavigationSelectionModel._calculateSelectedPageDefault", [ found ] );					
					return {found: found, maybe: maybe};	
				},
				
				_calculateSelectedPageSSABehavior: function(/*NavigationNode*/ navNode){	
				
					ibm.portal.debug.entry( "NavigationSelectionModel._calculateSelectedPageSSABehavior", [ navNode ] );
					var children = navNode.getChildren();
					var found = null, maybe = null;
					var i = 0;
					var ret = null;
					
					while ( i < children.length && !found ) {
						if ( !children[i].isLabel ) {
							maybe = children[i].URI;
							if(!children[i].isExternalURL && !children[i].isInternalURL) {
								found = maybe;
							}
						} else if(children[i].isLabel) {
						
						        ret = this._calculateSelectedPageSSABehavior( children[i] );
						   	found = ret.found;
							if(!maybe) maybe = ret.maybe;
						
						}
						
						i++;
					}
					

					
					ibm.portal.debug.exit( "NavigationSelectionModel._calculateSelectedPageSSABehavior", [ found ] );						
					
					return {found: found, maybe: maybe};				
				
				
				
				},
				
				_calculateSelectedPage: function( /*NavigationNode*/ navNode ) {
				
				       var ssaBehavior = ibmPortalConfig.selectedPageSSAbehavior;
				       if(ssaBehavior != null && ssaBehavior){
					return this._calculateSelectedPageSSABehavior(navNode)
				      }
				          return this._calculateSelectedPageDefault(navNode);
				},

				_validateSelection: function( /*String*/ selectedNodeID) {
				    ibm.portal.debug.entry( "NavigationSelectionModel._validateSelection", [ selectedNodeID ] );
					
					//Strip out the "cm:oid:"
					selectedNodeID = selectedNodeID.substring( 7 );
					// create a new state
				    var stateManager = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
					var state = stateManager.newState();
				
				    // write the ID of the selected node into the state
				    var selectionAcc = stateManager.newSelectionAccessor(state);
				    selectionAcc.setSelection(selectedNodeID);
				
				    // validate the state using the serialization manager
				    var serializationManager = stateManager.getSerializationManager();
				    var result = serializationManager.serialize(state, true);
				    
				    // retrieve the validated state from the serialization result
				    var validatedState = result["state"];
				    
				    // read the validated selection and return it
				    selectionAcc = stateManager.newSelectionAccessor(validatedState);
				    
					ibm.portal.debug.exit( "NavigationSelectionModel._validateSelection", [ "cm:oid:" + selectionAcc.getSelection() ] );
					return "cm:oid:" + selectionAcc.getSelection();
				}
	
				
			  }
);

}
