/**
 * Copyright  1999-2004 AlphaBlox Corporation. All rights reserved.
 *
 * Script library of string utilities
 */


String.prototype.equalsIgnoreCase = function( str ) {
	return ( this.toLowerCase() == str.toLowerCase() );
}


String.prototype.endsWith = function( str ) {
	return ( this.substring( this.length - str.length, this.length ) == str );
}


String.prototype.startsWith = function( str ) {
	return ( this.substring( 0, str.length ) == str );
}


String.prototype.regionMatches = function( ignoreCase, start1, str, start2, len ) {
	if (ignoreCase)
		return this.regionMatchesIgnoreCase( start1, str, start2, len );
	else
		return ( this.substring(start1, start1+len) == str.substring(start2, start2+len) );
}


String.prototype.regionMatchesIgnoreCase = function( start1, str, start2, len ) {
	return ( this.toLowerCase().substring(start1, start1+len) == str.toLowerCase().substring(start2, start2+len) );
}


// Version of replace() built-in with MSIE JScript only replaces the first instance then exits.
// This recursively replaces all instances of a character within a string.
String.prototype.replaceChar = function( chr1, chr2 ) {
	var idx = 0;
	var modifiedString = this;
	for (idx = 0; idx < modifiedString.length; idx++) {
		if (modifiedString.charAt(idx) == chr1) {
			modifiedString = ([ modifiedString.substring(0,idx), chr2, modifiedString.substring(idx+1) ]).join("");
		}
	}
	return modifiedString;
}


String.prototype.trim = function() {
	var whitespace = new Array(" ","\t","\n","\r");
	var trimmedString = this;
	while (trimmedString.charAt(0) == whitespace[0]
			|| trimmedString.charAt(0) == whitespace[1]
			|| trimmedString.charAt(0) == whitespace[2]
			|| trimmedString.charAt(0) == whitespace[3]) {
		trimmedString = trimmedString.substring(1);
	}
	while (trimmedString.charAt(trimmedString.length - 1) == whitespace[0]
			|| trimmedString.charAt(trimmedString.length - 1) == whitespace[1]
			|| trimmedString.charAt(trimmedString.length - 1) == whitespace[2]
			|| trimmedString.charAt(trimmedString.length - 1) == whitespace[3]) {
		trimmedString = trimmedString.substring(0, trimmedString.length-1);
	}
	return trimmedString;
}


String.prototype.toCharArray = function() {
	var charArray = [];
	for (var n=0;n<this.length;n++)
		charArray.push( this.charAt(n) );
	return charArray;
}


String.prototype.compareTo = function( anotherString ) {

	var len1 = this.length;
	var len2 = anotherString.length;
	var n = Math.min(len1, len2);
	var v1 = this.toCharArray();
	var v2 = anotherString.toCharArray();
	var i = 0;
	var j = 0;

	if (i == j) {
	    var k = i;
	    var lim = n + i;
	    while (k < lim) {
			var c1 = v1[k].charCodeAt(0);
			var c2 = v2[k].charCodeAt(0);
			if (c1 != c2) {
			    return c1 - c2;
			}
			k++;
	    }
	} else {
	    while (n-- != 0) {
			var c1 = v1[i++].charCodeAt(0);
			var c2 = v2[j++].charCodeAt(0);
			if (c1 != c2) {
			    return c1 - c2;
			}
	    }
	}
	return len1 - len2;
}



function trimWhitespace(str) {	
	var whitespace = new Array(" ","\t","\n","\r");
	while (str.charAt(0) == whitespace[0]
			|| str.charAt(0) == whitespace[1]
			|| str.charAt(0) == whitespace[2]
			|| str.charAt(0) == whitespace[3]) {
		str = str.substring(1);
	}
	while (str.charAt(str.length - 1) == whitespace[0]
			|| str.charAt(str.length - 1) == whitespace[1]
			|| str.charAt(str.length - 1) == whitespace[2]
			|| str.charAt(str.length - 1) == whitespace[3]) {
		str = str.substring(0, str.length-1);
	}
	return str;
}
