/*
 * Copyright 2001 AlphaBlox Corporation. All rights reserved.
 */

/*
 * Context menu
 */
var CLASS_CHOICE		= "choice";
var CLASS_CHOICE_HOVER	= "choicehover";
var MENU_DELAY = 500;

var currentMenu = null;  // TODO : Get rid of these globals - how ?

function showMenu( target, menu, needToOffset, xPosition, yPosition) {

	if( commandPending )
		return;

	setSelected( target, true );

	_hideMenu( );

	// The mouse over may have occured on a descendent of the *true* target
	while( target.tagName != "TH" && target.tagName != "TD" ){
		target = target.parentNode;
	}

	target.appendChild(menu);

	menu.target = target;

	menu.mouseX = xPosition;
	menu.mouseY = yPosition;

	// Show menu after a delay
	menu.activateTime = new Date( );
	
	// Fudge factor to get around bug in bounds checking :-(
	menu.needToOffset = needToOffset;

	menu.report = getReport( target );
	currentMenu = menu;

	currentMenu.timeoutId = setTimeout( showMenuDelayed, MENU_DELAY );
}

function showMenuDelayed( ) {

	var menu = currentMenu;
	if( menu != null ) {
		var now = new Date( );

		var delay = now.getTime( ) - menu.activateTime.getTime( );
		
		if( delay >= MENU_DELAY ) {
			_setVisible( menu, true );

			var menuBounds = getBounds( menu );
			var targetBounds = getBounds( menu.target );
			var bodyBounds = getBounds( document.body );

			menuBounds.top = targetBounds.top + targetBounds.height;
			menuBounds.left = menu.mouseX +  document.body.scrollLeft;

			var bodyX;
			var bodyY;

			if(isIE())
			{
				bodyY =  bodyBounds.height + document.body.scrollTop;
				bodyX =  bodyBounds.width + document.body.scrollLeft;
			}
			else
			{
				bodyY =  bodyBounds.height;
				bodyX =  bodyBounds.width;
			}


			if(menu.needToOffset)
			{
				menuBounds.top = targetBounds.top + 1/2 * targetBounds.height;
 			        menuBounds.left = menu.mouseX +	document.body.scrollLeft;


				if( menuBounds.top + menuBounds.height > bodyY) 
				{
					menuBounds.top = menuBounds.top - menuBounds.height;
				}

				if( menuBounds.left + menuBounds.width > bodyX )
				{
					menuBounds.left = menuBounds.left - menuBounds.width;
				}
			}

			else
			{
				// Pin the menu to the screen if it goes outside
				if( menuBounds.top + menuBounds.height > bodyY ) {
				    menuBounds.top = targetBounds.top - menuBounds.height;
				}
				if( menuBounds.left + menuBounds.width > bodyX ) {
				    menuBounds.left = targetBounds.left - menuBounds.width;
				}    
			}

			setBounds( menu, menuBounds );
		}
		else {
			currentMenu = null;
		}
	}
}

function shouldHideMenu( event, item, menu ) {
	return	item != null 
		&&  ! isWithinBounds( menu, getXPos(event), getYPos(event));
}

function _hideMenu( ) {

	if( currentMenu != null ) {
		var report = getReport( currentMenu.target );

		clearTimeout( currentMenu.timeoutId );

		setSelected( currentMenu.target, false );				
		_setVisible( currentMenu, false );

		if(isIE())
		{
		   var cells = currentMenu.cells;

	 	   if(report.menuItemSelected != -1)
		   {
		      var prevItem = cells[report.menuItemSelected];
		      handleMenuDeSelect(prevItem);

		      report.menuItemSelected = -1;
		      report.mi = null;
		   }
		}

		report.parentNode.appendChild(currentMenu );
		currentMenu = null;
	}
}

function hideMenu( ) {
	if( currentMenu != null && isVisible(currentMenu)) {
		currentMenu.timeoutId = setTimeout( _hideMenu, MENU_DELAY )	}
	else {
		_hideMenu();
	}
}

function menu_mouseOver( event ) {
	var item = getEventSource(event);
	handleMenuSelect(item);
	var report = getReport( item );

	if(isIE())
	{
	    report.menuItemSelected = parseInt(item.getAttribute('menuItemIndex'));
	    report.mi = item;	
        }

	return false;
}

function handleMenuSelect(item)
{	
	var report = getReport( item );
	if( currentMenu != null) {
		clearTimeout( currentMenu.timeoutId );
	}

	switch ( item.className ) {
		case CLASS_CHOICE:
			highlightChoice( item );
			return true;
		default:
			return false;
	}
}

function menu_cleanup (event )
{
	var item = getEventSource(event);
		
	switch ( item.className ) {
		case CLASS_CHOICE_HOVER:
			_hideMenu( );
			releaseChoice( item );
			break;
		default:
			break;
	}

	return false;
}

function menu_mouseOut( event ) {

	var item = getEventSource(event);
	handleMenuDeSelect(item);
	var report = getReport(item);
	var menu = currentMenu;

	if( menu != null && shouldHideMenu( event, menu.target, menu ) ) {
		hideMenu( );
	}

	return false;
}

function handleMenuDeSelect(item)
{
	switch ( item.className ) {
		case CLASS_CHOICE_HOVER:
			releaseChoice( item );
			break;
		default:
			break;
	}
}

function menu_selectStart() {
	return false;
}

function highlightChoice( choice ) {
	choice.className = CLASS_CHOICE_HOVER;
}

function releaseChoice( choice ) {
	choice.className = CLASS_CHOICE;
}

function getMenuTarget( choice, event) {
	var target = _getParent( choice ).target;
	menu_cleanup( event );

	return target;
}

