/*
 * Copyright 2001 AlphaBlox Corporation. All rights reserved.
 */


/* css classes */
var CLASS_REPORT		= "report";
var CLASS_SEPARATOR		= "separator";
var CLASS_COLUMN		= "column";
var CLASS_GROUP			= "group";
var CLASS_GROUP_TOTAL	= "grouptotal";

/* element IDs */
var ID_DRAGGED_COLUMN 	= "draggedcolumn";
var ID_SELECTION_ELEMENT= "selectionElement";
var ID_EDITOR			= "editor";
var ID_RPC_FRAME		= "rpc";
var ID_MENU_COLUMN 		= "columnmenu";
var ID_MENU_GROUP 		= "groupmenu";
var ID_MENU_TOTAL 		= "totalmenu";

var UIEnabled = true;

function initializeEditor( reportBloxName ) {

	report = getElement(reportBloxName);

	report.columnMenu	= getElement( reportBloxName + ID_MENU_COLUMN );
	report.groupMenu	= getElement( reportBloxName + ID_MENU_GROUP );
	report.totalMenu	= getElement( reportBloxName + ID_MENU_TOTAL );
	report.editor		= getElement( reportBloxName + ID_EDITOR );
	report.draggedColumn= getElement( reportBloxName + ID_DRAGGED_COLUMN );
	report.selectionElement= getElement( reportBloxName + ID_SELECTION_ELEMENT );
        report.refresh = _refresh;

	//index of menu item that has been selected for the currently on focus
	//menu, the menu can be any of the above three
	report.menuItemSelected = -1;
	report.mi = null;

	if(isIE())
        {
		report.rpc = document.frames[ reportBloxName + ID_RPC_FRAME ];
		document.onkeydown = fnKeyDown;
        }
	else
	{
		report.rpc = getElement( reportBloxName + ID_RPC_FRAME );
		document.onkeydown = new Function("event", "fnKeyDownFF(event);");
	}

	//ensure that no menu/item shows up right after loading the page in
	//order to work around IBM HPR problem.
	_setVisible(report.columnMenu, false);
	_setVisible(report.groupMenu, false);
	_setVisible(report.totalMenu, false);
	_setVisible(report.draggedColumn, false);

}

function _refresh(){
    parent.refreshReport( this.id );
}

function isIE()
{
	var index = navigator.userAgent.indexOf( "MSIE" );
	if(index != -1)
	    return true;
	else
	    return false;
}

/*
 * Focus handlers
 */
 
function fnFocus ( event )
{
	var elementOnFocus = getEventSource(event);

	if( currentMenu != null && currentMenu.target == elementOnFocus)
	     return true;

	var offsetParent = elementOnFocus.offsetParent;
	var x = elementOnFocus.offsetLeft;
	var y = elementOnFocus.offsetTop;
	while (offsetParent) {
	      x += offsetParent.offsetLeft;
	      y += offsetParent.offsetTop;
	      offsetParent = offsetParent.offsetParent;
	}

	handleMouseOver(event, x, y);	
}

function fnKeyDownFF(event)
{
	if(currentMenu == null)
	    return true;

	if(currentMenu == null)
	    return true;

	var menu = currentMenu;

	var isHotKey = handleAcceleratorKey(event, menu, menu.target); 
        if(isHotKey)
	{
	    event.preventDefault();
	    return false;
        }
	else
	    return true;
}

function fnKeyDown(  )
{
	if(currentMenu == null)
	    return true;

	var report = getReport( currentMenu );
      
	var menu = currentMenu;
        var cells = menu.cells;
	var mi = null;

  	switch (event.keyCode) {
		case 38: //up arrow key
		     var startIndex = (report.menuItemSelected == -1)?
		     		cells.length -1 : report.menuItemSelected - 1;

		     if(report.menuItemSelected != -1)
		     {
		         var prevItem = cells[report.menuItemSelected];
			 handleMenuDeSelect(prevItem);
		     }	

		     if(startIndex == -1)
		         startIndex = cells.length -1;
		     for (var i= startIndex; i > -1; i-- ) {
			 var item = cells[i];
			 if(handleMenuSelect(item))
                         {
			     report.menuItemSelected = i;
			     report.mi = item;
			     event.cancelBubble = true;
			     return false;
                         }
                     }
		     break;

	        case 40: //down arrow key
		     var startIndex = (report.menuItemSelected == -1)? 
				    0 : report.menuItemSelected + 1;
		     if(report.menuItemSelected != -1)
		     {
		         var prevItem = cells[report.menuItemSelected];
			 handleMenuDeSelect(prevItem);
		     }	

		     if(startIndex == cells.length)
		         startIndex = 0;
		     for (var i= startIndex; i < cells.length; i++ ) {
			 var item = cells[i];
			 if(handleMenuSelect(item))
                         {
			     report.menuItemSelected = i;
			     report.mi = item;
			     event.cancelBubble = true;
			     return false;
                         }
                     }
		     break;

		case 13: //enter key
		     if(report.mi != null)
		     {
		         report.mi.fireEvent( "onclick" );
		         return false;
	             }
		     else
		         break;

		case 27: //esc key
		     _hideMenu();
		     break;

                default:
		     var isHotKey = handleAcceleratorKey(event, menu, menu.target); 
		     if(isHotKey)
		     {
		           event.cancelBubble = true;
			   return false;
                     }
		     else
		           break;
         }
	 return true;
}

/*
 * Mouse handlers
 */

function fnMouseOver( event ) {
    handleMouseOver(event, getXPos(event), getYPos(event));
}

function handleMouseOver(event, xPosition, yPosition)
{
	var item = getEventSource(event);

	var report = getReport( item );

	if(item.tagName == "TABLE")
	     return;

	if( UIEnabled && editor == null)  {
		switch( item.className ) {
			case CLASS_COLUMN :
			        showMenu( item, report.columnMenu, false, xPosition, yPosition );
				break;
		}

		if( isTotal( item ) ) {
			if( isParentTotal(item) ) {
				item = item.parentNode;
			}
			showMenu( item, report.totalMenu, true, xPosition, yPosition );
		}
		else if( isGroup( item ) ) {
			showMenu( item, report.groupMenu, true, xPosition, yPosition );
		}
	}
}

function fnMouseOut( event ) {
	var item = getEventSource(event);
	var report = getReport( item );

	if(item.tagName == "TABLE")
	     return;

	switch( item.className ) {
		case CLASS_COLUMN :
			hideMenu( );
			break;
	}

	if( isTotal( item ) ) {
		if( isParentTotal(item) ) {
			item = item.parentNode;
		}

		hideMenu( );
	}
	else if (isGroup( item ) ) {
		hideMenu( );
	}
}

function fnMouseDown( event ) {

	//disable right click mouse
	if(event.button > 1)
   	    return true;

	var item = getEventSource(event);
	var className = item.className;

	switch( className ) {
		case CLASS_COLUMN :
		        var report = getReport( item );
			if(currentMenu != null)
			    report.parentNode.appendChild(currentMenu);
			startDrag( event, item );
			break;
	}

	return true;
}


function isGroup( item ) {
	return item.className.indexOf( CLASS_GROUP ) == 0;
}

function isTotal( item ) {
	return item.className.indexOf( CLASS_GROUP_TOTAL ) == 0 || isParentTotal( item );
}

// because there could be an edit tag inside the TD
function isParentTotal( item ) {
	return item.parentNode.className.indexOf( CLASS_GROUP_TOTAL ) == 0;
}

/*
 * Edit text
 */

var editor;
function edit( item ) {
	 
	var report = getReport( item );

	editor = report.editor;

	setBounds( editor, getBounds( item ) );

	var content = item.innerHTML;
	item.content = content;

	if( content == "&nbsp;"){
		content ="";
	}

	var mask = item.mask;
	if (mask != null)
	    mask = decodeURIComponent(mask);
    else
        mask = "<member/>";

	editor.onchange = null;
	editor.value = mask;
	editor.targetItem = item;
	editor.onblur = editor_onChange;
	if(isIE())
	    editor.onkeypress = editor_onKeyPress;
	else
	    editor.onkeypress = new Function("event", "editor_onKeyPressFF(event);");

	setTimeout( editor_requestFocus, 0 );
}

function edit_ReaderMode( choice, event ) {
	 
	var item = _getParent( choice ).target;

	var report = getReport(item);

	editor = report.editor;

	setBounds( editor, getBounds( item ) );

	var content = item.innerHTML;
	item.content = content;

	if( content == "&nbsp;"){
		content ="";
	}

	var mask = item.mask;
	if (mask != null)
	    mask = decodeURIComponent(mask);
        else
            mask = "<member/>";

	editor.onchange = editor_onChange_ReaderMode;
	editor.value = mask;
	editor.targetItem = item;
	editor.src = choice;

	_setVisible( editor, true );
	choice.appendChild(editor);
}

function editor_requestFocus( ) {
	if(editor != null) {

		_setVisible( editor, true );
		editor.select( );
		editor.focus( );

		setUIEnabled(false);
	}
}

function editor_onKeyPress() {
	switch(window.event.keyCode) {
		case 13:
			editor_onChange( );
			break;

		case 27:
			cancelEdit();
			break;
	}
}

function editor_onKeyPressFF(event) {
	switch(event.which) {
		case 13:
			editor_onChange( );
			break;

		case 27:
			cancelEdit();
			break;
	}
}


function cancelEdit( ) {
	if( editor != null ) {
		_setVisible( editor, false );
		editor = null;

		setUIEnabled(true);
	}
}

function editor_onChange( ) {
	if( editor != null ) {
		_setVisible( editor, false );
		var item = editor.targetItem;
		var content = editor.value;
		editor.targetItem = null;
		editor = null;

		setUIEnabled(true);

		// If you edit a column header... and click off the editor... the following fn may not return!  ???
		storeText( item, content );
	}
}

function editor_onChange_ReaderMode( ) {
	if( editor != null ) {
		var report = getReport( editor.targetItem );

		report.parentNode.appendChild(editor);
		_setVisible( editor, false );

		releaseChoice( editor.src );
		_hideMenu( );

		var item = editor.targetItem;
		var content = editor.value;
		editor.targetItem = null;
		editor = null;

		setUIEnabled(true);

		// If you edit a column header... and click off the editor... the following fn may not return!  ???
		storeText( item, content );
	}
}

function setSelected( item, selected ) {
	if( selected ) {
		item.selected = true;

		// Copy the style from a dummy element on the pages
		var report = getReport( item );
		var selectionElement = report.selectionElement;

	    if(isIE()){

		item.runtimeStyle.backgroundColor = selectionElement.currentStyle.backgroundColor;
		item.runtimeStyle.color = selectionElement.currentStyle.color;
		item.runtimeStyle.borderColor = selectionElement.currentStyle.borderColor;
		item.runtimeStyle.cursor = selectionElement.currentStyle.cursor;
            }
	    else
	    {
		item.style.backgroundColor = document.defaultView.getComputedStyle(selectionElement, '').getPropertyValue("background-color");
		item.style.color = document.defaultView.getComputedStyle(selectionElement, '').getPropertyValue("color");
		item.style.cursor = document.defaultView.getComputedStyle(selectionElement, '').getPropertyValue("cursor");
		item.style.borderColor = document.defaultView.getComputedStyle(selectionElement, '').getPropertyValue("border-color");
	    }
	}
	else {
  	    item.selected = false;
            if(isIE())
		item.runtimeStyle.cssText = "";
	    else
		item.style.cssText = "";
	}
}

/*
 * Dragging columns
 */
function startDrag( event, column ) {
	var report = getReport( getEventSource(event) );
	if (!UIEnabled)
	        return;

	var report = getReport( getEventSource(event) );
	
	var draggedColumn = report.draggedColumn;

	draggedColumn.sourceColumn	= column.getAttribute("columnIndex");
	//draggedColumn.columns		= column.parentNode.children;
	draggedColumn.columns		= column.parentNode.childNodes;
	draggedColumn.innerHTML		= column.innerHTML;
	draggedColumn.report		= report;

	var bounds = getBounds( column );
	setBounds( draggedColumn, bounds );

	_setVisible( draggedColumn, true );
	draggedColumn.x = getXPos(event);

	if(!isIE())
		event.preventDefault();
}

function onDrag( event, draggedColumn ) {
	   _hideMenu( );
 	   var deltaX = getXPos(event) - draggedColumn.x;
 	   draggedColumn.style.left = parseInt(draggedColumn.style.left) + deltaX;
	   draggedColumn.x = getXPos(event);
}

function getDropTarget( event, draggedColumn ) {
	var columns = draggedColumn.columns;

	var x = getXPos(event) + document.body.scrollLeft;
	var targetIndex = -1;
	var childPos = -1;

	for( var i = 0; i < columns.length; i++ ) {
		var target = columns[ i ];

		//since the ie and firefox returns different length for the
		//collection, this filters out the non-text nodes for firefox
		switch( target.className ) {
			case CLASS_COLUMN :
			        childPos++;
				break;
		}

		if(withinTarget(x, target))
	        {
	            targetIndex = childPos;
	            break;
                }
	}

	return targetIndex;
}

function onDrop( event, draggedColumn ) {
	 var targetColumn = getDropTarget( event, draggedColumn );
	 cancelDrag( draggedColumn );
	 switchColumns( report, draggedColumn.sourceColumn, targetColumn );
}

function cancelDrag( draggedColumn ) {
	_setVisible( draggedColumn, false );
}

function isDragging( ) {
	return isVisible( getDraggedColumn( ) );
}

function getDraggedColumn( ) {
	return getElement( ID_DRAGGED_COLUMN );
}

//function getReport( ){
//	var reportId = SERVLET_PREFIX['bloxid'];

//	return getElement(reportId);
//}

function getReport( item ){
     if(isIE())
	return getAncestorWithClass( item, CLASS_REPORT );
     else
        return getAncestorWithClassFF( item, CLASS_REPORT );
}

function setUIEnabled( enabled ){
	UIEnabled = enabled;
}

function getEventSource( event )
{
	if(isIE())
	    return event.srcElement;
        else
	    return event.target;
}

