/*
 * Copyright 2001 AlphaBlox Corporation. All rights reserved.
 */

/*
 * Dialog for changing styles
 */
var ID_STYLES_DIALOG = "styles";
var ID_STYLES_DIALOG_TITLE = "stylestitle";

var ID_STYLES_DIALOG_OK_BUTTON = "okButton";
var ID_STYLES_DIALOG_APPLY_BUTTON = "applyButton";
var ID_STYLES_DIALOG_CANCEL_BUTTON = "cancelButton";

var ID_STYLES_DIALOG_TEXT_COLOR = "stylestextcolor";
var ID_STYLES_DIALOG_BACKGROUND_COLOR = "stylesbackgroundcolor";

var ID_STYLES_DIALOG_FONT_LIST = "stylesfont";
var ID_STYLES_DIALOG_FONT_SIZE = "stylessize";
var ID_STYLES_DIALOG_FONT_BOLD = "stylesbold";
var ID_STYLES_DIALOG_FONT_ITALIC = "stylesitalic";

var ID_STYLES_DIALOG_ALIGN_GROUP = "stylesaligngroup";

var ID_STYLES_DIALOG_SCOPE_COLUMN_HEADERS = "stylesscopecolumnheaders";
var ID_STYLES_DIALOG_SCOPE_DATA = "stylesscopedata";
var ID_STYLES_DIALOG_SCOPE_SELECTED_DATA = "stylesscopeselecteddata";
var ID_STYLES_DIALOG_SCOPE_SELECTED_COLUMN_HEADER = "stylesscopeselectedcolumnheader";
var ID_STYLES_DIALOG_SCOPE_GROUP_HEADER = "stylesscopegroupheader";
var ID_STYLES_DIALOG_SCOPE_GROUP_FOOTER = "stylesscopegroupfooter";
var ID_STYLES_DIALOG_SCOPE_GROUP_TOTAL = "stylesscopegrouptotal";

var KEY_ENTER 	= 13;
var KEY_ESCAPE 	= 27;

var currentDialog = null;
var dragging = null;

function showStyleDialog( target ) {

	var report = getReport( target );
	UIEnabled = false;

	var ID_PREFIX = report.id;
	var dialog = getElement( ID_PREFIX + ID_STYLES_DIALOG );
	dialog.target = target;
	dialog.report = report;

	// change the scope box depending on what the target is
	dialog.scopeheading			= getElement(ID_PREFIX + ID_STYLES_DIALOG_SCOPE_COLUMN_HEADERS);
	dialog.scopedata			= getElement(ID_PREFIX + ID_STYLES_DIALOG_SCOPE_DATA);
	dialog.scopeselecteddata	= getElement(ID_PREFIX + ID_STYLES_DIALOG_SCOPE_SELECTED_DATA);
	dialog.scopeselectedcolumnheader	= getElement(ID_PREFIX + ID_STYLES_DIALOG_SCOPE_SELECTED_COLUMN_HEADER);
	dialog.scopegroupheader		= getElement(ID_PREFIX + ID_STYLES_DIALOG_SCOPE_GROUP_HEADER);
//	dialog.scopegroupfooter		= getElement(ID_PREFIX + ID_STYLES_DIALOG_SCOPE_GROUP_FOOTER);
	dialog.scopegrouptotal		= getElement(ID_PREFIX + ID_STYLES_DIALOG_SCOPE_GROUP_TOTAL);

	dialog.textColor			= getElement(ID_PREFIX + ID_STYLES_DIALOG_TEXT_COLOR);
	dialog.backgroundColor		= getElement(ID_PREFIX + ID_STYLES_DIALOG_BACKGROUND_COLOR);
	dialog.fontList				= getElement(ID_PREFIX + ID_STYLES_DIALOG_FONT_LIST);
	dialog.fontSize				= getElement(ID_PREFIX + ID_STYLES_DIALOG_FONT_SIZE);
	dialog.bold					= getElement(ID_PREFIX + ID_STYLES_DIALOG_FONT_BOLD);
	dialog.italic				= getElement(ID_PREFIX + ID_STYLES_DIALOG_FONT_ITALIC);

        dialog.alignGroup	= document.getElementsByName(ID_PREFIX + ID_STYLES_DIALOG_ALIGN_GROUP);	

	var isGroup = dialog.target.getAttribute("levelIndex") != null;

	dialog.scopeheading.checked 	= false;
	dialog.scopedata.checked 		= ! isGroup;
	dialog.scopeselecteddata.checked 	= false;
	dialog.scopeselectedcolumnheader.checked 	= false;

	dialog.scopegroupheader.checked = isGroup && ! isTotal(target);
//	dialog.scopegroupfooter.checked = isGroup;
	dialog.scopegrouptotal.checked 	= isGroup && isTotal(target);

	dialog.scopeselecteddata.disabled 	= isGroup;
	dialog.scopeselectedcolumnheader.disabled 	= isGroup;

	dialog.scopegroupheader.disabled 	= ! isGroup;
//	dialog.scopegroupfooter.disabled 	= ! isGroup;
	dialog.scopegrouptotal.disabled 	= ! isGroup;

	var titleBar = getElement( ID_PREFIX + ID_STYLES_DIALOG_TITLE );

	if(isIE())
		document.onmousemove 	= stylesDialog_onDrag;
	else
		document.onmousemove 	= new Function("event", "stylesDialog_onDragFF(event);");

	if(isIE())
		document.body.onscroll 	= stylesDialog_onScroll;
	else
		window.onscroll = stylesDialog_onScrollFF;

	if(isIE())
	{
		dialog.scrollX = document.body.scrollLeft;
		dialog.scrollY = document.body.scrollTop;
	}
	else
	{
		dialog.scrollX = window.scrollX;
		dialog.scrollY = window.scrollY;
	}

	populateDialogStyle(target, dialog);

	_setVisible( dialog, true );
	
	// Center on page
	centerOnPage( dialog );

	var fontList = getElement( ID_PREFIX + ID_STYLES_DIALOG_FONT_LIST );

	currentDialog = dialog;
	
	if(isIE())
		dialog.focus();

	setTimeout( selectTargetDelayed, 0 );
}

function populateDialogStyle(target, dialog)
{
	if(target.style.color != '')
	{	   
	  if( target.style.fontWeight != 'normal')
	      dialog.bold.checked = true;
	  else
	      dialog.bold.checked = false;

	  if( target.style.fontStyle != 'normal')
	      dialog.italic.checked = true;
	  else
	      dialog.italic.checked = false;

          setStyleSelected(dialog.textColor, target.style.color);

          setStyleSelected(dialog.backgroundColor, target.style.backgroundColor);

	  fontString = target.style.fontSize;
	  var pos = fontString.indexOf('px');
	  var fontSize = fontString.substring(0, pos);
	  setStyleSelected(dialog.fontSize, fontSize);


	  var fontString = target.style.fontFamily;
	  var pos = fontString.indexOf(',');
	  var fontName = fontString.substring(0, pos);
	  setStyleSelected(dialog.fontList, fontName);

	  var textAlign = target.style.textAlign;

	  if(textAlign == "left")
	      dialog.alignGroup[0].checked = true;
	  else if(textAlign == "center")
	      dialog.alignGroup[1].checked = true;
	  else
	      dialog.alignGroup[2].checked = true;
        }
	else //return the default
	{
	    dialog.bold.checked = false;
	    dialog.italic.checked = false;
	    dialog.textColor.selectedIndex = 0; //the default
	    dialog.backgroundColor.selectedIndex = 1; //the default
	    dialog.fontSize.selectedIndex = 6; //the default
	    dialog.fontList.selectedIndex = 9; //the default
	    dialog.alignGroup[0].checked = true;
	}
}

function setStyleSelected( selectList, value)
{
	var size = selectList.length;

	for(var i = 0; i < size; i++)
	{
	    if(selectList.options[i].text == value)
	    {
	       selectList.selectedIndex = i;
	       return;
	    }
	}
}

function selectTargetDelayed(){
	setSelected( currentDialog.target, true );
}

function stylesDialog_onKeypress() {
	if(window.event.keyCode == KEY_ENTER) {
	        id_prefix = currentDialog.report.id;
		if(event.srcElement == getElement( id_prefix + ID_STYLES_DIALOG_CANCEL_BUTTON ))
		    cancel_onClick( );
		else if(event.srcElement == getElement( id_prefix + ID_STYLES_DIALOG_APPLY_BUTTON ))
		    apply_onClick( ); 
		else
		    ok_onClick( );
	}
	if(window.event.keyCode == KEY_ESCAPE) {
		cancel_onClick( );
	}
}

function cancel_onClick( ) {
	hideStylesDialog( currentDialog );
	currentDialog = null;
	setUIEnabled(true);
}

function ok_onClick( ) {
        setUIEnabled(true);
	hideStylesDialog( currentDialog );
	apply( );
	currentDialog = null;
}

function apply_onClick( ) {
	apply( );
}

function apply( ) {
	var dialog = currentDialog;
	var column = dialog.target.getAttribute("columnIndex");
	var level = dialog.target.getAttribute("levelIndex");

	var style = "";
	var scope = "";

	style += createStyle("color", getSelectedColorOption( dialog.textColor ) );
	style += createStyle("background-color", getSelectedColorOption( dialog.backgroundColor ) );
	style += createStyle( "font-family", getSelectedOption( dialog.fontList	) + ",sans-serif" );
	style += createStyle( "font-size", getSelectedOption( dialog.fontSize ) );
	style += createStyle( "font-weight", dialog.bold.checked ? "bold" : "normal" );
	style += createStyle( "font-style", dialog.italic.checked ? "italic" : "normal" );

	var alignOptions = [ "left", "center", "right" ];
	style += createStyle( "text-align", alignOptions[ getRadioSelection( dialog.alignGroup ) ] );

	if( dialog.scopeheading.checked ) {
		scope += "header;";
	}
	if( dialog.scopedata.checked ) {
		scope += "data;"
		;
	}
	if( dialog.scopeselecteddata.checked ) {
		scope += "column" + column + ";"
		;
	}
	if( dialog.scopeselectedcolumnheader.checked ) {
		scope += "header" + column + ";"
		;
	}
	if( dialog.scopegroupheader.checked ) {
		scope += "groupheader" + level + ";"
		;
	}
//	if( dialog.scopegroupfooter.checked ) {
//		scope += "groupfooter" + level + ";"
//		;
//	}
	if( dialog.scopegrouptotal.checked ) {
		scope += "grouptotal" + level + ";"
		;
	}

	applyStyles( dialog.report, scope, style );
}

function createStyle(  name, value ) {
	return name + ":" + value + ";";
}

function getSelectedOption( selectList ) {
	return selectList.options[ selectList.selectedIndex ].text;
}

function getSelectedColorOption( selectList ) {
    switch(selectList.selectedIndex) {
        case 0:
            return "black";
        case 1:
            return "white";
        case 2:
            return "red";
        case 3:
            return "green";
        case 4:
            return "blue";
        case 5:
            return "magenta";
        case 6:
            return "cyan";
        case 7:
            return "yellow";
        case 8:
            return "gray";
        default:
            return "white";
    }
}

function getRadioSelection( radio ) {
	for( var i = 0; i < radio.length; i++ ) {
		if( radio[i].checked ) {
			return i;
		}
	}
	return -1;
}

function hideStylesDialog( dialog ) {
	setSelected( dialog.target, false );
	_setVisible( dialog, false );
}

function stylesDialog_onScroll( ) {
	if(dragging) {
		return false;
	}

	if( currentDialog != null) {
		var dialog = currentDialog;
		var bounds = getBounds( dialog );

		bounds.left += document.body.scrollLeft - dialog.scrollX;
		bounds.top += document.body.scrollTop - dialog.scrollY;

		dialog.scrollX = document.body.scrollLeft;
		dialog.scrollY = document.body.scrollTop;

		setBounds( dialog, bounds  );
	}
}

function stylesDialog_onScrollFF( ) {
	if(dragging) {
		return false;
	}

	if( currentDialog != null) {
	    window.status = "scroll: " + document.body.scrollLeft;
		var dialog = currentDialog;

		var deltaX = window.scrollX - dialog.scrollX;
		var deltaY = window.scrollY - dialog.scrollY;

		dialog.style.left = parseInt(dialog.style.left) + deltaX;
		dialog.style.top = parseInt(dialog.style.top) + deltaY;

		dialog.scrollX = window.scrollX;
		dialog.scrollY = window.scrollY;
	}
}

function stylesDialog_startDrag(event) {

	var dialog = currentDialog;
	dialog.x = getXPos(event);
	dialog.y = getYPos(event);

	dialog.dragging = true;
	dragging = true;

	return false;
}

function stylesDialog_onDrag() {
	var dialog = currentDialog;

	if( dialog != null && dialog.dragging && dragging ) {
		dialog.style.pixelLeft += getXPos(event) - dialog.x;
		dialog.style.pixelTop += getYPos(event) - dialog.y;

		dialog.x = getXPos(event);
		dialog.y = getYPos(event);
		return false;
	}
}

function stylesDialog_onDragFF(event) {

	var dialog = currentDialog;

	if( dialog != null && dialog.dragging && dragging ) {

		var deltaX = getXPos(event) - dialog.x;
		var deltaY = getYPos(event) - dialog.y;

		dialog.style.left = parseInt(dialog.style.left) + deltaX;
		dialog.style.top =  parseInt(dialog.style.top) + deltaY;

		dialog.x = getXPos(event);
		dialog.y = getYPos(event);

		return false;
	}
}

function stylesDialog_stopDrag(event) {
	var dialog = currentDialog;
	dialog.dragging = false;
	dragging = false;

	return false;
}
