package com.ibm.alphablox.tooling.querytools;
/**
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: hujianh
 * Date: Jul 25, 2007
 */
import com.alphablox.blox.DataBlox;
import com.alphablox.blox.DataBloxCannotConnectException;
import com.alphablox.blox.PresentBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.ServerBloxMissingResourceException;
import com.alphablox.blox.ViewBlox;
import com.alphablox.blox.uimodel.BloxModel;
import com.alphablox.blox.uimodel.core.ComboBox;
import com.alphablox.blox.uimodel.core.Controller;
import com.alphablox.blox.uimodel.core.event.ModelEvent;
/**
 * used to response to the two buttons on the queryDesigner.jsp
 * 
 * 1.setPresentBlox
 * 2.setQueryDesignerBean
 * 2.setContainer
 * 
 * @author hujianh@cn.ibm.com
 *
 */
public class DesignerController extends Controller{
    private static final boolean DEBUG = false;
    private PresentBlox presentBlox = null;
    private String lastCubeName="";
    private String defaultQuery = null;
    private BloxModel model = null;
    private ComboBox combo = null;
    private QueryDesignerBean queryDesignerBean;
    private boolean initialized = false;
    
    public DesignerController(){
    }
    
    public void setQueryDesignerBean(QueryDesignerBean bean){
        queryDesignerBean = bean;
    }
    
    public void actionRunDefaultQuery(ModelEvent event) throws Exception {
        f("actionRunDefaultQuery:"+this);
        if(!initialized){
            throw new RuntimeException("DesignerController is not initialized yet!");
        }
        try{
            DataBlox db = presentBlox.getDataBlox();
  
            queryDesignerBean.connect(false);
            if(queryDesignerBean.isMDXDataSource()){
                getMDXDefaultQuery();
            }else{
                defaultQuery = queryDesignerBean.getDefaultQuery();
            }
            db.setQuery(defaultQuery);
            queryDesignerBean.connect(true);
        }catch(Exception ex){
            System.out.println("QueryToolsDesignerController:actionRunQuery error!");
            ex.printStackTrace();
        }
        f("run default query is clicked!:"+defaultQuery);
    }

    /**
     * 
     * @throws Exception
     * @throws ServerBloxMissingResourceException
     * @throws ServerBloxException
     */
    private void getMDXDefaultQuery() throws Exception, ServerBloxMissingResourceException, ServerBloxException {
        int idx = combo.getSelected();
        lastCubeName = combo.get(idx);
        defaultQuery = queryDesignerBean.getDefaultQuery(presentBlox.getDataBlox(), lastCubeName);
    }

    /**
     * @return the presentBlox
     */
    public PresentBlox getPresentBlox() {
        return presentBlox;
    }

    /**
     * @param presentBlox the presentBlox to set
     */
    public void setPresentBlox(ViewBlox presentBlox) {
        if(presentBlox instanceof PresentBlox ){
            this.presentBlox = (PresentBlox)presentBlox;
        }else
            f("Wrong type of the argument:"+presentBlox);
    }

    /**
     * @return the queryDesignerBean
     */
    public QueryDesignerBean getQueryDesignerBean() {
        return queryDesignerBean;
    }

    /**
     * @return the model
     */
    public BloxModel getModel() {
        return model;
    }

    /**
     * @param model the model to set
     */
    public void setModel(BloxModel model) {
        this.model = model;
        this.combo = model.getComboBox("cubes");
        combo.clear();
        boolean showCombo = queryDesignerBean.isMDXDataSource();
        String[] cubes = queryDesignerBean.getCubeList();
        
        f("setModel, cubelength="+cubes.length);
        
        for(int i=0;i<cubes.length;i++){
            combo.add(cubes[i]);
        }
        if( cubes.length>0){
            combo.setSelected(0);
        }else{
            f("No cubes!!!");
        }
        try{
            combo.setVisible(showCombo);
            model.getStatic("CubeStatic").setVisible(showCombo);
            model.changed();
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }
    
    public void initialize(ViewBlox presentBlox, QueryDesignerBean bean, BloxModel model)throws Exception {
        f("initialize:"+this);
        try{
            setPresentBlox(presentBlox);
            DataBlox db = getPresentBlox().getDataBlox();
            bean.setDataBlox(db);
            
            bean.setMDispatcher(model.getDispatcher());
            this.initialized = bean.connect(true);
            
            setQueryDesignerBean(bean);
            setModel(model);
            model.setController(this);
            initialized  = true; 
            f("initialized  = true");
        }catch(DataBloxCannotConnectException  ex){
            f("Cannot connect to the database server!!!");
            ex.printStackTrace();
        }catch(ServerBloxException  ex){
            f("Cannot initialize the data blox peer!");
            ex.printStackTrace();
        }catch(Exception ex){
            f("cannot initialize it for unknow exception!");
            ex.printStackTrace();
        }
    }
    private void f(String s){
        if(DEBUG){
            System.out.println(s);
        }
    }
}
