package com.ibm.alphablox.tooling.querytools;

import com.alphablox.blox.DataBlox;
import com.alphablox.blox.DataBloxCannotConnectException;
import com.alphablox.blox.MemberFilterBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.ViewBlox;
import com.alphablox.blox.data.mdb.Member;
import com.alphablox.blox.event.MemberSelectEvent;
import com.alphablox.blox.event.MemberSelectListener;
import com.alphablox.blox.uimodel.BloxModel;
import com.alphablox.blox.uimodel.core.ComboBox;
import com.alphablox.blox.uimodel.core.event.IEventHandler;
import com.alphablox.blox.uimodel.core.event.SelectionChangedEvent;
import com.ibm.alphablox.bloxbuilder.tooling.querytools.QueryToolsBlox;

/**
 * setup the cube list
 * get the cube the user select;
 * generat a default query for the cube;
 * set the default query to the datablox;
 * 
 * The user may change the cube he selected during usage.
 * @author hujianh@cn.ibm.com
 *
 */
public class MetadataBrowserHandler implements IEventHandler, MemberSelectListener{
    private static final boolean DEBUG = false;
    private MemberFilterBlox memberFilterBlox;
    private boolean initialized;
    private QueryDesignerBean queryDesignerBean;
    private BloxModel model;
    private ComboBox combo;
    private QueryToolsBlox qtBlox;
    
    public MetadataBrowserHandler(QueryToolsBlox qtBlox)throws Exception{
        this.qtBlox = qtBlox;
    }
    
    public boolean initialize(ViewBlox mfBlox, QueryDesignerBean bean, BloxModel model)throws Exception {
        f("initialize:"+this);
        try{
            this.memberFilterBlox = (MemberFilterBlox)mfBlox;
            
            DataBlox db = memberFilterBlox.getDataBlox();
            
            bean.setDataBlox(db);
            bean.setMDispatcher(model.getDispatcher());
            try{
                if(db.isConnected()){
                    db.disconnect(true);
                }
                bean.connect(false);
                f("bean.connect(); OK!");
            }catch(DataBloxCannotConnectException ex){
                throw ex;
            }catch(Exception ex){
                f("db.connect(); get other exceptions!");
            }
            
            this.queryDesignerBean = bean;
            
            setModel(model);
            //change cubes
            model.addEventHandlerToEnd(this);

            db.addEventListener(this);
            
            String query = "";
            try{
                query = bean.getDefaultQuery();
                db.setQuery(query);
                bean.connect(true);
            }catch(Exception ex){
                f("BadQuery:"+query);
                throw ex;
            }
            initialized  = true; 
            f("initialized  = true");
        }catch(DataBloxCannotConnectException  ex){
            f("Cannot connect to the database server!!!");
            initialized  = false; 
            ex.printStackTrace();
        }catch(ServerBloxException  ex){
            f("Cannot initialize the data blox peer!");
            initialized  = false; 
            ex.printStackTrace();
        }catch(Exception ex){
            f("cannot initialize it for unknow exception!");
            initialized  = false; 
            ex.printStackTrace();
        }
        return initialized;
    }
    
    /**
     * Set model, setup the combo box!
     * @param model
     */
    public void setModel(BloxModel model) {
        this.model = model;
        this.combo = model.getComboBox("cubes");
        combo.clear();
        boolean showCombo = queryDesignerBean.isMDXDataSource();
        String[] cubes = queryDesignerBean.getCubeList();
        
        f("setModel, cubelength="+cubes.length);
        
        for(int i=0;i<cubes.length;i++){
            combo.add(cubes[i]);
        }
        if( cubes.length>0){
            combo.setSelected(0);
        }else{
            f("No cubes!!!");
        }
        try{
            combo.setVisible(showCombo);
            model.getStatic("CubeStatic").setVisible(showCombo);
            model.changed();
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }

    private void f(String s){
        if(DEBUG){
            System.out.println(s);
        }
    }
    
    public boolean handleSelectionChangedEvent(SelectionChangedEvent event) {
        if(!initialized){
            f("Error, not initialized!!!");
            return false;
        }
        f("selection changed:"+this);
        Integer[] selections=event.getSelection();
        if( selections.length==0){
            f("No selection!!!");
        }else{
            String cubeName = combo.get(selections[0].intValue());
            try{
                String str = queryDesignerBean.getDefaultQuery(null, cubeName);
                DataBlox dataBlox = this.memberFilterBlox.getDataBlox();
                dataBlox.setQuery(str);
                
                queryDesignerBean.connect(true);
                memberFilterBlox.getBloxModel().changed();
            }catch(Exception ex){
                f("Error when connect to the Server!!");
                ex.printStackTrace();
            }
        }
       
        return false;
    }

    /* (non-Javadoc)
     * @see com.alphablox.blox.event.MemberSelectListener#memberSelect(com.alphablox.blox.event.MemberSelectEvent)
     */
    public void memberSelect(MemberSelectEvent event) throws Exception {
        Member[] members = event.getNewMemberSelections();
        StringBuffer sb = new StringBuffer(256);
        for(int i=0;i<members.length;i++){
            if(i!=0){
                sb.append(",");
            }
            sb.append(members[i].getUniqueName());
        }
        qtBlox.setMemberSelected(sb.toString());
        String selectedDimension = event.getDimension().getDisplayName();
        this.memberFilterBlox.setSelectedDimension(selectedDimension);
        f("member select:"+sb.toString()+"; dimension="+selectedDimension);
    }
    
}
