package com.ibm.alphablox.tooling.relational;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.alphablox.blox.BloxContext;
import com.alphablox.blox.DataBlox;
import com.alphablox.blox.uimodel.core.Component;
import com.alphablox.blox.uimodel.core.ComponentContainer;
import com.alphablox.blox.uimodel.core.Controller;
import com.alphablox.blox.uimodel.core.RadioButton;
import com.alphablox.blox.uimodel.core.event.ClickEvent;
import com.alphablox.blox.uimodel.core.tree.TreeNode;
import com.ibm.alphablox.bloxbuilder.tooling.querytools.QueryToolsBlox;
import com.ibm.alphablox.tooling.relational.util.SQLFactory;

/**
 * This is the info-center for saving user configuration.
 * @author hujianh@cn.ibm.com
 *
 */
public class OperationContext extends Controller{
    private BloxContext context;

    private int nameLevel = RMBConstants.SCHEMA_NODE_LEVEL;
    private String encloseChars ="  "; //$NON-NLS-1$
    private boolean useTableAlias = false;
    
    private final SQLFactory sqlFactory;
    
    private PageController pageController;
    
    private final QueryToolsBlox qtBlox;
    
    private static final Map insertMap = new HashMap(11);
    private static final Map encloseMap = new HashMap(11);
    static{
        insertMap.put(RMBConstants.INSERT_SCHEMA, new Integer(RMBConstants.SCHEMA_NODE_LEVEL));
        insertMap.put(RMBConstants.INSERT_TABLE, new Integer(RMBConstants.TABLE_NODE_LEVEL));
        insertMap.put(RMBConstants.INSERT_COL, new Integer(RMBConstants.COLUMN_NODE_LEVEL));
        
        encloseMap.put(RMBConstants.ENCLOSE_NONE, "  "); //$NON-NLS-1$
        encloseMap.put(RMBConstants.ENCLOSE_DOUBLE_QUOTE, "\"\"");
        encloseMap.put(RMBConstants.ENCLOSE_SINGLE_QUOTE, "''"); //$NON-NLS-1$
        encloseMap.put(RMBConstants.ENCLOSE_BRACKET, "[]"); //$NON-NLS-1$
    };
    
    public OperationContext(BloxContext context, DataBlox db, QueryToolsBlox qtBlox){
        this.context = context;
        this.sqlFactory = SQLFactory.getInstance(db);
        this.qtBlox = qtBlox;
    }
    
    public void setTarget(ComponentContainer container) throws Exception 
    {
        container.setController(this);
        List list = container.searchForAllComponents(RadioButton.class);
        for(Iterator iter = list.iterator();iter.hasNext();){
            RadioButton rb = (RadioButton)iter.next();
            if(rb.isChecked()){
                setSettings(rb);
            }
        }
    }
    
    public boolean handleClickEvent(ClickEvent e){
        Component src = e.getComponent();
        if( src instanceof RadioButton ){
            setSettings(src);
        }
        return false;
    }

    /**
     * @param src
     */
    private void setSettings(Component src) {
        String gName = ((RadioButton)src).getGroupName();
        if( gName.equalsIgnoreCase(RMBConstants.GROUP_NAME_INSERT_POLICY)){
            nameLevel = ((Integer)insertMap.get(src.getName())).intValue();
        }else if(gName.equalsIgnoreCase(RMBConstants.GROUP_NAME_ENCLOSE_STYLE)){
            encloseChars  = encloseMap.get(src.getName()).toString();
        }else if(gName.equalsIgnoreCase(RMBConstants.GROUP_TABLE_ALIAS)){
            useTableAlias = (src.getName().equalsIgnoreCase(RMBConstants.RADIO_USE_ALIAS));
        }
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        StringBuffer sb = new StringBuffer(128);
        sb.append("nameLevel=").append(nameLevel);
        sb.append(";encloseChars=").append(encloseChars);
        sb.append(";useAlias=").append(useTableAlias);
        sb.append("@").append(super.toString());
        return sb.toString();
    }

    /**
     * @return the context
     */
    public BloxContext getContext() {
        return context;
    }

    /**
     * @param context the context to set
     */
    public void setContext(BloxContext context) {
        this.context = context;
    }

    /**
     * @return the encloseChars
     */
    public String getEncloseChars() {
        return encloseChars;
    }

    /**
     * @param encloseChars the encloseChars to set
     */
    public void setEncloseChars(String encloseChars) {
        this.encloseChars = encloseChars;
    }

    /**
     * @return the nameLevel
     */
    public int getNameLevel() {
        return nameLevel;
    }

    /**
     * @param nameLevel the nameLevel to set
     */
    public void setNameLevel(int nameLevel) {
        this.nameLevel = nameLevel;
    }

    public SQLFactory getSqlFactory() {
        return this.sqlFactory;
    }

    /**
     * @return the pageController
     */
    public PageController getPageController() {
        return pageController;
    }

    /**
     * @param pageController the pageController to set
     */
    public void setPageController(PageController pageController) {
        this.pageController = pageController;
    }

    /**
     * @return the useTableAlias
     */
    public boolean isUseTableAlias() {
        return useTableAlias;
    }

    /**
     * @param useTableAlias the useTableAlias to set
     */
    public void setUseTableAlias(boolean useTableAlias) {
        this.useTableAlias = useTableAlias;
    }

    private String encloseItemName(String itemName){
        String encloseStr = getEncloseChars();
        
        if(encloseStr.trim().length()==2){
            return encloseStr.charAt(0)+itemName+encloseStr.charAt(1);
        }
        
        return itemName;
    }

    public void getColumnNodeAsStr(TreeNode tn, StringBuffer sb) {
        if( tn.getNodeLevel()!=RMBConstants.COLUMN_NODE_LEVEL){
            return;
        }
        switch(getNameLevel()){
        case RMBConstants.SCHEMA_NODE_LEVEL:
            sb.append(tn.getParentNode().getParentNode().getName());
            sb.append("."); //$NON-NLS-1$
        case RMBConstants.TABLE_NODE_LEVEL:
            sb.append(
                    encloseItemName(
                            tn.getParentNode().getName()
                            )
                     );
            sb.append("."); //$NON-NLS-1$
        case RMBConstants.COLUMN_NODE_LEVEL:
            sb.append(encloseItemName(tn.getName()));
        }
        
    }

    public void getTableNodeAsStr(TreeNode tn, StringBuffer sb) {
        if(tn.getNodeLevel()!=RMBConstants.TABLE_NODE_LEVEL){
            return;
        }
        String tblName = encloseItemName(tn.getName());
        sb.append(tn.getParentNode().getName())
            .append(".")
            .append(tblName); //$NON-NLS-1$
        if( isUseTableAlias() ){
            sb.append(" "+tblName); //$NON-NLS-1$
        }
    }

    /**
     * @return the qtBlox
     */
    public QueryToolsBlox getQtBlox() {
        return qtBlox;
    }
}
