package com.ibm.alphablox.tooling.relational;

import com.alphablox.blox.BloxContext;
import com.alphablox.blox.ContainerBlox;
import com.alphablox.blox.DataBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.repository.Log;
import com.alphablox.blox.uimodel.BloxModel;
import com.alphablox.blox.uimodel.core.ComponentContainer;
import com.alphablox.blox.uimodel.core.HorizontalLayout;
import com.alphablox.blox.uimodel.core.ModelException;
import com.alphablox.blox.uimodel.core.Static;
import com.alphablox.blox.uimodel.core.Style;
import com.alphablox.blox.uimodel.core.VerticalLayout;
import com.ibm.alphablox.bloxbuilder.tooling.querytools.QueryToolsBlox;
import com.ibm.alphablox.tooling.AppContext;
import com.ibm.alphablox.tooling.relational.view.IViewPartBuilder;
import com.ibm.alphablox.tooling.relational.view.ViewPartBuilderFactory;

/**
 * Construct the page frame work, store the reference to each component.
 * @author hujianh@cn.ibm.com
 *
 */
public class PageIntegrattor {
    public static final String INVISIBLE_BORDER = "padding:1; border: 0; border-style: solid;"; //$NON-NLS-1$
    public static final String VISIBLE_BORDER = "padding:1; border: 1; border-color:gray; border-style: solid;"; //$NON-NLS-1$
    private static int ScreenHeight = 600;//650
    private static int ScreenWidth = 790;//850
    
    private DataBlox dataBlox;

    //important
    private BloxContext context;
    private OperationContext opContext;
    //create
    private PageController pageController;

    private IViewPartBuilder treeContainerBuilder;
    private BloxModel topContainer;
    private IViewPartBuilder opContainerBuilder;
    private IViewPartBuilder sqlEditorBuilder;
    private ComponentContainer dsInfoContainer;
    private IViewPartBuilder radioConBuilder;
   
    public PageIntegrattor(){
    }
    /**
     * Add all the components into the container. 
     * Before this method is called, the appContext, dataBlox, bloxContext must be set 
     * @param container
     * @param appContext
     * @throws Exception
     */
    public void initContainer(ContainerBlox container, AppContext appContext) throws Exception{      
        topContainer = container.getBloxModel();
        topContainer.clear();

        topContainer.setLayout(new VerticalLayout());
        
        ComponentContainer dsInfoContainer = buildDsInfo((ScreenHeight/40), ScreenWidth, INVISIBLE_BORDER);
        ComponentContainer mainContainer = buildMain((ScreenHeight*25/40)+5, ScreenWidth);
        ComponentContainer itemInfoContainer = ViewPartBuilderFactory.getDBItemInfoContainerBuilder()
            .build((ScreenHeight/10+5), ScreenWidth, INVISIBLE_BORDER);
        
        topContainer.add(dsInfoContainer);
        
        mainContainer.setStyle(VISIBLE_BORDER);
        topContainer.add(mainContainer);
        
        topContainer.add(new Static(Messages.getString("PageIntegrattor.DBItemInfo"))); //$NON-NLS-1$
        topContainer.add(itemInfoContainer);     
        
        topContainer.setScrollingEnabled(false);
    }

    private ComponentContainer buildMain(int h,int w) throws Exception{
        ComponentContainer mainContainer = new ComponentContainer();
        mainContainer.setLayout(new VerticalLayout());
        mainContainer.setHeight(h);
        mainContainer.setWidth(w);
        
        ComponentContainer buildContainer = new ComponentContainer("ControllContainer"); //$NON-NLS-1$
        buildContainer.setLayout(new HorizontalLayout());
        
        treeContainerBuilder = ViewPartBuilderFactory.getTreeContainerBuilder();
        ComponentContainer treeContainer = treeContainerBuilder.build( h-5, (w*3/10), INVISIBLE_BORDER);
        
        opContainerBuilder = ViewPartBuilderFactory.getOperatorContainerBuilder();
        ComponentContainer opContainer = opContainerBuilder.build( h-5, (w*3/20)-10, INVISIBLE_BORDER);
        
        sqlEditorBuilder = ViewPartBuilderFactory.getSQLEditorContainerBuilder();
        ComponentContainer sqlContainer = sqlEditorBuilder.build( h-5, w*11/20+10, INVISIBLE_BORDER);
        
        radioConBuilder = ViewPartBuilderFactory.getRadioContainerBuilder(); 
        ComponentContainer radioContainer = radioConBuilder.build(5, w, INVISIBLE_BORDER);
        
        buildContainer.add(treeContainer);
        buildContainer.add(opContainer);
        buildContainer.add(sqlContainer);
        
        mainContainer.add(buildContainer);
        mainContainer.add(radioContainer);
        return mainContainer;
    }

    private ComponentContainer buildDsInfo(int h,int w, String style) throws Exception{
        dsInfoContainer = new ComponentContainer("DSInfoContainer"); //$NON-NLS-1$
        dsInfoContainer.setHeight(h);
        dsInfoContainer.setWidth(w);
        dsInfoContainer.setStyle(style);
        
        Static st = new Static("DSInfoStatic",""); //$NON-NLS-1$ //$NON-NLS-2$
        dsInfoContainer.add(st);

        return dsInfoContainer;
    }
//chagne every time
    private void changeDSInfo() throws ServerBloxException, ModelException {
        Static st2 = dsInfoContainer.getStatic("DSInfoStatic"); //$NON-NLS-1$
        StringBuffer sb = new StringBuffer(128);
        sb.append(Messages.getString("PageIntegrattor.DSInfoStart")) //$NON-NLS-1$
            .append(" ")
            .append(dataBlox.getDataSourceName())
            .append(Messages.getString("PageIntegrattor.DSInfoUser")) //$NON-NLS-1$
            .append(dataBlox.getUserName());
        st2.setTitle(sb.toString());
        st2.setStyle(new Style(true, false, false));
        st2.changed();
    }    

    /**
     * The chagned part of the Tool. It executed every time the browser is open.
     * @param db new datablox
     * @param bloxContext new bloxContext
     * @param appContext new appContext
     * @param qtBlox new queryToolsBlox
     */
    public void changeDataBlox(DataBlox db, BloxContext bloxContext, AppContext appContext, QueryToolsBlox qtBlox){

        this.dataBlox = db;
        this.context = bloxContext;
        this.opContext = new OperationContext(context,dataBlox,qtBlox);

        try{
            if( pageController == null ){
                pageController  = new PageController(dataBlox, opContext);
                pageController.initControllers(topContainer);
            }
            changeDSInfo();
            radioConBuilder.setDataBlox(db);
            treeContainerBuilder.setDataBlox(db);
            sqlEditorBuilder.setDataBlox(db);
            pageController.setDataBlox(db);
            pageController.changeOperationContext(opContext);
        }catch(Exception ex){
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, "RMB Build Tree", ex); //$NON-NLS-1$
        }
    }
    
    public int getEdtUID()
    {
        int edtUID ;
        edtUID = topContainer.searchForComponent(RMBConstants.SQL_EDITOR_ID).getUID(); 
            //pageController.getSqlEditorHandler().getSqlText().getUID();
        return edtUID;
    }
    
}
