package com.ibm.alphablox.tooling.relational.view;

import com.alphablox.blox.DataBlox;
import com.alphablox.blox.repository.Log;
import com.alphablox.blox.uimodel.core.ComponentContainer;
import com.alphablox.blox.uimodel.core.GroupBox;
import com.alphablox.blox.uimodel.core.HorizontalLayout;
import com.alphablox.blox.uimodel.core.ModelException;
import com.alphablox.blox.uimodel.core.RadioButton;
import com.alphablox.blox.uimodel.core.Spacer;
import com.ibm.alphablox.tooling.relational.Messages;
import com.ibm.alphablox.tooling.relational.RMBConstants;

public class RadioContainerBuilder implements IViewPartBuilder {

    private ComponentContainer radioContainer;
    private DataBlox dataBlox;
    public ComponentContainer build(int h, int w, String style) throws ModelException {
        radioContainer = new ComponentContainer(RMBConstants.RADIO_CONTAINER_ID);
        radioContainer.setLayout(new HorizontalLayout());
        
        GroupBox colNamePolicy = new GroupBox(RMBConstants.GROUP_NAME_INSERT_POLICY,
                Messages.getString("PageIntegrattor.ColumnNamePolicy")); //$NON-NLS-1$
        buildColFormatGroup(colNamePolicy);
        
        GroupBox encloseFormat = new GroupBox(RMBConstants.GROUP_NAME_ENCLOSE_STYLE, 
                Messages.getString("PageIntegrattor.EncloseColumnName")); //$NON-NLS-1$
        buildEnclosingCharsGroup(encloseFormat);
        
        GroupBox aliasTableGroup = new GroupBox(RMBConstants.GROUP_TABLE_ALIAS,
                Messages.getString("RadioContainerBuilder.TableAlias")); //$NON-NLS-1$
        buildTableAliasGroup(aliasTableGroup);
        
        radioContainer.add(aliasTableGroup);
        radioContainer.add(new Spacer());
        radioContainer.add(colNamePolicy);
        radioContainer.add(new Spacer());
        radioContainer.add(encloseFormat);
        
        return radioContainer;
    }

    private void buildTableAliasGroup(GroupBox gb) throws ModelException {
        gb.setLayout(new HorizontalLayout());
        RadioButton rb1 = new RadioButton(RMBConstants.RADIO_USE_ALIAS,
                Messages.getString("RadioContainerBuilder.Yes"), //$NON-NLS-1$
                RMBConstants.GROUP_TABLE_ALIAS);
        gb.add(rb1);
        RadioButton rb2 = new RadioButton(
                RMBConstants.DONT_USE_ALIAS, //$NON-NLS-1$
                Messages.getString("RadioContainerBuilder.No"), //$NON-NLS-1$
                RMBConstants.GROUP_TABLE_ALIAS);
        gb.add(rb2);
        rb2.setChecked(true);
    }
    
    private void buildEnclosingCharsGroup(GroupBox gb2) throws ModelException {
        gb2.setLayout(new HorizontalLayout());
        RadioButton rb2 = new RadioButton(RMBConstants.ENCLOSE_NONE, 
                Messages.getString("PageIntegrattor.None"), //$NON-NLS-1$
                RMBConstants.GROUP_NAME_ENCLOSE_STYLE); //$NON-NLS-1$
        rb2.setChecked(true);
        gb2.add(rb2); //$NON-NLS-1$
        gb2.add(new RadioButton(RMBConstants.ENCLOSE_DOUBLE_QUOTE, 
                Messages.getString("PageIntegrattor.EncloseWithDoubleQuote"),  //$NON-NLS-1$
                RMBConstants.GROUP_NAME_ENCLOSE_STYLE)); //$NON-NLS-1$
        gb2.add(new RadioButton(RMBConstants.ENCLOSE_BRACKET, 
                Messages.getString("PageIntegrattor.EncloseWithBracket"),  //$NON-NLS-1$
                RMBConstants.GROUP_NAME_ENCLOSE_STYLE)); //$NON-NLS-1$
    }

    private void buildColFormatGroup(GroupBox gb) throws ModelException {
        gb.setLayout(new HorizontalLayout());
        gb.add(new RadioButton(RMBConstants.INSERT_COL,
                Messages.getString("PageIntegrattor.Column"), //$NON-NLS-1$
                RMBConstants.GROUP_NAME_INSERT_POLICY));
        gb.add(new RadioButton(RMBConstants.INSERT_TABLE,
                Messages.getString("PageIntegrattor.TableColumn"), //$NON-NLS-1$
                RMBConstants.GROUP_NAME_INSERT_POLICY)); //$NON-NLS-1$
        RadioButton rb = new RadioButton(RMBConstants.INSERT_SCHEMA,
                Messages.getString("PageIntegrattor.SchemeTableColumn"), //$NON-NLS-1$
                RMBConstants.GROUP_NAME_INSERT_POLICY); //$NON-NLS-1$
        rb.setChecked(true);
        gb.add(rb); //$NON-NLS-1$
    }

    public void setDataBlox(DataBlox db) {
        try{
            if(dataBlox!=null && dataBlox.getDataSourceName().equalsIgnoreCase(db.getDataSourceName())){
                //nothing to do
            }else{
                dataBlox = db;
                radioContainer.getRadioButton(RMBConstants.INSERT_SCHEMA).setChecked(true);
                radioContainer.getRadioButton(RMBConstants.ENCLOSE_NONE).setChecked(true);
                radioContainer.getRadioButton(RMBConstants.DONT_USE_ALIAS).setChecked(true);
            }
        }catch(Exception ex){
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, "", ex); //$NON-NLS-1$
        }
        
//        this.dataBlox = db;

    }

}
