/**
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: hujianh
 * Date: Oct 22, 2007
 */
package com.ibm.alphablox.tooling.relational.view;

import java.io.PrintStream;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import com.alphablox.blox.DataBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.data.rdb.JDBCConnection;
import com.alphablox.blox.repository.Log;
import com.alphablox.blox.uimodel.core.Button;
import com.alphablox.blox.uimodel.core.ComponentContainer;
import com.alphablox.blox.uimodel.core.ModelException;
import com.alphablox.blox.uimodel.core.Static;
import com.alphablox.blox.uimodel.core.tree.Tree;
import com.alphablox.blox.uimodel.core.tree.TreeNode;
import com.alphablox.util.DataException;
import com.ibm.alphablox.tooling.relational.Messages;
import com.ibm.alphablox.tooling.relational.PageIntegrattor;
import com.ibm.alphablox.tooling.relational.RMBConstants;

/**
 * @author hujianh@cn.ibm.com
 * 
 */
public class TreeContainerBuilder implements IViewPartBuilder {
    private final JDBCConnection jBean = new JDBCConnection();

    final PrintStream out = System.out;

    private DatabaseMetaData dbMetaData;

    private ComponentContainer treeContainer;

    private Tree tree;
    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.alphablox.tooling.relational.view.IViewPartBuilder#build(java.lang.Object,
     *      int, int, java.lang.String)
     */
    public ComponentContainer build( int h, int w, String style)
            throws ModelException {
        treeContainer = new ComponentContainer(RMBConstants.TREE_CONTAINER_ID);
        treeContainer.setHeight(h);
        treeContainer.setWidth(w);
        treeContainer.setStyle(style);

        treeContainer.setScrollingEnabled(false);
        
        treeContainer.add(new Static(Messages.getString("TreeContainerBuilder.Title")));
        try{
            tree = new Tree(RMBConstants.DBFRAME_TREE_ID,"DBFrame"); //$NON-NLS-1$
            tree.setHeight(treeContainer.getHeight()-46);
            tree.setWidth(w);
            tree.setScrollingEnabled(true);
            tree.setMultipleSelect(true);   
            tree.setEnableDragDrop(false);
            tree.setStyle( PageIntegrattor.VISIBLE_BORDER );
            treeContainer.add(tree);
        }catch(Exception ex){
            ex.printStackTrace();
        }
        Button sampleButton = new Button(RMBConstants.SAMPLE_INSERT, Messages.getString("TreeContainerBuilder.ShowValues"));
        sampleButton.setTooltip(Messages.getString("OperatorContainerBuilder.SampleData")); //$NON-NLS-1$
        treeContainer.add(sampleButton);
        
        treeContainer.setUserObject(this);
        return treeContainer;
    }

    /* (non-Javadoc)
     * @see com.ibm.alphablox.tooling.relational.view.IViewPartBuilder#setDataBlox(com.alphablox.blox.DataBlox)
     */
    public void setDataBlox(DataBlox db) {
        try{
            buildTree(db);
        }catch(Exception ex){
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, "RMB build tree error!", ex);
        }
    }

    private void buildTree(DataBlox db) throws ServerBloxException, DataException,SQLException,ModelException {
        
        String dsName = db.getDataSourceName();
        tree.setMultipleSelect(true);
        jBean.setDataSourceName(dsName);
        Connection connection = jBean.createConnection();

        tree.removeAllNodes();
        if (connection != null) {
            dbMetaData = connection.getMetaData();
            buildTree(tree);
        }else{
            Log.sendMessage(Log.MESSAGE_LEVEL_ERROR, 
                    "RMB", Messages.getString("TreeContainerBuilder.DSConnectError")+dsName); //$NON-NLS-1$ //$NON-NLS-2$
        }
    }

    protected void outputTree(TreeNode node, String head){
        if(node.isLeafNode()){
            out.println(head+node.getName());
        }else{
            TreeNode[] nodes = node.getNodes();
            out.println(head+node.getName());
            head+="\t"; //$NON-NLS-1$
            for(int i=0;i<nodes.length;i++){
                outputTree(nodes[i],head);
            }
            head = head.substring(1);
        }
    }
    
    /**
     * Build the schemas.
     * @param tree
     * @throws DataException
     * @throws SQLException
     * @throws ModelException
     */
    private void buildTree(Tree tree) throws DataException, SQLException, ModelException {
        tree.setTitle("*");
        tree.setUserObject("*");
        tree.setExpanded(false);
        tree.setRootVisible(true);
    }
    
    /**
     * Append the tree nodes for the schemas
     * @param rootNode
     * @throws DataException
     * @throws SQLException
     * @throws ModelException
     */
    public void buildSchemas(TreeNode rootNode)throws DataException, SQLException, ModelException {
        ResultSet schemas = dbMetaData.getSchemas();
        StringBuffer sb = new StringBuffer(128);
        
        while(schemas.next()){
            String schemaName = schemas.getString(1);
            TreeNode schemaNode = new TreeNode(schemaName, schemaName, false);
            rootNode.addNode(schemaNode);
            
            String imageURL = RMBConstants.SCHEMA_GIF;
            schemaNode.setThemeBasedImage(false);
            schemaNode.setNodeImageURL(imageURL);
            //user object
            sb.setLength(0);
            sb.append(Messages.getString("TreeContainerBuilder.SchemaName")).append(schemaName); //$NON-NLS-1$
            schemaNode.setUserObject(sb.toString());
        }
    }
    /**
     * Build the table information about the schema being selected
     * @param schemaName
     * @param schemaNode
     * @throws DataException
     * @throws SQLException
     * @throws ModelException
     */
    public void buildSchemaChild( String schemaName, TreeNode schemaNode) throws DataException, SQLException, ModelException { 
        StringBuffer sb = new StringBuffer(128);
        String imageURL = RMBConstants.TABLE_GIF;
        ResultSet tables = dbMetaData.getTables(null, schemaName, "%", null); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        
        while(tables.next()){
            String tableName = tables.getString(3);
            TreeNode tableNode = new TreeNode( tableName, tableName ,false );
            schemaNode.addNode(tableNode);
            
            tableNode.setThemeBasedImage(false);
            tableNode.setNodeImageURL(imageURL);
            
            sb.setLength(0);
            sb.append(Messages.getString("TreeContainerBuilder.TableName")).append(tableName); //$NON-NLS-1$
            sb.append(Messages.getString("TreeContainerBuilder.PrimaryKeys")); //$NON-NLS-1$
            ResultSet rs = dbMetaData.getPrimaryKeys(null,schemaName,tableName);
            while(rs.next()){
                sb.append(rs.getString(4));
                sb.append(","); //$NON-NLS-1$
            }
            
            tableNode.setUserObject(sb.toString());
        }
    }
    
    /**
     * Build the columns information about the table being selected.
     * @param schemaName
     * @param tableName
     * @param tableNode
     * @throws DataException
     * @throws SQLException
     * @throws ModelException
     */
    public void buildTableChild(String schemaName, String tableName, TreeNode tableNode) throws DataException, SQLException, ModelException {        
        String imageURL;
        String sqlStr = "select * from "+schemaName+"."+tableName+" where 1<0"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        Connection connection = dbMetaData.getConnection();
        PreparedStatement ps = connection.prepareStatement(sqlStr);
        ResultSet rs = ps.executeQuery();
        ResultSetMetaData rsMetadata = rs.getMetaData();
        StringBuffer sb = new StringBuffer(128);
        
        for(int i=1;i<=rsMetadata.getColumnCount();i++){
            sb.setLength(0);
            String colName = rsMetadata.getColumnName(i);
            sb.append(Messages.getString("TreeContainerBuilder.ColumnDisplayName")).append(rsMetadata.getColumnLabel(i)); //$NON-NLS-1$
            sb.append(Messages.getString("TreeContainerBuilder.ColName")).append( colName ); //$NON-NLS-1$
            sb.append(Messages.getString("TreeContainerBuilder.TypeCode")).append(rsMetadata.getColumnType(i)); //$NON-NLS-1$
            sb.append(Messages.getString("TreeContainerBuilder.TypeName")).append(rsMetadata.getColumnTypeName(i)); //$NON-NLS-1$
            sb.append(Messages.getString("TreeContainerBuilder.IsNullable")).append(rsMetadata.isNullable(i)); //$NON-NLS-1$
            sb.append(Messages.getString("TreeContainerBuilder.IsAutoIncrement")).append(rsMetadata.isAutoIncrement(i)); //$NON-NLS-1$
            sb.append(Messages.getString("TreeContainerBuilder.IsCaseSensitive")).append(rsMetadata.isCaseSensitive(i)); //$NON-NLS-1$
            
            TreeNode colNode = new TreeNode( colName, colName,true );
            imageURL = RMBConstants.COLUMN_GIF;//RMBConstants.getImageURL(RMBConstants.COLUMN_GIF).toString();
            colNode.setThemeBasedImage(false);
            colNode.setNodeImageURL(imageURL);
            
            colNode.setUserObject(sb.toString());
            tableNode.add(colNode);
        }
    }

    protected ResultSetMetaData getMetaData(ResultSet cols) throws SQLException {
        ResultSetMetaData metadata = cols.getMetaData();
        return metadata;
    }

    public ComponentContainer getContainer() {
        return this.treeContainer;
    }
}
