/*
 * IBM Confidential
 * OCO Source Materials
 * 5724-L14
 * © Copyright IBM Corp. 2004, 2006
 * The source code for this program is not published or otherwise divested of its trade secrets, irrespective of what has been deposited with the U.S. Copyright Office.
 */
package fastforward;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;


/**
 * Provides a method to access localized strings with helper methods that assist in
 * formatting methods. <BR>
 *
 * There is also a static convenience method to save creating an instance.
 */

public class FastForwardResourceBundle {
    private static final String BUNDLE = "fastforward.FastForwardBundle";

    private ResourceBundle bundle;

    public static final String STRING_FOR_NULL = "null";

    // Small optimization - cache this to save creating it every 3 seconds !
    private static final MessageFormat formatter = new MessageFormat("");

    /**
     * Allow for i18n by accepting the locale in the constructor
     * @param locale
     */
    public FastForwardResourceBundle(Locale locale, ClassLoader classLoader) throws MissingResourceException {
        bundle = ResourceBundle.getBundle(BUNDLE, locale, classLoader);
    }

    /**
     * Get a localized string from this resource, substituting parameters as necessary <BR>
     * @param key The key to the resource
     * @param args the values to substitute for the args in the string
     * @return the localized, expanded value for this key if there is one - if not, return an error message containing the key
     */
    public String getString(String key, Object[] args) {
        StringBuffer buffer = new StringBuffer();

        // The formatter cannot deal with nulls in the args
        if (args != null) {
            for (int i = 0; i < args.length; i++) {
                if (args[i] == null)
                    args[i] = STRING_FOR_NULL;
            }
        }

        try {
            String message = getString(key);

            // The formatter truncates text if there are no arguments
            if (args == null || args.length == 0) {
                return message;
            } else {
                // Don't use the *simple* version of format() so we can handle the IllegalArgumentException correctly
                synchronized (formatter) {
                    // The docs say that the FieldPosition parameter is not used so we can safely pass in null
                    formatter.applyPattern(message);
                    formatter.format(args, buffer, null);
                }
            }
        } catch (IllegalArgumentException iae) {
            // This occurs when the number of arguments specified in the message does not match the args[] passed in
            // A sneak peak at the MessageFormat code shows that the buffer will be populated up to but not including the arg that failed
            // (Which is good - but not perfect)
        } catch (Exception e) {
            // Don't know when this would occur, but we need to make sure that an error in here does not break the calling routine
        }

        return buffer.toString();
    }

    /**
     * Get a localized string from this resource <BR>
     * @param key The key to the resource
     * @return the localized value for this key if there is one - if not, return the key itself
     */
    public String getString(String key) {
        return bundle.getString(key);
    }
}



