package fastforward;

import org.w3c.dom.Document;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.util.XmlFile;

import java.util.Vector;
import java.io.StringReader;

/**
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: rraghavan
 * Date: Jan 17, 2003 9:41:09 AM
 */
public class GroupConfiguration extends AbstractConfiguration {
    private UserImpl user;
    private String groupName;
    /**
     * deprecated
     */
    public static final String GROUP_REPORT_LABEL = "Group Reports";
    /**
     * deprecated
     */
    public static final String GROUP_REPORT_DESC = "Group Reports";

    public void init(AppContextImpl appContext, String groupName) {
        super.init(appContext);
        user = (UserImpl)appContext.getUser();
        this.groupName = groupName;
    }

    public String getGroup() {
        return groupName;
    }

    public void readConfig() throws XmlParseException {
        try {
            Document userConfigXml = null;

            try {
                userConfigXml = user.readConfigFile(groupName, appContext.getContextPath());
                if (userConfigXml == null) {
                    setReports(new Vector());
                    setParameters(new ParameterSet());
                    NavigationTree navTree = new NavigationTree();
                    navTree.setNavigationRoot(new NavigationTree.Folder(NavigationTree.GROUP_ROOT_ID, appContext.getString("GROUP_REPORT_LABEL"), appContext.getString("GROUP_REPORT_DESC")));
                    setNavigationTree(navTree);

                    return;
                }
            } catch (ServerBloxException e) {
                e.printStackTrace();
            }

            XmlFile xmlFile = new XmlFile();

            String xml = addDocumentDTD(userConfigXml, "fastforward", "file:///" + appContext.getRealPath(AppContextImpl.CONFIG_DTD_FILE));
            StringReader reader = new StringReader(xml);
            xmlFile.setXmlReader(reader);

            XMLErrorHandler errorHandler = new XMLErrorHandler(xmlFile);
            xmlFile.setErrorHandler(errorHandler);

            userConfigXml = xmlFile.getDocument();

            readConfig(userConfigXml);

            if (errorHandler.exceptionThrown())
                throw errorHandler.getException();
        } catch (Exception exc) {
            throw new XmlParseException(appContext.getString("ERROR_LOADING_GROUP_CONFIG",new Object[]{groupName}));
        }
    }

    public void saveConfig() throws XmlParseException, ServerBloxException {
        // save the config to the repository
        XMLConfiguration xmlConfig = new XMLConfiguration();
        xmlConfig.init(this, appContext.getResourceBundle());
        Document document = xmlConfig.generateXmlDocument();

        user.saveConfigFile(groupName, document, appContext.getContextPath());
    }

    public void deleteReport(String id) {
        super.deleteReport(id);
        ((SavedStateImpl)appContext.getSavedState()).delete();
    }

}
