/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Aug 27, 2002 2:32:21 PM
 */
package fastforward;

import org.w3c.dom.*;
import org.apache.xpath.XPathAPI;

import java.util.HashMap;
import java.util.Map;

import fastforward.ParameterSupport;
import com.alphablox.util.HTMLUtils;

public class ParameterSet implements ParameterSupport, Cloneable {
    private Map parameters=new HashMap(); // paramName=paramValues[]

    public ParameterSet() {
    }

    public void setParameter(String parameterName, String parameterValue) {
        if (parameterValue == null) {
            setParameter(parameterName,(String[])null);
            return;
        }

        parameters.put(parameterName, new String[]{parameterValue});
    }

    public void setParameter(String parameterName, String[] parameterValues) {
        if (parameterValues == null) {
            parameters.remove(parameterName);
            return;
        }

        parameters.put(parameterName, parameterValues);
    }

    public String getParameter(String parameterName) {
        String[] parameterValues = (String[]) parameters.get(parameterName);
        if (parameterValues != null && parameterValues.length > 0)
            return parameterValues[0];

        return null;
    }

    public void parseParameters(NodeList parameterList) throws XmlParseException {
        try {
            for (int paramIndex = 0; paramIndex < parameterList.getLength(); paramIndex++) {
                Node paramElement = parameterList.item(paramIndex);

                String paramName = XPathAPI.selectSingleNode(paramElement, "param-name/node()").getNodeValue();
                NodeList nodeList = XPathAPI.selectNodeList(paramElement, "param-value/node()");
                if (nodeList == null) continue;
                String[] paramValues = new String[nodeList.getLength()];
                for (int i = 0; i < nodeList.getLength(); i++) {
                      paramValues[i] = HTMLUtils.translateString( HTMLUtils.translateCDATA(nodeList.item(i).getNodeValue()));
                }
                if (parameters.containsKey(paramName)) {
                    throw new XmlParseException("Duplicate param entry found for "+paramName);
                }

                parameters.put(paramName, paramValues);
            }
        } catch (Exception exc) {
            throw new XmlParseException(exc);
        }
    }

    public String getParameter(String parameterName, String defaultValue) {
        String value=getParameter(parameterName);

        return (value != null) ? value : defaultValue;
    }

    public String[] getParameterValues(String parameterName) {
        return (String[]) parameters.get(parameterName);
    }

    public String[] getParameterNames() {
        String[] parameterNames=new String[parameters.size()];

        Object[] keys = parameters.keySet().toArray(parameterNames);
        for (int keyIndex = 0; keyIndex < keys.length; keyIndex++) {
            parameterNames[keyIndex] = (String)keys[keyIndex];
        }

        return parameterNames;
    }

    protected Object clone() {
        ParameterSet clone=new ParameterSet();

        clone.parameters.putAll(this.parameters);

        return clone;
    }

    public void clear() {
        parameters.clear();
    }

}
