package fastforward;

import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.form.IFormBlox;

import java.util.HashMap;
import java.util.Set;

/*
 * Copyright 2003 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Feb 19, 2003 11:25:01 AM
 */

public class PreviewReport implements Report {
    private Template template;
    private String reportName;
    private String description;
    private String reportId;
    private HashMap parameters = new HashMap();

    public PreviewReport(Template template, String reportId, String reportName, String description, IFormBlox[] formBlox) {
        this.template = template;
        this.reportName = reportName;
        this.description = description;
        this.reportId = reportId;

        for (int i = 0; i < formBlox.length; i++) {
            String bloxName;
            try {
                bloxName = formBlox[i].getBloxName();
            } catch (ServerBloxException e) {
                continue;
            }

            parameters.put(bloxName, formBlox[i].getFormValues());
        }
    }

    public String getId() {
        return reportId;
    }

    public String getLabel() {
        return reportName;
    }

    public String getDescription() {
        return description;
    }

    public Template getTemplate() {
        return (TemplateImpl) template;
    }

    public void setParameter(String parameterName, String parameterValue) {
        parameters.put(parameterName, new String[]{parameterValue});
    }

    public void setParameter(String parameterName, String[] parameterValues) {
        parameters.put(parameterName,parameterValues);
    }

    public String getParameter(String parameterName) {
        String[] values = getParameterValues(parameterName);
        if (values == null)
            return null;

        return (values.length > 0 ? values[0] : null);
    }

    public String getParameter(String parameterName, String defaultValue) {
        String paramValue = getParameter(parameterName);
        return (paramValue != null) ? paramValue : defaultValue;
    }

    public String[] getParameterValues(String parameterName) {
        return (String[]) parameters.get(parameterName);
    }

    public String[] getParameterNames() {
        Set keys=parameters.keySet();
        return (String[]) keys.toArray(new String[keys.size()]);
    }
}
