/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Aug 22, 2002 12:08:29 PM
 */
package fastforward;

public class ReportImpl implements  ParameterSupport, Report {
    private String id;
    private String description;
    private String label;
    private TemplateImpl template;
    private String owner;   // owner of this report
    private ParameterSet parameters = new ParameterSet();
    private Acl acl = new Acl(null);
    private SavedStateData savedStateData;
    private IConfiguration config;

    public void setId(String name) {
        this.id = name;
    }

    public String getId() {
        return id;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public void setTemplate(TemplateImpl template) {
        this.template = template;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public String getOwner() {
        return owner;
    }

    public void setOwner(String owner) {
        this.owner = owner;
    }

    public Template getTemplate() {
        return template;
    }

    public void setAcl(Acl acl) {
        this.acl = acl;
    }

    public Acl getAcl() {
        return this.acl;
    }

    public IConfiguration getConfig() {
        return config;
    }

    public void setConfig(IConfiguration config) {
        this.config = config;
    }

    public void copyTo(ReportImpl otherReport) {
        otherReport.setTemplate(template);
        otherReport.setParameters((ParameterSet) parameters.clone());
        otherReport.setAcl(acl);
        otherReport.setOwner(owner);
        otherReport.setConfig(config);
    }

    public void setParameter(String parameterName, String parameterValue) {
        parameters.setParameter(parameterName, parameterValue);
    }

    public void setParameter(String parameterName, String[] parameterValues) {
        parameters.setParameter(parameterName, parameterValues);
    }

    public String getParameter(String parameterName) {
        Parameter parameter=template.getParameter(parameterName);

        return parameters.getParameter(parameterName,
                (parameter != null ? parameter.getDefaultValue() : null));
    }

    public String getParameter(String parameterName, String defaultValue) {
        return parameters.getParameter(parameterName,defaultValue);
    }

    public String[] getParameterValues(String parameterName) {
        return parameters.getParameterValues(parameterName);
    }

    public String[] getParameterNames() {
        return parameters.getParameterNames();
    }

    public void setParameters(ParameterSet parameters) {
        this.parameters = parameters;
    }

    public boolean canViewReport(User user) {
        return acl.hasAccess(user);
    }

    public void clearParameters() {
        parameters.clear();
    }

    public void setSavedStateData(SavedStateData savedState) {
        this.savedStateData = savedState;
    }

    public SavedStateData getSavedStateData() {
        return savedStateData;
    }

    public String getVisibility() {
        if (config instanceof GroupConfiguration)
            return Report.GROUP;
        else if (config instanceof UserConfiguration)
            return Report.PRIVATE;
        else
            return Report.PUBLIC;
    }
}
