/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: rraghavan
 * Date: Dec 5, 2002 11:19:46 AM
 */
package fastforward;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.apache.xpath.XPathAPI;

import javax.xml.transform.TransformerException;
import java.util.Vector;

public class SavedStateData {
    private Vector controls = new Vector();
    private Vector bookmarks = new Vector();

    public SavedStateData() {
    }

    public void setControlValue(String controlName, String controlValue) {
        setControlValue(controlName, new String[] {controlValue});
    }

    public void setControlValue(String controlName, String[] controlValue) {
        Control control = getControl(controlName);
        if (control == null) {
            control = new Control(controlName, controlValue);
            controls.add(control);
        } else {
            control.setValue(controlValue);
        }
    }

    public String getControlValue(String controlName) {
        Control control = getControl(controlName);
        if (control != null)
            return control.getValue();

        return null;
    }

    public String[] getControlValues(String controlName) {
        Control control = getControl(controlName);
        if (control != null)
            return control.getValues();

        return null;
    }

    public void setBookmarkName(String bloxName, String bookmarkName) {
        Bookmark bookmark = getBookmark(bloxName);
        if (bookmark == null) {
            bookmark = new Bookmark(bloxName, bookmarkName);
            bookmarks.add(bookmark);
        } else {
            bookmark.setBookmarkName(bookmarkName);
        }
    }

    public String getBookmarkName(String bloxName) {
        Bookmark bookmark = getBookmark(bloxName);
        if (bookmark != null)
            return bookmark.getBookmarkName();

        return null;
    }

    private Control getControl(String controlName) {
        for (int i = 0; i < controls.size(); i++) {
            Control control = (Control) controls.elementAt(i);
            if (control.getName().equals(controlName))
                return control;
        }
        return null;
    }

    protected Bookmark getBookmark(String bloxName) {
        for (int i = 0; i < bookmarks.size(); i++) {
            Bookmark bookmark = (Bookmark) bookmarks.elementAt(i);
            if (bookmark.getBloxName().equals(bloxName))
                return bookmark;
        }
        return null;
    }

    public Bookmark[] getBookmarks() {
        return (Bookmark[]) bookmarks.toArray(new Bookmark[bookmarks.size()]);
    }

    public Control[] getControls() {
        return (Control[]) controls.toArray(new Control[controls.size()]);
    }

    protected Bookmark[] getReportBloxBookmarks() {
        Vector reportBloxBookmarks = new Vector();
        Bookmark[] allBookmarks = getBookmarks();

        for (int x = 0; x < allBookmarks.length; x++) {
            Bookmark bookmark = allBookmarks[x];
            if (bookmark.isReportBlox())
                reportBloxBookmarks.add(bookmark);
        }

        return (Bookmark[]) reportBloxBookmarks.toArray(new Bookmark[reportBloxBookmarks.size()]);
    }

    protected void parse(Node node) throws TransformerException {
        NodeList controls = XPathAPI.selectNodeList(node, "controls/control");
        for (int i = 0; controls != null && i < controls.getLength(); i++) {
            Node control = controls.item(i);
            Node name = control.getChildNodes().item(0).getChildNodes().item(0);
            Node value = control.getChildNodes().item(1).getChildNodes().item(0);
            setControlValue(name.getNodeValue(), value.getNodeValue());
        }
        NodeList bookmarks = XPathAPI.selectNodeList(node, "bookmarks/blox");
        for (int i = 0; bookmarks != null && i < bookmarks.getLength(); i++) {
            Node bookmark = bookmarks.item(i);
            Node name = bookmark.getChildNodes().item(0).getChildNodes().item(0);
            Node bmarkName = bookmark.getChildNodes().item(1).getChildNodes().item(0);
            setBookmarkName(name.getNodeValue(), bmarkName.getNodeValue());
            Node isReportBlox = bookmark.getChildNodes().item(2);
            if (isReportBlox != null)
                getBookmark(name.getNodeValue()).setReportBlox("true".equals(isReportBlox.getChildNodes().item(0).getNodeValue()));
        }
    }

    static class Control {
        private String name;
        private Vector values = new Vector();

        public Control() {
        }

        public Control(String name, String value) {
            this.name = name;
            this.values.add(value);
        }

        public Control(String name, String[] value) {
            this.name = name;
            for (int i = 0; i < value.length; i++) {
                this.values.add(value[i]);
            }
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getValue() {
            if (values.size() == 0)
                return null;
            else
                return (String) values.get(0);
        }

        public String[] getValues() {
            if (values.size() == 0)
                return null;
            else
                return (String[]) values.toArray(new String[values.size()]);
        }

        public void setValue(String value) {
            this.values.clear();
            this.values.add(value);
        }

        public void setValue(String[] value) {
            this.values.clear();
            for (int i = 0; i < value.length; i++) {
                this.values.add(value[i]);
            }
        }
    }

    static class Bookmark {
        private String bloxName;
        private String bookmarkName;
        private boolean isReportBlox = false;

        public Bookmark() {
        }

        public Bookmark(String bloxName, String bookmarkName) {
            this.bloxName = bloxName;
            this.bookmarkName = bookmarkName;
        }

        public String getBloxName() {
            return bloxName;
        }

        public void setBloxName(String bloxName) {
            this.bloxName = bloxName;
        }

        public String getBookmarkName() {
            return bookmarkName;
        }

        public void setBookmarkName(String bookmarkName) {
            this.bookmarkName = bookmarkName;
        }

        public boolean isReportBlox() {
            return isReportBlox;
        }

        public void setReportBlox(boolean reportBlox) {
            isReportBlox = reportBlox;
        }
    }
}
