/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Sep 11, 2002 4:28:53 PM
 */
package fastforward;

import com.alphablox.blox.form.TreeFormBlox;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

public class TreeFormBloxHelper {
    public TreeFormBlox.Folder createGroupTree(GroupConfiguration[] groupConfigs, User user, FastForwardResourceBundle resource) {
        TreeFormBlox.Folder allGroupRoot = new TreeFormBlox.Folder(resource.getString("GROUP_REPORT_LABEL"), resource.getString("GROUP_REPORT_DESC"), resource.getString("GROUP_REPORT_DESC"), null, null);

        for (int i = 0; i < groupConfigs.length; i++) {
            GroupConfiguration groupConfig = groupConfigs[i];
            if (groupConfig.getReports().size() > 0) {
                // We're using the createTree method here... but we really want to name the root
                TreeFormBlox.Folder root = createTree(groupConfig, user, resource);
                // So now we'll move the root's children to a new root with the name we want
                TreeFormBlox.Folder namedRoot = new TreeFormBlox.Folder(root.getName(),
                        groupConfig.getGroup(),
                        groupConfig.getGroup() + " Group Reports",
                        root.getHref(), root.getTarget(), root.getUserObject());
                // Copy all of the root children to the named root
                List items = new Vector();
                items.addAll(root.getChildren());
                for (int x = 0; x < items.size(); x++) {
                    namedRoot.add((TreeFormBlox.Item) items.get(x));
                }
                allGroupRoot.add(namedRoot);
            }
        }
        if (allGroupRoot.getChildren().isEmpty()) {
            allGroupRoot = new TreeFormBlox.Folder("NoReports", resource.getString("NO_REPORTS_AVAILABLE"), resource.getString("NO_REPORTS_AVAILABLE"), null, null);
        }

        return allGroupRoot;
    }

    public TreeFormBlox.Folder createTree(IConfiguration config, User user, FastForwardResourceBundle resource) {
        NavigationTree.Folder root = config.getNavigationTree().getNavigationRoot(user);
        if (root == null)
            root = new NavigationTree.Folder("NoReports", resource.getString("NO_REPORTS_AVAILABLE"), null);

        TreeFormBlox.Folder rootFolder = createRootFolder(root, false, config);

        return rootFolder;
    }

    public TreeFormBlox.Folder createAdminNavTree(IConfiguration config, FastForwardResourceBundle resource) {
        NavigationTree.Folder root = config.getNavigationTree().getNavigationRoot();

        TreeFormBlox.Folder rootFolder = createRootFolder(root, true, config);

        return rootFolder;
    }

    private TreeFormBlox.Folder createRootFolder(NavigationTree.Report root, boolean isAdminTree, IConfiguration config) {
        return copyNavigationTree(root, null, isAdminTree, config);
    }

    private TreeFormBlox.Folder copyNavigationTree(NavigationTree.Report item, TreeFormBlox.Folder parent, boolean isAdminTree, IConfiguration config) {
        String href = getHref(item);
        String target = getTarget(item);

        if (item instanceof NavigationTree.Folder) {
            TreeFormBlox.Folder folder = new TreeFormBlox.Folder(item.getId(), item.getLabel(), item.getDescription(), href, target);
            if (parent != null)
                parent.add(folder);

            folder.setDraggable(isAdminTree);

            Iterator childList = ((NavigationTree.Folder) item).getChildren().iterator();
            while (childList.hasNext()) {
                copyNavigationTree((NavigationTree.Report) childList.next(), folder, isAdminTree, config);
            }

            return folder;
        } else {
            TreeFormBlox.Item newItem = new TreeFormBlox.Item(item.getId(), item.getLabel(), item.getDescription(), href, target);
            // associate the Report with the node
            newItem.setUserObject(config.getReport(item.getId()));
            if (parent != null)
                parent.add(newItem);

            newItem.setDraggable(isAdminTree);

            return parent;
        }
    }

    public String getHref(NavigationTree.Report item) {
        return null;
    }

    public String getTarget(NavigationTree.Report item) {
        return null;
    }

    protected NavigationTree.Report getTopLevelRoot(NavigationTree.Report item) {
        NavigationTree.Report root = item;
        while (!root.isRoot()) {
            root = root.getParent();
        }
        return root;
    }
}
