/**
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: CEUBANKS
 * Date: Nov 7, 2002
 * Time: 7:02:21 PM
 */
package fastforward;

import fastforward.ParameterSupport;

/**
 * Represents the user currently logged into the FastForward application. This User object is an abstraction of the User
 * defined within the Alphablox Application Server.
 */

public interface User extends ParameterSupport {
    /**
     * Returns the name of the currently logged in user.
     * @return a String representing the current user
     */
    public String getName();

    /**
     * Returns true if the user is a member of the admistrator role as defined in the AppContext.
     * @see AppContext
     * @return true if the current user as access to admin tasks
     */
    public boolean isAdministrator();

    /**
     * Returns true if the current user is a member of the given Alphablox Analysis Server group name. This is used to personalize report
     * templates to groups of users.
     * @param group the group to test membership of the current user
     * @return true if the current user is a member of the given group
     */
    public boolean isInGroup(String group);

    /**
     * Returns an array of all groups that include the current user as a member.
     * @return an array of Strings.  These are  the group names that include the current user
     */
    public String[] getGroups();

    /**
     * Sets the named user parameter to the given value. User parameters may be used to personalize report templates
     * specifically for the current user.
     * @param paramName the name of the user parameter to assign
     * @param paramValue the value of the user parameter to assign
     */
    public void setParameter(String paramName, String paramValue);
}
