package fastforward;

import org.w3c.dom.Document;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.util.XmlFile;

import java.util.Vector;
import java.io.StringReader;

/**
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: ceubanks
 * Date: Dec 5, 2002 1:41:09 PM
 */
public class UserConfiguration extends AbstractConfiguration {
    private UserImpl user;
    /**
     * deprecated
     */
    public static final String PRIVATE_REPORT_LABEL = "My Reports";
    /**
     * deprecated
     */
    public static final String PRIVATE_REPORT_DESC = "My Reports";

    public void init(AppContextImpl appContext) {
        super.init(appContext);
        user = (UserImpl)appContext.getUser();
    }

    public void readConfig() throws XmlParseException {
        Document userConfigXml = null;

        try {
            try {
                userConfigXml = user.readConfigFile((appContext.getContextPath()));
                if (userConfigXml == null) {
                    setReports(new Vector());
                    setParameters(new ParameterSet());
                    NavigationTree navTree = new NavigationTree();
                    navTree.setNavigationRoot(new NavigationTree.Folder(NavigationTree.PRIVATE_ROOT_ID, appContext.getString("PRIVATE_REPORT_LABEL"), appContext.getString("PRIVATE_REPORT_DESC")));
                    setNavigationTree(navTree);

                    return;
                }
            }
            catch (ServerBloxException e) {
                throw new XmlParseException(appContext.getString("ERROR_LOADING_USER_CONFIG", new Object[]{user.getName()}), e);
            }

            XmlFile xmlFile = new XmlFile();

            String xml = addDocumentDTD(userConfigXml, "fastforward", "file:///" + appContext.getRealPath(AppContextImpl.CONFIG_DTD_FILE));

            StringReader reader = new StringReader(xml);
            xmlFile.setXmlReader(reader);

            XMLErrorHandler errorHandler = new XMLErrorHandler(xmlFile);
            xmlFile.setErrorHandler(errorHandler);

            userConfigXml = xmlFile.getDocument();

            readConfig(userConfigXml);

            if (errorHandler.exceptionThrown())
                throw errorHandler.getException();
        }
        catch (XmlParseException xpe) {
            throw xpe;
        }
        catch (Exception e) {
            throw new XmlParseException(appContext.getString("ERROR_LOADING_USER_CONFIG", new Object[]{user.getName()}), e);
        }
    }

    public void saveConfig() throws XmlParseException, ServerBloxException {
        // save the config to the repository
        XMLConfiguration xmlConfig = new XMLConfiguration();
        xmlConfig.init(this, appContext.getResourceBundle());
        Document document = xmlConfig.generateXmlDocument();

        user.saveConfigFile(document, appContext.getContextPath());
    }

    public void deleteReport(String id) {
        super.deleteReport(id);
        ((SavedStateImpl)appContext.getSavedState()).delete();
    }
}
