#!/bin/sh

# ***************************************************************
#  Licensed Material - Property of IBM
# 
#  5724-M24 
# 
#  Copyright IBM Corp. 2010.  All rights reserved.
# 
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM
#  Corp.
# ***************************************************************

# ***************************************************************
#  Be sure to update dmgrConfig.properties to your configuration
# ***************************************************************

. dmgrConfig.properties

if [ "$ADMINUSERNAME" == "" ]
then
   echo ""
   echo " Please provide needed settings in dmgrConfig.properties"
   exit 1
fi

# ******************************************************
echo "-"                            
echo  START of install task execution
# ******************************************************

CURRENT_DIR=`pwd`
cd ..
WBM_LOC=`pwd`
cd $CURRENT_DIR
         
# ******************************************************
#  Copy files to the Deployment Manager 
# ******************************************************

echo "-"                            
echo "-"                            
echo "move the WAS plugins to be used"
if [ -d $WBM_LOC/plugins ]; then
   rm -r $WBM_LOC/plugins
fi                                                
mkdir $WBM_LOC/plugins
if [ $WAS_VERSION == 61 ]; then 
   cp $WBM_LOC/abx.62.plugins/plugins/* $WBM_LOC/plugins 
   elif [ $WAS_VERSION == 70 ]; then
     cp $WBM_LOC/abx.70.plugins/plugins/* $WBM_LOC/plugins 
fi
echo "move logging support jar"
cp $WBM_LOC/temp/was602/lib/wbminstall_log.jar $WAS_LOC/lib
if [ "$?" -ne "0" ]; then
  exit 1
fi
  


echo "-"                            
echo "-"                            
echo "Install files task"
$WAS_LOC/bin/ws_ant.sh -Dwbm.portdash.installed.id=700 -Dwbm.install.instance.id=70 -Dwbm.installType.PortalDashboard=installNewPortalDashboard -Dinstall.widgets.nativePortlets=true -Dwbm.install.abx.portalviews=$ABX_INSTALL -Dwas602.install.root=$WAS_LOC -Dwas.install.root=$WBM_LOC -Dportal.install.root=$WBM_LOC -Dportal.profileType=deploymentManager -DtraceLevel=all -f $WBM_LOC/actions/CopyPortalDashboardFiles.ant
if [ "$?" -ne "0" ]; then
  exit 1
fi

echo "-"                            
echo "-"                            
echo "Initialize new plugins"
$WAS_PROFILE_PATH/bin/osgiCfgInit.sh

# ******************************************************
echo "-"                            
echo "-"                            
echo "Deploy REST Proxy and Human Task settings"
# ******************************************************

$WAS_LOC/bin/ws_ant.sh -DconfigureHumanTask=$CONFIGHUMANTASK -DhumanTaskWPSHostname=$HUMANTASKHOST -DhumanTaskWPSRMIPort=$HUMANTASKPORT -Dbspace.hostName=$BSPACEHOST -Dbspace.serverPort=$BSPACEPORT -Dbspace.secureServerPort=$BSPACESECUREPORT -Drest.hostName=$RESTHOST -Drest.serverPort=$RESTPORT -Drest.secureServerPort=$RESTSECUREPORT -Dwas602.admin.user=$ADMINUSERNAME -Dwas602.admin.password=$ADMINPASSWORD -Dportal.cluster.name=$CLUSTERNAME -Dportal.server.name= -Dportal.profile.nodename=$NODENAME -Dportal.profile.cellname=$CELLNAME -Dportal.profile.name=$WAS_PROFILE -Dwbm.installType.PortalDashboard=installNewPortalDashboard -Dinstall.widgets.nativePortlets=true -Dwbm.install.abx.portalviews=$ABX_INSTALL -Dwas602.install.root=$WAS_LOC -Dwas.install.root=$WBM_LOC -Dportal.install.root=$WBM_LOC -Dportal.profile.path=$WBM_LOC -Dportal.profileType=deploymentManager -DtraceLevel=all -f $WBM_LOC/actions/AugmentPortalDashboardProfile.ant 
if [ "$?" -ne "0" ]; then
  echo "check log ${WAS_LOC}/logs/manageprofiles/${WAS_PROFILE}_augment.log" 
  echo "check log ${WAS_LOC}/profiles/${WAS_PROFILE}/logs/installPortalDashboardRestProxy.log"  
  exit 1
fi


# ******************************************************
echo "-"                            
echo "-"                            
echo "Create Monitor data source for Alphablox"
# ******************************************************
 
$WAS_LOC/bin/ws_ant.sh -Ddb.dbName=$DBNAME -Ddb.dbUserId=$DBUSERID -Ddb.dbPassword=$DBPASSWORD -Ddb.dbHostName=$DBHOSTNAME -Ddb.dbServerPort=$DBPORT -Ddb.dbType=$DBTYPE -Ddb.dbSchemaName=$DBSCHEMA -Ddb.portalJdbcClassPath=$DBCLASSPATH -Ddb.dbJdbcDriverType=$DBDRIVERTYPE -DconfigureHumanTask=$CONFIGHUMANTASK -DhumanTaskWPSHostname=$HUMANTASKHOST -DhumanTaskWPSRMIPort=$HUMANTASKPORT -Dbspace.hostName=$BSPACEHOST -Dbspace.serverPort=$BSPACEPORT -Dbspace.secureServerPort=$BSPACESECUREPORT -Drest.hostName=$RESTHOST -Drest.serverPort=$RESTPORT -Drest.secureServerPort=$RESTSECUREPORT -Dwas602.admin.user=$ADMINUSERNAME -Dwas602.admin.password=$ADMINPASSWORD -Dportal.cluster.name=$CLUSTERNAME -Dportal.server.name= -Dportal.profile.nodename=$NODENAME -Dportal.profile.cellname=$CELLNAME -Dportal.profile.name=$WAS_PROFILE -Dwbm.installType.PortalDashboard=installNewPortalDashboard -Dinstall.widgets.nativePortlets=true -Dwbm.install.abx.portalviews=$ABX_INSTALL -Dwas602.install.root=$WAS_LOC -Dwas.install.root=$WBM_LOC -Dportal.install.root=$WBM_LOC -Dportal.profile.path=$WBM_LOC -Dportal.profileType=deploymentManager -DtraceLevel=all -f $WBM_LOC/actions/AugmentABXPortalDashboardProfile.ant
if [ "$?" -ne "0" ]; then
  echo "check log ${WAS_LOC}/logs/manageprofiles/${WAS_PROFILE}_augment.log" 
  echo "check log ${WAS_LOC}/profiles/${WAS_PROFILE}/logs/installABXPortalDashboard.log" 
  exit 1 
fi


# ******************************************************
echo "-"                            
echo "-"                           
echo "Deploy Alphablox service"
# ******************************************************
 
$WAS_LOC/bin/ws_ant.sh -DenableAdminSecurity=true -Ddb.dbSchemaName=$DBSCHEMA -Dportal.soap.port=8880 -Dwas602.admin.user=$ADMINUSERNAME -Dwas602.admin.password=$ADMINPASSWORD -Dportal.cluster.name=$CLUSTERNAME -Dportal.server.name=dummy -Dportal.profile.nodename=$NODENAME -Dportal.profile.cellname=$CELLNAME -Dportal.profile.name=$WAS_PROFILE -Dwbm.installType.PortalDashboard=installNewPortalDashboard -Dinstall.widgets.nativePortlets=true -Dwbm.install.abx.portalviews=$ABX_INSTALL -Dwas602.install.root=$WAS_LOC -Dwas.install.root=$WBM_LOC -Dportal.install.root=$WBM_LOC -Dportal.profile.path=$WAS_PROFILE_PATH -Dportal.profileType=deploymentManager -DtraceLevel=all -f $WBM_LOC/actions/ConfigABX.ant
if [ "$?" -ne "0" ]; then
  echo "check log ${WAS_LOC}/profiles/${WAS_PROFILE}/logs/deployABXservice.log"  
  exit 1
fi

# ******************************************************
echo "-"                            
echo "-"                           
echo "Add the Monitor Theme to Portal"
# ******************************************************

# ** get the portal ear
$WAS_PROFILE_PATH/bin/wsadmin.sh -user $ADMINUSERNAME -password $ADMINPASSWORD -conntype NONE -c "\$AdminApp export wps $WBM_LOC/temp/wptheme/wps.ear" 
if [ "$?" -ne "0" ]; then
   echo ""
   echo " Problem exporting the wps ear"
   exit 1
fi

# ** expand the ear
$WAS_LOC/bin/EARExpander.sh -ear  $WBM_LOC/temp/wptheme/wps.ear -operationDir $WBM_LOC/temp/wptheme/wps_expanded -operation expand
if [ "$?" -ne "0" ]
then
   echo ""
   echo " Problem expanding the wps ear"
   exit 1
fi

# ** add the monitor theme 
cp -R $WBM_LOC/theme/* $WBM_LOC/temp/wptheme/wps_expanded/wps.war/themes/html

# ** delete the exported ear
rm $WBM_LOC/temp/wptheme/wps.ear

# ** rebuild the ear
$WAS_LOC/bin/EARExpander.sh -ear  $WBM_LOC/temp/wptheme/wps.ear -operationDir $WBM_LOC/temp/wptheme/wps_expanded -operation collapse
if [ "$?" -ne "0" ]
then
   echo ""
   echo " Problem creating the new wps ear"
   exit 1
fi

# ** update the ear 
$WAS_PROFILE_PATH/bin/wsadmin.sh -user $ADMINUSERNAME -password $ADMINPASSWORD -conntype NONE -c "\$AdminApp install $WBM_LOC/temp/wptheme/wps.ear {-update -appname wps -nodeployejb}" 
if [ "$?" -ne "0" ]
then
   echo ""
   echo " Problem deploying the new wps ear"
   exit 1
fi

# ** update the mappings 
$WAS_PROFILE_PATH/bin/wsadmin.sh -user $ADMINUSERNAME -password $ADMINPASSWORD -conntype NONE -lang jython -f "$WBM_LOC/actions/updateServerMappings.py" 
if [ "$?" -ne "0" ]
then
   echo ""
   echo " Problem updating server mappings"
   exit 1
fi

 
echo "-"
echo " log files are located at /tmp/portal.dashboard/"
echo "-"                            
echo "-"   
