######################################################################
# Licensed Materials - Property of IBM
# 5724-M24
# Copyright IBM Corporation 2005, 2009. All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
######################################################################

import java

#
# imports for the logger
#
from com.ibm.wbmonitor.install.common.log import LogFactory
from com.ibm.wbmonitor.install.common.log import ILogger
from com.ibm.wbmonitor.install.common.log import ITracer
from com.ibm.wbmonitor.install.common.log import LogConstants

#
# set up the logger
#
trc = LogFactory.createTracer(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH)
log = LogFactory.createLogger(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH)
className = "uninstallABXPortalDashboard_py"



from java.io import ByteArrayOutputStream

#############################################################
# main
#
# Unconfigure the Alphablox Library / Plugin on
# the WAS server.  
#
#############################################################

methodName = "main"

trc.entry(className, methodName)

if ((len(sys.argv) != 1)):
        #print "This script requires 1 parameter: profilePath"
        #print "e.g.: <scriptname>.py /opt/WebSphere/Monitor/profiles/MonSrv01"
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "This script (uninstallABXPortalDashboard.py) requires 1 parameter: profilePath")
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "e.g.: uninstallABXPortalDashboard.py /opt/WebSphere/Monitor/profiles/MonSrv01")
	raise ValueError	

# get the profile path
profilePath = sys.argv[0]
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "profilePath="+profilePath)

# 
# Load the installation parameters 
#
execfile(profilePath+'/config.wbm/MonitorProfileVals.py')
execfile(moncommonPath+'/MonitorServerUtil.py')

#
# get parameters we need
#
lifeCycleDataSourceName = "MONITOR"
trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Preset Vales required for Alphablox datasource creation:")
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceJndi                = "+ datasourceJndi)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  jdbcProviderName              =" + jdbcProviderName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  clusterName                   =" + clusterName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  cellName                      =" + cellName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  nodeName                      =" + nodeName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  serverName                    =" + serverName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceName                =" + datasourceName)

trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  lifeCycleDataSourceName         =" + lifeCycleDataSourceName)

installStep = "Delete Alphablox datasource for Portal Dashboard"
success = "true"

log.msg(ILogger.TYPE_INFO, className, methodName, "CWMLP0010I", installStep)


UtilWsadminHolder.AdminConfig = AdminConfig
UtilWsadminHolder.AdminTask = AdminTask
    
trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Determine scope")
scope = Scope(clusterName, cellName, nodeName, serverName)    
trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Got scope")

try:
    trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Removing Datasource: " + datasourceName)
    
    trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Get config id...")
    configId = getDatasourceConfigId(scope, Scope.CELL_SCOPE, jdbcProviderName, datasourceName)
    trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Got config id")
    if (configId == None):
        trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Datasource: " + datasourceName + " not found.")
    else:
        trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Found datasource: " + datasourceName + " with config id: " + configId + ". Removing...")
	AdminConfig.remove(configId)
        trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Datasource: " + datasourceName + " successfully removed.")
    
    AdminConfig.save()

except:
    # Exception occured... failed to delete ABX datasource
    success = "false"
    trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Exception caught removing Datasource: " + datasourceName)


try:
    trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Removing Datasource Provider: " + jdbcProviderName)
    
    trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Get config id...")
    configId = getJdbcProviderConfigId(scope, Scope.CELL_SCOPE, jdbcProviderName)
    if (configId == None):
        trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Datasource provider: " + jdbcProviderName + " not found.")
    else:
        trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Found datasource provider: " + jdbcProviderName + " with config id: " + configId + ". Removing...")
       	AdminConfig.remove(configId)
        trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Datasource Provider: " + jdbcProviderName + " successfully removed." )
  
    AdminConfig.save()
      
except:
    # Exception occured... failed to delete ABX datasource provider
    success = "false"
    trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Exception caught removing Datasource provider: " + jdbcProviderName)
  

try:
    # Remove WAS Environment variable WBM_JDBC_DRIVER_PATH
    trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Removing WAS Environment variable WBM_JDBC_DRIVER_PATH...")
    
    execfile(moncommonPath+'/setWASVar.py')
    removeEnvironment(cellName,None,serverName,"WBM_JDBC_DRIVER_PATH")
    removeEnvironment(cellName,nodeName,serverName,"WBM_JDBC_DRIVER_PATH")
    trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "WAS Environment variable WBM_JDBC_DRIVER_PATH successfully removed")

    AdminConfig.save()

except:
    # Exception occured... failed to remove variable
    success = "false"
    trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Error removing variable WBM_JDBC_DRIVER_PATH.")


if (success == "true"):
    # successful completion
    log.msg(ILogger.TYPE_INFO, className, methodName, "CWMLP0011I", installStep)
else:
    # successful completion
    log.msg(ILogger.TYPE_ERROR, className, methodName, "CWMLP2130E", installStep, profilePath+"/logs/uninstallABXPortalDashboard.log")
    


trc.exit(className, methodName)

