#  Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2010. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#

import os
import sys
import java
from java.io import ByteArrayOutputStream 


#-------------------------------------------------------------------------------
# This jython script installs the referenced EAR
#-------------------------------------------------------------------------------

def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split(" ")
        else:
                tmpList = inStr.split("\n")  #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();        #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#endDef

#######################################
# updateEar subroutine
# location - location where the EAR resides
# earName - name of the ear in the location directory
# appName - name of the Application
#######################################
def updateEar ( location, earName, appName ):
        rc = 0;
        print "INFO:  Entering updateEar"
        print "INFO:  location="+location+",earName="+earName+",appName="+appName

        #--------------------------------------------------------------
        # set up globals
        #--------------------------------------------------------------
        global AdminConfig
 

        #---------------------------------------------------------
        # First, list the existing applications 
        #---------------------------------------------------------
        apps = AdminApp.list( )
        print "INFO:  Installed applications: "
        print apps

        # Check if App is already installed on the target server
        # Comparing the application name.
        appExist = "False"
        for tmpAppName in wsadminToList(apps):
       	        if (cmp(tmpAppName, appName) == 0):
               	        appExist = "True"
                       	break 
                #endIf 
        #endFor 
        if (cmp(appExist, "True") == 0):
                print 
                print "INFO:  Found "+earName+".ear  installed."
        else:
           print "ERROR:  Did not find "+earName+".ear installed."
           print "ERROR:  Did not find "+earName+".ear installed."
           return -1

        #---------------------------------------------------------
        # Here is the ear file we'll be dealing with... 
        #---------------------------------------------------------
        defapp = location+"/"+earName+".ear"

        #endIf
         
        #---------------------------------------------------------
        # Update 
        #---------------------------------------------------------
        print "INFO:  Update Application "+appName+" beginning."
        try:
            AdminApp.update(appName, 'app', ["-operation", "update", "-contents", defapp] )
        except java.lang.Exception,e: 
            print "Exception occurs in AdminApp.update.",e
            return -1

        # check for success
        appExist = "False"
        for tmpAppName in wsadminToList(AdminApp.list()):
            if (cmp(tmpAppName, appName) == 0):
                appExist = "True"
                break 
            #endIf 
        #endFor 
        if (cmp(appExist, "True") == 0):
            print "INFO:  Application "+appName+" successfully updated."
            AdminConfig.save( )
        else:
            print "ERROR: Application "+appName+" could not be updated."
            rc = -1
        #endElse 

        AdminConfig.save( )
        print "INFO:  Exiting updateEar"
        return rc
#endDef 
#######################################
# setAllAuthenticated subroutine
# app - name of the application 
# role - name of the user role
#######################################
def setAllAuthenticated ( app, role ):
    rc = 0
    print "INFO:  Entering setAllAuthenticated"
    print "INFO:  app="+app+",role="+role

    #--------------------------------------------------------------
    # set up globals
    #--------------------------------------------------------------
    global AdminConfig
    global AdminApp
 
    #---------------------------------------------------------
    # A very simple edit command 
    #---------------------------------------------------------
    print "INFO:  Set the AllAuthenticated Security to role: "+role+" for the "+app+" application."
    try:
        AdminApp.edit(app, '[ -MapRolesToUsers [[ '+role+' AppDeploymentOption.No AppDeploymentOption.Yes "" "" ]]]' )
        print "INFO:  AllAuthenticated set successfully for the "+role+" role for the "+app+" application."
        AdminConfig.save( )
    except:
        print "INFO:  Exception caught executing AdminApp.edit() to set the AllAuthenticated Security to role: "+role+" for the "+app+" application."
        rc = -1
    
    print "INFO:  Exiting setAllAuthenticated"            
    return rc
#endDef 

#-----------------------------------------------------------------
# Main
#-----------------------------------------------------------------
print "updateABXears"
   # need the location of ears as a parameter     
rc = 0
if ((len(sys.argv) != 1)):
        print "updateABXears: this script requires the location of WebSphere as a parameter"
        print "e.g.:     updateABXears.py /opt/WebSphere/AppServer"
        print "1 FatalErrors:" 
        rc = 1
if(rc == 0):
        location = sys.argv[0]+"/Alphablox/installableApps"        
        try:        
            rc =  updateEar(location, "AlphabloxPlatform","AlphabloxPlatform" )
            if(rc == 0):
                rc =  updateEar(location, "ApplicationStudio", "ApplicationStudio")
             
            if (rc != 0):
                print "1 FatalErrors:" 
                print " Failed to update Alphablox ears"   
        except java.lang.Exception,e:
            print "1 FatalErrors:" 
            print " Failed to update Alphablox:",e
            raise


