######################################################################
#  Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2005, 2009. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
######################################################################

import java

#
# imports for the logger
#
from com.ibm.wbmonitor.install.common.log import LogFactory
from com.ibm.wbmonitor.install.common.log import ILogger
from com.ibm.wbmonitor.install.common.log import ITracer
from com.ibm.wbmonitor.install.common.log import LogConstants

#
# set up the logger
#
trc = LogFactory.createTracer(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH)
log = LogFactory.createLogger(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH) 
className = "installHumanTask_py"

#############################################################
# main
#
# Install & Configure the REST Service Proxy Application on
# The WAS 6.0.2 server.  
#
#############################################################

methodName = "main"

trc.entry(className, methodName)

if ((len(sys.argv) != 1)):
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "This script (installHumanTask.py) requires 1 parameter: profilePath")
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "e.g.: installHumanTask.py /opt/WebSphere/Monitor/profiles/MonSrv01")
        raise ValueError

# get the profile path
profilePath = sys.argv[0]
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "profilePath="+profilePath)

# Load the installation parameters 
execfile(profilePath+'/config.wbm/MonitorProfileVals.py')

# parameters needed from presets
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Parameters ==> clusterName="+clusterName+", cellName="+cellName+", nodeName="+nodeName+", serverName="+serverName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Parameters ==> wpsHost="+wpsHost+", wpsRMIPort="+wpsRMIPort)

installStep = "Portal Dashboard HumanTask"
log.msg(ILogger.TYPE_INFO, className, methodName, "CWMLP0010I", installStep)

execfile(moncommonPath+'/MonitorServerUtil.py')    
# set custom jvm properties for human task
jvmProperties = {}
jvmProperties["DashboardBPCHost"] = wpsHost
jvmProperties["DashboardBPCRMIPort"] = wpsRMIPort
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Custom Property jvmProperties[DashboardBPCHost]    : " + jvmProperties["DashboardBPCHost"])
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Custom Property jvmProperties[DashboardBPCRMIPort] : " + jvmProperties["DashboardBPCRMIPort"])
    
cellName = AdminConfig.showAttribute(AdminConfig.list("Cell"), "name")
UtilWsadminHolder.AdminConfig = AdminConfig
UtilWsadminHolder.AdminTask = AdminTask

# add default port if not set
if(wpsRMIPort == ""):
   wpsRMIPort = "2809"

# Define a JVM custom property.
if(wpsHost != ""):
   try:
      # we will not do anything when a managed node
      if (ndType == "managed"):
         trc.exit(className, methodName)
      elif (ndType == "standalone"):
         # configure with custom jvm properties for human task
         scope = Scope("", cellName, nodeName, serverName)    
         trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: scope=" + str(scope))
         trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Setting Custom Properties.")
         setSystemProperties(scope, jvmProperties)
         trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Custom Properties created.")
         trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Save the configuration changes")
         AdminConfig.save()         
      elif (ndType == "dmgr"):
         # Create our system properties for node/server              
         cId = AdminConfig.getid("/ServerCluster:"+clusterName+"/" )
         cList = AdminConfig.list("ClusterMember", cId ).split(lineSeparator)
         for sId in cList:
            cMemName = AdminConfig.showAttribute(sId, "memberName" )
            cMemNodeName = AdminConfig.showAttribute(sId, "nodeName" ) 
            memScope =   Scope(clusterName, cellName, cMemNodeName, cMemName)            		
            trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Create System Properties for: "+cMemName+":"+cMemNodeName , jvmProperties)
            setSystemProperties(memScope, jvmProperties)
         trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Save the configuration changes")
         AdminConfig.save()            
   except:    
      trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "ERROR: AdminControl may not be available.")
      raise

log.msg(ILogger.TYPE_INFO, className, methodName, "CWMLP0011I", installStep)
trc.exit(className, methodName)

