######################################################################
#
#  Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2008. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#  
######################################################################

import java

#
# imports for the logger
#
from com.ibm.wbmonitor.install.common.log import LogFactory
from com.ibm.wbmonitor.install.common.log import ILogger
from com.ibm.wbmonitor.install.common.log import ITracer
from com.ibm.wbmonitor.install.common.log import LogConstants

#
# set up the logger
#
trc = LogFactory.createTracer(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH)
log = LogFactory.createLogger(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH)
className = "uninstallPortalDashboardRestProxy_py"


#############################################################
# unconfigureProxy subroutine
#
# Remove the REST Service Proxy custom JVM properties.
#
#############################################################
def unconfigureProxy (clusterName, nodeName, proxyServer):
       
	methodName = "unconfigureProxy"

	trc.entry(className, methodName)
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Parameters passed ==> clusterName="+clusterName+", nodeName="+nodeName+", proxyServer="+proxyServer)

	# Set custom JVM porperty to enable proxy to find the rest services host
	#
	# Application servers > WebSphere_Portal > Process Definition > Custom Properties > 
	#
		
	UtilWsadminHolder.AdminConfig = AdminConfig
	UtilWsadminHolder.AdminTask = AdminTask
	
	cellName = AdminConfig.showAttribute(AdminConfig.list("Cell"), "name")
	
	# define a scope object
	scope = Scope(clusterName, cellName, nodeName, serverName)

	systemProperties = {}
        systemProperties["DashboardRESTServiceTarget"] = restServicesHost +":"+ restServicesPort
        systemProperties["DashboardRESTServiceSecureTarget"] = restServicesHost +":"+ restServicesSecurePort
        systemProperties["BusinessSpaceTarget"] = bspaceServicesHost +":"+ bspaceServicesPort
        systemProperties["BusinessSpaceSecureTarget"] = bspaceServicesHost +":"+ bspaceServicesSecurePort
	
	try:
	
		# remove system properties		
		trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Remove System Properties: ", systemProperties)
		removeSystemProperties(scope, systemProperties)
		trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: System Properties removed successfully.")
		
		trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Save the configuration changes")
		AdminConfig.save()
	except:
		trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "ERROR: Unable to remove System Properties: ", systemProperties)
       
	trc.exit(className, methodName)
        return

#enddef



#############################################################
# main
#
# Install & Configure the REST Service Proxy Application on
# The WAS 6.0.2 server.  
#
#############################################################

methodName = "main"

trc.entry(className, methodName)

if ((len(sys.argv) != 1)):
        #print "This script requires 1 parameter: profilePath"
        #print "e.g.: <scriptname>.py /opt/WebSphere/Monitor/profiles/MonSrv01"
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "This script (installRestServiceProxy.py) requires 1 parameter: profilePath")
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "e.g.: uninstallPortalDashboardRestProxy.py /opt/WebSphere/Monitor/profiles/MonSrv01")
        raise ValueError	

# get the profile path
profilePath = sys.argv[0]
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "profilePath="+profilePath)


# 
# Load the installation parameters 
# Problem resolving path?? with blanks??
#
execfile(profilePath+'/config.wbm/MonitorProfileVals.py')

#
# get parameters we need
#
app            = restproxyAppName
node           = nodeName
cluster        = clusterName
proxyServer    = serverName

        
# load the utility scripts to uninstall the application ear
execfile(moncommonPath+'/uninstallEar.py')
execfile(moncommonPath+'/MonitorServerUtil.py')


# unconfigure REST Services Proxy
unconfigureProxy(cluster, node, proxyServer)

installStep = "Uninstall "+ app

# uninstall EAR
try:
	log.msg(ILogger.TYPE_INFO, className, methodName, "CWMLP0010I", installStep)
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Uninstalling REST Services Proxy App...")
	uninstallEar(app)
	log.msg(ILogger.TYPE_INFO, className, methodName, "CWMLP0011I", installStep)
except:
	log.msg(ILogger.TYPE_ERROR, className, methodName, "CWMLP2130E", installStep, profilePath+"/logs/uninstallPortalDashboardRestProxy.log")


trc.exit(className, methodName)
       