/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.api.ServerGenerated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.api.ServerGenerated"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.api.ServerGenerated");

// API for resources that have server-generated values that are only known
// after the item is saved to the server and then updated with the server-
// generated values on the response.
// Is useful for checking if an item has client-side temporary values
// because the resource may not have been created on the server yet,
// and therefore those values are unknown to the client for the 
// particular item.  An example could be a unique ID which is generated
// on the server when an item is created, but the client has not
// yet sent the request to the server to create the item, but simply has
// a client-side stub for the item.


dojo.declare("com.ibm.data.api.ServerGenerated",
	null,
	{
		isCreated: function(/*item*/ item) {
			// summary: Indicates whether or not the item has been created on the server and
			//		the server-generated values were properly applied to the client-side
			//		object.	NOTE: Any client-side modifications	to the values of an item for 
			// 		which isCreated returns true can potentially modify the server-generated 
			// 		values to be out-of-sync with the server.
			// 		Therefore, it's important to note that isCreated merely indicates whether
			//		or not the client-side item has a server-side resource associated with it,
			// 		and any values that the server generated for the resource (such as a unique
			// 		id) were applied to the client-side item object when it was saved to the server.
			// item: Item in the store
		},
		addOnCreate: function(/*item*/ item, /*Function*/ callback) {
			// summary: Register callback to be called when the given item is created on the server.
			//		If the item is already created, this will call the callback immediately.
			// item: Item in the store
			// callback: Function to be called when this item's onCreate event is fired, due to the
			//		process of creating this item on the server being completed successfully.
		},
		getUniqueCID: function() {
			// summary: Returns a uniquely generated CID that can be used in new items for this store.
		},
		registerCID: function(/*String*/ cid, /*item*/ item) {
			// summary: Registers a CID with an item in this store so that lookups by that CID are possible.
		},
		byCID: function(/*String*/ cid) {
			// summary: Looks up an item in this store by a given CID and returns that item if it exists,
			//		otherwise returns null.
		}
	}
);

}
