/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.resolver.ResolverQueryRules"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.resolver.ResolverQueryRules"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.resolver.ResolverQueryRules");

dojo.require("com.ibm.query.rules.Atom");
dojo.require("com.ibm.query.rules.OpenSearch");

com.ibm.data.resolver.ResolverQueryRules = {
	
	namespaces: dojo.mixin({}, com.ibm.query.rules.OpenSearch.namespaces, {
        "xsl" : "http://www.w3.org/1999/XSL/Transform",
        "thr" : "http://purl.org/syndication/thread/1.0",
        "xhtml" : "http://www.w3.org/1999/xhtml",
        "service": "http://www.ibm.com/xmlns/prod/sw/model/service/1.0",
        "ac": "http://www.ibm.com/xmlns/prod/lotus/access-control/v1.0",
        /*
        "model" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model-elements",
        "base" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0/ibm-portal-composite-base",
        "ext" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model",
        "creationcontext" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1.0/portal-creationcontext",
        */
        "xsi" : "http://www.w3.org/2001/XMLSchema-instance"
    }),
	
	// atom rules are meant to be used with XML query plugin in conjunction with atom feed data
	atom: {
		modelExt: {
			modelNodeType: {selector: "./atom:content/*", field: "$localName", readOnly: true}
		},
		uniqueNameExt: {
			uniqueName: {selector: "./atom:id[@ext:uniquename]", field: "@ext:uniquename", setter: 
				function(data, rule, value){
					this.createFromJson(data, {
						name: "atom:id",
						attributes: {
							"ext:uniquename": value
						}
					});
					return true;
				}
			}
		},
		threadExt: {
			parent: {selector: "./thr:in-reply-to", field: "@ref", type: "Item"},
			parentId: {selector: "./thr:in-reply-to", field: "@ref"},
			parentURI: {selector: "./thr:in-reply-to", field: "@href"}
		},
		treeExt: {
			// for full tree support, combine this with the threadExt map also
			childrenURI: {selector: "./atom:link[@rel='replies']", field: "@href", readOnly: true},
			next: {selector: "./atom:link[@rel='next']", field: "@ext:uri", type: "Item", unset: "!"},
			nextId: {selector: "./atom:link[@rel='next']", field: "@ext:uri", unset: "!"},
			nextURI: {selector: "./atom:link[@rel='next']", field: "@href", unset: "!"}
		},
		L10nExt: {
			// localizedTitle and localizedDescription are parameterized by language code
			localizedTitle$: {selector: "./atom:content/*[model:title]/model:title/base:nls-string[" + (dojo.isIE ? "@xml:lang = '${0}'": "lang('${0}')") + "]", field: "$textContent", locale: "${0}"},
			localizedDescription$: {selector: "./atom:content/*[model:description]/model:description/base:nls-string[" + (dojo.isIE ? "@xml:lang = '${0}'": "lang('${0}')") + "]", field: "$textContent", locale: "${0}"}
		},
		acExt: {
			userOwned: {selector: "./atom:content/*/ac:allowed-access", field: "@ac:user-owned"},
			accessLevel: {selector: "./atom:content/*/ac:allowed-access/ac:access-level", field: "@ac:type"},
			accessLevel$: {selector: "./atom:content/*/ac:allowed-access/ac:access-level[@ac:type='${0}']", field: "@ac:type", acType: "${0}"}
		},
		operationsExt: {
			contextMenu: {selector: "./atom:link[@ext:rel='contextMenu']", field: "@ext:uri", readOnly: true},
			contextMenuURI: {selector: "./atom:link[@ext:rel='contextMenu']", field: "@href", readOnly: true}
		},
		contentExt: {
			layoutURI: {selector: "./atom:link[@ext:rel='layout']", field: "@href", readOnly: true},
			theme: {selector: "./atom:link[@ext:rel='theme']", field: "@ext:uri", setter: function(data, rule, value) {
				this.createFromJson(data, {
					name: "atom:link",
					attributes: {
						"ext:rel": "theme",
						"ext:uri": value,
						"href": value
					}
				});
				return true;
			}}			
		},
		metaExt: {
			metadata$: {selector: "./atom:content/*/model:metadata[@name='${0}']/base:value", field: "@value", mdname: "${0}", deferSet: true, setter: 
				function(data, rule, value, multiValued){
					var valueDef = {
						name: "base:value",
						attributes: {
							"xsi:type": "base:String"
						}
					};
					
					var modelNode = this.select("./atom:content/*", data, true);
					var metaNode = modelNode.select("model:metadata[@name='" + rule.mdname + "']", true);
					if(metaNode.length == 0) {
						modelNode.createFromJson({
							name: "model:metadata",
							attributes: {
								"name": rule.mdname
							}
						});
					}
					metaNode = modelNode.select("model:metadata[@name='" + rule.mdname + "']", true);
					metaNode.select("*").destroy();	// clear existing values
					if(multiValued) {
						dojo.forEach(value, function(val){
							valueDef.attributes.value = val;
							metaNode.createFromJson(valueDef);
						}, this);
					}
					else {
						valueDef.attributes.value = value;
						metaNode.createFromJson(valueDef);
					}
					
					return true;
				}
			}
		}
	}
};

// minimize code
(function(){
	var atomBase = com.ibm.data.resolver.ResolverQueryRules.atom;
	
	var atomParentSetter = function(data, rule, value, multiValued){
		var nodeSet = this.select(rule.selector, data, false);	// XmlNodeList
		if(nodeSet.length == 0) {
			this.createFromJson(data, {
				name: "thr:in-reply-to",		// change to atom:link style
				attributes: {
					"ext:uri": value,
					"type": "application/atom+xml",
					"ref": value,
					"href": value
				}
			});
		}
		else {
			nodeSet.attr("ext:uri", value).attr("ref", value).attr("href", value);
		}
		return true;
	};
	
	atomBase.threadExt.parent.setter = atomParentSetter;
	atomBase.threadExt.parentId.setter = atomParentSetter;
	atomBase.threadExt.parentURI.setter = atomParentSetter;
	
	atomBase.threadExt.parent.deferSet = true;
	atomBase.threadExt.parentId.deferSet = true;
	atomBase.threadExt.parentURI.deferSet = true;
	
	var atomNextSetter = function(data, rule, value){
		var attrs = {
			"ext:uri": value,
			"rel": "next", 
			"type": "application/atom+xml"
		};
		attrs[rule.field.substr(1)] = value;
		this.createFromJson(data, {
			name: "atom:link",
			attributes: attrs
		});
		return true;
	};
	
	atomBase.treeExt.next.setter = atomNextSetter;
	atomBase.treeExt.nextId.setter = atomNextSetter;
	atomBase.treeExt.nextURI.setter = atomNextSetter;
	
	var atomL10nExtSetter = function(nodeName, data, rule, value){
		var contentNode = this.select("./atom:content/*", data, true);
		var titleNode = contentNode.select("model:title", true);
		var stringNodeDef = {
			name: "base:nls-string",
			attributes: {
				"xml:lang": rule.locale
			},
			children: [value]
		};
		
		if(titleNode.length == 0) {
			contentNode.createFromJson({
				name: nodeName,
				children: [stringNodeDef]
			});
		}
		else {
			titleNode.createFromJson(stringNodeDef);
		}
		return true;
	};
	
	atomBase.L10nExt.localizedTitle$.setter = dojo.partial(atomL10nExtSetter, "model:title");
	atomBase.L10nExt.localizedDescription$.setter = dojo.partial(atomL10nExtSetter, "model:description");
	
}());

}
