/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.resolver.tree.TreeModeledHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.resolver.tree.TreeModeledHandler"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.resolver.tree.TreeModeledHandler");

// Module for managing tree-modeled data from a particular POC handler.
// Handler implementations can mix this in to get special support for
// tree modeled attributes.  This is written to support extensions of
// the QueryRulePocHandler by default, but in concept can support
// any POC handler which provides a cacheContext on each item and a
// childrenCacheKey.

dojo.require("com.ibm.data.ItemsLoader");
dojo.require("com.ibm.data.resolver.strategy.UpdateStrategy");

dojo.declare("com.ibm.data.resolver.tree.TreeModeledHandler",
	null,
	{
		// parentMgmtStrategy: 
		//		com.ibm.strategy.Strategy that manages dependencies between parents
		//		and children operations in the tree model
		parentMgmtStrategy: "com.ibm.data.resolver.tree.ParentDependencyStrategy",
		
		// siblingMgmtStrategy: 
		//		com.ibm.strategy.Strategy that manages dependencies between siblings
		//		in the tree model
		siblingMgmtStrategy: "com.ibm.data.resolver.tree.SiblingDependencyStrategy",
		
		// cacheContext: Property on the item which is used to store cached values
		cacheContext: "_cachedMeta",
		
		// childrenCacheKey: Property on an item's cacheContext which is used to store
		//		the children of that item
		childrenCacheKey: "children",
		
		constructor: function() {
			this.customAttributes = dojo.mixin({}, this.customAttributes, {
				mayHaveChildren: {
					get: function(item, multiValued) {
						var children = item[this.cacheContext][this.childrenCacheKey];
						if(children && children.loaded) return children.length > 0;
						if(item[this.cacheContext].mayHaveChildren == true) return true;
						return this.getValue(item, "childrenURI") != null;
					},
					set: function(item, value, multiValued) {
						// allow store seeding with this attribute
						if(value === 1) item[this.cacheContext].mayHaveChildren = true;
						// can't write to this attribute as it is derived from others
						return false;
					}
				},
				children: {
					get: function(item, multiValued) {
						// returns ItemsLoader
						if(!item._cachedMeta.childrenLoader) {
							var query = this.getChildrenQuery(item);
							
							item._cachedMeta.childrenLoader = new com.ibm.data.ItemsLoader({
								store: this.resolver,
								query: query,
								fetchOnReload: true,
								cacheContext: item[this.cacheContext],
								cacheProperty: this.childrenCacheKey,
								filter: dojo.hitch(this, "filterChildren", item)
							});
						}
						return multiValued ? [item._cachedMeta.childrenLoader] : item._cachedMeta.childrenLoader;
					},
					set: function(item, value, multiValued) {
						if(dojo.isArray(value) && multiValued && value.preloaded) {
							// leverage to preload children... usually not used for normal
							// processing besides seeding the store with items
							item[this.cacheContext][this.childrenCacheKey] = value.slice();
						}
						return false;	// writing to this should never flag the item dirty
					}
				},
				treeModel: {
					get: function(item, multiValued) {
						if(!item._cachedMeta.treeModel) {
							item._cachedMeta.treeModel = new com.ibm.data.resolver.tree.PocTreeModel({
								root: item,
								handler: this,
								resolver: this.resolver
							});
						}
						return multiValued ? [item._cachedMeta.treeModel] : item._cachedMeta.treeModel;
					},
					set: function(item, value, multiValued) {
						// can't write to this attribute
						return false;
					}
				}
			});
			
			
			com.ibm.strategy.Strategy.applyStrategy(this, new (dojo.getObject(this.parentMgmtStrategy))());
			com.ibm.strategy.Strategy.applyStrategy(this, new (dojo.getObject(this.siblingMgmtStrategy))());
			
		},
		
		processFetch: function(/*DOMDoc*/ xmlDoc, /*Request*/ request) {
			// summary: Extension point for descendants or consumers of this store to handle
			//		responses from server requests for data.  Most subclasses can leave this
			// 		function as-is, but if special handling is required that cannot be done
			// 		with other extension points, this can be overridden as well.
			// xmlDoc: DOMDoc of the response from an IO request
			// request: Request used to fetch the result
			// returns: Object map with two properties:
			//		items: Array of the items fetched
			//		meta: Object map of meta data associated with the request
			var map = this.inherited(arguments);
			
			if(request.query.levels > 1 || request.query.levels == "all") {
				var topItem = this.resolver.byId(request.query.uri);
				dojo.forEach(map.items, function(item){
					
				}, this);
			}
			return map;
		},
		
		updateReferences: function(item) {
			// see the QueryRulePocHandler@updateReferences
			this.inherited(arguments);
			var parentId = this.getValue(item, "parentId");
			if(parentId) {
				// get the "real" id of my parent, which at this point should no longer be a cid
				var parentItem = this.resolver.byId(parentId);
				if(parentItem) this.setValue(item, "parentId", this.resolver.getIdentity(parentItem));
			}
		},
		
		modifyChildren: function(children) {
			// summary: Updates the last modified time of the children array so consumers will
			//		know when the children have been updated last.
			children._lastModified = (new Date()).getTime();
		},
		
		getChildrenQuery: function(item) {
			return this.getValue(item, "childrenURI", 
				dojo.objectToQuery({
					uri: this.getIdentity(item),
					levels: 2,
					mode: "download"
				})
			);
		},
		
		filterChildren: function(parentItem, items) {
			var id = this.getIdentity(parentItem);
			items = dojo.filter(items, function(item){
				// only push the results whose parent has the same id as parentItem
				return id == this.resolver.getValue(item, "parentId");
			}, this);
			items.loaded = true;
			return items;
		}
	}
);

dojo.declare("com.ibm.data.resolver.tree.ParentDependencyStrategy",
	com.ibm.data.resolver.strategy.UpdateStrategy,
	{
		extensions: com.ibm.data.resolver.strategy.UpdateStrategy.prototype.extensions.concat(["processFetch", "newItem", "createStub"]),
		
		extensionsMap: dojo.mixin({}, com.ibm.data.resolver.strategy.UpdateStrategy.prototype.extensionsMap, {
			"newItem": "updateNewItem",
			"createStub": "updateNewItem"
		}),
		
		_setValue: function(item, attribute, value) {
			// wraps setValue and setValues... so check both
			var success = false;
			if(attribute == "parent" || attribute == "parentId" || attribute == "parentURI") {
				var multi = this.getStrategyContext().joinPoint.targetName == "setValues";
				var oldValue = this.handler[multi ? "getValues" : "getValue"](item, attribute),
					newValue = value;
				success = this.proceed(arguments);
				if(success) {
					if(multi) {
						oldValue = oldValue ? oldValue[0] : null;
						newValue = newValue ? newValue[0] : null;
					}
					if(attribute == "parent") {
						if(oldValue) oldValue = this.handler.getIdentity(oldValue);
						if(newValue) newValue = this.handler.getIdentity(newValue);
					}
					this._changeDependencies(this.handler.getIdentity(item), newValue, oldValue);
				}
			}
			else {
				success = this.proceed(arguments);
			}
			
			return success;
		},
		
		_checkAndUpdateDependencies: function(item) {
			// summary: Utility to check and update the parent dependency of an item
			var pId = this.handler.getValue(item, "parentId");
			if(pId) {
				this._changeDependencies(this.handler.getIdentity(item), pId);
			}
		},
		
		updateNewItem: function() {
			// summary: Direct wrapping
			var item = this.proceed(arguments);
			this._checkAndUpdateDependencies(item);
			return item;
		},
		
		processFetch: function() {
			// summary: Direct wrapping
			var map = this.proceed(arguments);
			dojo.forEach(map.items, function(item){
				this._checkAndUpdateDependencies(item);
			}, this);
			return map;
		},
		
		_changeDependencies: function(childId, newParentId, oldParentId) {
			// summary: Utility to update dependencies
			if(oldParentId == newParentId) return;
			if(oldParentId) {
				this.handler.depMgr.removeDependency(oldParentId, childId, com.ibm.data.DependencyManager.types["delete"]);
				this.handler.depMgr.removeDependency(childId, oldParentId, com.ibm.data.DependencyManager.types["create"]);
			}
			if(newParentId) {
				this.handler.depMgr.addDependency(newParentId, childId, com.ibm.data.DependencyManager.types["delete"]);
				this.handler.depMgr.addDependency(childId, newParentId, com.ibm.data.DependencyManager.types["create"]);
			}
		}
	}
);

dojo.declare("com.ibm.data.resolver.tree.SiblingDependencyStrategy",
	com.ibm.data.resolver.strategy.UpdateStrategy,
	{
		// summary: Manages dependencies in ordering when next attribute values are updated
		//		on items, particularly when the next value is changed from or to an item
		//		that is not created yet.
		
		extensions: com.ibm.data.resolver.strategy.UpdateStrategy.prototype.extensions.concat(["processSave"]),
		
		// blocked: Boolean indicating whether or not to handle setValue/setValues calls
		//		by this strategy.  This strategy can block itself so that it does not perform
		//		normal advising of these calls in certain circumstances, such as cases where
		//		infinite loops are possible or when dependency management is already performed
		//		and does not need to be performed twice.
		blocked: false,
		
		odepType: com.ibm.data.DependencyManager.types["order"],
		cdepType: com.ibm.data.DependencyManager.types["create"],
		
		processSave: function(/*Item*/ item) {
			this.blocked = true;
			this.handler.setValue(item, "nextId", this._getEffectiveNextId(item));
			this.blocked = false;
			return this.proceed(arguments);
		},
		
		_getEffectiveNextId: function(item) {
			if(!item) return null;
			var next = this.handler.getValue(item, "next");
			if(next) {
				if(!this.handler.resolver.isCreated(next)) {
					return this._getEffectiveNextId(next);
				}
				else return this.handler.getIdentity(next);
			}
			else return null;
		},
		
		_setValue: function(item, attribute, value) {
			// wraps setValue and setValues... so check both
			var success = false;
			if(!this.blocked && (attribute == "next" || attribute == "nextId")) {
				var multi = this.getStrategyContext().joinPoint.targetName == "setValues";
				var oldValue = this.handler[multi ? "getValues" : "getValue"](item, attribute),
					newValue = value;
				success = this.proceed(arguments);
				if(success) {
					if(multi) {
						oldValue = oldValue ? oldValue[0] : null;
						newValue = newValue ? newValue[0] : null;
					}
					if(attribute == "next") {
						if(oldValue) oldValue = this.handler.getIdentity(oldValue);
						if(newValue) newValue = this.handler.getIdentity(newValue);
					}
					this._changeDependencies(this.handler.getIdentity(item), newValue, oldValue);
				}
			}
			else {
				success = this.proceed(arguments);
			}
			
			return success;
		},
		
		_changeDependencies: function(id, newNextId, oldNextId) {
			// summary: Utility
			if(oldNextId == newNextId) return;
			// dependencies here go from source -> target like next -> previous because of
			// ordering constraints						
			this._remove(id);
			this._insert(id, newNextId);			
		},
		
		_remove: function(id) {
			// summary: Remove an item with id from an abstract ordered list denoted
			//		by order dependency relationships
			var targetDeps = this.handler.depMgr.getTargetsByType(id, this.odepType);
			var sourceDeps = this.handler.depMgr.getSourcesByType(id, this.odepType);
			
			// relink dependencies in updated list (A <- B <- C becomes A <- C)
			
			// add links (B <- C becomes A <- C)
			dojo.forEach(targetDeps, function(target){
				dojo.forEach(sourceDeps, function(source){
					this.handler.depMgr.addDependency(source, target, this.odepType);
				}, this);
				this.handler.depMgr.removeDependency(id, target, this.odepType);
			}, this);
			
			// remove dependencies in updated list (A <- B gets removed)
			dojo.forEach(sourceDeps, function(source){
				this.handler.depMgr.removeDependency(source, id, this.odepType);
			}, this);
		},
		
		_insert: function(id, nextId) {
			// summary: Insert an item with id into an abstract ordered list denoted
			//		by order dependency relationships
			if(!nextId) {
				var createDeps = this.handler.depMgr.getSourcesByType(this.handler.getValue(this.handler.resolver.byId(id), "parentId"), this.cdepType);
				
				dojo.forEach(createDeps, function(depId){
					if(this.handler.resolver._getHandlerByUri(depId) == this.handler) {
						var item = this.handler.resolver.byId(depId);
						if(!this.handler.getValue(item, "nextId")) {
							this.handler.depMgr.addDependency(id, depId, this.odepType);
						}
					}
				}, this);
			}
			else {
				var targetDeps = this.handler.depMgr.getTargetsByType(nextId, this.odepType);
				
				// move dependencies (A <- C becomes A <- B <- C)
				dojo.forEach(targetDeps, function(target){
					this.handler.depMgr.removeDependency(nextId, target, this.odepType);
					this.handler.depMgr.addDependency(id, target, this.odepType);
				}, this);
				
				this.handler.depMgr.addDependency(nextId, id, this.odepType);
			}
		}
	}
);


dojo.declare("com.ibm.data.resolver.tree.AtomModelTreeStrategy",
	com.ibm.data.resolver.strategy.UpdateStrategy,
	{
		// summary: Strategy used to manage data referring to relative position information
		// 		in a tree model in the remote ATOM feeds.  This strategy takes care to set
		//		data correctly in the reflected ATOM XML document when data APIs are called
		//		to change attribute values on items.  It also understands and parses the
		//		semantics of fetched data with regards to parent-children relationships
		//		and associates children in a chuck of data with their respective parents.
		
		extensions: com.ibm.data.resolver.strategy.UpdateStrategy.prototype.extensions.concat(["processFetch"]),
		
		_setValue: function(item, attribute, value) {
			var success = false;
			switch(attribute) {
				case "next":
				case "nextId":
					if(!value) {
						success = this.handler.unsetAttribute(item, attribute);
					}
					else success = this.proceed(arguments);
					if(success) {
						// parent is required in data when setting the next sibling
						var parentId = this.handler.getValue(item, "parentId");
						if(parentId) this.handler.setValue(item, "parentId", parentId);
					}
					break;
				default:
					success = this.proceed(arguments);
			}
			return success;
		},
		
		processChildren: function(/*Object*/ map, /*String|Number*/ levels, /*String*/requestURI) {
			if(levels > 1 || levels == "all") {
				var treeMap = {};
				dojo.forEach(map.items, function(item){
					var id = this.handler.getIdentity(item),
						parentId;
					treeMap[id] = {
						item: item
					};
					if(id != requestURI) {
						parentId = this.handler.getValue(item, "parentId");
						if(treeMap[parentId]) {
							treeMap[id].level = treeMap[parentId].level + 1;
							if(treeMap[parentId].children) treeMap[parentId].children.push(item);
						}
					}
					else {
						treeMap[id].level = 1;
					}
					
					if(levels == "all" || treeMap[id].level < levels) {
						if(!item._cachedMeta.children) {
							treeMap[id].children = item._cachedMeta.children = [];
						}
						item._cachedMeta.children.loaded = true;
					}
				}, this);
			}
		},
		
		processFetch: function(/*DOMDoc*/ xmlDoc, /*Request*/ request) {
			// summary: Extension point for descendants or consumers of this store to handle
			//		responses from server requests for data.  Most subclasses can leave this
			// 		function as-is, but if special handling is required that cannot be done
			// 		with other extension points, this can be overridden as well.
			// xmlDoc: DOMDoc of the response from an IO request
			// request: Request used to fetch the result
			// returns: Object map with two properties:
			//		items: Array of the items fetched
			//		meta: Object map of meta data associated with the request
			var map = this.proceed(arguments);
			var levels = request.query.levels;
			if(dojo.isString(levels)) levels = levels.toLowerCase();
			this.processChildren(map, levels, request.query.uri);
			return map;
		}
	}
);

}
