/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation"] = true;
dojo.provide( "com.ibm.portal.aggregation" );

dojo.require( "com.ibm.aggregation" );
dojo.require( "com.ibm.portal.aggregation.common" );
dojo.require( "com.ibm.portal.navigation.controller");
dojo.require( "com.ibm.portal.xslt" );
dojo.require( "com.ibm.portal.xpath" );
dojo.require( "com.ibm.portal.EventBroker" );
dojo.require( "com.ibm.portal.utilities.html" );


dojo.declare( "com.ibm.portal.aggregation.PortalAggregator", 
              com.ibm.aggregation.Aggregator,
                          {
                // summary: Implementation of the portal aggregation.
                // description: The aggreagtion consists of two elements, the page itself and the
                //      navigation for navigating between pages. Subscribes to the state change topic
                //      so that any navigational state change triggers the rendering process. Each element
                //      involved in the rendering process is responsible for determining if it has changed
                //      based on the new state.
                // navigation: the navigation implementation
                // page: the page implementation.
                
                constructor: function () {
                    //Initialize instance variables.
                    this.navigation = new com.ibm.portal.aggregation.theme.PortalNavigation();
                    this.page = new com.ibm.portal.aggregation.PortalPage();
                    this.statusBar = new com.ibm.portal.aggregation.StatusBar();
                    
                    //Subscribe to state changes. Any time the navigational state changes, we want
                    //to start the aggregation again.
                    //dojo.event.topic.subscribe( "/portal/NavigationalStateController/stateChanged", this, this.start );
                    this._stateChangedStartHandle = com.ibm.portal.EVENT_BROKER.stateChanged.register( this, this.start );
                },
                                
                start: function () {
                    // summary: Starts the aggregation rendering process.
                    // description: Pulls the aggregation information and starts
                    //      the rendering process. The information is based on the
                    //      current navigational state provided by the Navigation 
                    //      object.
                    ibm.portal.debug.entry( "PortalAggregator.start" );
                    //Get the current state.
                    var currentState = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                    ibm.portal.debug.text( "current navigational state is: " + dojox.data.dom.innerXML( currentState.stateDOM ) );
                    //Start the rendering process given the current state.
                    this.navigation.render( currentState );
                    this.page.render( currentState );
                    
                    
                    ibm.portal.debug.exit( "PortalAggregator.start" );
                }
              }
);

dojo.declare( "com.ibm.portal.aggregation.PortalPage", 
              com.ibm.aggregation.Page,
              {
                // summary: Implementation of a portal page.
                // description: Responsible for rendering the layout and any fragments (portlets)
                //      on the selected page. 
                layoutManager: null,
                portletMoveController: null,
                selectedTheme: null,
                _skins: [],
                _defaultSkinId: null,
                _fragments: [],
                _fragmentIdIndex: [],
                _previousSelection: null,
                _renderedFragments: 0,
                _resourceBaseURL: dojo.moduleUrl("com.ibm.portal.aggregation", ""),
                _ssaPagesCookieName: "com.ibm.wps.web2.ssaPages",
                
                constructor: function () {
                    //Subscribe to portlet render events so that we know when the page is completely rendered.
                    this._portletCompleteHandler = dojo.subscribe( "/portal/Portlet/complete", this, function () { this._renderedFragments++; } );  
                    this._portalPageReloadHandler = dojo.subscribe("/portal/PortalPage/reload", this, "reload");
                    this.layoutManager = new com.ibm.portal.aggregation.LayoutManager();
                    this.portletMoveController = new com.ibm.portal.aggregation.PortletMoveController();

                },
                render: function ( /*NavigationalState*/ state ) {
                    // summary: Renders the page.
                    // description: Renders the page and all its fragments in its specified layout.
                    // state: com.ibm.aggregation.NavigationState - the current navigation state
                    ibm.portal.debug.entry( "PortalPage.render", [state] );
                    this._firePageRenderPhaseStart();
                    
                    var selectedPage = state.selectionModel.getSelectedNode();
                    ibm.portal.debug.text( "Selected page is: " + selectedPage.URI );
                    if ( this._hasPageChanged( state.getPageSelection() ) ) {
                        this._clearHTMLElement();
                        this.setLoading();
                        this._destroyFragments();
                        if ( selectedPage.isLabel ){
                            
                            var problemProperties = dojo.i18n.getLocalization("com.ibm.portal", "problem", csaUserLocale);
                            var newTextNode = document.createTextNode(problemProperties["content.not.available"] + "  " + problemProperties["content.not.available.answer"]);
                            
                            var newDiv = document.createElement("DIV");
                            newDiv.style.cssText = "text-align: center; padding: 20px;";
                            newDiv.className = "generalErrorText";
                            newDiv.appendChild(newTextNode);
                            
                            var mainContent = dojo.byId("mainContent");
                            mainContent.innerHTML = "";
                            mainContent.appendChild(newDiv);
                            
                        }else{
                    
                            selectedPage.getLayoutModel( dojo.hitch( this, "applyXslt" ), { state: state } );
                        
                        }
                        
                    }
                    else {
                        //Page hasn't changed but maybe one of the fragments has
                        dojo.forEach( this._fragments, function ( frag ) { frag.render( state ); } );
                    }   
                    
                    ibm.portal.debug.exit( "PortalPage.render" );
                },
                reload: function(){
                    com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page._previousSelection = null;
                    portalAggregator.page.render( com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState() );
                },
                setLoading: function () {
                    this._getHTMLElement().innerHTML = "<div id='mainContentLoading' ><img src='" + this._resourceBaseURL + "/progress-anim.gif' />&nbsp;Loading...</div>";
                },
                clearLoading: function () {
                    //dojo.dom.destroyNode( dojo.dom.removeNode( dojo.byId( 'mainContentLoading' ) ) );
                    var nodeToDelete = dojo.byId( 'mainContentLoading' );
                    if ( nodeToDelete ) {
                    	nodeToDelete.parentNode.removeChild( nodeToDelete );
                    	delete nodeToDelete;
                    }
                },
                applyXslt: function ( /*String*/type, /*dojo.dom.Document*/xmlData, /*XMLHttpRequest*/xhr, /*Array*/callbackargs ) {
                    // summary: Apply the XSLT to create the page's layout.
                    // type: the type of response
                    // xmlData: the xml of the Feed
                    // xhr: the XMLHttpRequest used to retrieve the feed
                    // callbackargs: any arguments passed through to this function
                    ibm.portal.debug.entry( "PortalPage.applyXslt", [xmlData,xhr,callbackargs] );
                    
                    var state = callbackargs["state"];
                    
                    var selection = state.getPageSelection();
                    var forceSSA = state.selectionModel.getSelectedNode().isStaticPage;
                    
                    ibm.portal.debug.text( "selection is: " + selection + "; isStaticPage? " + state.selectionModel.getSelectedNode().isStaticPage );
                                                            
                    //First, check to see if we have any portlets that require server-side aggregation.
                    if ( ibmPortalConfig["enforceSSAOnPortletLevel"] && !forceSSA) {
                         forceSSA = (this._ssaPagesCookieContainsSelection( selection ) || this._pageContainsSSAPortlets( xmlData ));
                    }
                    
                    if ( forceSSA ) {
                        ibm.portal.debug.text( "A condition exists which requires us to switch to server side aggregation mode for page: " + selection );
                        this._updateSSAPagesCookieIfNecessary( selection );
                        com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.fullPageRefresh( selection );
                    }
                    else {
                    
                        this.clearLoading();
                        // render the page layout
                        var trf = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer(ibmPortalConfig["mainContentXslt"]);
                        var xsltParams = { "loadingImagePath": this._resourceBaseURL + "progress-anim.gif"};
                        trf.transformToRegion( xmlData, xsltParams, this._getHTMLElement(), document );
                        // initialize the layoutManager with the new page's layoutmodel
                        this.layoutManager.setLayoutModel(xmlData);
                        ibm.portal.debug.text( "PortalPage.applyXslt after creating LayOutManager is "+this.layoutManager );
                        // find portlets that need to be rendered within the page layout
                        this._extractAndStoreFragments( xmlData );
                        
                        // render the portlets
                        this._renderedFragments = 0;    // setting here before we start rendering fragments
                        for ( var i = 0; i < this._fragments.length; i++ ) {
                            this._fragments[i].render( state ); 
                        }
                        
                        CSADND_ScanForNewSources();
                        this._firePageLayoutComplete();
                        this._watchForFragmentRenderComplete();
                    }   
                    
                    ibm.portal.debug.exit( "PortalPage.applyXslt" );
                },
                getFragment: function ( /*String*/ fragId ) {
                    // summary: Retrieves the Fragment on this page that matches the given id.
                    // description: Retrieves the Fragment with the given id and returns the
                    //      Fragment object. Returns null if no fragment with the given id can
                    //      be found.
                    // fragId: String - the fragment id
                    // returns: com.ibm.aggregation.Fragment or null
                    ibm.portal.debug.entry( "Page.getFragment", [ fragId ] );
                    
                    var retVal = null;
                    //dojo.debugShallow( this._fragmentIdIndex );
                    ibm.portal.debug.text( "Found index: " + this._fragmentIdIndex[ fragId ] );
                    retVal = this._fragments[ this._fragmentIdIndex[fragId] ];                  
                    
                    ibm.portal.debug.exit( "Page.getFragment", [ retVal ] );
                    return retVal; 
                },
                addSkin: function ( /*Skin*/ skinTemplate, /*Boolean*/isDefault ) {
                    // summary: Registers a new skin  for use in rendering the page.
                    // description: Adds the skinTemplate to the Page object for use in rendering.
                    //      The Page object stores an associative array of skin templates, and a reference to the 
                    //      default skin to use if one is not specified for the portlet. There is one tempate object
                    //      for each 'skin', but a factory method: skinTemplate.instance(fullPortletId) is used to
                    //      create an actual skin for a specific portlet.
                    // skinTemplate: com.ibm.portal.aggregation.Skin - the skin template object.
                    // isDefault: Boolean - whether the skin should be registered as the default for the theme.
                    ibm.portal.debug.entry( "Page.addSkin", [ skinTemplate, isDefault ] );
                    //validate before adding. most likely scenario for failure is the xsltUrl is empty if the findInTheme failed in bootstrap
                    if(skinTemplate != null && skinTemplate.validate()){
                        this._skins[skinTemplate._skinId] = skinTemplate;
                        if(isDefault) {
                            this._defaultSkinId = skinTemplate._skinId;
                        }
                    }
                    else {
                        //report error
                        var statusMessage = new com.ibm.portal.aggregation.StatusMessage(
                            "warning", 
                            "A skin defined for this theme could not be added. The default skin will be used in its place.", 
                            "This is likely due to the xslt not being found in the theme.\nThe skins attributes:\n resourceRoot:"+skinTemplate._resourceRoot+"\n xsltUrl: "+skinTemplate._xsltUrl);
                        dojo.publish("/portal/status",[{message: statusMessage}]);

                    }
                    ibm.portal.debug.exit( "Page.addSkin" );
                },
                getSkin: function ( /*String*/ skinId ) {
                    // summary: Retrieves the skin definition that matches the given id.
                    // description: Retrieves the skin with the given id and returns the
                    //      skin object. Returns null if no skin with the given id can
                    //      be found.
                    // skinId: String - the skin id
                    // returns: com.ibm.portal.aggregation.SkinDefinition or null
                    ibm.portal.debug.entry( "Page.getSkin", [ skinId ] );
                    var retVal = this._skins[skinId];
                    if(retVal == null){
                        retVal = this._skins[this._defaultSkinId];
                    }
                    ibm.portal.debug.exit( "Page.getSkin: return"+retVal._resourceRoot, [ retVal ] );
                    return retVal; 
                },
                setTheme: function(/*String*/ theme){
                    // summary: Set the page's selected theme resource root
                    this.selectedTheme = theme;
                },
                getTheme: function(){
                    // returns: String - page's selected theme resource root
                    return this.selectedTheme;
                },
                
                _getHTMLElement: function () {
                    // summary: Retrieve the main content area in which to render the page.
                    // returns: HTMLElement - the main content area
                    return dojo.byId( "mainContent" ); //HTMLElement
                },
                _clearHTMLElement: function () {
                    // summary: Clears the main content area where the page is rendered.
                    CSADND_RemoveDropZones();
                    var node = this._getHTMLElement();
                    while(node && node.childNodes.length > 0) {
                    	var elem = node.childNodes[0];
                    	elem.parentNode.removeChild( elem ); 
                    	delete elem;
                    }
                },
                _destroyFragments: function () {
                    dojo.forEach( this._fragments, function ( frag ) { frag.destroy(); } );
                    this._fragments = new Array();
                },
				addFragment: function (/*PortletControl*/ portletControl) {
					this._fragments.push( portletControl);
                    this._fragmentIdIndex[ portletControl.id ] = (this._fragments.length - 1);

				},
                _extractAndStoreFragments: function ( /*dojo.dom.Document*/ xmlData ) {
                    // summary: Determines the fragments that need to be rendered on this page.
                    // description: Uses xpath expressions to pull out all the portlets on this
                    //      page and stores them in an array.
                    ibm.portal.debug.entry( "PortalPage._extractAndStoreFragments", [xmlData] );
                    var xpathExpr = "/atom:feed/atom:entry[atom:content[model:layout-control[model:templateName='Control']]]";
                    var nodes = com.ibm.portal.xpath.evaluateXPath( xpathExpr, xmlData, com.ibm.portal.services.LAYOUT_MODEL.namespaces );
                    
                    ibm.portal.debug.text( "found " + nodes.length + " portlets on this page." )
                    
                    if (nodes != null && nodes.length > 0) {
                       var nodeCount = nodes.length;
                        for (var i = 0; i < nodeCount; i++) {
                            var atomIdStr = "id";
                            if ( dojo.isIE || dojo.isFF >= 3) {
                                atomIdStr = "atom:id";
                            }
                            var atomId = nodes[i].getElementsByTagName( atomIdStr )[0];
                            var skinId = this.layoutManager.getAssignedSkinId(atomId.childNodes[0].nodeValue);
                            var skin = this.getSkin(skinId);
                            ibm.portal.debug.text("getAssignedSkin ResourceRoot:"+skin._resourceRoot); 
							this.addFragment(new com.ibm.portal.aggregation.PortletControl( atomId.childNodes[0].nodeValue, nodes[i], skin ) );
                        }
                    }
                                    
                    ibm.portal.debug.exit( "PortalPage._extractAndStoreFragments" );
                },
                _hasPageChanged: function ( /*String*/ newSelection ) { 
                    // summary: Determines if the element has changed.
                    // description: Indicates if the element has changed since the last time
                    //      it was rendered.
                    // newState: the state that is being rendered
                    ibm.portal.debug.entry( "PortalPage.hasPageChanged", [ newSelection ] );
                    
                    var retVal = true;
                    
                    if ( this._previousSelection != null ) {
                        retVal = ( this._previousSelection != newSelection );   
                    }
                    
                    if ( retVal ) {
                        this._previousSelection = newSelection;
                    }
                    
                    ibm.portal.debug.exit ( "PortalPage.hasPageChanged", [ retVal ]);
                    return retVal;
                },
                _pageContainsSSAPortlets: function ( /*XMLDocument*/xmlData ) {
                    // summary: Determines if the page has portlets which require server-side aggregation.
                    // xmlData: the layout model xml feed
                    // returns: true if the page contains any portlets which require SSA mode
                    ibm.portal.debug.entry( "PortalPage._pageContainsSSAPortlets", [xmlData] );
                    
                    //Extract all the portlet definition links from the ATOM feed.
                    var xpathExpr = "/atom:feed/atom:entry[atom:content[model:layout-control[model:templateName='Control']]]/atom:link[@portal:rel='portlet-definition']";
                    var nodes = com.ibm.portal.xpath.evaluateXPath( xpathExpr, xmlData, com.ibm.portal.services.LAYOUT_MODEL.namespaces );
                    
                    ibm.portal.debug.text( "found " + nodes.length + " portlet definitions on this page." )
                    
                    //We want to cycle through the portlet definitions until we find any one that is marked
                    //as requiring SSA. 
                    var retVal = false;
                    
                    if (nodes != null && nodes.length > 0) {
                       var nodeCount = nodes.length;
                       
                       for (var i = 0; (i < nodeCount && !retVal); i++) {
                            //Get the href for retrieving the portlet definition.
                            var portletDefHref = nodes[i].getAttribute( "href" );
                            //Send the request for the portlet definition.
                            var url = new com.ibm.portal.services.ContentHandlerURL( portletDefHref );
                            var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url, null, null, true );
                            serviceReq.read( function ( type, xmlData, xhr ) {
                                //See if our special config parameter is set for this portlet.
                                var prefExpr = "/atom:feed/atom:entry/atom:content/model:portlet-definition/model:portletpreferences[@name='com.ibm.wps.web2.renderMode']/base:value";
                                var prefNodes = com.ibm.portal.xpath.evaluateXPath( prefExpr, xmlData, com.ibm.portal.services.LAYOUT_MODEL.namespaces );
                                if ( prefNodes && prefNodes.length > 0 ) {
                                    var value = prefNodes[0].getAttribute( "value" );
                                    if ( value == com.ibm.portal.navigation.controller.CONSTANTS.forceSsaRenderMode ) {
                                        retVal = true;  
                                    }
                                }
                                else {
                                    //Now check our blacklist, if it exists.
                                    if ( portletsForServerSideOnly ) {
                                        prefExpr = "/atom:feed/atom:entry/atom:id[@portal:guid]";
                                        prefNodes = com.ibm.portal.xpath.evaluateXPath( prefExpr, xmlData, com.ibm.portal.services.LAYOUT_MODEL.namespaces );
                                        if ( prefNodes && prefNodes.length > 0 ) {
                                            var value = "";
                                            if(prefNodes[0].getAttributeNS) value = prefNodes[0].getAttributeNS( com.ibm.portal.services.LAYOUT_MODEL.namespaces.portal, "guid" );
                                            else value = prefNodes[0].getAttribute( "portal:guid" );
                                            value = com.ibm.portal.utilities.string.strip( value, ".webmod" );
                                            if ( portletsForServerSideOnly[value] == true ) {
                                                retVal = true;
                                            }
                                        }
                                    }
                                }
                            } );
                        }
                    }
                    
                    return retVal; // Boolean
                },
                _ssaPagesCookieContainsSelection: function ( /*String*/selection ) {
                    // summary: Determines if the current list of ssa pages contains the selected page.
                    // selection: the current selection
                    // returns: true if the page is in the selection
                    var list = dojo.cookie( this._ssaPagesCookieName );
                    return ( list && list.indexOf( selection ) >= 0 );
                },
                _updateSSAPagesCookieIfNecessary: function ( /*String*/selectionToAdd ) {
                    // summary: Adds the selection to the ssa pages list stored in the cookie if
                    //      it isn't already in the list.
                    // selectionToAdd: the page to add to the list
                    var list = dojo.cookie( this._ssaPagesCookieName );
                    if ( list == null ) {
                        list = "";
                    }
                    else {
                        list += ",";
                    }
                    
                    if ( list.indexOf( selectionToAdd ) == -1 ) {
                        list += selectionToAdd;
                    }   
                    dojo.cookie( this._ssaPagesCookieName, list, {path: "/"} ); 
                },
                _watchForFragmentRenderComplete: function () {
                    // summary: Waits for all fragments to render before firing the page render
                    //      complete event.
                    var me = this;
                    com.ibm.portal.utilities.waitFor( function () {
                        return me._renderedFragments == me._fragments.length;   
                    },
                    500,
                    me._firePageRenderPhaseComplete  
                    );
                },
                _firePageLayoutComplete: function () {
                    dojo.publish( "/portal/PortalPage/layoutComplete" );
                },
                _firePageRenderPhaseStart: function () {
                    //dojo.event.topic.publish( "/portal/PortalPage/start" );
                    com.ibm.portal.EVENT_BROKER.startPage.fire();
                },
                _firePageRenderPhaseComplete: function () {
                    //dojo.event.topic.publish( "/portal/PortalPage/complete" );
                    com.ibm.portal.EVENT_BROKER.endPage.fire();
                }
              }
);

//dojo.require( "com.ibm.portal.aggregation.javascript.uneval" );

dojo.declare( "com.ibm.portal.aggregation.PortletControl",
              com.ibm.aggregation.Fragment, {
              constructor: function ( fullPortletId, atomEntryXml, /*com.ibm.portal.aggregation.Skin*/skin ) {
                ibm.portal.debug.entry( "PortletControl.constructor", [ fullPortletId ] );            
                
                //Initialize instance variables.
                this.id = fullPortletId;
                this.portletWindowId = com.ibm.portal.utilities.getControlId( fullPortletId );
                this._atomEntryXml = atomEntryXml;
                this.portlet = new com.ibm.portal.aggregation.Portlet(fullPortletId);
                this.skin = skin.instance(fullPortletId, this, atomEntryXml);
                this._resourceBaseURL = dojo.moduleUrl("com.ibm.portal.aggregation", "");
                
                ibm.portal.debug.text( "Subscribing portletControl to: /portal/PortletFragmentService/portletInfoRetrieved/" + this.id.substring(3) );
                //Subscribe to event notifications for this portlet.
                /*dojo.subscribe( "/portal/PortletFragmentService/portletInfoRetrieved/" + this.id.substring(3), 
                                            this, 
                                            "handlePortletInfoRetrieved"                             
                );*/
                //com.ibm.portal.EVENT_BROKER.startFragment.register( this, "setLoading" );
                this._endFragmentHandle = com.ibm.portal.EVENT_BROKER.endFragment.register( this, "handlePortletInfoRetrieved" );
                
                ibm.portal.debug.exit( "PortletControl.constructor" );
              },
              
                // summary: The portlet implementation of a fragment.
                // description: Responsible for rendering the portlet markup. Any updates to the portlet
                //      markup are received by subscribing to the PortletFragmentService's topic for this
                //      particular portlet.
                handlePortletInfoRetrieved: function ( /*Object*/args ) {
                    // summary: Called whenever this portlet's info is retrieved. Pulls out the relevant
                    //      arguments and calls the update functions.
                    var pInfo = args["portletInfo"];
                    if ( !(pInfo instanceof Error) ) {
                        if ( pInfo.windowId == this.portletWindowId ) {
                            this.clearLoading();
                            this._update( pInfo );
                        }
                    }
                    else {
                        if ( this.id == args["id"] ) {
                            //renaming the variable here just to make it clear that we are working with
                            //an Error object and not the typicaly PortletInfo object.
                            var error = pInfo;
                            this.clearLoading( args );
                            var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
                            dojo.byId( this.id ).innerHTML = v3themeProperties["portletRequestFailed"];
                            
                            var errorMessage = new com.ibm.portal.aggregation.StatusMessage( 'error', v3themeProperties["portletRequestFailed"], this.id + " - " + error.message );
                    		dojo.publish("/portal/status",[{message: errorMessage}]);
                            //Fire the render complete event.
                    		this._firePortletRenderComplete();
                        }   
                    }       
                },
                destroy: function () {
                    /*dojo.unsubscribe( "/portal/PortletFragmentService/portletInfoRetrieved/" + this.id.substring(3), 
                                            this, 
                                            "handlePortletInfoRetrieved"                             
                    );*/
                    //com.ibm.portal.EVENT_BROKER.startFragment.unregister( this, "setLoading" );
                    com.ibm.portal.EVENT_BROKER.endFragment.unregister( this._endFragmentHandle );
                    this.portlet.destroy();
                    this.skin.destroy();    
                },
                render: function ( /*NavigationalState*/ state) {
                    // summary: Renders this HTML fragment.
                    // description: Renders this HTML fragment. Processes any included javascript using 
                    //      the specified javascript filter. 
                    // state: the current navigational state
                    ibm.portal.debug.entry( "PortletControl.render", [ state ] );
                    
                    var solo = state.getSoloPortlet();
                    if(!solo || solo == this.portletWindowId) {
                    	// if we're in solo, remember it
                    	if(solo == this.portletWindowId) this._inSolo = true;
                    	else this._inSolo = false;
						if (!this.skin.isRendered){
	                        this.skin.renderSkin();
	                    }
	                    if ( this.portlet._needsRefresh( state.getPortletState( this.portletWindowId ), this.portlet._currentState ) ) {
	                        //We want to ignore dependent portlets when this is the initial rendering of 
	                        //the portlet because all the portlets might not have rendered yet. The only 
	                        //case we run into this is when 2 portlets on the page use a shared render 
	                        //parmaeter (JSR286 support).
	                        var portletService = null;
	                        if ( this.portlet._currentState == null ) {
	                            portletService = new com.ibm.portal.services.IndependentPortletFragmentService();
	                        }
	                        else {
	                            portletService = new com.ibm.portal.services.PortletFragmentService();
	                        }
	                        
	                        portletService.getPortletInfo( this.id, state.newPortletRenderURL( this.portletWindowId ) );
	                    }
                    }
                    ibm.portal.debug.exit( "PortletControl.render" );
                },
                /* This function will render the view mode without a portlet state update */
                renderViewMode: function( pInfo, elem){
                    // summary: renders the portlets view mode, without a state update.
                    // description: if a skin supports inline edit modes, this method can
                    //     be used to update the view mode markup without changing the portlets
                    //     state to view mode. This is not called from normal page aggregation, but is 
                    //     initiated from the Skin object itself
                    ibm.portal.debug.entry( "PortletControl.renderViewMode", [pInfo, elem] );
                    var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                    var url = state.newPortletRenderURL( this.portletWindowId, "view" );
                    var service = new com.ibm.portal.services.PortletFragmentService();
                    var me = this;
                    ibm.portal.debug.text("PortletControl.renderViewMode: calling PortletFragmentService.getPortletInfo(id:"+this.id+",url:"+url+",elem:"+elem+")")
                    service.getPortletInfo( this.id,
                                            url,
                                            function ( pInfo ) { 
                                                //me.portlet._markup.update( elem , pInfo.markup );
                    							var args = me.portlet.portletMarkup.prepareUpdate( elem , pInfo.markup, false, me.id );
                    							me.portlet.portletMarkup.update( args ); 
                                            },
                                            null,
                                            true
                                        );
                },
                
                setLoading: function ( args ) {
                },
                clearLoading: function ( args ) {
                    if ( !args ) {
                        this.skin.hideLoadingIfNecessary();
                    }
                    else if ( args.id.indexOf( this.portletWindowId ) != -1 ) {
                        this.skin.hideLoadingIfNecessary();
                    }    
                },
                _update: function( /*PortletInfo*/portletInfo, /*HTMLElement?*/elemToUpdate ) {
                    // summary: Updates the page with the actual markup and skin information.
                    ibm.portal.debug.entry( "PortletControl._update", [portletInfo] );
                    //Insert the markup.
                    //skin.render, just updates the skins state(activediv,max,min, etc), 
                    // and in case its not in view mode, requests a view mode update as well                    
                    this.skin.updateSkin( portletInfo );
                    if(this._inSolo) {
                    	// we are in solo mode, so tell the skin to maximize itself... won't change the state
                    	// or send off any requests.
                    	dojo.publish("/portal/Skin/maximized/"+this.id);
                    }
                    // this actually updates the active div with the markup in portletInfo
                    if(this.skin.getActiveDiv() != null){
                        this.portlet.update(portletInfo,this.skin.getActiveDiv());
                    }
                    
                    //IN IE, we need to run this every time we do an update.
                    if ( !this._dampened || window.attachEvent ) {
                        var dampener = new com.ibm.portal.aggregation.event.PortletEventDampener(function (e) { 
                                                                                                    ibm.portal.debug.entry ( "_cancelDefault", [ e ] );
                                                                                                    var form = e.target;
                                                                                                    if ( form && ! form.tagName.toLowerCase()=="form") {
                                                                                                        form = e.target.form;    
                                                                                                    }                                                                                                    
                                                                                                    ibm.portal.debug.text( "e.target = " + form );
                                                                                                    ibm.portal.debug.text( "form encoding type = " + form.enctype );
                                                                                                    return ( form.enctype != "multipart/form-data"); 
                                                                                                 });
                        var elem = dojo.byId( this.id );
                        //In Firefox (and other W3C compliant browsers), the onsubmit event bubbles. This allows us to 
                        //simply attach to the highest point in the browser DOM and catch the event there. Unfortunately,
                        //in IE, the onsubmit event does not bubble, so we have to attach the dampener to every form
                        //element in the portlet DOM.  
                        if ( elem.addEventListener ) {
                             elem.addEventListener( "submit", 
                                                               dojo.hitch( dampener, "dampen" ), 
                                                               false );
                        }
                        else if ( elem.attachEvent ) {
                            var forms = elem.getElementsByTagName( "FORM" );
                            dojo.forEach( forms, function ( formElem ) { 
                                                          formElem.attachEvent( "onsubmit", dojo.hitch( dampener, "dampen" ) );    
                                                      } );    
                        }
                                                               
                        this._dampened = true;
                    }    
                    //Update dependent portlets, if any.
                    this._processDependentPortlets( portletInfo.dependentPortlets );
                    //Fire the render complete event.
                    this._firePortletRenderComplete();
                    
                    ibm.portal.debug.exit( "PortletControl._update" );
                },
                _processDependentPortlets: function ( /*Array*/dependents ) {
                    // summary: Processes any dependent portlets, sending off requests to update
                    //      those portlets. 
                    // dependents: the array of dependent portlets
                    ibm.portal.debug.entry( "PortletControl._processDependentPortlets", [dependents] );
                    
                    //We want to ignore any dependent portlets that come back as a result of a dependent
                    //portlet update (got all that??). The reason is that with shared render parameters,
                    //if one portlet using a shared render param is updated, the other portlets that use 
                    //that shared param will always come back as dependent. Then when the dependent portlet
                    //is updated it will claim that the original portlet that was updated is dependent. Thus,
                    //we end up in a nasty circular dependency.
                    var portletService = new com.ibm.portal.services.IndependentPortletFragmentService();
                    
                    for ( var i = 0; i < dependents.length; i++ ) {
                        var portletid = dependents[i].uri;
                        portletid = portletid.replace( /fragment:pm:/g, "lm:" );
                        com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.getFragment( portletid ).setLoading();
                        portletService.getPortletInfo( portletid, dependents[i].link ); 
                    }   
                    
                    ibm.portal.debug.exit( "PortletControl._processDependentPortlets" );
                },
                _firePortletRenderComplete: function () {
                    dojo.publish( "/portal/Portlet/complete" );
                }       
              }
);

dojo.declare( "com.ibm.portal.aggregation.Portlet",
              com.ibm.aggregation.Fragment,
              {
                // summary: The portlet implementation of a fragment.
                // description: Responsible for rendering the portlet markup. Any updates to the portlet
                //      markup are received by subscribing to the PortletFragmentService's topic for this
                //      particular portlet.

                constructor: function ( fullPortletId ) {
                    var _method = "Portlet.constructor"
                    ibm.portal.debug.entry( _method, [ fullPortletId ] );           
                    
                    //Initialize instance variables.
                    this.id = fullPortletId;
                    this.portletMarkup = new com.ibm.portal.aggregation.PortletMarkup();
                    this._currentState = null;
                    this._restorePoints = [];
                    this.cleanUp = true;
                                    
                    ibm.portal.debug.exit( _method );
                },
                _cleanupJavascript: function ( /*String*/mode ) {
                	var _method = "Portlet._cleanupJavascript"
                    ibm.portal.debug.entry( _method, [ mode ] );
                	
                	var restorePoints = this._restorePoints[mode];
                    if ( restorePoints ) {
                        var scrubbers = com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS;
                    	var length = scrubbers.length;
                        for ( var i = 0; i < length; i++ ) {
                        	var scrubber = scrubbers[i];
                        	for ( var j = 0; j < restorePoints.length; j++ ) {
                        		restorePoints[j].restore( scrubber );
                        	}
                        }	
                    }
                    
                    ibm.portal.debug.exit( _method );
                },
                _executeInlineCSAScript: function(/*HTMLElement*/scriptElem) {
                	var script = scriptElem.innerHTML;
                	var f = new Function(script);
                	var me = this;
                	f.call(me);
                },
                registerUpdateHandler: function(/*Function*/ func) {
            		this._customUpdateHandler = func;
                },
                unregisterUpdateHandler: function() {
                	delete this._customUpdateHandler;
                },
                update: function( /*PortletInfo*/portletInfo, /*HTMLElement?*/elemToUpdate ) {
                    // summary: Updates the page with the actual markup and information.
                    var _method = "Portlet.update";
                    ibm.portal.debug.entry( _method, [portletInfo] );
                    if(this._timer) {
                    	try {
		                	clearTimeout(this._timer);
                    	}
                    	catch(e) {
                    	}
                    	this._timer = null;
                    }
                    					
                    //Store all the before and after snapshots; later we will create the restore points from
                    //these. These are unique to the portlet mode.
                    var me = this;
                    var currentMode = portletInfo.currentMode;
                    var beforePoints = [];
                    var afterPoints = [];
                    //Since all javascript code is executed asynchronously now, we need to use an event-based approach to take our
                    //snapshots. The startScriptExecution event fires directly before the script code is evaluated and the 
                    //endScriptExecution event fires directly after the script code is evaluated, with no relinquishing of control.
                    //This should keep our snapshots honest. 
                    var startHandle = com.ibm.portal.EVENT_BROKER.startScriptExecution.register( function ( args ) { 
                    	if ( args.portletId == me.id ) {
                    		beforePoints.push( new com.ibm.portal.aggregation.javascript.uneval.Snapshot() );
                    	}	
                    } );
                    var endHandle = com.ibm.portal.EVENT_BROKER.endScriptExecution.register( function ( args ) { 
                    	if ( args.portletId == me.id ) {
                    		afterPoints.push( new com.ibm.portal.aggregation.javascript.uneval.Snapshot() );
                    	}	
                    } );
                    var completeHandle = com.ibm.portal.EVENT_BROKER.endScriptHandling.register( function ( args ) {
                    	if ( args.portletId == me.id ) {
                    		//Clear out any existing restore points.
                    		me._restorePoints[currentMode] = [];
                    		ibm.portal.debug.text( "There are " + beforePoints.length + " before points and " + afterPoints.length + " after points.", _method );
                    		if ( beforePoints.length == afterPoints.length ) {
	                    		//Create each restore point from a before and after snapshot pair.
	                    		for ( var i = 0; i < beforePoints.length; i++ ) {
		                    		me._restorePoints[currentMode].push( new com.ibm.portal.aggregation.javascript.uneval.RestorePoint( beforePoints[i], afterPoints[i] ) ); 
	    	                	}
                    		}
                    		else {
                    			//Something bad happened - the number of before and after snapshots are not the same. We can not operate reliably here so just don't try and
                    			//clean anything up.
                    			beforePoints = null;
                    			afterPoints = null;
                    		}	
        	            	//Clean up after ourselves now that we don't care anymore.
            	        	var eb = com.ibm.portal.EVENT_BROKER;
                	    	eb.startScriptExecution.unregister( startHandle );
                    		eb.endScriptExecution.unregister( endHandle );
                    		eb.endScriptHandling.unregister( completeHandle );
                    	}	
                    });
                    
                    var args = this.portletMarkup.prepareUpdate( elemToUpdate, portletInfo.markup, false, this.id );
                    args.currentMode = portletInfo.currentMode;
                    for(var i = 0; i < args._CSAScripts.length; i++) {
                    	this._executeInlineCSAScript(args._CSAScripts[i]);
                    }
                    delete args._CSAScripts;
                    
                    //this._markup.update( elemToUpdate, portletInfo.markup, false, this.id );
                    this.preUpdate(portletInfo, elemToUpdate);
                    var me = this;
                    args.onScriptCompleted = function () {
                    	me.postUpdate(portletInfo, elemToUpdate);
                    }	
                    
                    if(this._customUpdateHandler) {
                    	this._customUpdateHandler.call(this.portletMarkup, args );
                    }
                    else {
                    	this.portletMarkup.update( args );
                    }
                    this._currentState = portletInfo.updatedState;
                    
                    //this.postUpdate( portletInfo, elemToUpdate );
                    
                    ibm.portal.debug.exit( _method );
                },
                preUpdate: function(/*PortletInfo*/portletInfo, /*HTMLElement?*/elemToUpdate) {
                	if(this.cleanUp) {
                        this._cleanupJavascript( portletInfo.currentMode );
                	}
                },
                postUpdate: function(/*PortletInfo*/portletInfo, /*HTMLElement?*/elemToUpdate) {

					// look for meta tags that indicate a timed refresh should occur, and refresh this portlet
					// with the URL specified if one exists
					try {
						var metas = dojo.query("meta", elemToUpdate);
						for(var i = 0; i < metas.length; i++) {
							var httpEquiv = dojo.isIE ? metas[i].httpEquiv : metas[i].getAttribute("http-equiv");
			            	if(httpEquiv != "Refresh") continue;
			            	var meta = metas[0];
							var content = meta.getAttribute("content");
							var match = content.match(/^\s*(\d+)\s*(?:;\s*url=\s*(.+)\s*)?\s*$/im);
							if(match && match[1]) {
								var time = match[1] * 1000;
								var url = match[2];
								if(!url) {
									var state = new com.ibm.portal.navigation.controller.NavigationalState(this._currentState);
									url = state.newPortletRenderURL( com.ibm.portal.utilities.getControlId( this.id ) );
								} 
								url = dojo.string.trim(url);
								
								var serviceReq = new com.ibm.portal.services.PortletFragmentService();
								ibm.portal.debug.text( "Setting auto-refresh timer on portlet " + this.id + " for " + time + " ms");
								this._timer = setTimeout(dojo.hitch(this, function(){serviceReq.getPortletInfo(this.id, url)}), time);
								break;
							}
			            }
					}
					catch(e){
						// not mission critical, so don't expose any errors, just don't break everything else
					}		
					
					this.unregisterUpdateHandler();
					ibm.portal.debug.exit( "Portlet._update" );
                },
                _needsRefresh: function( /*XMLDocument*/newState, /*XMLDocument*/currentState ) {
                    ibm.portal.debug.entry( "Portlet._needsRefresh", [ newState, currentState ] );
                    var retVal = true;
                    
                    if ( currentState != null && newState != null ) {
                        retVal = ( dojox.data.dom.innerXML( currentState ) != dojox.data.dom.innerXML( newState ) );
                    }
                    ibm.portal.debug.exit( "Portlet._needsRefresh", [ retVal ] );
                    return retVal;
                },
                destroy: function () { 
                    var _method = "destroy";
                    ibm.portal.debug.entry( _method );
                	
                	for ( mode in this._restorePoints ) {
                    	if ( this._restorePoints.hasOwnProperty( mode ) ) {
                    		this._cleanupJavascript( mode );
                    	}	
                    }
                	
                	ibm.portal.debug.exit( _method );
                }
              }
);

dojo.declare( "com.ibm.portal.aggregation.PortletMarkup",
              com.ibm.aggregation.FragmentMarkup,
              {
                // summary: The portlet markup must be handled specially when used in client-side
                //      aggregation. This implementation is responsible for making all necessary 
                //      changes.
                // description: Handles inserting the portlet markup into the current page. The 
                //      javascript handler is responsible for handling any <script> elements in 
                //      the portlet markup. When the markup is complete, the the portlet markup event
                //      is fired to allow for any further modifications that must occur.
                //          /portal/navigation/portlet/markup - portlet markup update is complete    
                constructor: function () {
                    this._onloadHandler = new com.ibm.portal.aggregation.event.markup.OnloadRewriter();
                },
                //_jsHandler: com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER,
                _jsHandler: com.ibm.portal.aggregation.javascript.ASYNC_JAVASCRIPT_HANDLER,
                prepareUpdate: function ( /*HTMLElement*/elementToUpdate, /*String*/markup, /*Boolean*/append, /*String*/portletId ) {
                	// summary: Update the specified element with the given markup.
                    // elementToUpdate: HTMLElement - the element to update
                    // markup: the markup string.
                    var _method = "PortletMarkup.prepareUpdate";
                	ibm.portal.debug.entry( _method, [ elementToUpdate, markup, append, portletId ] );
                    // clean up any forms that were submitted before in case we have a custom update handler that doesn't
                    // update the entire portlet but appends markup or something; if we don't do this, those forms won't
                    // work anymore
                    dojo.query("form", elementToUpdate).forEach(
                    	function(formElem) {
                    		if(formElem.submitted) formElem.submitted = false;
                    	}
                    );
                    var markupMap = this.prepareMarkup( markup );
                    
                    var args = {portletId: portletId, append: append, markupMap: markupMap, elementToUpdate: elementToUpdate};
                    args = this.prepareScripts(args);
                   
                    ibm.portal.debug.exit( _method );
                    return args;
                },
                update: function ( args ) {
                    // summary: Default update handler to update the specified element with the given markup.
                    // args: Object - Map specifying the following properties.
                    //		
                    //	-	portletId: String - portletId
                    // 	- 	elementToUpdate: HTMLElement - the element to update
                    //  -	append: Boolean - True if the markup should be appended to the existing markup, false to replace.
                    // 	-	markupMap: Object - contains these properties:
                    //		-		node: A HTMLElement containing the filtered markup to insert
                    // 		-		objects: OBJECT elements stripped from the markup to be added by the 
                    // 		-			insertMarkup function
                    var _method = "PortletMarkup.update";
                    ibm.portal.debug.entry( _method, [ args ] );
                	
                	if ( !args.append ) {
                        args.elementToUpdate.innerHTML = "";
                    }
                    this.prepareOnloadHandlers( args );
                    this.insertMarkup( args );
                    this.runScripts( args );
                    
                    ibm.portal.debug.exit( _method );
                },
                prepareScripts: function( args ) {
                	var _method = "PortletMarkup.prepareScripts";
                	ibm.portal.debug.entry( _method, [ args ] );
                	
                	if(args.scriptElems) return;
                    args._CSAScripts = new Array();
                    // find the script elements
                    args.scriptElems = args.markupMap.node.getElementsByTagName( "script" );
                    if ( args.scriptElems != null ) {
                        // iterate over script elements and attach an id attribute
                        for ( var i = 0; i < args.scriptElems.length; i++ ) {
                            var scriptElem = args.scriptElems[i];
                            // include an id if there is none so far
                            // (needed to lookup script in original DOM later on)
                            var id = scriptElem.getAttribute( "id" );
                            if (id == null || id == "" ) {
                                scriptElem.setAttribute( "id", "_scr#" + i );
                            }
                            
                            if(scriptElem.getAttribute("type") == "ibmCSA/portlet") {
                            	args._CSAScripts.push(scriptElem);
                            	scriptElem.parentNode.removeChild(scriptElem);
                            	i--;
                            }
                        }
                    }
                    
                    ibm.portal.debug.exit( _method );
                    return args;
                },
                prepareMarkup: function( markup ) {
                    var _method = "PortletMarkup.prepareMarkup";
                    ibm.portal.debug.entry( _method, [ markup ] );
                	
                	// parse the markup into' a temporary div element
                    var retVal = com.ibm.portal.utilities.html.createTemporaryMarkupDiv( markup );
                    
                    ibm.portal.debug.exit( _method, retVal );
                    return retVal;
                },
                prepareOnloadHandlers: function( args ) {
                	var _method = "PortletMarkup.prepareOnloadHandlers";
                	ibm.portal.debug.entry( _method, [ args ] );
                	// find and rewrite all onload handlers to run them later
                    this._onloadHandler.process( args.markupMap.node );
                    ibm.portal.debug.exit( _method );
                },
                insertMarkup: function( args ) {
                	var _method = "PortletMarkup.insertMarkup";
                	ibm.portal.debug.entry( _method, [ args ] );
                	
                	// update the DOM before executing the scripts
                    // (the scripts might rely on elements in the DOM)
                    args.elementToUpdate.innerHTML = args.markupMap.node.innerHTML;
                    if(dojo.isIE) {
                    	com.ibm.portal.utilities.html.replaceObjectElementsInMarkup(args.markupMap.objects);
                    }
                    
                    ibm.portal.debug.exit( _method );
                },
                runScripts: function( args ) {
                	var _method = "PortletMarkup.runScripts";
                	ibm.portal.debug.entry( _method, [ args ] );
                	
                	com.ibm.portal.EVENT_BROKER.startScriptHandling.fire( { portletId: args.portletId } );
                    
                	var scriptElems = args.scriptElems;
                	var portletId = args.portletId;
                	var elementToUpdate = args.elementToUpdate;
                	var markupMap = args.markupMap;

                	// execute the scripts
                    if ( scriptElems != null ) {
                    	var me = this;
                        ibm.portal.debug.text( "There are " + scriptElems.length + " script elements in this markup.", _method );
                        var scriptHandler = new com.ibm.portal.aggregation.javascript.AsyncPortletScriptElementsHandler( scriptElems, portletId, this._jsHandler );
                        scriptHandler.stopOnError = true;
                        scriptHandler.setEventHandler( {
                        	error: function ( error ) { 
                        		if ( !this._firstErrorLogged ) {
                        			elementToUpdate.innerHTML = error.message;
                        			this._firstErrorLogged = true;
                        		}
                        		else {
                        			elementToUpdate.innerHTML += error.message;
                        		}	
                        		//elementToUpdate.innerHTML = "<div style=\"color: red; font-weight: bold;\">" + error.message + "</div>" + elementToUpdate.innerHTML;
                        	},
                        	scriptDone: function () { },
                        	completed: function () { 
                        		ibm.portal.debug.entry( "ScriptEventHandler.completed" );
                        		// finally destroy the temporary div element
			                    if ( markupMap ) {
			                    	delete tempDiv;
			                    	delete markupMap.node;
			                    	delete markupMap.objects;
			                    }
			
			                    // now run the onload handlers that were detected before
			                    me.runOnloadHandlers( args );
			                    
			                    com.ibm.portal.EVENT_BROKER.endScriptHandling.fire( { portletId: portletId } );
			                    
			                    // flag the element as loaded.
			                    me.finalizeMarkup( args );
			                    
			                    if ( args && args.onScriptCompleted ) {
			                    	args.onScriptCompleted();
			                    }	
                        		
                        		ibm.portal.debug.exit( "ScriptEventHandler.completed" );
                        	},
                        	started: function () { }
                        } );
                        scriptHandler.start();
                    }
                    
                    ibm.portal.debug.exit( _method );
                },
                runOnloadHandlers: function( args ) {
                    var _method = "PortletMarkup.runOnloadHandlers";
                    ibm.portal.debug.entry( _method, [ args ] );
                	// now run the onload handlers that were detected before
                    this._onloadHandler.runHandlers();
                    ibm.portal.debug.exit( _method );
                },
                finalizeMarkup: function( args ) {
                	var _method = "PortleMarkup.finalizeMarkup";
                	ibm.portal.debug.entry( _method, [ args ] );
                	
                	if(args.node) delete args.node;
                	
                    // flag the element as loaded.
                    args.elementToUpdate.setAttribute( "loaded", "true" );
                    this._firePortletMarkupUpdated( args.elementToUpdate );
                    ibm.portal.debug.exit( _method );
                },
                _firePortletMarkupUpdated: function ( markupRootElement ) {
                    dojo.publish( "/portal/PortletMarkup/updated", [ {newMarkup: markupRootElement} ] );
                }
            }   
);




/**
 * Layout Manager reads the atom feed and creates the layout tree in the form of a hidden div.
 * The PortletMove Controller uses this hidden div as a reference layout tree to show the move menu.
 */
dojo.declare( "com.ibm.portal.aggregation.LayoutManager", 
    null,
    {
        constructor: function() {  
            ibm.portal.debug.entry("LayoutManager.constructor",[]);
            this._id = "layoutmanager";
            this.moveWithinContainer = null;
            this.targetPortletId = null;
            this._lmgrPrefix = "lmgr";
            this._hiddenDivId = "wptheme-lmgr-root";
            this._hiddenDiv = null; //needs to be created after initial html page has loaded, so we create it lazily
            this._xsltURL = dojo.moduleUrl( "com", "ibm/portal/");
            this._xslt = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer(this._xsltURL + "LayoutManager.xsl");
            this._xsltParams = { "lmgrPrefix": this._lmgrPrefix };
            ibm.portal.debug.exit("LayoutManager.constructor",[this]);
        },
        setLayoutModel: function(xmlData){
            if (this._hiddenDiv == null){ this._createHiddenDiv();} //lazy creation of div
            this._xslt.transformToRegion( xmlData, this._xsltParams, this._hiddenDiv, document );
			com.ibm.portal.utilities.html.removeNodesOnCondition(this._hiddenDiv, function(node) {
				if(node && node.nodeType != 1) return true;
				else return false;
			});
        }, 
        getAssignedSkinId: function(/*String*/ fullPortletId) {
            ibm.portal.debug.entry("LayoutManager.getAssignedSkin",[fullPortletId]);
            // modify the portletId to match layoutmanager DOM prefixing        
            var prefixedFullPortletId = this._lmgrPrefix+"dndSrc"+fullPortletId;
            // find the control node for this portlet
            var control = dojo.byId(prefixedFullPortletId);
            // see if a skin is assigned (if not, then it has the default skin)  
            var result = null;
            if (control != null) {
                ibm.portal.debug.text("LayoutManager.getAssignedSkin: found control: "+dojox.data.dom.innerXML(control));
                result = control.getAttribute("skin");
            }
            ibm.portal.debug.exit("LayoutManager.getAssignedSkin",[result]);
            return result;
    
        },

        _createHiddenDiv: function() {
            //create the container div for all of model information       
            var div = document.createElement( "DIV" );
            div.style.display = "none";
            div.setAttribute( "id", "wptheme-lmgr-root" ); 
            dojo.body().appendChild( div );
            this._hiddenDiv = div;
            return div;
        },

        _getRootContainerDiv: function() {
            var rootDiv = document.getElementById(this._hiddenDivId).childNodes[0];
            return rootDiv;
        },      

        _getHiddenDivId: function() {
            window.console.log("_getHiddenDivId: this._hiddenDivId: " + this._hiddenDivId);
            return this._hiddenDivId;
        },      

        _getHiddenDiv: function() {
            window.console.log("_getHiddenDiv: this._hiddenDiv: " + this._hiddenDiv);
            return this._hiddenDiv;
        },      

        _hasChildcontainers: function(container) {
            if(container.hasChildNodes){
                if(container.lastChild.getAttribute("containertype") == "C"){
                    return false;
                }
                else{
                    return true;
                }
            }
            else{
                return false;
            }
        },

//Update the LayoutManager div after deleting a portlet on a page.      
        deletePortletInLayoutManager: function(selectedSourceId) {
            var lmgrPrefix = "lmgrdndSrc";
            
            var parent = document.getElementById(lmgrPrefix+selectedSourceId).parentNode;
            parent.removeChild(document.getElementById(lmgrPrefix+selectedSourceId));
        },

//Update the LayoutManager div after moving a portlet on a page.        
        movePortletInLayoutManager: function(selectedSourceId, tgtPortletId, containerId) {
            var lmgrPrefix = "lmgr";
            
            if(tgtPortletId != "emptyDZ"){
                var parent = document.getElementById(lmgrPrefix+tgtPortletId).parentNode;   
                parent.insertBefore(document.getElementById(lmgrPrefix+selectedSourceId), document.getElementById(lmgrPrefix+tgtPortletId));
            }
            else{
                var parent = document.getElementById(lmgrPrefix+containerId);   
                parent.appendChild(document.getElementById(lmgrPrefix+selectedSourceId));               
            }
        },
        

//Update the LayoutManager div after dragging a portlet from a palette to a page.       
        addControl: function(nextControlId, id, title, containerId, pageId, skinId) {
            var lmgrPrefix = "lmgr";
            if(nextControlId == ""){
                var newPortletDivElement = document.createElement("div");
                newPortletDivElement.setAttribute("id", lmgrPrefix+'dndSrc'+id);
                newPortletDivElement.setAttribute("containertype", "C");
                newPortletDivElement.setAttribute("skin", skinId);
                newPortletDivElement.setAttribute("portlettitle", title);

                var parent = document.getElementById(lmgrPrefix+'lm:oid:'+containerId+'@oid:'+pageId);  
                parent.appendChild(newPortletDivElement);               
            }
            else{
                var newPortletDivElement = document.createElement("div");
                newPortletDivElement.setAttribute("id", lmgrPrefix+'dndSrc'+id);
                newPortletDivElement.setAttribute("containertype", "C");
                newPortletDivElement.setAttribute("skin", skinId);
                newPortletDivElement.setAttribute("portlettitle", title);

                var parent = document.getElementById(lmgrPrefix+'dndSrclm:oid:'+nextControlId+'@oid:'+pageId).parentNode;
                parent.insertBefore(newPortletDivElement, document.getElementById(lmgrPrefix+'dndSrclm:oid:'+nextControlId+'@oid:'+pageId));
            }

        }
        
    }
);      

/**
 * PortletMoveController controls the move menu for the portlet in the skin.
 * If a move target id is found, the appropriate move direction is a loaded into the skin along with the target id.
 * If not, the direction is not shown.
 */

dojo.declare( "com.ibm.portal.aggregation.PortletMoveController", 
    null, 
    {
        constructor: function() {  
            this._id = "portletmovecontroller";
            this._lmgrPrefix = "lmgr";
        },
        _getRootContainerDiv: function() {
            var rootDiv = document.getElementById(com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.layoutManager._getHiddenDivId()).firstChild;
            return rootDiv;
        },      

        _hasChildContainers: function(container) {
            ibm.portal.debug.entry ("_hasChildContainers");
            if(container.hasChildNodes){
                if(container.firstChild != null){
                    if(container.firstChild.getAttribute("containertype") != null && container.firstChild.getAttribute("containertype") == "C"){
                        return false;
                    }
                    else{
                        return true;
                    }
                }
                else{
                    return false;
                }
            }
            else{
                return false;
            }
        },
            
        _getPreviousSiblingContainerMoveId: function(container) {
            ibm.portal.debug.entry( "_getPreviousSiblingContainerMoveId" );
            var firstUnlockedContainerId = null;
            var sibling = container.previousSibling;

//          for(var i = 0; i < container.parentNode.childNodes.length; i++){
            while(sibling != null){             
                if(sibling != null && sibling.getAttribute && sibling.getAttribute("modifiableFlag") == "true"){
                    if(this._hasChildContainers(sibling)){
                        firstUnlockedContainerId = this._getContainerMoveId(sibling);
                        if(firstUnlockedContainerId == null){
                            sibling = sibling.previousSibling;
                        }
                    }
                    else{
                        firstUnlockedContainerId = sibling.getAttribute("id");
                    }
                }
                else{
                    sibling = sibling.previousSibling;
                }
                if(firstUnlockedContainerId != null){break}
            }
            
            return firstUnlockedContainerId;
        },      

        _getNextSiblingContainerMoveId: function(container) {
            ibm.portal.debug.entry( "_getNextSiblingContainerMoveId" );
            var firstUnlockedContainerId = null;
            var sibling = container.nextSibling;

//          for(var i = 0; i < container.parentNode.childNodes.length; i++){
            while(sibling != null){             
                if(sibling != null && sibling.getAttribute && sibling.getAttribute("modifiableFlag") == "true"){
                    if(this._hasChildContainers(sibling)){
                        firstUnlockedContainerId = this._getContainerMoveId(sibling);
                        if(firstUnlockedContainerId == null){
                            sibling = sibling.nextSibling;
                        }
                    }
                    else{
                        firstUnlockedContainerId = sibling.getAttribute("id");
                    }
                }
                else{
                    sibling = sibling.nextSibling;
                }

                if(firstUnlockedContainerId != null){break;}
            }
            
            return firstUnlockedContainerId;
        },      


        _getContainerMoveId: function(container) {
            ibm.portal.debug.entry ("_getContainerMoveId");
            if(container.hasChildNodes && container.firstChild.getAttribute("containertype") != "C"){
                var firstUnlockedContainerId = null;
                var child = null;
                var i = 0;
                for(child = container.firstChild; i < container.childNodes.length; i++){
                    if(child.getAttribute("modifiableFlag") == "true"){
                        if(this._hasChildContainers(child)){
                            firstUnlockedContainerId = this._getContainerMoveId(child);
                        }
                        else{
                            firstUnlockedContainerId = child.getAttribute("id");
                        }
                        if(firstUnlockedContainerId == null){
                            child = child.nextSibling;
                        }
                    }
                    else{
                        child = child.nextSibling;
                    }
                    if(firstUnlockedContainerId != null){break}
                }
                return firstUnlockedContainerId;
            }
            else{
                return container.getAttribute("id");
            }
        },


        _getLeftUCHorUpUCVneighbour: function(id){
            ibm.portal.debug.entry ("_getLeftUCHorUpUCVneighbour");

            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;
            var parentContainer2 = parentContainer1.parentNode;
            var parentContainer3 = parentContainer2.parentNode;
            var parentContainer4 = parentContainer3.parentNode;

            if(parentContainer1.getAttribute("modifiableFlag") == "false"){
                return null;
            }
            else{
                if(parentContainer1.firstChild != portletDiv){
                    return portletDiv.previousSibling.getAttribute("id");
                }
                else{
                    if(parentContainer2 != null && parentContainer2 == this._getRootContainerDiv()){
                        return null;
                    }
                    else{
//                      var targetId = null;

                        while(parentContainer2 != null && parentContainer3 != null)
                        {
                            if(parentContainer3 == this._getRootContainerDiv() || (parentContainer4 != null && parentContainer4 == this._getRootContainerDiv())){
                                if(parentContainer3.firstChild != parentContainer2){
                                    if(parentContainer2.previousSibling != null ){
                                        return this._getPreviousSiblingContainerMoveId(parentContainer2);
//                                      targetId = this._getPreviousSiblingContainerMoveId(parentContainer2);
                                    }
                                    else{
                                        return null;
//                                      targetId = null;
                                    }
                                }
                                else{
                                    return null;
//                                  targetId = null;
                                }
                            }
                            else{
                                parentContainer2 = parentContainer2.parentNode;
                                if(parentContainer2 != null) parentContainer2 = parentContainer2.parentNode;
                                if(parentContainer2 != null) parentContainer3 = parentContainer2.parentNode;
                                if(parentContainer3 != null) parentContainer4 = parentContainer3.parentNode;
                                
                            }
                        }
                    }
                }
            }
            
        },


//same as _getUpNeighbourForUCH
//Left UCV

        _getLeftUCVorUpUCHNeighbour: function(id){
            ibm.portal.debug.entry ("_getLeftUCVorUpUCHNeighbour");

            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;
            var parentContainer2 = parentContainer1.parentNode;
            var parentContainer3 = parentContainer2.parentNode;
            var parentContainer4 = parentContainer3.parentNode;
            var parentContainer5 = parentContainer4.parentNode;

            if(parentContainer1.getAttribute("modifiableFlag") == "false"){
                return null;
            }
            else{
                if((parentContainer2 != null && parentContainer2 == this._getRootContainerDiv()) || (parentContainer3 != null && parentContainer3 == this._getRootContainerDiv())){
                    if(parentContainer2.firstChild == parentContainer1){
                        return null;
                    }
                    else{
//TODO look for previous sibling unlocked container
                        if(parentContainer1.previousSibling!= null){
                            return this._getPreviousSiblingContainerMoveId(parentContainer1);
                        }
                        else{
                            return null;
                        }
                    }
                }
                else{
                    while(parentContainer4 != null){
                    if(parentContainer4 == this._getRootContainerDiv() || (parentContainer5 != null && parentContainer5 == this._getRootContainerDiv())){
                        if(parentContainer2.firstChild == parentContainer1){
                            if(parentContainer4.firstChild == parentContainer3){
                                return null;
                            }
//TODO look for previous sibling unlocked container
                            else if(parentContainer3.previousSibling!= null){
                                return this._getPreviousSiblingContainerMoveId(parentContainer3);
                            }
                        }
                        else{
                            if(parentContainer1.previousSibling!= null ){
                                var id = this._getPreviousSiblingContainerMoveId(parentContainer1);
                                if(id != null){
                                    return id;
                                } 
                                else{
//TODO look for previous sibling unlocked container
                                    if(parentContainer3.previousSibling!= null){                                    
                                        return this._getPreviousSiblingContainerMoveId(parentContainer3);
                                    }
                                }
                            }
                            else{
                                return null;
                            }
                        }
                    }
                    else{
                        parentContainer4 = parentContainer4.parentNode;
                        if(parentContainer4 != null) parentContainer4 = parentContainer4.parentNode;
                        if(parentContainer4 != null) parentContainer5 = parentContainer5.parentNode;
                    }
                }
                }
            }
        },

        _isValidLeftMove: function(id) {
            ibm.portal.debug.entry ("_isValidLeftMove: dojo.byId(this._lmgrPrefix+id): " + document.getElementById(this._lmgrPrefix+id));
            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;

            if(parentContainer1.getAttribute("containertype") == "UCH"){
//              return this._getPreviousNeighbourForUCH(id);
                return this._getLeftUCHorUpUCVneighbour(id);
            }
            else if(parentContainer1.getAttribute("containertype") == "UCV"){
//              return this._getPreviousNeighbourForUCV(id);
                return this._getLeftUCVorUpUCHNeighbour(id);

            }
        },

        _isValidUpMove: function(id) {
            ibm.portal.debug.entry ("_isValidUpMove: dojo.byId(this._lmgrPrefix+id): " + document.getElementById(this._lmgrPrefix+id));
            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;

//Previous UCV similar to up UCH            
            if(parentContainer1.getAttribute("containertype") == "UCH"){
//              return this._getPreviousNeighbourForUCV(id);
                return this._getLeftUCVorUpUCHNeighbour(id);
            }
//Previous UCH similar to up UCV            
            else if(parentContainer1.getAttribute("containertype") == "UCV"){
//              return this._getPreviousNeighbourForUCH(id);
                return this._getLeftUCHorUpUCVneighbour(id);
            }
        },


        _getRightUCHorDownUCV: function(id){
            ibm.portal.debug.entry ("_getRightUCHorDownUCV");

            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;
            var parentContainer2 = parentContainer1.parentNode;
            var parentContainer3 = parentContainer2.parentNode;
            var parentContainer4 = parentContainer3.parentNode;

            if(parentContainer1.getAttribute("modifiableFlag") == "false"){
                return null;
            }
            else{
                if(parentContainer1.lastChild != portletDiv){
                    return portletDiv.nextSibling.getAttribute("id");
                }
                else{
                    if(parentContainer2 != null && parentContainer2 == this._getRootContainerDiv()){
                        return null;
                    }
                    else{
                        while(parentContainer2 != null && parentContainer3 != null)
                        {
                            if(parentContainer3 == this._getRootContainerDiv() || (parentContainer4 != null && parentContainer4 == this._getRootContainerDiv())){
                                if(parentContainer3.lastChild != parentContainer2){
                                    if(parentContainer2.nextSibling != null){
                                        return this._getNextSiblingContainerMoveId(parentContainer2);
                                    }
                                    else{
                                        return null;
                                    }
                                }
                                else{
                                    return null;
                                }
                            }
                            else{
                                parentContainer2 = parentContainer2.parentNode;
                                
                                if(parentContainer2 != null) parentContainer2 = parentContainer2.parentNode;
                                if(parentContainer2 != null) parentContainer3 = parentContainer2.parentNode;
                                if(parentContainer3 != null) parentContainer4 = parentContainer3.parentNode;
                            }
                        }
                    }
                }
            }
            
        },



        _getRightUCVorDownUCH: function(id){
            ibm.portal.debug.entry ("_getRightUCVorDownUCH");
            
            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;
            var parentContainer2 = parentContainer1.parentNode;
            var parentContainer3 = parentContainer2.parentNode;
            var parentContainer4 = parentContainer3.parentNode;
            var parentContainer5 = parentContainer4.parentNode;

            if(parentContainer1.getAttribute("modifiableFlag") == "false"){
                return null;
            }
            else{
                if((parentContainer2 != null && parentContainer2 == this._getRootContainerDiv()) || (parentContainer3 != null && parentContainer3 == this._getRootContainerDiv())){
                    if(parentContainer2.lastChild == parentContainer1){
                        return null;
                    }
                    else{
//TODO look for next sibling unlocked container
                        if(parentContainer1.nextSibling!= null){
                            return this._getNextSiblingContainerMoveId(parentContainer1);
                        }
                        else{
                            return null;
                        }
                    }
                }
                else{
                    while(parentContainer4 != null)
                    {
                    if(parentContainer4 == this._getRootContainerDiv() || (parentContainer5 != null && parentContainer5 == this._getRootContainerDiv())){
                        if(parentContainer2.lastChild == parentContainer1){
                            if(parentContainer4.lastChild == parentContainer3){
                                return null;
                            }
//TODO look for next sibling unlocked container
                            else if(parentContainer3.nextSibling!= null){
                                var id = this._getNextSiblingContainerMoveId(parentContainer3);
                                if(id != null){
                                    return id;
                                } 
//fallback
                                else{
                                    return this._getNextSiblingContainerMoveId(parentContainer3.nextSibling);
                                }
                            }
                        }
                        else{
                            if(parentContainer1.nextSibling!= null ){
                                var id = this._getNextSiblingContainerMoveId(parentContainer1);
                                if(id != null){
                                    return id;
                                } 
                                else{
//TODO look for next sibling unlocked container
                                    if(parentContainer3.nextSibling!= null){                                    
                                        return this._getNextSiblingContainerMoveId(parentContainer3);
                                    }
                                }
                            }
                            else{
                                return null;
                            }
                        }
                    }
                    else{
                        parentContainer4 = parentContainer4.parentNode;

                        if(parentContainer4 != null) parentContainer4 = parentContainer4.parentNode;
                        if(parentContainer4 != null) parentContainer5 = parentContainer5.parentNode;
                    }
                    }
                }
            }
        },

        _isValidRightMove: function(id) {
            ibm.portal.debug.entry ("_isValidRightMove: dojo.byId(this._lmgrPrefix+id): " + document.getElementById(this._lmgrPrefix+id));
            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;

            if(parentContainer1.getAttribute("containertype") == "UCH"){
//              return this._getNextNeighbourForUCH(id);
                return this._getRightUCHorDownUCV(id);
            }
            else if(parentContainer1.getAttribute("containertype") == "UCV"){
//              return this._getNextNeighbourForUCV(id);
                return this._getRightUCVorDownUCH(id);
            }
        },


        _isValidDownMove: function(id) {
            ibm.portal.debug.entry ("_isValidDownMove: dojo.byId(this._lmgrPrefix+id): " + document.getElementById(this._lmgrPrefix+id));
            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;

//Next UCH similar to down UCV          
            if(parentContainer1.getAttribute("containertype") == "UCV"){
//              return this._getNextNeighbourForUCH(id);
                return this._getRightUCHorDownUCV(id);
            }
//Next UCV similar to down UCH          
            else if(parentContainer1.getAttribute("containertype") == "UCH"){
//              return this._getNextNeighbourForUCV(id);
                return this._getRightUCVorDownUCH(id);
            }
        },


        _findAndMovePortlet: function(sourceContainer, movePortletId, targetId, direction) {
            ibm.portal.debug.entry( "_findAndMovePortlet" );

            ibm.portal.debug.text ("sourceContainer:" + sourceContainer);
            ibm.portal.debug.text ("movePortletId:" + movePortletId);
            ibm.portal.debug.text ("direction:" + direction);
            ibm.portal.debug.text ("targetId:" + targetId);

//Move Left
            if(direction == "left"){
//if move within Container
                if(document.getElementById(targetId).getAttribute("containertype") == "C"){
                    window.console.log("_findAndMovePortlet: targetId.substring(4) " +targetId.substring(4));
                    CSADND_alternateSubmitFunction(sourceContainer.id, targetId.substring(4), movePortletId);
                    
                }
                else{
                    CSADND_alternateSubmitFunction(targetId.substring(4), "emptyDZ", movePortletId);
                }
            }
            else if(direction == "right"){              
                if(document.getElementById(targetId).getAttribute("containertype") == "C"){
                    //loop through the portlets(ignoring dropzones) and find the movePortlet
                    //if the move portlet is last but one, send emptyDZ for targetPortletID
                    var i = sourceContainer.childNodes[0].childNodes[0].childNodes.length;
                    for (var j = 1; j < i; j=j+2) {
                        if(sourceContainer.childNodes[0].childNodes[0].childNodes[j].id == movePortletId){
                            //check to see if if it is the one before last, 
                            //then need to send emptyDZ for targetPortletID
                            if(sourceContainer.childNodes[0].childNodes[0].childNodes[i-4].id == movePortletId){
                                CSADND_alternateSubmitFunction(sourceContainer.id, "emptyDZ", movePortletId);
                                break;
                            }
                            else{
                                CSADND_alternateSubmitFunction(sourceContainer.id, sourceContainer.childNodes[0].childNodes[0].childNodes[j+4].id, movePortletId);
                                break;                                  
                            }
                        }                   
                    }

                }
                else{
                    CSADND_alternateSubmitFunction(targetId.substring(4),"emptyDZ" , movePortletId);
                }
            }
            else if(direction == "up"){
//if move within Container
                if(document.getElementById(targetId).getAttribute("containertype") == "C"){
                    ibm.portal.debug.text("_findAndMovePortlet: targetId.substring(4) " +targetId.substring(4));
                    CSADND_alternateSubmitFunction(sourceContainer.id, targetId.substring(4), movePortletId);
                }
                else{
                    CSADND_alternateSubmitFunction(targetId.substring(4), "emptyDZ", movePortletId);
                }
            }
            else if(direction == "down"){               
                if(document.getElementById(targetId).getAttribute("containertype") == "C"){
                    var i = sourceContainer.childNodes[0].childNodes.length;
                        //loop through the portlets(ignoring dropzones) and find the movePortlet
                        //if the move portlet is last but one, send emptyDZ for targetPortletID
                        for (var j = 1; j < i; j=j+2) {
                            if(sourceContainer.childNodes[0].childNodes[j].childNodes[0].id == movePortletId){
                                if(sourceContainer.childNodes[0].childNodes[i-4].childNodes[0].id == movePortletId){
                                    CSADND_alternateSubmitFunction(sourceContainer.id, "emptyDZ", movePortletId);
                                    break;                                  
                                }
                                else{
                                    CSADND_alternateSubmitFunction(sourceContainer.id, sourceContainer.childNodes[0].childNodes[j+4].childNodes[0].id, movePortletId);
                                    break;                                  
                                }
                            }                   
                        }
                }
                else{
                    CSADND_alternateSubmitFunction(targetId.substring(4),"emptyDZ" , movePortletId);
                }
            }
        }
    }

);

com.ibm.portal.aggregation.PORTAL_AGGREGATOR = new com.ibm.portal.aggregation.PortalAggregator();

}
