/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.javascript"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.javascript"] = true;
dojo.provide( "com.ibm.portal.aggregation.javascript" );

dojo.require( "com.ibm.portal.aggregation.javascript.parser" );
dojo.require( "com.ibm.portal.EventBroker" );
dojo.require( "dojo.dnd.common");
//NOTE: This function is defined using the Function constructor because it needs a clean scope. The build process munges all of these js files
//	inside of an anonymous function which is immediately executed to control the scope. This is what enables the dojo scopesMap support. This 
//  means that the current scope from inside that gigantic anonymous function is used when the function is executed so certain things may be
//	defined that aren't really (i.e. the "dojo" namespace). 

// summary: Evaluates a string of javascript in the global scope.
// description: Calling eval.call( self, ... ); from within a JS object apparently makes "this" point to the object from which the
//      eval call was made even though the scope is set using the "call" function. Therefore we need to have this function
//      available in the global scope and not as a member variable on a JS object.

if ( !dojo.isIE ) {
	window.com_ibm_portal_aggregation_javascript_globalEvalNonIE = new Function( "javascript", "dojo.eval( javascript );" );
}	

function com_ibm_portal_aggregation_javascript_windowLocationHref( /*String*/ portlet, /*String*/newLocation ) {
    var service = new com.ibm.portal.services.PortletFragmentService();
    service.getPortletInfo( portlet, newLocation );
    
    //Anytime a window.location change is handled, it effectively stops the processing of the current event/page
    //since in a non-CSA environment, we would load an entirely new document at this point. However, some portlets
    //seem to depend on this behavior and don't properly cancel an event which causes a window.location change -- so
    //we have to do it for them.
    com.ibm.portal.EVENT_BROKER.stopEvent.fire();
}

document.com_ibm_portal_aggregation_javascript_windowLocationHref = com_ibm_portal_aggregation_javascript_windowLocationHref;
//Necessary for IE because it does not correctly add global functions to the window object.
window.com_ibm_portal_aggregation_javascript_windowLocationHref = com_ibm_portal_aggregation_javascript_windowLocationHref;

// this class is a wrapper class around the script element
// it is used to insert script elements into the DOM without actually executing the script
// this class has to be instantiated through the FakeScriptFactory
dojo.declare( "com.ibm.portal.aggregation.javascript.FakeScript",
                    null,
                    {
                        setFakeSrc: function(source)
                        {
                            this.fakeSrc = source;
                        },
                        getAttribute: function(attr)
                        {
                            if (attr == 'src') {
                                return this.fakeSrc;
                            }
                            // this call is calling the super class of this class
                            return this.super_getAttribute(attr);
                        }
                    }
);

// this is the factory call to be used to create the fake scripts element
dojo.declare( "com.ibm.portal.aggregation.javascript.FakeScriptFactory",
                    null,
                    {
                        createFakeScriptElement: function () {
                             return com.ibm.portal.aggregation.javascript.INHERITANCE.applyInherit(
                                 document.createElement("script"),
                                 new com.ibm.portal.aggregation.javascript.FakeScript());
                        }
                    }
);

// SINGLETON to access the factory
com.ibm.portal.aggregation.javascript.FAKE_SCRIPT_FACTORY = new com.ibm.portal.aggregation.javascript.FakeScriptFactory();

// allows to do class inheritance in javascript and at the same time makes existing functions names available as super_XYZ
dojo.declare( "com.ibm.portal.aggregation.javascript.Inheritance",
                    null,
                    {
                        applyInherit: function(original, inheritee)
                        {
                            // first we iterate over the function of "inheritee" which is the new class that wants to inherit from original
                            // and create new functions that point to the original ones
                            for (method in inheritee) {
                                if (typeof(original[method])!="undefined") {
                                    inheritee["super_"+method] = original[method];
                                }
                            }
                            for (method in inheritee) {
                                original[method] = inheritee[method];
                            }
                            return original;
                        }
                    }
);

com.ibm.portal.aggregation.javascript.INHERITANCE = new com.ibm.portal.aggregation.javascript.Inheritance();

dojo.declare( "com.ibm.portal.aggregation.javascript.Filter",
              null,
              {
                // summary: Base interface for a filter. Each implementation must implement
                //      the doFilter function.
                // description: Passes the given event through to give the implementation of
                //      this filter a chance to alter or handle the event in some way. 
                
                doFilter: function ( /*String or HTMLElement*/script, portletId ) {
                    // summary: Filter the given event.
                    // description: Allows this filter to handle or alter the event. If this function
                    //      returns true, processing is stopped and the event is considered "handled".
                    // script: the script block (string or HTMLElement) 
                    dojo.unimplemented( "com.ibm.portal.aggregation.JavascriptFilter.doFilter" );
                },
                evalGlobal: function ( /*String*/javascript, /*String*/portletId, /*boolean?*/defer, /*Function?*/onreadystatechange ) {
                    // summary: Evaluates the given javascript in the global context.
                    // javascript: a string of javascript to evaluate.
                    // portletId: the id of the portlet that the javascript belongs to
                    // defer: whether the script has the defer attribute (queues the eval operation up with
                    //      the onload handler)
                    // onreadystatechange: IE only. If the script element has an onreadystatechange handler,
                    //      this should be a wrapper function which takes the readyState value as a parameter.                          
                    ibm.portal.debug.entry( "Filter.evalGlobal", [ javascript, portletId, defer, onreadystatechange ] );
                    
                    javascript = this._replaceWindowLocationReplaces( javascript, portletId );
                    javascript = this._replaceWindowLocationHrefs( javascript, portletId );
                    
                    var me = this;
                    var evalFunction = function () {
                        com.ibm.portal.EVENT_BROKER.startScriptExecution.fire( { portletId: portletId } );
                        
                        try {
	                        if ( window.execScript ) {
	                            window.execScript( me._stripHTMLComments( javascript ), "JavaScript" );
	                        }
	                        else {
	                            //Using eval.call( window, javascript ) was apparently not really eval'ing
	                            //in the global scope since 'this' was still referring to the Filter object 
	                            //and not the window object as it should have (or I think it should have)...
	                            com_ibm_portal_aggregation_javascript_globalEvalNonIE( javascript );
	                        }
	                        
	                        if ( onreadystatechange ) {
	                            onreadystatechange( "complete" );
	                        }
                        } 
                        //Don't catch for two reasons: 1. we want to surface any error in the portlet code as soon as 
                        //possible. 2. we want to avoid the potential performance hit entering the catch block frequently
                        //would cause. This results in an odd message in IE6 about an "unsupported operation", as IE6 seems
                        //to report to the browser the actual javascript error while execScript is executing and then generate a new error
                        //with the "unsupported operation" message that actually gets thrown. IE6 also ignores finally statements with
                        //no catch, but we really want to let the browser handle any errors so instead of doing some browser
                        //sniff here, we'll just check the number of before and after snapshots before attempting to create any
                        //restore points.
                        finally {
                        	com.ibm.portal.EVENT_BROKER.endScriptExecution.fire( { portletId: portletId } );
                        }	
                    }
                    
                    //If the script is "deferred", we want to execute it "onload". 
                    if ( defer ) {
                        com.ibm.portal.aggregation.javascript.JS_ONLOAD_HANDLER._registerHandler( evalFunction );   
                    }
                    else {
                        evalFunction();                     
                    }
                    
                    ibm.portal.debug.exit( "Filter.evalGlobal" );   
                },
                _stripHTMLComments: function( str ) {
                    // summary: Strip all HTML comments from a script block.
                    // description: Although HTML comments are allowed in a script block by the browser
                    //      IE throws up whenever you try and eval a javascript string in the global context
                    //      that has HTML comments in the string.
                    ibm.portal.debug.entry( "stripHTMLComments", [ str ] );
                    
                    var resultStr = "";
                    var startIndex = str.indexOf( "<!--" );
                    
                    if ( startIndex != -1 ) {
                        resultStr = str.substring( 0, startIndex );
                    }
                    else {
                        resultStr = str;
                    }
                    
                    while ( startIndex >= 0 ) {
                        var endIndex = startIndex;
                        //Look at the preceding character to see if we are inside a string
                        if ( startIndex - 1 >= 0 ) {
                            var openQuote = str.charAt( startIndex - 1 );
                            var closeQuote = str.charAt( startIndex + 4 );
                            if ( openQuote == "\"" && closeQuote == "\"" ) {
                                startIndex = str.indexOf( "<!--", endIndex + 3 );
                                if ( startIndex >= 0 ) {
                                    //found another comment later, add everything in between
                                    resultStr += str.substring( endIndex, startIndex );
                                }
                                else {
                                    //found no more comments, add the rest of the script
                                    resultStr += str.substring( endIndex );
                                }   
                                
                                continue;
                            }
                        }
                        
                        var endIndex = str.indexOf( "-->", startIndex );
                        if ( endIndex < 0 ) {
                            //no closing tag found for the comment, something's wrong
                            throw new Error( "Unclosed HTML comment found!!" );
                        }
                        
                        var lastNewLineIndex = str.substring( startIndex, endIndex ).lastIndexOf( "\n", endIndex );
                        if ( lastNewLineIndex < 0 ) {
                            lastNewLineIndex = startIndex;
                        }
                        else {
                            lastNewLineIndex += startIndex;
                        }
                        var doubleSlashIndex = str.substring( lastNewLineIndex, endIndex ).indexOf( "//" );
                        if ( doubleSlashIndex > -1 ) {
                            // This comment block needs to be executed as javascript, but don't add the comment tags themselves
                            resultStr += str.substring( startIndex + 4, lastNewLineIndex + doubleSlashIndex );
                        }
                        
                        
                        startIndex = str.indexOf( "<!--", endIndex + 3 );
                        if ( startIndex >= 0 ) {
                            //found another comment later, add everything in between
                            resultStr += str.substring( endIndex + 3, startIndex );
                        }
                        else {
                            //found no more comments, add the rest of the script
                            resultStr += str.substring( endIndex + 3 );
                        }
                    }
                    
                    ibm.portal.debug.exit( "stripHTMLComments", [ resultStr ] );
                    
                    return resultStr;
                },
                _setNewWindowLocationHrefFunction: function ( /*String*/portletId ) {
                    // summary: Sets a new function with a unique name to handle a window.location.href call.
                    // portletId: the portlet id for the portlet to update
                    // returns: the function name of the created function
                    var funcName = "com_ibm_portal_aggregation_javascript_windowLocationHref" + dojo.dnd.getUniqueId();
                    window[ funcName ] = function ( url ) {
                        ibm.portal.debug.entry( funcName, [ url ] );
                        com_ibm_portal_aggregation_javascript_windowLocationHref( portletId, url );
                        ibm.portal.debug.exit( funcName );
                    }
                    
                    return funcName;
                },
                _replaceWindowLocationHrefs: function ( /*String*/javascript, /*String*/ portletId ) {
                    // summary: Replaces window.location.href references with a function call that CSA can handle.
                    // javascript: the javascript string to process
                    // portletId: the portlet the javascript belongs to
                    // returns: the processed javascript string with all window.location.href references replaced.
                    
                    ibm.portal.debug.entry( "_replaceWindowLocationRefs", [ javascript ] );
                    
                    var resultStr = javascript;
                    var searchArray = [ "self.location.href", "top.location.href", "window.location.href", "window.location", "top.location", "self.location", "location.href" ];
                    var indexOf = com.ibm.portal.utilities.string.findNext( resultStr, searchArray, null );
                    
                    var parser = new com.ibm.portal.aggregation.javascript.parser.AssignmentStatementParser();
                    while( indexOf >= 0 ) {
                        //Always have to reset the parser.
                        parser.reset();
                        var result = parser.parse( resultStr, indexOf );
                        
                        if ( result.parsingSuccessful ) {
                            //Reconstruct the javascript string by taking the portion around the window.location.href reference and
                            //combining that around a new function call which takes the place of the window.location.href reference.
                            var beforePortion = resultStr.substring( 0, indexOf );
                            //Create a new function object. This must be a global, unique function for each portlet.
                            var funcName = this._setNewWindowLocationHrefFunction( portletId );
                            ibm.portal.debug.text( "Replacing: " + resultStr.substring(result.startIndex, result.endIndex - 1) + " with: " + funcName + "( "+ result.variableValue + " )" ); 
                            //Re-construct the javascript string.
                            resultStr = beforePortion + funcName + "( "+ result.variableValue + " )" + resultStr.substring( result.endIndex - 1 );
                        }
                            
                        //Check for the next instance.
                        indexOf = com.ibm.portal.utilities.string.findNext( resultStr, searchArray, indexOf );  
                    }
                    
                    ibm.portal.debug.exit( "_replaceWindowLocationRefs", [ resultStr ] );
                    return resultStr; // String
                },
                _replaceWindowLocationReplaces: function ( /*String*/javascript, portletId ) {
                    // summary: Replaces window.location.replace() function calls
                    // javascript: the javascript string to process
                    // portletId: the portlet the javascript belongs to
                    // returns: the processed javascript string with all window.location.replace() function calls replaced
                    ibm.portal.debug.entry( "_replaceWindowLocationReplaces", [ javascript ] );
                    
                    var str = javascript.replace( /location.replace\(/g, "com_ibm_portal_aggregation_javascript_windowLocationHref('"+ portletId + "'," );
                    
                    ibm.portal.debug.exit( "_replaceWindowLocationReplaces", [str] );
                    return str; // String
                },  
                prepareDocumentWrite: function( buffer ) {
                    // summary: Prepares support for document.write() and document.writeln
                    // description: Overrides the original document.write() and document.writeln()
                    //       functions with functions writing to the buffer
                    ibm.portal.debug.entry( "prepareDocumentWrite" );
                    var me = this;
                    document.write = function() {
                        me._documentWrite( buffer, document.write.arguments );
                    };
                    document.writeln = function(str) {
                        me._documentWrite( buffer, document.writeln.arguments );
                    };
                    ibm.portal.debug.exit( "prepareDocumentWrite" );
                },
                _documentWrite: function( buffer, /*String[]*/args ) {
                    // summary: Internal document.write() function writing to the given buffer
                    // description: Just appends the given String arguments to our buffer
					var _method = "Filter._documentWrite";
                	ibm.portal.debug.entry( _method, [ buffer, args ] );
                	var content = ""+buffer.innerHTML;
					var str = "";
                    for ( var i = 0; i < args.length; i++ ) {
                        str += args[i];
                    }
                    var markupMap = {markup: str, objects: {}};
                    if(dojo.isIE) {
                    	markupMap = com.ibm.portal.utilities.html.extractObjectElementsFromString(str);
                    }
					buffer.innerHTML = content + markupMap.markup;
					if(dojo.isIE) {
						// this if block is just for good measure since the markupMap.objects would be an
						// empty map if we weren't in IE anyway
						com.ibm.portal.utilities.html.replaceObjectElementsInMarkup(markupMap.objects);
					}
					ibm.portal.debug.exit( _method );
                },
                applyDocumentWrite: function( /*HTMLElement*/script, buffer) {
                    // summary: Injects the markup in the given buffer into the DOM
                    // description: Looks up the script element in the original DOM
                    //       and injects the markup before that element
                    var _method = "Filter.applyDocumentWrite";
                	ibm.portal.debug.entry( _method, [ script, buffer.innerHTML ] );
                    var cont = buffer.innerHTML;
                    var id = script.getAttribute( "id" );
                    var realScript = document.getElementById( id );
                    if ( cont != null && cont.length > 0 ) {
                        var tempDIV = buffer; //com.ibm.portal.utilities.html.createTemporaryMarkupDiv( cont );
                    	//var markupMap = com.ibm.portal.utilities.html.createTemporaryMarkupDiv( cont );
                        //var tempDIV = markupMap.markup;
                        var children = tempDIV.childNodes;
                        if ( children != null && children.length > 0 ) {
                            // copy the nodes over into the DOM
                            var pred = realScript;
                            // <PK92843>
                            if (pred != null) { // </PK92843>
			        var parent = pred.parentNode; // need parent to insert relative to pred
			        // we don't need an i++ at this point because
			        // a) the insertBefore call of a non-cloned element removes that element from the children list and
			        // b) in the script case we create a copy which is added to the DOM and therefore not removed from the children list
			        //    therefore we increment i to get the next element
			        for ( var i = 0; i < children.length; ) {
				    var node = children[i];
				    if (node.tagName != "SCRIPT") {
				    	// we need to inject the markup before the script element
				    	// to properly handle table updates
				    	//dojo.dom.insertAfter( node, pred );
					pred == parent.lastChild ? parent.appendChild(node) : parent.insertBefore(node, pred.nextSibling);
				    	// removes the element from the original children list, therefore we do not need to do i++
				    	pred = node;
				    }
				    else {
				    	// execute the script
				    	var fakeNode = 
						com.ibm.portal.aggregation.javascript.FAKE_SCRIPT_FACTORY.createFakeScriptElement(node.src);
				    	// this section is very important because in later script evals we read out the
				    	// id and use it as a reference to insert new statements
				    	// also it needs to be unique also in a nested use case since we remove the id
				    	// at the end of this function for cleanup reasons again
				    	node.setAttribute( "id", id + "_" + i );
				    	fakeNode.setAttribute( "id", id + "_" + i );

				    	//dojo.dom.insertAfter( fakeNode, pred );
					// insert after
					pred == parent.lastChild ? parent.appendChild(node) : parent.insertBefore(node, pred.nextSibling);
				        fakeNode.setFakeSrc(node.src);
				        pred = fakeNode;
				        i++;

				        com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER.handle( node );
				    }
			        }
			    } // end of if (pred != null)
                        }

                        //dojo.dom.destroyNode( tempDIV );
						if(tempDIV.parentNode) tempDIV.parentNode.removeChild(tempDIV);
						
						delete tempDIV;
						delete buffer;
                    }
                    // remove id attribute if it is an internal id
                    if (id.indexOf("_scr#") == 0) {
                        realScript.setAttribute( "id", "empty" );
                    }
                    ibm.portal.debug.exit( _method );
                }
              } 
);

dojo.declare( "com.ibm.portal.aggregation.javascript.ExternalScriptFilter",
              com.ibm.portal.aggregation.javascript.Filter,
              {
                doFilter: function ( /*HTMLElement*/script, portletId, dfd ) {
                    // summary: Filter the given event.
                    // description: Allows this filter to handle or alter the event. If this function
                    //      returns true, processing is stopped and the script is considered "handled".
                    // script: the script block (HTMLElement) 
                    ibm.portal.debug.entry( "ExternalScriptFilter.doFilter", [ script, portletId, dfd ]); 
                    
                    var url = this._getScriptUrl( script );
                    //Normally prefer to check for capabilities instead of a straight isIE check, but in this
                    //case, it's simply just a browser behavior issue.
                    var deferred = ( script.defer && ( dojo.isIE > 0 ) );
                    ibm.portal.debug.text( "deferred: \"" + deferred + "\"; script.defer: \"" + script.defer + "\"");
                    var onreadystatechange = script.onreadystatechange;
                    var handled = false;
                    
                    if ( url ) {
                        // check if the AJAX proxy is enabled
                        var proxyURL = ibmPortalConfig["proxyURL"];
                        if (proxyURL != null) {
                            url = this._rewriteURL(url, proxyURL);
                        }
                        // prepare bufferDiv for document.write/ln
                        var buffer = com.ibm.portal.utilities.html.createTemporaryMarkupDiv( "" ).node;
						dojo.style(buffer, "display", "none");
						dojo.body().appendChild(buffer);
                        this.prepareDocumentWrite(buffer);
                        
                        //handle onreadystatechange values
                        var onreadyFn = null;
                        if ( onreadystatechange && dojo.isIE ) {
                            ibm.portal.debug.text( "wrapping onreadystatechange handler: " + onreadystatechange );
                            onreadyFn = function ( /*String*/readyState ) {
                                ibm.portal.debug.entry( "onreadystate", [ readyState ] );
                                try {
                                    ibm.portal.debug.text( "Executing: " + onreadystatechange );
                                    onreadystatechange.apply( { readyState: readyState } );
                                }
                                catch ( error ) {
                                    ibm.portal.debug.text( "Unable to call script's onreadystatechange handler!" );
                                    com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError( "Unable to execute script tag's onreadystatechange handler!", error.message );
                                }
                                ibm.portal.debug.exit( "onreadystate" );
                            }
                        }
                        
                        var status = this._loadExternalScript( url, portletId, deferred, onreadyFn, dfd );
                        if ( status && status.addCallback ) {
                        	// apply document.write/ln to DOM
                        	var me = this;
                        	status.addCallback( function () { me.applyDocumentWrite(script, buffer); } );
                        }
                        else {
                        	if ( status instanceof Error ) {
                        		throw status;
                        	}
                        }	
                        handled = true;
                    }
                    
                    ibm.portal.debug.exit( "ExternalScriptFilter.doFilter", handled );
                    return handled;
                },
                _getScriptUrl: function ( /*HTMLElement*/script ) {
                    // summary: Retrieve the src attribute value of a script tag if it exists.
                    // returns: the url for the script tag or null if the script tag doesn't specify a URL
                    var url = null;
                    
                    if ( script.getAttribute ) {
                        url = script.getAttribute( "src" ); 
                    }
                    else {
                        var start = script.toLowerCase().indexOf( "<script" );
                        var end = script.toLowerCase().indexOf( ">" );
                        
                        var scriptTagSubstr = script.substring( start, end );
                        var srcIndex = scriptTagSubstr.toLowerCase().indexOf("src");
                        
                        
                        if (srcIndex != -1)
                        {
                            // figure out whether it's a src='blah' or src="blah"
                            var quoteIndex = scriptTagSubstr.indexOf("'", srcIndex);
                            var dblQuoteIndex = scriptTagSubstr.indexOf('"', srcIndex);
                            var markerChar = '"';
                            var markerIndex = dblQuoteIndex;
                            if (dblQuoteIndex == -1 || (quoteIndex != -1 && quoteIndex < dblQuoteIndex))
                            {
                                markerChar = "'";
                                markerIndex = quoteIndex;
                            }
                            var markerIndex2 = scriptTagSubstr.indexOf(markerChar, markerIndex + 1);
                            url = scriptTagSubstr.substring(markerIndex + 1, markerIndex2);
                        }
                    }
                    
                    return url;
                },
                _loadExternalScript: function ( /*String*/url, /*String*/portletId, /*boolean?*/deferred, /*Function?*/onreadystatechange, /*dojo.Deferred?*/dfd ) {
                    // summary: Loads an external script.
                    var _method = "ExternalScriptFilter._loadExternalScript";
                    ibm.portal.debug.entry( _method, [ url, portletId, deferred, onreadystatechange ] ); 
                    
                    var status = null;
                    var me = this;
                    try {
                        if ( url == "//:" ) {
                            //probably a hack to detect onload
                            ibm.portal.debug.text( "Empty protocol url detected. Probably a hack to detect onload!", _method );
                            /*if ( onreadystatechange ) {
                                try {
                                    ibm.portal.debug.text( "Executing onreadystatechange handler for state \"loaded\": " + onreadystatechange );
                                    onreadystatechange( "loaded" );
                                }
                                catch ( error ) {
                                    com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError( "Unable to execute script tag's onreadystatechange handler!", error.message );
                                }
                            }*/
                            me.evalGlobal( "void(0);", portletId, deferred, onreadystatechange );
                            
                            if ( dfd ) { dfd.callback(); }
                        }
                        else {
						  var ioArgs = { 
						  	url: url, 
						  	method: "GET",
						  	handle: function ( response, ioArgs ) {
						  		var xhr = ioArgs.xhr;
						  		if ( xhr.status == 200 ) {
									var data = xhr.responseText;
									ibm.portal.debug.text( "Retrieved JS file: " + data );
								  	if ( onreadystatechange ) {
								    	onreadystatechange( "loaded" );
								    }
								    me.evalGlobal( data, portletId, deferred, onreadystatechange );
								    if ( dfd ) { dfd.callback(); }
								  }
								  else if ( xhr.status == 403 ) {
									var messages = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
									status = new Error( messages[ "javascriptProxyNotEnabled" ] );
									if ( dfd ) { dfd.errback( status ); }
								  }
								  else {
									//Should likely never happen.
									status = new Error( "Javascript error from: " + url + ". Error: " + xhr.statusText );
									if ( dfd ) { dfd.errback( status ); }
								  }
						  		} 
                        	}
                        	status = dojo.xhrGet( ioArgs );
                        }      
                    } catch ( error ) {
                        ibm.portal.debug.text( "Unable to load javascript file: " + url, _method );
                        com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError( "Error while loading javascript file: " + url, error.message );
                        if ( dfd ) { dfd.errback( "Unable to load javascript file: " + url, _method ); }
                    } 
                    
                    ibm.portal.debug.exit( _method, status );
                    return status;      
                },
                _rewriteURL: function (/*String*/ targetUrl, /*String*/ proxyUrl) {
                    var newURL = proxyUrl;
                    
                    // check if this is a proxy request:
                    if ( !com.ibm.portal.utilities.isExternalUrl( targetUrl ) ) {
                        return targetUrl;
                    }

                    // rewrite
                    if (targetUrl.indexOf("https") == 0) 
                        newURL += "https/";
                    else
                        newURL += "http/";

                    newURL += targetUrl.substr(targetUrl.indexOf("://") + 3);
                    return newURL;
                }
              }   
);

dojo.declare( "com.ibm.portal.aggregation.javascript.InlineScriptFilter",
              com.ibm.portal.aggregation.javascript.Filter,
              {
                doFilter: function ( /*HTMLElement*/script, /*String*/portletId, /*dojo.Deferred?*/deferred ) {
                    // summary: Filter the given event.
                    // description: Allows this filter to handle or alter the script. If this function
                    //      returns true, processing is stopped and the script is considered "handled".
                    // script: the script block (HTMLElement) 
                    

                    var handled = false, tokenContents = "";
                    
                    if ( !dojo.isString( script ) ) {
                        //Use innerHTML because IE doesn't correctly provide the textContent for a script tag.
                        tokenContents = script.innerHTML;
                    }
                    else {
                        var firstIndex = scriptStr.indexOf(">");
                        var lastIndex = scriptStr.lastIndexOf("<");
                        var scriptStr = script;
                        tokenContents = scriptStr.substring(firstIndex+1, lastIndex);
                    }
                    
                    ibm.portal.debug.text( "Stripped HTML tags out: " + tokenContents, "processScriptArray" );
                    if ( tokenContents ) {
                        // prepare bufferDiv for document.write/ln
                        var buffer = com.ibm.portal.utilities.html.createTemporaryMarkupDiv( "" ).node;
						dojo.style(buffer, "display", "none");
						dojo.body().appendChild(buffer);
                        this.prepareDocumentWrite(buffer);
                        // Eval the contents
                        this.evalGlobal( tokenContents, portletId );
                        handled = true;
                        // apply document.write/ln to DOM
                        this.applyDocumentWrite(script, buffer);
                        if ( deferred ) { deferred.callback(); }
                    }
                    
                    return handled; 
                }
              }  
);

dojo.declare( "com.ibm.portal.aggregation.javascript.PlainScriptFilter",
              com.ibm.portal.aggregation.javascript.Filter,
              {
                doFilter: function ( /*String*/script, /*String*/portletId, /*dojo.Deferred*/deferred ) {
                    // summary: Filter the given event.
                    // description: Allows this filter to handle or alter the script. If this function
                    //      returns true, processing is stopped and the script is considered "handled".
                    // script: the script block (String) 
                    
                    if (!dojo.isString(script) || (script && script.toLowerCase() == "script") )
                        return false; // not handled

                    var tokenContents = script;
                    
                    ibm.portal.debug.text( "Executed script: " + tokenContents, "processScriptArray" );
                    if ( tokenContents ) {
                        // prepare buffer for document.write/ln
                        //var buffer = {content: ""};
                        //this.prepareDocumentWrite(buffer);
                        // Eval the contents
                        this.evalGlobal( tokenContents, portletId );
                        
                        // apply document.write/ln to DOM
                        //this.applyDocumentWrite(script, buffer);
                        if ( deferred ) { deferred.callback(); }
                    }

                    return true;    
                }
              }  
);

dojo.declare( "com.ibm.portal.aggregation.javascript.FilterChain",
              null,          
              {
                // summary: Controls the registration and execution of the filters associated
                //      with this filter chain.
                constructor: function () {
                    this._filters = new Array();
                },
                addFilter: function ( filter ) {
                    // summary: Register a filter at the end of the filter chain.
                    // filter: the filter to append
                    if ( !this._filters ) { this._filters = new Array(); }
                    this._filters.push( filter );
                },
                applyFilters: function ( script, portletId, deferred ) {
                    // summary: Execute the filter chain.
                    // description: Calls doFilter on every filter registered in this filter chain
                    //      until all filters have been called or one of the filters returns true.
                    // e: the event object
                    // returns: true if the event was handled, false if it was not
                    var i = 0;
                    var returnValue = false;
                    while ( i < this._filters.length && !returnValue )  {
                        returnValue = this._filters[i].doFilter(script, portletId, deferred);
                        i = i + 1;      
                    }
                    
                    return returnValue; //Boolean
                }   
              } 
);

dojo.declare( "com.ibm.portal.aggregation.javascript.PortletJavascriptHandler",
              null,
              {
                // summary: Examines all javascript included by a portlet, modifies it if 
                //      necessary and evaluates it in the global context.
                // description: All script elements must be loaded manually since the portlet
                //      markup is added asynchronously. This handler will evaluate a script
                //      element in the global context or load an external script file if necessary.
                constructor: function () {
                    this.filterChain = new com.ibm.portal.aggregation.javascript.FilterChain();
                },
                handle: function ( script, portletId ) {
                    // summary: Handles a script element added asynchronously.
                    // script: the script element
                    ibm.portal.debug.entry( "JavascriptHandler.handle", [ script, portletId ] );
                    //try {
                    var val = this.filterChain.applyFilters( script, portletId );
                    //}
                    //catch ( error ) {
                    //   com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError( "Error encountered while processing script tags for portlet: " + portletId, error.message );
                    //}   
                    ibm.portal.debug.exit( "JavascriptHandler.handle" );
                }
              }  
);

dojo.declare( "com.ibm.portal.aggregation.javascript.AsyncJavascriptProcessor",
			  null,
			  {
			  	constructor: function () {
			  		this.filterChain = new com.ibm.portal.aggregation.javascript.FilterChain();	
			  	},
			  	handle: function ( script, portletId ) {
                    // summary: Handles a script element added asynchronously.
                    // script: the script element
                    ibm.portal.debug.entry( "AsyncJavascriptProcessor.handle", [ script, portletId, deferred ] );

                    var deferred = new dojo.Deferred();
                    this.filterChain.applyFilters( script, portletId, deferred );

                    ibm.portal.debug.exit( "AsyncJavascriptProcessor.handle" );
                    return deferred;
                }
			  }	
);

dojo.declare( "com.ibm.portal.aggregation.javascript.AsyncPortletScriptElementsHandler",
			  null,
			  {
			  	constructor: function ( /*Array*/scriptElements, /*String*/pId, /*AsyncJavascriptProcessor*/scriptProcessor ) {
			  		// summary: Constructs a new object for handling an array of script elements associated with a single portlet instance. Each script
			  		//		element is looped over and evaluated asynchronously (non-blocking) to allow other scripts to continue to execute.
			  		// description: A consumer/interested party interacts with the asynchronous script handler through the event handler object. Before
			  		//		"start"ing the script handling process, the consumer or interested party should register an event handler object using the 
			  		//		setEventHandler method. The event handler object should define a method for "error", "scriptDone", "completed", and "started".
			  		//		The implementation does not have to DO anything for every event, but each method MUST be defined, even if it is empty. The script
			  		//		handler will call each method as each event occurs - "started" when script handling begins; "scriptDone" when an individual script
			  		//		element is done; "error" when a script element causes an error; and "completed" when all script elements have been executed.
			  		// scriptElements: the array of script elements to process
			  		// pId: the portlet ID of the portlet instance to which the script elements belong
			  		// scriptProcessor: the object which will actually process the individual script elements
			  		var _method = "AsyncPortletSCriptElementsHandler.constructor";
			  		ibm.portal.debug.entry( _method, [ scriptElements, pId, scriptProcessor ] );
			  		
			  		this.index = 0;
			  		this.elements = scriptElements;
			  		this.eventHandler = null;
			  		this.portletId = pId;
			  		this.processor = scriptProcessor;
			  		this.stopOnError = false;
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	setEventHandler: function ( /*Object*/handler ) {
			  		// summary: Sets the event handler object. If a required method is not defined, the handler will be ignored.
			  		// description: The event handler object should define a method for "error", "scriptDone", "completed", and "started". The implementation 
			  		// 		does not have to DO anything for every event, but each method MUST be defined, even if it is empty.
			  		// handler: the event handler object
			  		var _method = "AsyncPortletScriptElementsHandler.setEventHandler";
			  		ibm.portal.debug.entry( _method, [ handler ] );
			  		
			  		if ( handler.error && handler.scriptDone && handler.completed && handler.started ) {
			  			this.eventHandler = handler;
			  		}
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	_fireEvent: function ( /*String*/eventName, /*Array?*/args ) {
			  		// summary: Fires the given event on the provided event handler.
			  		// eventName: the name of the event to fire
			  		var _method = "_fireEvent";
			  		ibm.portal.debug.entry( _method, [ eventName ] );
			  		
			  		if ( this.eventHandler && this.eventHandler[eventName] ) {
			  			var eventFunc = this.eventHandler[eventName];
			  			//IE doesn't like apply calls with an undefined array. Firefox seems to fail gracefully in this case.
			  			if ( args ) {
			  				eventFunc.apply( eventFunc, args );
			  			}
			  			else {
			  				eventFunc();
			  			}	
			  		}
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	start: function () {
			  		// summary: Starts processing the script elements provided in the constructor.
			  		var _method = "AsyncPortletScriptElementsHandler.start";
			  		ibm.portal.debug.entry( _method );
			  		
			  		this._fireEvent( "started" );
			  		this.nextScript();
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	scriptDone: function () {
			  		// summary: Indicates an individual script element has completed.
			  		var _method = "AsyncPortletScriptElementsHandler.scriptDone";
			  		ibm.portal.debug.entry( _method );
			  		
			  		this._fireEvent( "scriptDone" );
			  		
			  		this.index = this.index+1;
			  		this.nextScript();
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	scriptError: function ( error ) {
			  		// summary: Indicates an error has occurred during script processing, not execution.
			  		// description: Please note that this refers to script processing, not execution. Errors that occur
			  		//		during script execution will interrupt this process.
			  		// error: the error that was generated
			  		var _method = "AsyncPortletScriptElementsHandler.scriptError";
			  		ibm.portal.debug.entry( _method, [ error ] );
			  		
			  		this._fireEvent( "error", [ error ] );
			  		
			  		if ( this.stopOnError ) {
			  			this.completed();
			  		}
			  		else {
			  			this.index = this.index+1;
			  			this.nextScript();
			  		}	
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	completed: function () {
			  		// summary: Indicates that the all script elements have been processed.
			  		var _method = "AsyncPortletScriptElementsHandler.completed";
			  		ibm.portal.debug.entry( _method );
			  		
			  		this._fireEvent( "completed" );
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	nextScript: function () {
			  		// summary: Called to indicate that the next script can be processed.
			  		// description: The actual call to execute the script element is wrapped inside a timeout to allow
			  		//		all running javascript to complete.
			  		var _method = "AsyncPortletScriptElementsHandler.nextScript";
			  		ibm.portal.debug.entry( _method );
			  		
			  		var me = this;
			  		setTimeout( function () {
			  			if ( me.index < me.elements.length ) {
				  			var d = me.processor.handle( me.elements[me.index], me.portletId );
				  			d.addCallback( dojo.hitch( me, "scriptDone" ) );
				  			d.addErrback( dojo.hitch( me, "scriptError" ) );
				  		} else {
				  			me.completed();
				  		}
			  		}, 5);
			  		
			  		
			  		ibm.portal.debug.exit( _method );
			  	}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.FunctionExecutor",
        null,
        {
         	// summary: Executes the appropriate event broker events before and after the execution
			// 		of the provided function.
			// description: The primary use case for this executor is when executing onload handlers. We need
			//		to be able to clean up any javascript variables or widgets created in an onload handler 
			//		(for example, the dojo parser is usually called in an onload handler). The executor fires
			//		the startScriptExecution/endScriptExecution events around executing the provided function.
			// func: the function to execute
			// portletId: the id of the portlet executing the function
			// deferred: a deferred object (OPTIONAL)
			// returns: returns the return value from the provided function
		   	run: function ( /*Function*/func, /*String*/portletId, /*dojo.Deferred*/deferred ) {
              // summary: Filter the given event.
              // description: Allows this filter to handle or alter the script. If this function
              //      returns true, processing is stopped and the script is considered "handled".
              // script: the script block (String) 
              var _method = this.declaredClass + ".run";
			  ibm.portal.debug.entry( _method, [ func, portletId, deferred ] );
              
              ibm.portal.debug.text( "Executing function: " + func.toString() );
              var retVal = undefined;
              if ( func ) {
                  com.ibm.portal.EVENT_BROKER.startScriptExecution.fire( { portletId: portletId } );
                  retVal = func();
                  com.ibm.portal.EVENT_BROKER.endScriptExecution.fire( { portletId: portletId } );
                  if ( deferred ) { deferred.callback(); }
              }

              ibm.portal.debug.exit( _method, retVal );
              return retVal;    
          }
        }  
);

dojo.declare( "com.ibm.portal.aggregation.javascript.OnloadHandler",
              null,
              {
                // summary: Handles any onload handlers a portlet might register.
                // description: Looks for any onload handlers set using the W3C methods or added by
                //        setting an attribute on the document body.
                
                constructor: function () {
                    this._onLoadMap = [];
                    this._activePortlet = null;
                
                    this._startScriptHandlingHandle = com.ibm.portal.EVENT_BROKER.startScriptHandling.register( this, "startListening" );
                    this._setContextHandle = com.ibm.portal.EVENT_BROKER.startScriptExecution.register( this, "setRegistrationContext" );
                    this._clearContextHandle = com.ibm.portal.EVENT_BROKER.endScriptExecution.register( this, "clearRegistrationContext" );
                    this._endScriptHandlingHandle = com.ibm.portal.EVENT_BROKER.endScriptHandling.register( this, "stopListening" );    
                    //In the case where a portlet with an inline-mode enabled skin has one of the inline modes open upon initial rendering,
                    //(portlet is put into Edit Shared Settings mode and then the user hits Refresh), there may be multiple listen begin notifications
                    //for the case where we render the view mode as well as the edit mode. We only want to override the native functions once so
                    //we'll track the number of listeners and only restore (or override) the native functions when the number of listeners is zero.
                    this.listening = 0;
                },                
                startListening: function ( args ) {
                    // summary: Starts listening for the registration of onload handlers.
                    // description: Overwrites the addEventListener/attachEvent functions on the window
                    //        object, as well as, the setAttribute function on the body object.
                    ibm.portal.debug.entry( "OnloadHandler.startListening", [ args ] );
                	
                	if ( this.listening == 0 ) {
	                	var me = this;
	                    //this._activePortlet = args["portletId"];
	                    if ( window.addEventListener ) {
	                        this.window_addEventListener = window.addEventListener;
	                        window.addEventListener = function (a,b,c) { me._addEventListener( a,b,c ); };
	                    }
	                    else {
	                        window.window_attachEvent = window.attachEvent;                
	                        window.attachEvent = function(a,b) { me._attachEvent(a,b); };
	                    }   
	    
	                    this.original_setAttribute = document.body.setAttribute;
	                    document.body.setAttribute = function ( a,b,c) { me._interceptSetAttribute(a,b,c ); };
                	}
                	
                	this.listening++;
                    
                	ibm.portal.debug.text( "Number of listening handlers: " + this.listening );
                	
                    ibm.portal.debug.exit( "OnloadHandler.startListening" );
                },
                stopListening: function ( args ) {
                    // summary: Stops listening for onload handlers.
                    // description: Restores all overwritten functions to the original state. startListening should
                    //        never be called without a corresponding stopListening call.
                    ibm.portal.debug.entry( "OnloadHandler.stopListening" );
                    
                	this.listening--;
                
                	ibm.portal.debug.text( "Number of listening handlers: " + this.listening );
                	
                    if ( this.listening == 0 ) {
	                	if ( window.addEventListener ) {
	                        window.addEventListener = this.window_addEventListener;
	                    }
	                    else {
	                        window.attachEvent = window.window_attachEvent;                
	                    }   
	    
	                    document.body.setAttribute = this.original_setAttribute;
                    }    
                    //com.ibm.portal.EVENT_BROKER.startScriptHandling.unregister( this, "startListening" );
                    //com.ibm.portal.EVENT_BROKER.endScriptHandling.unregister( this, "stopListening" );    
                    
                    this.executeHandlers( args ); 
                    this._activePortlet = null;
                    
                    ibm.portal.debug.exit( "OnloadHandler.stopListening" );
                },
                executeHandlers: function( args ) {
                    // summary: Executes the registered onload handlers for the given portletid.
                    // portletid: the portlet id to execute onload handlers for
                    ibm.portal.debug.entry( "OnloadHandler.executeHandlers" );
                    
                    var portletid = args["portletId"];
                    var handlers = this._onLoadMap[ portletid ];
                    if ( handlers ) { 
                    	var executor = new com.ibm.portal.aggregation.javascript.FunctionExecutor();
                    	
                    	executor.run( function () { 
	                    	for (var i=0; i<handlers.length; i++) {
	                             var obj = handlers[i];
	
	                             try {
	                                 var fkt = obj.fkt;
	                                 var pthis = obj.pthis;
	                                 fkt.call(pthis);
	                             }
	                             catch (error)
	                             {
	                                    com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError( "Unable to execute onload function!", error.message );
	                             }
	                         }
	                         
	                         
                    	}, portletid);
                    	
                    	delete this._onLoadMap[portletid];
                    }
                    
                    ibm.portal.debug.exit( "OnloadHandlers.executeHandlers" );   
                },
                setRegistrationContext: function ( /*Object*/ args ) {
                    // summary: Sets the active portlet id.
                    this._activePortlet = args["portletId"];    
                },
                clearRegistrationContext: function ( /*Object*/ args ) {
                    // summary: Clears the active portlet id.
                    this._activePortlet = null;
                },
                _registerHandler: function ( /*Function*/ handler ) {
                    // summary: Registers an onload handler.
                    // handler: the onload handler function
                    ibm.portal.debug.entry( "OnloadHandler._registerHandler", [ handler ] );
                    
                    var map = this._onLoadMap[this._activePortlet];
                    
                    //Create the map for this portlet if necessary. 
                    if ( !map ) {
                        this._onLoadMap[this._activePortlet] = [];
                        map = this._onLoadMap[this._activePortlet];
                    }
                    //Create the function that will be called when the onload handlers
                    //are executed. We need to create an event object to simulate the
                    //actual onload call.
                    var _fkt = function () {
                        ibm.portal.debug.text( "handler function is: " + handler );
                        if ( document.createEvent ) {
                            var event = document.createEvent("Events");
                            event.initEvent( "load", true, true );
                            handler.apply( window, [ event ] );
                        }
                        else {
                            var funct = function () { return handler(); };
                            document.body.attachEvent( "onload", funct );
                            document.body.fireEvent( "onload" );
                            document.body.detachEvent( "onload", funct );
                        }
                    };
                    //Add to the map.
                    map.push( {text: "",
                               fkt: _fkt,
                               pthis: window });    
                               
                    ibm.portal.debug.exit( "OnloadHandler._registerHandler" );
                },
                _addEventListener: function(eventType, handler, useCapture) {
                    // summary: Overwritten function for adding an onload handler.
                    if (eventType=="load" && this._activePortlet ) {
                        this._registerHandler( handler );                   
                    }
                    else {
                        this.window_addEventListener.apply(window, [eventType, handler, useCapture]);
                    }   
                },
                _attachEvent: function(event, handler) {
                    // summary: Overwritten function for adding an onload handler in IE.
                    if (event=="onload" && this._activePortlet ) {
                        this._registerHandler( handler );
                    }
                    else {
                        window.window_attachEvent( event, handler );
                    }
                },
                _interceptSetAttribute: function(name, value, caseSensitiv) {
                    // summary: Overwritten function for setting an onload handler on the body.
                    if (name=="onload" && this._activePortlet ) {
                        var fkt = null;
                                        
                        if ( typeof(value) == "string" && value.indexOf("function" ) != 0 ) {
                            //JSF Hack
                            var text = "var event = document.createEvent(\"Events\"); event.initEvent(\"load\", true, true); " + value;
                            fkt = new Function( text );
                        }
                        else if ( typeof(value) == "function" ){
                            fkt = value;    
                        }
                        
                        this._registerHandler( fkt );
                    }
                    else {
                        this.original_setAttribute.apply( document.body, [ name, value, caseSensitiv ]);
                    }
                }   
              }
);

com.ibm.portal.aggregation.javascript.JS_ONLOAD_HANDLER = new com.ibm.portal.aggregation.javascript.OnloadHandler();
com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER = new com.ibm.portal.aggregation.javascript.PortletJavascriptHandler();
com.ibm.portal.aggregation.javascript.ASYNC_JAVASCRIPT_HANDLER = new com.ibm.portal.aggregation.javascript.AsyncJavascriptProcessor();

//Register the filters. The order registered is the order executed.
com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.PlainScriptFilter() );
com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.ExternalScriptFilter() );
com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.InlineScriptFilter() );

com.ibm.portal.aggregation.javascript.ASYNC_JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.PlainScriptFilter() );
com.ibm.portal.aggregation.javascript.ASYNC_JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.ExternalScriptFilter() );
com.ibm.portal.aggregation.javascript.ASYNC_JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.InlineScriptFilter() );

}
