/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval"] = true;
dojo.provide( "com.ibm.portal.aggregation.javascript.uneval" );

dojo.require( "com.ibm.portal.EventBroker" );

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.Collector",
			  null,
			  {
				// summary: A collector is called before or after a piece of javascript is executed. The delta between the two
				//		collections determines what javascript variables/functions were declared in that piece of javascript. THis
				//		approach works for all browsers except IE because IE does not enumerate global javascript variables/functions
				//		as part of the global object/scope. In IE, any global javascript variables that need to be cleaned up should
				//		added to the com.ibm.portal.aggregation.javascript.uneval.collectors.COMMON_GLOBAL_VARIABLES collection.
				collect: function () {
					// summary: Collect the necessary information required to clean up a set of properties or objects.
					dojo.unimplemented( "com.ibm.portal.aggregation.javascript.uneval.Collector.collect" );
				},
				getValues: function () {
					// summary: Retrieves the values collected by the collect action.
					// return: implementation specific; a scrubber and a collector usually go hand in hand
					dojo.unimplemented( this.declaredClass + ".getValues" );
				}	
			  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.Scrubber",
			  null,
			  {
				// summary: A scrubber acts as a factory to create specific collector instances and will perform the process of 
				//		cleaning up or null'ing out the delta between the values collected by the collectors. 
				newCollector: function () {
					// summary: Creates a new collector appropriate for this scrubber.
					// returns: a new instance of a Collector
					dojo.unimplemented( "com.ibm.portal.aggregation.javascript.uneval.Scrubber.newCollector");
				},
				clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
					// summary: Given two collectors, clean the difference between the two.
					// collector1: the initial state to return to
					// collector2: the final state
					dojo.unimplemented( "com.ibm.portal.aggregation.javascript.uneval.Scrubber.clean" );
				},
				getId: function () {
					// summary: Retrieve an ID describing this Scrubber. The ID is unique to the class, but not
					//		the instance.
					return this.declaredClass;
				}	
			  }
);



dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.Snapshot",
			  null,
			  {
			  	// summary: A Snapshot cycles through all the registered scrubbers and executes the collector for
				//		that scrubber. A Snapshot is taken before and after each piece of JavaScript is executed.
				constructor: function () { 
			  		ibm.portal.debug.entry( "Snapshot.constructor" );
			  		
			  		this._collectors = {};
			  		var scrubbers = com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS;
			  		var length = scrubbers.length;
			  		ibm.portal.debug.text( "Executing " + length + " scrubbers.");
			  		for ( var i = 0; i < length; i++ ) {
			  			var scrubber = scrubbers[i];
			  			ibm.portal.debug.text( "Executing scrubber: " + scrubber.declaredClass );
			  			var collector = scrubber.newCollector();
			  			collector.collect();
			  			this._collectors[ scrubber.getId() ] = collector;
			  		}	
			  		
			  		ibm.portal.debug.exit( "Snapshot.constructor" );
			  	},
				getCollectors: function () {
			  		return this._collectors;
			  	}	
			  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.RestorePoint",
			  null, 
			  {
			  	// summary: Represents the current globally available javascript functions and
				//		variables.
				// description: The "recording" is made by essentially seeing what properties the window object has
				//		before the portlet's javascript is executed and then again after the portlet's
				//		javascript has been loaded. The new properties are the ones that will be 
				//		deleted when the "restoration" is performed. 
				SCRUBBERS: [],
				constructor: function ( /*Snapshot*/state1, /*Snapshot*/state2 ) { 
			  		this._state1 = state1;
			  		this._state2 = state2;
			  	},
				restore: function ( /*Scrubber*/scrubber ) {
					// summary: Restores the window object to the given restore point. If no deltaRestorePoint
					//		is specified, the current window object's properties are used as a comparison point.
					
			  		var collectors1 = this._state1.getCollectors();
			  		var collectors2 = this._state2.getCollectors();
			  		
			  		var collector1 = collectors1[scrubber.getId()];
			  		var collector2 = collectors2[scrubber.getId()];
			  		
			  		if ( collector1 && collector2 ) {
			  			scrubber.clean( collector1, collector2 );
			  		}	
					
					com.ibm.portal.EVENT_BROKER.javascriptCleanup.fire();
				},
				restoreAll: function () {
					var scrubbers = this.SCRUBBERS;
			  		var collectors1 = this._state1.getCollectors();
			  		var collectors2 = this._state2.getCollectors();
			  		
			  		var length = scrubbers.length;
			  		for ( var i = 0; i < length; i++ ) {
			  			var scrubber = scrubbers[i];
			  			var collector1 = collectors1[scrubber.getId()];
			  			var collector2 = collectors2[scrubber.getId()];
			  			
			  			if ( collector1 && collector2 ) {
			  				scrubber.clean( collector1, collector2 );
			  			}	
			  		}	
					
					com.ibm.portal.EVENT_BROKER.javascriptCleanup.fire();
				},	
				getSupportedScrubbers: function() {
					return this._state1.SCRUBBERS;
				}
			  }		
);

}
