/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.data.PumaStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.data.PumaStore"] = true;
dojo.provide( "com.ibm.portal.data.PumaStore" );

dojo.require( "com.ibm.data.AtomFeedStore" );

dojo.declare( "com.ibm.portal.data.PumaStore",
			com.ibm.data.AtomFeedStore,
			{
				// namespaces: Map
				//		Map of namespaces used by the Puma feeds
				namespaces: dojo.mixin({}, com.ibm.data.AtomFeedStore.prototype.namespaces, {
		            "xhtml" : "http://www.w3.org/1999/xhtml",
		            "xs" : "http://www.w3.org/2001/XMLSchema-datatypes",
		            "um" : "http://www.ibm.com/xmlns/prod/websphere/um.xsd"
	        	}),
		        
		        // methodTunneling: Boolean specifies whether or not to tunnel DELETE and PUT requests
		        //		through POST requests with special headers and/or request parameters.  Set this to
		        //		true in environments where DELETE and PUT requests are blocked.
		        methodTunneling: false,
		        
		        // itemClass: String the class used to represent items of this store
				itemClass: "com.ibm.portal.data.PumaItem",
				
				constructor: function( /*Map*/ args ) {
					// summary: Creates a new instance of a this data store
					// args: Map
					//		The args map will be used to replace any default values of this store with the
					//		values specified in the args map.
					this._features['dojo.data.api.Identity'] = true;
				},
				
				handleConstructs: function(/*DOMNode*/ node, /*String*/ prop) {
					switch(prop) {
						case "PumaAttributeConstruct": return new com.ibm.portal.data.PumaAttributeConstruct(node);
						case "PumaProfileConstruct": return new com.ibm.portal.data.PumaProfileConstruct(node);
						default: return this.inherited(arguments);
					}
				},
				_buildUrl: function(keywordArgs, baseUrl) {
					// summary: Builds a url string from this store's base url and the
					// 		query argument.
					// keywordArgs: Object
					//		A request object map as sent to the fetch function
					// returns: String The constructed url.
					var ret = "";
					keywordArgs = this._prepareQuery(keywordArgs);
					var query = keywordArgs.query;
					var qStart = baseUrl.indexOf("?");
					var baseQuery = "";
					if(qStart > -1) {
						baseQuery = baseUrl.substring(qStart);
						baseUrl = baseUrl.substr(0, qStart);
					}
					if(baseQuery.length > 0 && baseQuery.charAt(baseQuery.length - 1) == "/") baseQuery = baseQuery.substr(0, baseQuery.length - 1);
					
					// append a / if it's necessary
					if((baseUrl.length > 0 && baseUrl.charAt(baseUrl.length - 1) != "/")
						|| baseUrl.length == 0) baseUrl += "/";
					
					switch(query.type) {
						case "userAttributes": baseUrl += "attributes/users/"; 
							break;
						case "groupAttributes": baseUrl += "attributes/groups/"; 
							break;
						case "userProfiles": baseUrl += "users/profiles/";
							break;
						case "groupProfiles": baseUrl += "groups/profiles/";
							break;
						case "currentProfile": baseUrl += "currentuser/profile/";
							break;
						case "groupMembership": baseUrl += "groupmembership/";
							break;
					}
					
					delete query.type;
					
					if(query.id) {
						baseUrl += id;
					}
					
					var queryPart = dojo.objectToQuery(query);
					
					ret = baseUrl + "?" + queryPart + baseQuery;
					
					return (this.modifyUrl(keywordArgs, ret));
				},
				modifyRequestArgs: function(ioArgs, request) {
					// see com.ibm.data.AtomFeedStore@modifyRequestArgs
					// 
					if(!ioArgs.headers) {
						ioArgs.headers = {};
					}
					ioArgs.headers["accept"] = "application/atom+xml";
					return ioArgs;
				},			
				fetchItemByIdentity: function(/* object */ keywordArgs){
					//	summary:
					//		Given the identity of an item, this method returns the item that has 
					//		that identity through the onItem callback.  Conforming implementations 
					//		should return null if there is no item with the given identity.  
					//		Implementations of fetchItemByIdentity() may sometimes return an item 
					//		from a local cache and may sometimes fetch an item from a remote server, 
					//
					// 	keywordArgs:
					//		An anonymous object that defines the item to locate and callbacks to invoke when the 
					//		item has been located and load has completed.  The format of the object is as follows:
					//		{
					//			identity: string|object,
					//			onItem: Function,
					//			onError: Function,
					//			scope: object
					//		}
					//	The *identity* parameter.
					//		The identity parameter is the identity of the item you wish to locate and load
					//		This attribute is required.  It should be a string or an object that toString() 
					//		can be called on.
					//		
					//	The *onItem* parameter.
					//		Function(item)
					//		The onItem parameter is the callback to invoke when the item has been loaded.  It takes only one
					//		parameter, the item located, or null if none found.
					//
					//	The *onError* parameter.
					//		Function(error)
					//		The onError parameter is the callback to invoke when the item load encountered an error.  It takes only one
					//		parameter, the error object
					//
					//	The *scope* parameter.
					//		If a scope object is provided, all of the callback functions (onItem, 
					//		onError, etc) will be invoked in the context of the scope object.
					//		In the body of the callback function, the value of the "this"
					//		keyword will be the scope object.   If no scope object is provided,
					//		the callback functions will be called in the context of dojo.global.
					//		For example, onItem.call(scope, item, request) vs. 
					//		onItem.call(dojo.global, item, request)
					var id = keywordArgs.identity;
					
					// our IDs may come in as urn:id:# and all we want is the # part
					if(id.indexOf(":") > -1) id = id.substring(id.lastIndexOf(":") + 1);
					var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
					var args = {
						query: {
							id: id,
							collection: "ui.all"
						},
						onComplete: dojo.hitch(this, 
							function(items) {
								if(items.length == 0 && keywordArgs.onError) {
									keywordArgs.onError.call(scope, "No items returned matching identity " + id);
								}
								else {
									keywordArgs.onItem.call(scope, items[0]);
								}
							}
						)
					};
					if(keywordArgs.onError) args.onError = dojo.hitch(scope, keywordArgs.onError);

					return this.fetch(args);
				}
			}
);


dojo.declare( "com.ibm.portal.data.PumaItem",
			com.ibm.data.AtomEntryItem, 
			{
				// namespaces: Map
				//		Map of namespaces used by the Puma feeds
				namespaces: dojo.mixin({}, com.ibm.portal.data.PumaStore.prototype.namespaces),
				
		        attrMappings: dojo.mixin({}, com.ibm.data.AtomEntryItem.prototype.attrMappings, {
					pumaProfile: {xpath: "./atom:content/um:profile", targetName: "PumaProfileConstruct", type: "node"},
					pumaAttribute: {xpath: "./atom:content/um:profile/um:attribute'", targetName: "PumaAttributeConstruct", type: "node"},
					pumaAttribute$: {xpath: "./atom:content/um:profile/um:attribute[@name='${0}']", targetName: "PumaAttributeConstruct", type: "node"}
				})
			}
);

dojo.declare( "com.ibm.portal.data.PumaProfileConstruct",
			com.ibm.data._XmlConstruct,
			{
				namespaces: dojo.mixin({}, com.ibm.portal.data.PumaStore.prototype.namespaces),
				
		        constructor: function(/*DOMNode*/ root) {
					this.root = root;
				},
				handleConstructs: function(/*DOMNode*/ node, /*String*/ prop) {
					switch(prop) {
						case "PumaAttributeConstruct": return new com.ibm.portal.data.PumaAttributeConstruct(node);
						default: return this.inherited(arguments);
					}
				},
				getType: function() {
					// summary: Gets the type of this Puma Profile.
					// returns: String type.  Either user or group.
					return this._getAttribute(this.root, "type");
				},
				getIdentifier: function() {
					// summary: Gets the identifier of this Puma Profile.
					// returns: String identifier.
					return this._getAttribute(this.root, "identifier");
				},
				getPumaAttributes: function() {
					// summary: Gets an array of all the Puma Attributes in this Puma Profile.
					// returns: Array of PumaAttributeConstructs.
					var res = this._getNodesByXpath(this.root, "./um:attribute", "PumaAttributeConstruct");
					return res.value;
				},
				toString: function() {
					var ret = ["<", 
						this.__getClassName(), 
						" type=", 
						this.getType(), 
						" identifier='",
						this.getIdentifier(),
						"' >"].join('');
				}
			}
);

dojo.declare( "com.ibm.portal.data.PumaAttributeConstruct",
			com.ibm.data._XmlConstruct,
			{
				namespaces: dojo.mixin({}, com.ibm.portal.data.PumaStore.prototype.namespaces),
				
		        constructor: function(/*DOMNode*/ root) {
					this.root = root;
				},
				getName: function() {
					// summary: Gets the name of this Puma Attribute.
					// returns: String name.
					return this._getAttribute(this.root, "name");
				},
				getType: function() {
					// summary: Gets the type of this Puma Attribute which is an XmlSchema datatype.
					// returns: String type.
					return this._getAttribute(this.root, "type");
				},
				isMultiValued: function() {
					// summary: Returns true|false indicating if this Puma Attribute is multi-valued.
					// returns: Boolean indicating if this is multi-valued.
					var multiV = this._getAttribute(this.root, "multiValued");
					if(multiV) return multiV.toLowerCase() == "true";
					return false;
				},
				getValue: function() {
					// summary: Gets the first value of this Puma Attribute, which is the only value
					//		of a Puma Attribute that is not multi-valued.
					// returns: First value.
					var res = this._getNodeByXpath(this.root, "./um:attributeValue", "textContent");
					return res.value;
				},
				getValues: function() {
					// summary: Gets an array of the values of this Puma Attribute, which contains only
					//		one array object if the Puma Attribute is not multi-valued.
					// returns: Array of all values.
					var res = this._getNodesByXpath(this.root, "./um:attributeValue", "textContent");
					return res.value;
				},
				toString: function() {
					var ret = "<" + this.__getClassName() + " {" + this.getName() + "=";
					if(this.isMultiValued()) {
						var values = this.getValues();
						dojo.forEach(values, function(val, idx, arr){
							ret += val;
							if(idx < arr.length - 1) {
								ret += ",";
							}
						});
					}
					else {
						ret += this.getValue();
					}
					ret += "}>";
					return ret;
				}
			}
);

}
