/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.performance"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.performance"] = true;
//Define the package.
dojo.provide( "com.ibm.portal.performance" );

dojo.declare( "com.ibm.portal.performance.Timer",
			  null,
			  {
			  	constructor: function () {
					/*
					 * summary: A simple timer object that is manually started and stopped.
					 */
					this.startDate = 0;
					this.stopDate = 0;
				},
				start: function () {
					/*
					 * summary: Starts the timer.
					 */
					if ( this.startDate != 0 ) {
						throw new Error( "Timer already started!" );
					}
					
					this.startDate = new Date();
				},
				
				stop: function () {
					/*
					 * summary: Stops the timer. If the timer has already been stopped, this function
					 * will throw an exception.
					 */ 
					if ( this.stopDate != 0 ) {
						throw new Error("Timer already stopped!" );
					}
					
					this.stopDate = new Date();
				},
				
				isRunning: function () {
					/*
					 * summary: Indicates whether the timer is running or not.
					 */
					var retVal = false;
					
					if ( this.startDate != 0 && this.stopDate == 0  ) {
						retVal = true;
					}
					
					return retVal; /*boolean*/
				},
				
				getElapsed: function () {
					/*
					 * summary: Calculates the elapsed time in the current Timer. If the Timer is still running,
					 * the current time when this function is executed is used to calculate the elapsed time. If the 
					 * elapsed time cannot be calculated, -1 is returned.
					 */
					var elapsed = -1;
					
					if ( this.stopDate == 0 ) {
						elapsed = (new Date().getTime()) - this.startDate.getTime();
					}
					else {
						elapsed = this.stopDate.getTime() - this.startDate.getTime();
					}
					
					return elapsed; /*int*/
				}
			  }
);

dojo.declare( "com.ibm.portal.performance.PerformanceTestResults",
			  null,
			  {
			  	constructor: function () {
					/*
					 * summary: Stores timing results for interpretation.
					 */	
					this.results = new Array();
					this._maxValue = null;
					this._minValue = null;	
				},
				add: function ( /*Timer*/ timer ) {
					/*
					 * summary: Add a result to the list.
					 */
					var currentIndex = this.results.length;
					this.results.push(timer);
					
					if ( this._maxValue == null ) {
						this._maxValue = currentIndex;
					}
					else {
						if ( timer.getElapsed() > this.results[this._maxValue].getElapsed() ) {
							this._maxValue = currentIndex;
						}
					}
					
					if ( this._minValue == null ) {
						this._minValue = currentIndex;
					}
					else {
						if ( timer.getElapsed() < this.results[this._minValue].getElapsed() ) {
							this._minValue = currentIndex;
						}
					}
				},
				
				getResult: function ( /*int*/ pos ) {
					/*
					 * summary: Returns the result that matches the given id or NULL if no match is found.
					 */
					var result = null;
					
					if ( this.results[pos].stopTime ) { 
						result = this.results[pos]; 
					}
					
					return result; /*Timer*/
				},
				
				getMin: function () {
					/*
					 * summary: Returns the result with with the shortest elapsed time.
					 */
					var retVal = null;
					if ( this._minValue != null ) {
						retVal = this.results[this._minValue];	
					}
					
					return retVal; /*Timer*/
				},
				
				getMax: function () {
					/*
					 * summary: Returns the result with the greatest elapsed time.
					 */
					var retVal = null;
					if ( this._maxValue != null ) {
						retVal = this.results[this._maxValue];	
					}
					
					return retVal; /*Timer*/
				},
				
				createHtmlTable: function() {
					var table = document.createElement( "TABLE" );
					var tbody = document.createElement( "TBODY" );
					
					var row = document.createElement( "TR" );
					var headers = [ "Index", "Start", "Stop", "Elapsed" ]
					
					for ( var i = 0; i < headers.length; i++ ) {
						var th = document.createElement( "TH" );
						th.innerHTML = headers[i];
						row.appendChild( th );
					}
					
					tbody.appendChild( row );
					
					for ( var j = 0; j < this.results.length; j++ ) {
						row = document.createElement( "TR" );
						var td = document.createElement( "TD" );
						td.innerHTML = j;
						row.appendChild( td );
						
						td = document.createElement( "TD" );
						td.innerHTML = this.results[j].startDate.getTime();
						row.appendChild( td );
						
						td = document.createElement( "TD" );
						td.innerHTML = this.results[j].stopDate.getTime();
						row.appendChild( td ); 
						
						td = document.createElement( "TD" );
						td.innerHTML = this.results[j].getElapsed();
						row.appendChild( td );
						
						if ( j == this._maxValue ) {
							row.style.color = "white";
							row.style.backgroundColor = "red";
						}
						
						if ( j == this._minValue ) {
							row.style.color = "white";
							row.style.backgroundColor = "green";
						}
						
						tbody.appendChild( row );
					}
					
					//table.appendChild( thead );
					table.appendChild( tbody );
					
					return table;	
				},
				
				calculateAverageElapsed: function () {
					/*
					 * summary: Calculates the average elapsed time of all the results.
					 */
					var sum = 0;
					
					for ( result in this.results ) {
						sum += this.results[result].getElapsed();	
					}
					
					return (sum / this.results.length ); /*int*/
				},
				
				size: function () {
					/*
					 * summary: Returns the number of results.
					 */
					return this.results.length; /*int*/
				}
			  }
);

dojo.declare( "com.ibm.portal.performance.RepetitivePerformanceTest", 
			  null,
			  {
			  	constructor: function ( /*String*/ action, /*int*/ reps, /*int?*/ interval, /*String?*/ preAction, /*String?*/ postAction ) {
					/*
					 * summary: Executes a specific action a specified number of times and records the
					 * time it takes to execute each action. If an interval is specified, the test will 
					 * leave the specified interval of time in between executions. Otherwise, the interval
					 * defaults to zero and the tests will be executed consecutively, with no "rest" period.
					 * The status of the tests may be monitored by registering status listeners via the 
					 * addStatusListener method. All registered listeners will be notified on a regular basis
					 * of how many tests have been completed of the total batch. The "regular basis" is 
					 * determined by the property "statusNotificationInterval". It defaults to a value of 5 and
					 * so will notify all registered listeners after every 5 tests have been completed, unless
					 * changed.
					 * 
					 * action: the javascript action in string form to repeat
					 * reps: the number of repetitions to perform
					 * interval: *optional* the time between executions in milliseconds *default=0*
					 * preAction: *optional* An action to be executed every time before the main action is timed
					 * postAction: *optional* An action to be executed every time after the main action is timed
					 */
					this.performanceAction = action;
					this.performanceRepetitions = reps;
					this.performanceResults = new com.ibm.portal.performance.PerformanceTestResults();
					this._count = 0;
					this._interval = interval;
					this._statusListeners = new Array();
					//Notify of status every 5 executions.
					this.statusNotificationInterval = 5;
					this.isRunning = false;
					
					//If an interval is not specified, default to consecutive execution (interval = 0).
					if ( !this._interval || typeof(this._interval) == "undefined" ) {
						this._interval = 0;
					}
					
					//If an invalid or negative interval is specified, default to consecutive execution (interval=0).
					if ( typeof(this._interval) != "undefined" && this._interval < 0 ) {
						this._interval = 0;
					}
					
					if ( preAction != null && preAction != "" ) {
						this._preAction = preAction;
					} 
					else {
						this._preAction = null;
					}
					
					if ( postAction != null && postAction != "" ) {
						this._postAction = postAction;
					}
					else {
						this._postAction = null;
					}
				},
				run: function ( /*function*/callbackfn ) {
					/*
					 * summary: Executes the performance test.
					 * 
					 * callbackfn: the function to call when the test is complete
					 */
					if ( typeof(this._count) == "undefined" ) return;
					
					if ( this._count < this.performanceRepetitions ) {
						if ( !this.isRunning ) this.isRunning = true;
					 	
						var t = new com.ibm.portal.performance.Timer();
						
						if ( this._preAction != null ) {
							eval( this._preAction );
						}
						
						//Time the action.
						t.start();
						eval( this.performanceAction );
						t.stop();
						
						//Add the result to our collection.
						this.performanceResults.add( t );
						
						if ( this._postAction != null ) {
							eval( this._postAction );
						}
						
						this._count++;
						
						if ( (this._count % this.statusNotificationInterval ) == 0 ) {
							this._notifyAll();
						}
						
						//Create a variable to be used in the closure for the timeout.
						var p = this;
					 	setTimeout( function () { p.run( callbackfn ); }, this._interval );
					 }
					 else {
					 	this.isRunning = false;
					 	if ( callbackfn ) {
					 		callbackfn( this );
						}
					 }
				},
				addStatusListener: function ( /*function*/callbackfn ) {
					/*
					 * summary: Adds a function callback to be notified of status changes.
					 * 
					 * callbackfn: the function to be called when notification occurs. The function receives the current 
					 * performanceResults and the number of total repetitions as it's two parameters.
					 */
					this._statusListeners.push( callbackfn );	
				},
				_notifyAll: function () {
					/*
					 * summary: Notifies all registered status listeners of the current status. This should NOT be called
					 * by end users. This function should be treated as a private method.
					 */
					for ( var i = 0; i < this._statusListeners.length; i++ ) {
						this._statusListeners[i]( this.performanceResults, this.performanceRepetitions );
					}	
				}
			  }
);

com.ibm.portal.performance._RuntimePerformanceTestUniqueCounter = -1;

dojo.declare( "com.ibm.portal.performance.RuntimePerformanceTest",
			  null,
			  {
			  	constructor: function ( /*Object*/ srcObj, /*String*/ srcFunctionName ) {
					/*
					 * summary: Gathers performance information everytime the function specified by srcFunctionName
					 * is called on the specified source object. We use dojo's "around" advice to collect the performance
					 * statistics. At any point in time, the current results can be accessed through the "performanceResults"
					 * property.
					 * 
					 * srcObj: the source object
					 * srcFunctionName: the source function name
					 */
					this.sourceObject = srcObj;
					this.sourceFunction = srcFunctionName;
					this.performanceResults = new com.ibm.portal.performance.PerformanceTestResults();
					this._isConnected = false;
					this._uniqueId = "_runtimePerformanceTestTargetFunction" + (++com.ibm.portal.performance._RuntimePerformanceTestUniqueCounter );
				},
				_createEventTargetFunction: function () {
					/*
					 * summary: Creates the function that will be called for the "around" advice. We
					 * have to make sure this function is created the EXACT same way each time so that
					 * the dojo event system can properly connect and disconnect the events. This allows
					 * for monitoring to be started and stopped numerous times. The created function is
					 * assigned a unique name in the current window object so that the function can be found
					 * by dojo later on when disconnecting the event. Otherwise, we end up with multiple 
					 * connections for the same function call.
					 */
					var p = this;
					var targetFunction = function() { p.record(arguments[0]); };
					dj_global[this._uniqueId] = targetFunction;
					return targetFunction; /*function*/		
				},
				monitor: function () {
					/*
					 * summary: Starts monitoring function calls for the source object to the source function and recording the results. 
					 */
					if ( !this._isConnected ) {
						dojo.event.connect( "around", this.sourceObject, this.sourceFunction, this._createEventTargetFunction() );
						this._isConnected = true;	
					}
				},
				ignore: function () {
					/*
					 * summary: Stops the monitoring process for the source object and source function, but does not destroy the results. The
					 * monitoring can be started and stopped as many times as necessary.
					 */
					if ( this._isConnected ) {
						dojo.event.disconnect( "around", this.sourceObject, this.sourceFunction, this._createEventTargetFunction() );
						this._isConnected = false;
					}
				},
				record: function ( invocation ) {
					/*
					 * summary: Times the execution of the specified invocation and records the results. The return value of the invocation
					 * is used as the return value of this function.
					 */
					
					//If the dojo.event.disconnection failed for some reason, we just want to proceed with the invocation and avoid the overhead
					//of timing the execution.
					if ( !this._isConnected ) {
						return invocation.proceed();
					}	
					
					var t = new com.ibm.portal.performance.Timer();
					
					t.start();
					var result = invocation.proceed();
					t.stop();
					
					this.performanceResults.add( t );
					
					return result; //We don't really know what the return type is since we are just passing along the invocation's return value.	
				}
			  }
);


}
