/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.services.PortalRestServiceRequestQueue"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.services.PortalRestServiceRequestQueue"] = true;
dojo.provide( "com.ibm.portal.services.PortalRestServiceRequestQueue" );

dojo.declare( "com.ibm.portal.services.PortalRestServiceRequestQueue",
			  null,
			  {
			  	maxNumberOfActiveRequests: 4,
			  	constructor: function () {
			  		// summary: Constructs a new queue to limit the number of active requests at any given time.
			  		var _method = "PortalRestServiceRequestQueue.constructor"
			  		ibm.portal.debug.entry( _method );
			  		
			  		this._activeRequests = 0;
			  		this._requestQueue = [];
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	add: function ( /*Request*/req ) {
			  		// summary: Adds a new request to the queue. 
			  		// description: The request object should have a single method called "execute" that takes a function object
			  		//		as a parameter. The function should be called when the request is complete to let the queue know it can
			  		//		move on to the next request. 
			  		var _method = "PortalRestServiceRequestQueue.add";
			  		ibm.portal.debug.entry( _method, [ req ] );
			  		
			  		this._requestQueue.push( req );
			  		var me = this;
			  		setTimeout( function () { me._executeNextRequest(); }, 5 );
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	_executeNextRequest: function () {
			  		// summary: Execute the next request in the queue, if necessary.
			  		var _method = "PortalRestServiceRequestQueue._executeNextRequest";
			  		ibm.portal.debug.entry( _method );
			  		
			  		ibm.portal.debug.text( this._requestQueue.length + " request(s) in the queue. " + this._activeRequests + " active request(s) currently.", _method );
			  		if ( this._requestQueue.length > 0 && this._activeRequests < this.maxNumberOfActiveRequests ) {
				  		var reqToExec = this._requestQueue.shift();
			  			
			  			ibm.portal.debug.text( "Executing request: " + reqToExec, _method );
			  			
			  			var me = this;
			  			setTimeout( function () { reqToExec.execute( function () { me._notifyComplete(); } ); me.activeRequests = me._activeRequests+1; }, 1 );
			  		}
			  		else {
			  			ibm.portal.debug.text( "No request(s) pending or maximum number of requests already currently active.", _method );
			  		}
			  		
			  		ibm.portal.debug.exit( _method );	
			  	},
			  	_notifyComplete: function () {
			  		// summary: Called to notify the queue that an active request has completed and the next waiting request
			  		//		can now be executed.
			  		var _method = "PortalRestServiceRequestQueue._notifyComplete";
			  		ibm.portal.debug.entry( _method );
			  		
			  		this._activeRequests = this._activeRequests-1;
			  		if ( this._activeRequests < 0 ) {
			  			this._activeRequests = 0;
			  		}
			  		var me = this;
			  		setTimeout( function () { me._executeNextRequest(); }, 5 );
			  		
			  		ibm.portal.debug.exit( _method );
			  	}
			  }
);

}
