/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.state"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.state"] = true;
dojo.provide("com.ibm.portal.state");

dojo.require("dojo.string");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.portal.xpath");
dojo.require("com.ibm.portal.xslt");
dojo.require("ibm.portal.portlet.portlet");

dojo.declare("com.ibm.portal.state.StateManager", null, {

    /**
     * summary:
     *     Constructor which initializes our internal properties.
     */
    constructor: function(/*String*/ contentHandlerURL) {
        this.stateDOM = null;
        this.stateNode = null;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
        this.serializationManager = new com.ibm.portal.state.SerializationManager(contentHandlerURL);
    },

    /**
     * summary:
     *     Gets the current navigational state that can be read or modified
     *     using the offered state accessors.
     */
    getState: function() {
        return this.stateDOM; // dojo.dom.DOMDocument - DOM representing the navigational state
    },
	
    /**
     * summary:
     *     Creates a new state DOM. If a state is provided, this method will create a
     *     deep copy of the given state. Otherwise a new empty state DOM is created.
     *     
     * state: dojo.dom.DOMDocument - State that should be cloned. May be NULL.
     * xslDoc: dojo.dom.DOMDocument - XSL transformation which should be applied to the state copy. May be NULL.
     * xslParams: Map - Map of XSL parameters. May be NULL.
     */
    newState: function(/*dojo.dom.DOMDocument*/ state, /*dojo.dom.DOMDocument*/ xslDoc, /*Map*/ xslParams) {
        var resultState = null;
        // just create a new document if state is null
        if (state == null) {
            resultState = dojox.data.dom.createDocument();
        } else {
            // just clone the given state if no XSL transformation has been passed in
            if (xslDoc == null) {
                resultState = dojox.data.dom.createDocument(dojox.data.dom.innerXML(state));
            } else {
                // apply transformation; pass in XSL parameters
                var xslt = com.ibm.portal.xslt;
                var results = xslt.transform(state, xslDoc, null, xslParams, true);
                resultState = dojox.data.dom.createDocument(results);
            }
        }
        return resultState; /*dojo.dom.DOMDocument*/
    },
    
    /**
     * summary:
     *     Sets the current navigational state that can be read or modified
     *     using the offered state accessors.
     *     
     * state: dojo.dom.DOMDocument - the navigational state represented as a DOM.
     */
    reset: function(/*dojo.dom.DOMDocument*/ stateDOM) {        
        // remember the state
        this.stateDOM = stateDOM;
        // make sure that at least the "<state type='navigational'>" element exists
        this.stateNode = this._getStateNode(stateDOM);         
    },
	
    /**
     * summary:
     *     Returns a SerializationManager which cam be used to serialize /deserialize
     *     state. 
     */
    getSerializationManager: function() {
        return this.serializationManager;
    },
    
    /**
     * summary:
     *     Gets a state accessor that allows for reading and writing navigational
     *     state of a portlet. This includes portlet mode, window state, and render
     *     parameters.
     *     
     * portletWindowID: String - Unique identifier of the portlet window. Must not
     *     be NULL. 
     * state: dojo.dom.DOMDocument - the navigational state the requested portlet
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used.  
     */
    newPortletAccessor: function(/*String*/ portletWindowID, /*dojo.dom.DOMDocument*/ state) {
        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }
        
        // get the portlet node
        var expr = "state:portlet[@id='" + portletWindowID + "']";
        var portletNode = this._getSpecificStateNode("portlet", expr, stNode, stDOM);
        portletNode.setAttribute("id", portletWindowID);
        
        return new com.ibm.portal.state.PortletAccessor(portletNode, stDOM);
        // com.ibm.portal.state.PortletAccessor - Accessor to read and write portlet-specific state.
    },
	
    /**
     * summary:
     *     Gets a state accessor that allows for reading the set of portlet IDs that are
     *     part of the state DOM.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested portlet
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used.  
     */
    newPortletListAccessor: function(/*dojo.dom.DOMDocument*/ state) {
        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        return new com.ibm.portal.state.PortletListAccessor(stNode, stDOM);
        // com.ibm.portal.state.PortletListAccessor - Accessor to read portlet IDs.
    },    
    
    /**
     * summary:
     *     Gets a state accessor that allows for reading and writing page selection
     *     state information. This includes the currently selected page as well as label
     *     mappings.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested selection
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newSelectionAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the selection node
        var selectionNode = this._getSpecificStateNode("selection", "state:selection", stNode, stDOM);

        return new com.ibm.portal.state.SelectionAccessor(selectionNode, stDOM);
        // com.ibm.portal.state.SelectionAccessor - Accessor to read and write page selection state.
    },
    
    /**
     * summary:
     *     Gets a state accessor that allows to check whether or not a portlet window
     *     is in solo state or to set a portlet window to solo state.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested solo state
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newSoloStateAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the node keeping the solo state information
        var soloNode = this._getSpecificStateNode("solo", "state:solo", stNode, stDOM);

        return new com.ibm.portal.state.SoloStateAccessor(soloNode, stDOM);
        // com.ibm.portal.state.SoloStateAccessor - Accessor to read and write solo state.
    },
    /**
     * summary:
     *     Gets a state accessor that allows to check whether or not a theme-template
     * 	   has been specified and, if not, set one.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested theme-template
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newThemeTemplateAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the node keeping the theme-template information
        var templateNode = this._getSpecificStateNode("theme-template", "state:theme-template", stNode, stDOM);

        return new com.ibm.portal.state.ThemeTemplateAccessor(templateNode, stDOM);
        // com.ibm.portal.state.ThemeTemplateAccessor - Accessor to read and write theme-template state.
    },
    
    /**
     * summary:
     *     Gets a state accessor that allows to check whether or not a locale
     * 	   has been specified and, if not, set one.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested locale
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newLocaleAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the node keeping the locale information
        var localeNode = this._getSpecificStateNode("locale", "state:locale", stNode, stDOM);

        return new com.ibm.portal.state.LocaleAccessor(localeNode, stDOM);
        // com.ibm.portal.state.LocaleAccessor - Accessor to read and write locale information.
    },

    /**
     * summary:
     *     Returns a state accessor that allows to include a state partition id into the state.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested state partition
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newStatePartitionAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the node keeping the state partition information
        var partitionNode = this._getSpecificStateNode("statepartition", "state:statepartition", stNode, stDOM);

        return new com.ibm.portal.state.StatePartitionAccessor(partitionNode, stDOM);
        // com.ibm.portal.state.StatePartitionAccessor - Accessor to set a state partition.
    },    

    /**
     * summary:
     *     Internal method that resolves the <state type='navigational' /> node which is
     *     the root node of navigational state information. If not already present, this
     *     method will create the state node.
     *     
     * aStateDOM: dojo.dom.DOMDocument - the navigational state document that should be inspected.
     *    Must not be NULL. 
     */
    _getStateNode: function(aStateDOM) {
        var expr = "state:root/state:state[@type='navigational']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, aStateDOM, this.ns);
        var stateNode = null;         
        if (nodes == null || nodes.length <= 0) {
            var rootNode = aStateDOM.firstChild;
            
            //make sure we really have the root node and not any processing instructions
            while ( rootNode && rootNode.nodeType == 7 ) {
            	rootNode = rootNode.nextSibling;
            }
            
			if (rootNode == null) {
                // create the root element
                rootNode = this._createElement(aStateDOM, "root");
                this._prependChild(rootNode, aStateDOM);
			} 
            stateNode = rootNode.firstChild;
			if (stateNode == null) {
                // create the state element
                stateNode = this._createElement(aStateDOM, "state");
                this._prependChild(stateNode, rootNode);
            }
            // add the 'type' attribute
            stateNode.setAttribute("type", "navigational");
        } else {
            // in most cases we will end up here
            stateNode = nodes[0];
        }
        return stateNode; 
    },
   
    /**
     * summary:
     *     Internal method that resolves a specific state node with the given node name
     *     which is located as a child of the given <state type='navigational'> node.
     *     If the node does not exist, this method will create a new node. 
     *     
     * aNodeName: String - the name of the node that should be resolved. Must not be NULL.
     * aPathExpression: String - XPath expression that addresses the node to be resolved
     *     relative to the given state node.
     * aStateNode: Element - DOM element representing the <state type='navigational'> node.
     * aStateDOM: dojo.dom.DOMDocument - the navigational state document that should be
     *     inspected. Must not be NULL. 
     */
    _getSpecificStateNode: function(aNodeName, aPathExpression, aStateNode, aStateDOM) {
        // evaluate the expression starting from the state node
        var nodes = com.ibm.portal.xpath.evaluateXPath(aPathExpression, aStateNode, this.ns);
        var node;
        if (nodes == null || nodes.length <= 0) {
            // create the state element
            node = this._createElement(aStateDOM, aNodeName);
            this._prependChild(node, aStateNode);
        } else {
            node = nodes[0];
        }
        return node;
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},	    
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    }
    
});


dojo.declare("com.ibm.portal.state.PortletAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this acccessor with the
     *     state DOM and the DOM node that keeps the state information
     *     of the respective portlet window in its subtree.
     */
    constructor: function (/*Element*/ portletNode, 
                           /*dojo.dom.DOMDocument*/ stateDOM) {

        this.portletNode = portletNode;
        this.stateDOM = stateDOM;
        this.parameters = new com.ibm.portal.state.Parameters(portletNode, stateDOM);
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };

        this.xsltURL = dojo.moduleUrl( "com", "ibm/portal/state/");
    },

    /**
     * summary:
     *     Gets the current portlet mode of the portlet window this accessor
     *     operates on.
     */
    getPortletMode: function() {
        
        var expr = "state:portlet-mode";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        var portletMode = ibm.portal.portlet.PortletMode.VIEW; 
        if (nodes != null && nodes.length > 0) {
            var txtNode = nodes[0].firstChild;
            if (txtNode != null) {
                portletMode = txtNode.nodeValue;
            }
        }
        return portletMode; // ibm.portal.portlet.PortletMode - The current portlet mode.
    },
	
    /**
     * summary:
     *     Gets the current window state of the portlet window this accessor
     *     operates on.
     */
    getWindowState: function() {
        var expr = "state:window-state";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        var windowState = ibm.portal.portlet.WindowState.NORMAL; 
        if (nodes != null && nodes.length > 0) {
            var txtNode = nodes[0].firstChild;
            if (txtNode != null) {
                windowState = txtNode.nodeValue;
            }
        }
        return windowState; // ibm.portal.portlet.WindowState - The current window state.
    },

    /**
     * summary:
     *     Gets the current render parameters of the portlet window this accessor
     *     operates on. The returned Parameters object allows for querying, adding,
     *     modifying and removing parameters.
     */
    getRenderParameters: function() {
        return this.parameters; // com.ibm.portal.state.Parameters - The current render parameters. 
    },

    /**
     * summary:
     *     Sets the portlet mode of the portlet window this accessor operates on
     *     by changing the navigational state DOM accordingly.
     *     
     * portletMode: ibm.portal.portlet.PortletMode - the portlet mode to be set.
     *     Must not be NULL. 
     */
    setPortletMode: function(/*ibm.portal.portlet.PortletMode*/ portletMode) {
        var expr = "state:portlet-mode";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        if (nodes == null || nodes.length <= 0) {
            var portletModeNode = this._createElement(this.stateDOM, "portlet-mode");
            this._prependChild(portletModeNode, this.portletNode);
			var txtNode = this.stateDOM.createTextNode(portletMode);
            this._prependChild(txtNode, portletModeNode);
        } else {
            nodes[0].firstChild.nodeValue = portletMode;
        }
    },

    /**
     * summary:
     *     Sets the window state of the portlet window this accessor operates on
     *     by changing the navigational state DOM accordingly.
     *     
     * windowState: ibm.portal.portlet.WindowState - the window state to be set.
     *     Must not be NULL. 
     */
    setWindowState: function(/*ibm.portal.portlet.WindowState*/ windowState) {
        var expr = "state:window-state";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        if (nodes == null || nodes.length <= 0) {
            var windowStateNode = this._createElement(this.stateDOM, "window-state");
            this._prependChild(windowStateNode, this.portletNode);
            var txtNode = this.stateDOM.createTextNode(windowState)
            this._prependChild(txtNode, windowStateNode);
        } else {
            nodes[0].firstChild.nodeValue = windowState;
        }
    },
    
    /**
     * summary:
     *     Gets a dojo.dom.DOMDocument which contains only the state of the portlet
     *     this accessor operates on. The returned DOMDocument still complies with the
     *     state.DTD i.e. it contains the complete path to the portlet node that
     *     contains the portlet's state information. 
     */
    getPortletState: function() {
        // TODO implement this function using XSLT
        
        /*var xslt = com.ibm.portal.xslt;
        // load the xsl file
        var uri = this.xsltURL + "SelectPortletState.xsl";
        var xslDoc = xslt.loadXsl(uri);
	    // apply the transformation
	    if (xslDoc.documentElement == null) { alert("xslDoc is null"); }
    	var results = xslt.transform(this.stateDOM, xslDoc, null, {"id" : this.portletNode.getAttribute("id")}, true);
        return dojo.dom.createDocumentFromText(results);*/
        
        // get a new portlet accessor operating on a new empty state
        var newDOM = dojox.data.dom.createDocument();
        var portletAcc = com.ibm.portal.state.STATE_MANAGER.newPortletAccessor(this.portletNode.getAttribute("id"), newDOM);
        portletAcc.setPortletMode(this.getPortletMode());
        portletAcc.setWindowState(this.getWindowState());
        var paramMap = this.getRenderParameters().getMap();
        if (paramMap.length > 0) {
            portletAcc.getRenderParameters().putAll(paramMap);
        }
        return newDOM; /* dojo.dom.DOMDocument */
    },

    /**
     * summary:
     *     Sets the complete state of the portlet this accessor operates on.
     *     Ignores all state information in the given state that is not
     *     related to the portlet this accessor operates on. 
     *     
     *  state: dojo.dom.DOMDocument - The state DOM that contains state for the
     *      portlet this accessor operates on. Must not be NULL.
     *  merge: boolean - Flag that indicates whether the given state of the portlet
     *      should be merged into the state of the accessor or if it should be replaced.
     *      May be NULL which defaults to false, i.e. the current state of the portlet
     *      is replaced with the state retrieved from the given DOM. 
     */
    setPortletState: function(/* dojo.dom.DOMDocument */ state, /* boolean */ merge) {
        // TODO implement this function using XSLT
        
        // get a new portlet accessor operating on a new empty state
        var portletAcc = com.ibm.portal.state.STATE_MANAGER.newPortletAccessor(this.portletNode.getAttribute("id"), state);
        this.setPortletMode(portletAcc.getPortletMode());
        this.setWindowState(portletAcc.getWindowState());
        var paramMap = portletAcc.getRenderParameters().getMap();
        // clear parameters only if we should replace them
        if (merge == null || merge == false) {
            this.getRenderParameters().clear();
        }
        if (paramMap.length > 0) {
            this.getRenderParameters().putAll(paramMap);
        }
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},	
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    }
    
});	



dojo.declare("com.ibm.portal.state.Parameters", null, {

    /**
     * summary:
     *     Constructor which initializes this Parameters object with the
     *     state DOM and the DOM node that keeps the parameters information.
     */
    constructor: function(/*Element*/ baseNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.baseNode = baseNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary: 
     *     Returns a read-only parameter map represented as a Object[].
     *     Returns an empty map if no parameters are available.
     */
    getMap: function() {
        var names = this.getNames();
        var map = new Array(names.length);
        for (var i = 0; i < names.length; i++) {
            var name = names[i];
            map[i] = {name: name, values : this.getValues(name)};
        }
        return map; // Object[] {name: String, values: String[]}
    },

    /**
     * summary:
     *     Returns a String[] containing the names of all parameters that have an
     *     associated value. Returns an empty array if no parameters are available.
     */
    getNames: function() {
        var expr = "state:parameters/state:param";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        var names = new Array();
        if (nodes != null && nodes.length > 0) {
            var nodeCount = nodes.length;
            for (var i = 0; i < nodeCount; i++) {
                names[i] = nodes[i].getAttribute("name");
            }
        }
        return names;
    },

    /**
     * summary:
     *     Returns the String value associated with the parameter with the given name.
     *     
     * description:
     *     If there are one or more parameter values associated with the given name 
     * 	   it returns the first associated value.
     *
     * name: String - Name of the parameter for which the associated value is to
     *     be returned. Must not be NULL.
     */
    getValue: function(/*String*/ name) {
        var values = this.getValues(name);
        var firstValue = null;
        if (values != null && values.length > 0) {
            firstValue = values[0];
        }
		return firstValue;
    },

    /**
     * summary:
     *     Returns the String[] representing the values that are associated with
     *     the parameter with the given name.
     *
     * name: String - Name of the parameter for which the associated values are to
     *     be returned. Must not be NULL.
     */
    getValues: function(/*String*/ name) {
        var expr = "state:parameters/state:param[@name='" + name + "']/state:value";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        var values = null;
        if (nodes != null && nodes.length > 0) {
            values = new Array(nodes.length);
            var nodeCount = nodes.length;
            for (var i = 0; i < nodeCount; i++) {
                // get the text value
                var txtNode = nodes[i].firstChild;
                if (txtNode != null) {
                    values[i] = txtNode.nodeValue;
                }
            }
        }
        return values;
    },

    /**
     * summary:
     *     Removes the parameter with the given name.
     *
     * name: String - Name of the parameter that should be removed. Must not be NULL.
     */
    remove: function (/*String*/ name) {
        var expr = "state:parameters/state:param[@name='" + name + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        if (nodes != null) {
			var toRemove = nodes[0];
			if(toRemove && toRemove.parentNode){
				toRemove.parentNode.removeChild(toRemove);
			}
		} 
    },

    /**
     * summary:
     *     Adds all parameters that are contained in the given map. Overrides the values
     *     of parameters that already exist.
     *
     * map: Object[] {name: String, values: String[]} - Two dimensional object array
     *     representing the map of parameters that should be added. Must not be NULL.
     */
    putAll: function(/*Object[] {name: String, values: String[]}*/ map) {
        if (map != null && map.length > 0) {
            for (var i = map.length - 1; i >= 0; i--) {
                var paramName = map[i].name;
                var paramValues = map[i].values;
                this.setValues(paramName, paramValues);
            }
        }
    },

    /**
     * summary:
     *     Sets a simple String value for the parameter with the given name. Overrides
     *     the value of the parameter if it already exists.
     *
     * name: String - The name of the parameter. Must not be NULL.
     * value: String - The value of the parameter that should be set. Must not be NULL. 
     */
    setValue: function (/*String*/ name, /*String*/ value) {
        this.setValues(name, new Array(value));
    },

    /**
     * summary:
     *     Sets a String[] representing the value of the parameter with the given name.
     *     Overrides the value of the parameter if it already exists.
     *
     * name: String - The name of the parameter. Must not be NULL.
     * value: String[] - The value of the parameter that should be set. Must not be NULL. 
     */
    setValues: function(/*String*/ name, /*String[]*/ values) {
        var expr = "state:parameters/state:param[@name='" + name + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        var paramNode;
        if (nodes == null || nodes.length == 0) {
            // create the parameters node
            var paramsNode = null;// this._getFirstChildWithTag(this.baseNode, "state:parameters"); // do we really need to use the NS prefix here? bug??
            if (paramsNode == null) {
                paramsNode = this._createElement(this.stateDOM, "parameters");
                this._prependChild(paramsNode, this.baseNode);
            }
            // create the param node
            paramNode = this._createElement(this.stateDOM, "param");
            paramNode.setAttribute("name", name);
            this._prependChild(paramNode, paramsNode);
        } else {
            paramNode = nodes[0];
			// remove all children of paramNode
			dojox.data.dom.removeChildren(paramNode);
        }
        // putAll sometimes passes in null for values
        if (values != null) {
            for (var i = values.length - 1; i >= 0; i--) {
                var valueNode = this._createElement(this.stateDOM, "value");
                this._prependChild(valueNode, paramNode);
                var value = values[i];
                // make sure that we do not encode something like "undefined" into a value node 
                if (value != null) {
                    var txtNode = this.stateDOM.createTextNode(value);
                    this._prependChild(txtNode, valueNode);
                }
            }
        }
    },

    /**
     * summary:
     *     Removes all existing parameters.
     */
    clear: function() {
        var expr = "state:parameters";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        if (nodes != null) {
			var toRemove = nodes[0];
			if(toRemove && toRemove.parentNode){
				toRemove.parentNode.removeChild(toRemove);
			}
        } 
    },
	_getFirstChildWithTag: function(parentNode, tagName){
		// validate arguments
		if(!parentNode || !tagName) return null;
		var node = parentNode.firstChild;
		// iterate over all children
		while(node){
			// if an element node has a tag equal to tagname, return it
			if( node.nodeType == 1 && node.tagName && node.tagName.toLowerCase() == tagName.toLowerCase()){
				return node;
			}
			// otherwise, move on to the next sibling
			node = node.nextSibling;
		}
		// if we got here, we didn't find a match
		return null;
	},	
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},	    
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    }
	
});	


dojo.declare("com.ibm.portal.state.PortletListAccessor", null, {
   
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node representing the root of navigational state
     *     information in the state DOM.
     */
    constructor: function(/*Element*/stateNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.stateNode = stateNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary:
     *     Returns an array of Strings representing the IDs of the portlet windows
     *     that are part of the state DOM this accessor operates on. Returns null
     *     the state DOM does not contain any portlet-related state information.
     */
    getPortlets: function() {
        // get all portlet nodes
        var expr = "state:portlet";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.stateNode, this.ns);
        var portletIDs = null;
        if (nodes != null && nodes.length > 0) {
            portletIDs = new Array(nodes.length);
            for (var i=0; i < nodes.length; i++) {
               var node = nodes[i];
               portletIDs[i] = node.getAttribute("id");  
            }
        }
        return portletIDs; /*String[]*/
    }
    
});


dojo.declare("com.ibm.portal.state.SelectionAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the selection information.
     */
    constructor: function(/*Element*/selectionNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.selectionNode = selectionNode;
        this.stateDOM = stateDOM;
        this.parameters = new com.ibm.portal.state.Parameters(this.selectionNode, stateDOM);
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary:
     *     Returns a String representing the unique identifier of the
     *     navigation node that represents the currently selected page.
     */
    getPageSelection: function() {
        return this.selectionNode.getAttribute("selection-node"); // String
    },
	
    /**
     * summary:
     *     Returns a String representing the unique identifier of the
     *     selected fragment. Typically the fragment is a layout node
     *     that is part of the layout model of the currently selected page.
     */
    getFragmentSelection: function() {
        var params = this.getParameters();
        var fragmentParams = params.getValues("frg");
        var fragmentID = null;
        if (fragmentParams != null && fragmentParams.length > 0) {
            fragmentID = fragmentParams[0];
            if (fragmentParams.length > 1) {
                if (fragmentID == "pw") {
                    fragmentID = fragmentParams[1];
                }
            }    
        }
        return fragmentID; // String
    },
    
    /**
     * summary:
     *     Returns a String representing the unique ID of the navigation node
     *     that represents the page that should be selected instead of the
     *     label with the given identifier. Returns null if there is no
     *     mapping available for the given label ID.
     *     
     * srcID:
     *     String representing the ID of the label that should be resolved.
     *     Must not be NULL.
     */
    getMapping: function(/*String*/ srcID) {
        var expr = "state:mapping[@src='" + srcID + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.selectionNode, this.ns);
        var targetID = null;
        if (nodes != null && nodes.length > 0) {
            var mappingNode = nodes[0];
            targetID = mappingNode.getAttribute("dst");
        }
        return targetID; // String or null
    },

    /**
     * summary:
     *     Returns a com.ibm.portal.state.Parameters object which contains
     *     selection parameters. Does not return NULL.
     * 
     */
    getParameters: function() {
        return this.parameters; // com.ibm.portal.state.Parameters
    },
    
    /**
     * summary:
     *     Changes the selection according to the given navigation node ID.
     *     
     * navigationNodeID:
     *     String representing the identifier of the page or label that should
     *     be selected. Must not be NULL.
     */
    setPageSelection: function(/*String*/ navigationNodeID) {
        this.selectionNode.setAttribute("selection-node", navigationNodeID);
    },

    /**
     * summary:
     *     Changes the fragment selection according to the given layout node ID.
     *     
     * layoutNodeID:
     *     String representing the identifier of the layout node that represents
     *     the oage fragment that should be selected. Must not be NULL.
     * isPortletWindow:
     *     Flag which indicates whether or not the layout node represents a
     *     portlet window. May be NULL to indicate that the layout node is a
     *     portlet window.
     */
    setFragmentSelection: function(/*String*/ layoutNodeID, /*boolean*/ isPortletWindow) {
        var params = this.getParameters();
        if (isPortletWindow == null || isPortletWindow == true) {
            var values = new Array(2);
            values[0] = layoutNodeID;
            values[1] = "pw";
            params.setValues("frg", values);
        } else {
            params.setValue("frg", layoutNodeID);
        }
    },

    /**
     * summary:
     *     Sets a label mapping that specifies the page that should be
     *     selected instead of the label. The label is identified via the
     *     given source ID, the page via the target ID. If the given targetID
     *     is null, the method will completely remove the mapping from the state.
     *     
     * srcID:
     *     ID of the label that should be mapped to a page. Must not be NULL.
     * targetID:
     *     ID of the page the label should be mapped to. May be NULL..
     */
    setMapping: function(/*String*/ srcID, /*String*/ targetID ) {
        if (targetID != null) {
            var expr = "state:mapping[@src='" + srcID + "']";
            var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.selectionNode, this.ns);
            var mappingNode;
            if (nodes != null && nodes.length > 0) {
                mappingNode = nodes[0];
            } else {
                mappingNode = this._createElement(this.stateDOM, "mapping");
                this._prependChild(mappingNode, this.selectionNode);
                mappingNode.setAttribute("src", srcID);
            }
            mappingNode.setAttribute("dst", targetID);
        } else {
            // this is equivalent with removing the mapping
            this.removeMapping(srcID);
        }
    },

    /**
     * summary:
     *     Removes the label mapping with the given source ID.
     *     
     * srcID:
     *     ID of the label that should be removed. Must not be NULL.
     */
    removeMapping: function(/*String*/ srcID) {
        var expr = "state:mapping[@src='" + srcID + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.selectionNode, this.ns);
        var removed = false;
        if (nodes != null && nodes.length > 0) {
            // actually there should be only one such node
            // but clean up if we ran into an inconsistent state 
            for (var i=0; i < nodes.length; i++) {
	   			var toRemove = nodes[i];
				if(toRemove && toRemove.parentNode){
					toRemove.parentNode.removeChild(toRemove);
				}

            }
            removed = true;
        }
        return removed;
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},	    
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    },
    
    /**
     * @deprecated: use getPageSelection instead
     */
    getSelection: function() {
        return this.getPageSelection(); // String
    },

    /**
     * @deprecated: use setPageSelection instead
     */
    setSelection: function(/*String*/ navigationNodeID) {
        this.setPageSelection(navigationNodeID);
    }
    
});	


dojo.declare("com.ibm.portal.state.SoloStateAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the solo state information.
     */
    constructor: function(/*Element*/soloNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.soloNode = soloNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary:
     *     Sets the portlet window with the given ID into solo state.
     *     
     * portletWindowID:
     *     String representing the identifier of the portlet window which should
     *     be set to solo state. To entirely delete the solo state information from
     *     the state DOM, pass in NULL.
     */
    setSoloPortlet: function(/*String*/ portletWindowID) {
         // first remove child nodes
         dojox.data.dom.removeChildren(this.soloNode);
         if (portletWindowID != null) {
             // create a new text node and append it
             var txtNode = this.stateDOM.createTextNode(portletWindowID)
             this._prependChild(txtNode, this.soloNode);
         }
    },

    /**
     * summary:
     *     Returns the ID of the portlet window which is currently in solo state.
     *     Returns NULL if no such portlet window exists.
     */
    getSoloPortlet: function() {
        var txtNode = this.soloNode.firstChild;
        if (txtNode != null) {
            return txtNode.nodeValue;
        } else {
            return null;
        }
    },

    /**
     * summary:
     *     Sets the page the portal should navigate to after leaving
     *     Solo state.
     * 
     * pageID:
     *     ID of the return page. Must not be NULL.
     */ 
    setReturnSelection: function(/*String*/ pageID) {
        this.soloNode.setAttribute("return-selection", pageID);
    },

    /**
     * summary:
     *     Returns the ID of the page the portal navigates to
     *     after leaving Solo state.
     */
    getReturnSelection: function() {
        return this.soloNode.getAttribute("return-selection");
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	}   


});

dojo.declare("com.ibm.portal.state.ThemeTemplateAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the theme-template state information.
     */
    constructor: function(/*Element*/themeTemplateNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.themeTemplateNode = themeTemplateNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },
    
    /**
     * summary:
     *     Sets the theme-template to the given value.
     *     
     * themeTemplate:
     *     String representing the theme-template which should be set. To entirely delete the theme-template
     *     information from the state DOM, pass in NULL.
     */
    setThemeTemplate: function(/*String*/ themeTemplate) {
         // first remove child nodes
         dojox.data.dom.removeChildren(this.themeTemplateNode);
         if (themeTemplate != null) {
             // create a new text node and append it
             var txtNode = this.stateDOM.createTextNode(themeTemplate)
             this._prependChild(txtNode, this.themeTemplateNode);
         }
    },

    /**
     * summary:
     *     Returns the name of the current theme-template.
     *     Returns NULL if no such value exists.
     */
    getThemeTemplate: function() {
        var txtNode = this.themeTemplateNode.firstChild;
        if (txtNode != null) {
            return txtNode.nodeValue;
        } else {
            return null;
        }
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	}
	
});

dojo.declare("com.ibm.portal.state.LocaleAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the locale information.
     */
    constructor: function(/*Element*/localeNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.localeNode = localeNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },
    
    /**
     * summary:
     *     Sets the locale to the given value.
     *     
     * themeTemplate:
     *     String representing the locale which should be set. To entirely delete the locale
     *     information from the state DOM, pass in NULL.
     */
    setLocale: function(/*String*/ locale) {
         // first remove child nodes
         dojox.data.dom.removeChildren(this.localeNode);
         if (locale != null) {
             // create a new text node and append it
             var txtNode = this.stateDOM.createTextNode(locale)
             this._prependChild(txtNode, this.localeNode);
         }
    },

    /**
     * summary:
     *     Returns the name of the current locale in the state.
     *     Returns NULL if no such value exists.
     */
    getLocale: function() {
        var txtNode = this.localeNode.firstChild;
        if (txtNode != null) {
            return txtNode.nodeValue;
        } else {
            return null;
        }
    },

    _prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
        parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
    }
	
});


dojo.declare("com.ibm.portal.state.StatePartitionAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the state partition information.
     */
    constructor: function(/*Element*/statePartitionNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.statePartitionNode = statePartitionNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },
    
    /**
     * summary:
     *     Includes a state partition into the state.
     */
    includeStatePartition: function() {
         // first remove the exisiting partition ID
         dojox.data.dom.removeChildren(this.statePartitionNode);
         var txtNode = this.stateDOM.createTextNode(this._generateID());
         this._prependChild(txtNode, this.statePartitionNode);
    },
	
    _prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},

    _generateID: function() {
        return Math.floor(Math.random()*100);
    }
	
});


/**
 * The serialization manager is able to serialize /deserialize state documents. 
 */
dojo.declare("com.ibm.portal.state.SerializationManager", null, {
    
    /**
     * Scheme of the uri that addresses the state REST service.
     */
    STATE_URI_SCHEME: "state",
    STATE_URI_POST: "state:encode",
    
    /**
     * The download mode
     */
    DOWNLOAD_MODE: "download",
    
    STATUS_UNDEFINED: 0,
    STATUS_OK: 1,
    STATUS_ERROR: 2,
    
    STATE_NS_URI: "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state",

    /**
     * Maximum length the serialized state must not exceed.
     * The actual URL length limit is 2048 but we need to take into account the base URL incl. query params.
     */
    STATE_THRESHOLD: 1024,
    
    /**
     * summary:
     *     Constructor which initializes the serialization manager with a URL that
     *     can be used to access the state REST service of portal.
     */
    constructor: function(/*String*/ stateServiceURL) {
        this.serviceURL = stateServiceURL;
    },
    
    /**
     * summary:
     *     Serializes the given state DOM into a form that can be included into URLs.
     *     If a callback function is provided, the method will execute asynchronously.
     *     Otherwise it will execute synchronously and return a
     *     {status: STATUS_OK /STATUS_ERROR /STATUS_UNDEFINED, state: dojo.dom.DOMDocument, url: String} object. 
     *     
     * state: dojo.dom.DOMDocument - The state DOM that should be serialized. Must not
     *     be NULL.
     * validate: boolean - Optional flag specifying whether the server should validate the state or not.
     *     Defaults to false. 
     * callbackfn: function - The callback function which should be called in case of
     *     internal status changes. May be NULL to indcate that the method should execute
     *     synchronously.
     */
    serialize: function(/*dojo.dom.DOMDocument*/ state, /*boolean*/ validate, /*Function*/ callbackfn) {
         ibm.portal.debug.entry( "SerializationManager.serialize", [ dojox.data.dom.innerXML( state ), validate, callbackfn ] );
         
         // get the serialized state
         var stateXML = dojox.data.dom.innerXML(state).replace(/[\r\n]/mg, "");
         // apply ASCII encoding to state XML
         // because we want to use it as a scheme-specific part in a URI or as part of a POST body
		 //RES: this used to call dojo.string.encodeAscii(), which also included special handling for unicode characters
		 // if escape() is not sufficient, see the _encodeAscii function defined in this object (though it doesn't work yet :)
         var encodedStateXML = escape(stateXML);
         //var encodedStateXML = dojo.string.encodeAscii(stateXML);
         //var encodedStateXML = this._encodeAscii(stateXML);
         var mType = this._getMimeType();
         var result = null;
         var me = this;
         ibm.portal.debug.text( "Mime type for response: " + mType );
         ibm.portal.debug.text( "Length of encoded state XML is: " + encodedStateXML.length );
         ibm.portal.debug.text( "Encoded state XML is: " + encodedStateXML );
         var digest = com.ibm.portal.services.PortalRestServiceConfig.digest;
         ibm.portal.debug.text( "Digest: " + digest );
         // check if we need to send a GET or a POST based on the length of the serialized state
         if (encodedStateXML.length <= this.STATE_THRESHOLD) {
             // GET
             // create the uri
             var payload = this.STATE_URI_SCHEME + ":" + encodedStateXML;
             var params;
             validate = (validate != null && validate == true);
             if (validate == true) {
                 if (digest != null) {
                     params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "sessionDependencyAllowed": "true", "preprocessors": "true", "digest": digest};
                 } else {
                     params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "sessionDependencyAllowed": "true", "preprocessors": "true"};
                 }
             } else {
                 if (digest != null) {
                     params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "sessionDependencyAllowed": "true", "digest": digest};
                 } else {
                     params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "sessionDependencyAllowed": "true"};
                 }
             }
             
             ibm.portal.debug.text( "Doing a GET request: { url: \"" + this.serviceURL + "\", sync: " + ((callbackfn)?false:true) + ", content: " + params + ", handleAs: " + mType + ", transport: XMLHTTPRequest" );
             ibm.portal.debug.text( "Parameters: uri=\"" + params.uri + "\" mode=\"" + params.mode + "\" xmlns=\"" + params.xmlns + "\"" );
              
             dojo.xhrGet({
                 url: this.serviceURL,
                 sync: (callbackfn)?false:true,
                 content: params,
                 headers: {"X-IBM-XHR":"true"},
                 handleAs: mType,
                 handle: function(response, ioArgs) {
                      ibm.portal.debug.text( "Response: " + response );
                      if(com.ibm.ajax.auth && (ioArgs.xhr.status==401 || ioArgs.xhr.status==0 )) {
                         ibm.portal.debug.text( " 401 found, re-auth");
                         com.ibm.ajax.auth.authenticationHandler();
                         return;
                      } else {
                          result = me._handleSerializationResponse.call(me, response, callbackfn, state, validate);
                          return response;
                      }
                 },
                 transport: "XMLHTTPTransport"
             });
         } else {
             ibm.portal.debug.text( "Doing a POST request." );
             // tentative fix: content handler framework cannot deal with UTF-16 at the moment
             if (dojo.isIE) {
                 var idx = stateXML.indexOf("UTF-16");
                 if (idx >= 0) {
                     stateXML = stateXML.replace(/UTF-16/, "UTF-8");
                 }
             }
             var url = this.serviceURL + "?uri=" + this.STATE_URI_POST + "&xmlns=" + this.STATE_NS_URI + "&sessionDependencyAllowed=true";
             if (digest != null) {
                 url += "&digest=" + digest;
             }
             // POST
             dojo.rawXhrPost({
                 url: url,
                 sync: (callbackfn)?false:true,
                 postData: stateXML,
                 handleAs: mType,
                 headers: {"Content-Type":"text/xml", "X-IBM-XHR":"true"},
                 handle: function(response, ioArgs) {
                      if(com.ibm.ajax.auth && (ioArgs.xhr.status==401 || ioArgs.xhr.status==0 )) {
                         ibm.portal.debug.text( "401 found, re-auth");
                         com.ibm.ajax.auth.authenticationHandler();
                         return;
                      } else {
                         result = me._handleSerializationResponse.call(me, response, callbackfn, state, validate);
                         return response;
                      }
                 },
                 transport: "XMLHTTPTransport"
             });
         }
         
         ibm.portal.debug.exit( "SerializationManager.serialize", result );
         return result;
    },
    
    /**
     * summary:
     *     Deserializes the state from the given URL. If a callback function is provided,
     *     the method will execute asynchronously. Otherwise it will execute synchronously
     *     and return a
     *     {status: STATUS_OK /STATUS_ERROR /STATE_UNDEFINED, url: String, state: dojo.dom.DOMDocument}
     *     object. 
     *     
     * url: String - String representing the URL the state DOM should be retrieved from.
     *     Must not be NULL.
     * callbackfn: function - The callback function which should be called in case of
     *     internal status changes. May be NULL to indcate that the method should execute
     *     synchronously.
     */
    deserialize: function(/*String*/ url, /*Function*/ callbackfn) {
        var payload = this.STATE_URI_SCHEME + ":" + url;
        var result = null;
        var mType = this._getMimeType();
        var me = this;
        var digest = com.ibm.portal.services.PortalRestServiceConfig.digest;
        ibm.portal.debug.text( "Digest: " + digest );
        var params;
        if (digest != null) {
            params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "digest" : digest};
        } else {
            params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI};
        }
        dojo.xhrGet({
             url: this.serviceURL,
             sync: (callbackfn)?false:true,
             content: params,
             headers: {"X-IBM-XHR":"true"},
             handleAs: mType,
             handle: function(response, ioArgs){
                  var type = ( response instanceof Error ) ? "error" : "load";
                  if (type == "load") {
                      var stateDOM = me._getResponseXML(response);
                      //If we get a parser error, then valid XML was not returned by 
                      //the serialization service and we want to return an empty xml
                      //document in this case.
                      if ( stateDOM.documentElement.nodeName == "parsererror" ) {
                          stateDOM = dojox.data.dom.createDocument();
                      }
                      if (callbackfn) {
                          // TODO use this.STATUS_OK here
                          callbackfn(1, url, stateDOM);
                      } else {
                          // TODO use this.STATUS_OK here
                          result = {"status" : 1, "input" : me.serviceURL, "url" : me.serviceURL, "returnObject" : stateDOM, "state" : stateDOM};
                      }
                  } else if (type == "error") {
                      if(com.ibm.ajax.auth && (ioArgs.xhr.status==401 || ioArgs.xhr.status==0 )) {
                          ibm.portal.debug.text( " 401 found, reauth");
                          com.ibm.ajax.auth.authenticationHandler();
                          return;
                      }


                      if (callbackfn) {
                          // TODO use this.STATUS_ERROR here
                          callbackfn(2, url, null);
                      } else {
                          // TODO use this.STATUS_ERROR here
                          result = {"status" : 2, "input" : me.serviceURL, "url" : me.serviceURL, "returnObject" : null, "state" : null};
                      }
                  }
             },
             transport: "XMLHTTPTransport"
         });
         return result;
    },
    

    _handleSerializationResponse: function(response, callbackfn, state, validate) {
        var result = null;
        var type = ( response instanceof Error ) ? "error" : "load";
        if (type == "load") {
            var responseXML = this._getResponseXML(response);
            // get the serialized state from response XML
            var linkExpr = "atom:entry/atom:link";
            var ns = {"atom" : "http://www.w3.org/2005/Atom", "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state"};
            var serializedState = null;
            var linkNodes = com.ibm.portal.xpath.evaluateXPath(linkExpr, responseXML, ns);
            if (linkNodes != null && linkNodes.length > 0) {
                // take the fist node found and read its href attribute
                serializedState = linkNodes[0].getAttribute("href");
            }
            var resultState = state;
            // check if the state potentially changed
            if (validate == true) {
                // get the new state from the response XML
                var stateExpr = "atom:entry/atom:content/state:root";
                var stateNodes = com.ibm.portal.xpath.evaluateXPath(stateExpr, responseXML, ns);
                if (stateNodes != null && stateNodes.length > 0) {
                    var newState = dojox.data.dom.innerXML(stateNodes[0]); 
                    resultState = dojox.data.dom.createDocument(newState);
                }
            }
            if (callbackfn) {
                // TODO use this.STATUS_OK here
                callbackfn(1, resultState, serializedState);
            } else {
                // TODO use this.STATUS_OK here
                result = {"status" : 1, "input" : resultState, "state" : resultState, "returnObject" : serializedState, "url" : serializedState};
            }
        } else if (type == "error") {                      
            if (callbackfn) {
                // TODO use this.STATUS_ERROR here
                callbackfn(this.STATUS_ERROR, state, null);
            } else {
                // TODO use this.STATUS_ERROR here
                result = {"status" : this.STATUS_ERROR, "input" : state, "state" : state, "returnObject" : null, "url" : null};
            }
        }
        return result;
    },


    _getMimeType: function() {
        var mtype = "xml";
        // IE does not work with "text/xml"
        if (dojo.isIE) {
             mtype = "text";
        }
        return mtype;
    },
    
    _getResponseXML: function(data) {
        var responseDOM = data;
        if (dojo.isIE) {
            responseDOM = dojox.data.dom.createDocument(data);
        }
        return responseDOM;
    },
	_encodeAscii: function(str) {
		var ret = str;
		if(dojo.isString(ret)) {
			var escapedStr = escape(ret);
			// \u#### patterns will be replaced with %u#### patterns by the escape() function
			var unicodeRegx = /%u([A-F0-9][A-F0-9][A-F0-9][A-F0-9])/i;
			var unicodeVal = null;
			// now find all %u#### patterns and replace them with an 
			while((unicodeVal = escapedStr.match(unicodeRegx))) {
				ret += escapedStr.substring(0, unicodeVal.index) + escape(Number("0x"+unicodeVal[1]));
				escapedStr = escapedStr.substring(unicodeVal.index + unicodeVal[0].length);
			}
			ret += escapedStr;
			ret = ret.replace(/\+/g, "%2B");
		}
		return ret;
	}	
    
    
});

/**
 * Component that manages state vary expressions on an ID basis.
 */
dojo.declare("com.ibm.portal.navigation.controller.StateVaryManager", null, {
    
				/**
				 * summary:
				 *     Constructor which initializes the state vary manager with a URL
				 */
				constructor: function() {
					this._expr = new Array();
				},

				setExpressions: function (/*String*/ id, /*Array*/ expressions) {
					var bucket = this._findBucket(id);
					if (bucket == null) {
						// create a new empty bucket
						bucket = { "id": id, "expr": null };
						// update the cache
						this._expr.push(bucket);
					}
					// add /replace the expressions
					bucket.expr = expressions;
				},

				getExpressions: function(/*String*/ id) {
					var result = null;
					var bucket = this._findBucket(id);
					if (bucket != null) {
						result = bucket.expr;
					}
					return result;
				},

				_findBucket: function(/*String*/ id) {
					var bucket = null;
					for (i=0; i<this._expr.length; i++) {
						var temp = this._expr[i];
						if (temp.id == id) {
							bucket = temp;
							break;
						}
					}
			      	return bucket;
				}
    
});

com.ibm.portal.state.STATE_MANAGER = new com.ibm.portal.state.StateManager(); 	
com.ibm.portal.state.STATE_MANAGER.reset(dojox.data.dom.createDocument()); // workaround


}
