/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.utilities.html"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.utilities.html"] = true;
dojo.provide( "com.ibm.portal.utilities.html" );

dojo.require( "com.ibm.portal.utilities" );
dojo.require( "dojo.fx" );

com.ibm.portal.utilities.html = {
	createAnchor: function ( /*HTMLElement*/parent, /*String*/href, 
									     /*String*/id, /*String*/displayText, 
										 /*String*/className ) {
					
		// summary: Creates an anchor element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created anchor
		// href: the value of the href attribute
		// id: the id of the anchor element
		// displayText: the inner text of the anchor
		// className: the CSS style name to apply
		// returns: the created anchor object
		ibm.portal.debug.entry( "SkinRenderer.createAnchor", [ parent, href, id, displayText, className ] );
		
		//Create and set the attributes for the anchor.
		var anchor = document.createElement( "A" );
		anchor.href = href;
		if ( id ) { anchor.id = id; }
		if ( className ) { anchor.className = className; }
		if ( displayText ) { anchor.appendChild( document.createTextNode( displayText ) ); }
												 	
		//Append the anchor to the parent.
		parent.appendChild( anchor );
		
		ibm.portal.debug.exit( "SkinRenderer.createAnchor", [ anchor ]);
		return anchor; //HTMLElement																
	},
	createButton: function ( /*HTMLElement*/parent, /*String*/href, 
						     /*String*/id, /*String?*/displayText, 
							 /*String*/className ) {
		
		// summary: Creates a button element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created anchor
		// href: the value of the href attribute
		// id: the id of the anchor element
		// displayText: the inner text of the anchor
		// className: the CSS style name to apply
		// returns: the created anchor object
		ibm.portal.debug.entry( "SkinRenderer.createButton", [ parent, href, id, displayText, className ] );
		
		//Create and set the attributes for the anchor.
		var button = document.createElement( "BUTTON" );
		if ( href ) { button.href = href; }
		if ( id ) { button.id = id; }
		if ( className ) { button.className = className; }
		if ( displayText ) {
			button.appendChild( document.createTextNode( displayText ) );
		}	
												 	
		//Append the anchor to the parent.
		parent.appendChild( button );
		
		ibm.portal.debug.exit( "SkinRenderer.createButton", [ button ]);
		return button; //HTMLElement																
	},
	createImage: function ( /*HTMLElement*/parent, /*String*/src,
							/*String*/id, /*String*/altText,
							/*String*/className ) {
								
		// summary: Creates an img element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created image
		// src: the url to the image
		// id: the id of the created image element
		// altText: the alternate text for the image
		// className: the CSS style name to apply.
		ibm.portal.debug.entry( "SkinRenderer.createImage", [ parent, src, id, altText, className ] );
		
		//Create and set attributes for our image.
		var img = document.createElement( "IMG" );
		img.src = src;
		if ( id ) { img.id = id; }
		if ( altText ) { 
			img.alt = altText; img.setAttribute("title", altText); 
			//set the title for the button ("tooltip")
			if( parent.nodeName == "BUTTON") {
				parent.setAttribute("title", altText);
			}
		}
		if ( className ) { img.className = className; }
		
		//Append the image to the parent.
		parent.appendChild( img );
		
		ibm.portal.debug.exit( "SkinRenderer.createImage", [ img ] );
		return img; //HTMLElement	
	},
	createImageAnchor: function ( /*HTMLElement*/parent, /*String*/src,
							/*String*/id, /*String*/altText,
							/*String*/className ) {
								
		// summary: Creates an img element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created image
		// src: the url to the image
		// id: the id of the created image element
		// altText: the alternate text for the image
		// className: the CSS style name to apply.
		ibm.portal.debug.entry( "SkinRenderer.createImageAnchor", [ parent, src, id, altText, className ] );
		
		var anchor = com.ibm.portal.utilities.html.createAnchor( parent, "javascript:void(0);" );
		
		//Create and set attributes for our image.
		var img = document.createElement( "IMG" );
		img.src = src;
		if ( id ) { img.id = id; }
		if ( altText ) { img.alt = altText; img.title = altText; }
		if ( className ) { img.className = className; }
		
		//Append the image to the parent.
		anchor.appendChild( img );
		
		ibm.portal.debug.exit( "SkinRenderer.createImageAnchor", [ img ] );
		return anchor; //HTMLElement	
	},
	createTemporaryMarkupDiv: function ( /*String*/ markupStr ) {
		// summary: Creates a div for putting our markup into.  Returns a map with these properties:
		//		{ 	
		// 			node: the new div node,
		//			objects: a map of the extracted OBJECTS keyed by the unique id of the placeholder node in the div node
		//		}
		// description: Necessary because IE doesn't like it when you set innerHTML on a div and the first
		//		element is a script element. For some reason, any script elements up until the first actual
		//		markup element will not be inserted into the DOM. Our workaround is to create a hidden <p>
		//		element which will always be the first element in the markup string.  In IE, this will also
		//		extract all OBJECT elements and put them into the objects property map on the return object
		//		to later be put back into the DOM once the div in the node property of the return object
		//		has been placed into the DOM.  The works around an issue where using innerHTML or outerHTML
		//		of OBJECT elements doesn't retrieve their children in IE.
		// markupStr: the markup string.
		ibm.portal.debug.entry( "html.createTemporaryMarkupDiv" );
		
		var markupMap = {markup: markupStr, objects: {}};
		// fix for inserting OBJECT elements with contents in IE
		if(dojo.isIE) {
			markupMap = com.ibm.portal.utilities.html.extractObjectElementsFromString(markupStr);
		}
		
		var div = document.createElement( "DIV" );
		div.innerHTML = "<p style='display: none;'>&nbsp;</p>" + markupMap.markup;
		
		ibm.portal.debug.exit( "html.createTemporaryMarkupDiv", [ div ] );
		return {node: div, objects: markupMap.objects};
	},
	extractObjectElementsFromString: function ( /*String*/ markupStr ) {
		// summary: Extracts all OBJECT elements from the markupStr and returns an object with the following
		//		properties -
		//		{
		//			markup: the modified markup with uniquely identifiable placeholder nodes in place of the removed
		//				OBJECT elements,
		//			objects: a map of the extracted OBJECTS keyed by the unique id of the placeholder node in the new
		//				markup string
		//		}
		// markupStr: the markup string
		var objElems = {};
		var openRegExp = /<object/gi;
		var closeRegExp = /<\/object>/gi;
		var markup = markupStr;
		var match = null;
		try {
			match = openRegExp.exec(markup);
			if(match && match.index > -1) {
				var start = match.index;
				var buf;
				var end;
				var middle;
				var id;
				while(start > -1) {
					buf = markup.substring(0, start);
					end = markup.indexOf(">", start);
					if(markup.charAt(end - 1) == "/") {
						// we encountered an <OBJECT .... /> tag, so just leave it since it has no contents
						openRegExp.lastIndex = end;
						match = openRegExp.exec(markup);
						if(match) {
							start = match.index;
							continue;
						}
						else break;	// we didn't find anything after the last match
					}
					closeRegExp.lastIndex = start;
					match = closeRegExp.exec(markup);
					if(match) end = match.index;
					else break; // bad markup, OBJECT not closed, just let it go
					
					middle = markup.substring(start, end + 9);
					id = dojo.dnd.getUniqueId();
					markup = buf + "<div id='" + id + "'></div>" + markup.substring(end + 9);
					objElems[id] = middle;
					openRegExp.lastIndex = 0;	// start at the beginning since the markup string has changed
					match = openRegExp.exec(markup);
					
					if(match) start = match.index;
					else break;	// no more matches
				}
			}
			markupStr = markup;
		}
		catch(e) {
			// if we get a failure during the processing of the markup, then the markupStr will still be the same
			// that means we should clear out the objElems and just let the markup pass through as-is
			objElems = {};
		}
		return {markup: markupStr, objects: objElems};
	},
	replaceObjectElementsInMarkup: function ( /*Map*/ objects ) {
		// summary: For every key in the objects argument map, this function looks up a node in the DOM whose id
		// 		equals the key and replaces the outerHTML of that node with the string contents of the property
		//		value of the objects map with that key.  It's useful for putting markup from various nodes back 
		// 		into the DOM that was extracted earlier for some purpose.
		for(var id in objects) {
    		var objContainer = dojo.byId(id);
    		if(objContainer) {
    			objContainer.outerHTML = objects[id];
    		}
    	}
	},
	removeNodesOnCondition: function ( /*DOMElement*/node, /*Function*/condition ) {
		// summary: Walks down this node's descendants and removes all nodes that cause the given
		// condition function to return true, effectively satisfying the condition for removal.
		// Useful for delinting XSL transformations that have created unwanted nodes in the markup.
		if(!condition) condition = function(){return false;};
		if(node && node.childNodes) {
			for(var i = 0; i < node.childNodes.length; i++) {
				if(condition(node.childNodes[i])) {
					var nonNode = node.childNodes[i];
					node.removeChild(nonNode);
					delete nonNode;
					i--; // going back because the childNodes array has been updated
				}
				else {
					this.removeNodesOnCondition(node.childNodes[i], condition);
				}
			}
		}
	},
    getElementsByTagNames: function ( /*HTMLElement*/baseElem /*String 1...N*/ ) {
        // summary: Retrieves elements by tag name for 1 to N tag names.
        // description: Builds a single array with all elements of any tag name specified. A base element
        //        must be specified event if it's just the document object. Any number of tag names can be
        //        searched for, just simply keep appending parameters to the function call.
        // baseElem: the base element to search from
        // tagName1: a tag name
        // tagName2: a tag name
        // ...
        // tagNameN: a tag name
        ibm.portal.debug.entry( "html.getElementsByTagNames", [ baseElem ] );
        
        var retVal = new Array();
        
        for ( var i=1; i < arguments.length; i++ ) {
            var elements = baseElem.getElementsByTagName( arguments[i] );
            ibm.portal.debug.text( 'found ' + elements.length + ' ' + arguments[i] + ' tags.');
            for ( var j=0; j < elements.length; j++ ) {
                retVal.push( elements[j] );
            }    
        }
        
        ibm.portal.debug.exit( "html.getElementsByTagNames", [ retVal ] );
        return retVal;
    },
    getX: function ( /*HTMLElement*/elem ) {
    	// summary: Calculates the X coordinate of the top left corner of the given element. This
    	//		function was stolen from WCL and included here so that we have no dependency on 
    	//		WCL.
    	// elem: the html element to calculate the position of.
    	// returns: an integer with the X position.
    	ibm.portal.debug.entry( "html.getX", [ elem ] );
    	
    	var size = 0;
	    
	    if ( elem != null ) {
	       if ( elem.offsetParent != null ) {
	          size += com.ibm.portal.utilities.html.getX( elem.offsetParent );
	       }
	       if ( elem != null ) {
	          size += elem.offsetLeft;
	       }
	
	    }
	    
	    ibm.portal.debug.exit( "html.getX", [ size ] );
	    return size; // int
    },
    getY: function ( /*HTMLElement*/elem ) {
    	// summary: Calculates the Y coordinate of the top left corner of the given element. This
    	//		function was stolen from WCL and included here so that we have no dependency on 
    	//		WCL.
    	// elem: the html element to calculate the position of.
    	// returns: an integer with the Y position
    	ibm.portal.debug.entry( "html.getY" [ elem ] );
    	
    	var size = 0;

	    if ( elem != null ) {

	       if ( elem.offsetParent != null ) {
	          size += com.ibm.portal.utilities.html.getY( elem.offsetParent );
	       }
	       if ( elem != null ) {
	          size += elem.offsetTop;
	       }
	
 	    }
 	    
 	    ibm.portal.debug.exit( "html.getY", [ size ] );
 	    return size; // int
    },
    convertFormToQuery: function ( /*HTMLElement*/formObj, /*{name:, value:}*/submitTrigger ) {
    	// summary: Converts a given form object to a query string.
    	// description: Iterates through the given form object's "elements" and attempts to
    	//		a name/value pair to the query string. All parameters are encoded using the native
    	//		javascript function "encodeURIComponent". Since a form may contain multiple submit
    	//		inputs, the submit input name which triggered the current submission may be passed 
    	// 		to this function so that the submit input which triggered the submission may be properly
    	//		encoded into the query string. 
    	// formObj: the form object to decode
    	// submitTrigger: the name and value of the input which triggered the submission.
    	// returns: the query string with all name/value pairs that were successfully decoded
		ibm.portal.debug.entry( "html.convertFormToQuery", [ formObj, submitTrigger ] );
		
		//For some reason, formObj.elements didn't seem to always return ALL input elements.
		var elements = this.getElementsByTagNames( formObj, "input", "select", "textarea", "button" );
    	var queryStr = "";
    	var querySep = "&";
    	var queryNameValueSep = "=";
		var processed = 0;
    	for ( var i = 0; i < elements.length; i++ ) {
    		var inputNV = this.convertInputToNameValuePairs( elements[i], submitTrigger );
    		for ( var k = 0; k < inputNV.length; k++ ) {
				var pair = inputNV[k];
				if ( pair.name != "" ) {
					if ( processed != 0 ) {
						queryStr += querySep;
					}	
					queryStr += encodeURIComponent( pair.name );
		    		
					for ( var j = 0; j < pair.values.length; j++ ) {
		    			if ( j == 0 ) {
		    				queryStr += ( queryNameValueSep + encodeURIComponent( pair.values[j] ) ); 
		    			}
		    			else {
		    				queryStr += ( querySep + encodeURIComponent(pair.name) + queryNameValueSep + encodeURIComponent( pair.values[j] ) );
		    			}	
		    		}
					
					processed = processed+1;
				}
			}	
   		
    	}
		
    	ibm.portal.debug.exit( "html.convertFormToQuery", queryStr );
    	return queryStr;
    },
    convertInputToNameValuePairs: function ( /*HTMLElement*/input, /*{name:,value:}*/submitTrigger ) {
    	// summary: Converts a given form element to a name value pair.
    	// description: Given an HTML INPUT element, produces an object representation of the following format:
    	//		{ name: <input name>, values: [ <input value1>, <input value2, ..., <input valueN> ] } where values is likely
    	//		an array with one value. The array is used to cover the case where a single input has multiple values. Submit
    	//		inputs are only included if no submitTrigger is specified OR the submitTrigger variable value matches the 
    	//		submit input's "name" attribute.
    	// input: the HTML INPUT element to convert
    	// submitTrigger: the input which triggered the form submission
    	// returns: a name/value pair { name: "", values: [] }, guaranteed to be not NULL
    	ibm.portal.debug.entry( "html.convertInputToNameValuePairs", [ input, submitTrigger ]);
    	
    	var type = input.type;
    	ibm.portal.debug.text( "Input type is: " + type );
		ibm.portal.debug.text( "Input name is: " + input.name );
    	var name = "";
    	var values = [];
		var retPairs = [];
    	
    	if ( !input.disabled ) {
    		    		
			switch ( type.toLowerCase() ) {
				case "text":
				case "password":
				case "hidden":
					name = input.name; values.push( input.value );
					retPairs.push( { name: name, values: values } );
					break;
				case "reset":
				case "button": 
					if ( !submitTrigger || ( input.name == submitTrigger.name && input.value == submitTrigger.value ) ) {
						name = input.name; values.push( input.value );
						retPairs.push( { name: name, values: values } );				
					}
					break;		
				case "radio":
				case "checkbox": if ( input.checked ) {
						name = input.name;
						values.push( input.value );
					}
					retPairs.push( { name: name, values: values } );
					break;
				case "image": if ( !submitTrigger || input.name == submitTrigger ) {
						name = input.name;
						if ( input.value ) {
							values.push( input.value );
							retPairs.push( { name: name, values: values } );
						}
						
						retPairs.push( { name: name + ".x", values: [ this.getX( input ) ] } );
						retPairs.push( { name: name + ".y", values: [ this.getY( input ) ] } );
					}
					break;
				case "submit": if ( !submitTrigger || ( input.name == submitTrigger.name && input.value == submitTrigger.value ) ) {
						name = input.name;
						if ( input.value ) {
							values.push( input.value );
						}
						retPairs.push( { name: name, values: values } );
					}
					break;
				case "select-one": 
				case "select-multiple": name = input.name;
					for ( var i = 0; i < input.options.length; i++ ) {
						if ( input.options[i].selected ) {
							var value = input.options[i].value ? input.options[i].value : input.options[i].text;
							values.push( value );
						}
					}
					if ( values.length != 0 ) { 
						retPairs.push( { name: name, values: values } );
					}	
					
					break;
				case "file": break;
				default: name = input.name; values.push( input.value ); retPairs.push( { name: name, values: values } );
			}
    	}	
		
		ibm.portal.debug.exit( "html.convertInputToNameValuePairs", retPairs );
		return retPairs;	
    },
		isHidden: function(node) {
    	// summary: determines whether the specified node is hidden or shown.
    	// description: this is a lightweight replacement for the obsolete dojo.html.isHidden function. It checks the element
    	//		only, not its parents, for display:none being set.
    	// input: the dom node
    	// returns: true if the node has display:none style set, false otherwise	
			return dojo.style(node, 'display') == 'none';
		},
		hide: function(node) {
    	// summary:  hides the specified node from the rendered page.
    	// description: this is a replacement for the obsolete dojo.html.hide() function. 
    	// input: the dom node to hide
			dojo.fx.wipeOut({node: node, duration: 5}).play();
		},
		show: function(node) {
    	// summary:  shows the specified node from the rendered page.
    	// description: this is a replacement for the obsolete dojo.html.show) function. 
    	// input: the dom node to show
			dojo.fx.wipeIn({node: node, duration: 5}).play();
		},
		isDescendantOf: function(node, ref) {
			//summary: Checks to see if the given node descends from the reference node.
			//node: node to check
			//ref: the reference node
			// returns: true if the node is a descendant of the given reference node
			var node = node.parentNode;
			var found = false;
			
			while ( node && !found ) {
				if ( node == ref ) {
					found = true;
				}
				node = node.parentNode;
			}
			
			return found;
		}
	
};

}
