/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.query.QueryRuleEngine"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.query.QueryRuleEngine"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.query.QueryRuleEngine");

dojo.require("com.ibm.query.TypeHandler");

// Abstract module for providing support for querying data within other modules.
// Provides default implementations for looking up query rules, parameterizing them,
// and converting to and from arbitrary data types and String values.
// To be useful, this is meant to be extended to provide implementations for the getByRule,
// setByRule, and unsetByRule functions.  The toType and fromType functions provide no
// type-conversions by default and should be extended as well for custom types.


dojo.require("dojo.string");

dojo.declare("com.ibm.query.QueryRuleEngine",
	null,
	{
		queryRules: {},
		
		// context: Object
		//		Context in which to run manual getter/setter functions in query rules.  Defaults to this query
		//		engine instance if none is provided.
		context: null,
		
		types: {},
		
		constructor: function(args) {
			this.context = this;
			dojo.mixin(this, args);
		},
		
		isReadOnly: function(/*Object*/ rule) {
			return rule.readOnly;
		},
		toType: function(/*String*/ value, /*String*/ type) {
			// summary: Converts the String value to the given type as specified by the semantics of the
			//		implementation, then returns the converted value.  The type can be anything.
			// value: String original value from the data
			// type: String name of the type to convert the value to
			// returns: Anything converted value
			if(this.types[type]) {
				return this.types[type].to.call(this.context, value);
			}
			else return com.ibm.query.TypeHandler.toType(value);
		},
		fromType: function(/*Anything*/ value, /*String*/ type) {
			// summary: Converts the value from the given type to a String as specified by the semantics 
			// 		of the implementation, then returns the String value.
			// value: Anything typed value
			// type: String name of the type to convert the value from
			// returns: String value
			if(this.types[type]) {
				return this.types[type].from.call(this.context, value);
			}
			else return com.ibm.query.TypeHandler.fromType(value);
		},
		
		isParameterizable: function(/*String*/ key) {
			return key.charAt(key.length - 1) == '$';
		},
		getRulesWithParams: function(/*Object?*/ map) {
			// summary: Returns a map of the rules from another map that are parameterizable.  Takes an
			//		argument map or defaults to using this.queryRules if the argument is null.
			// map: Object map to use to lookup query rules.  If null, this.queryRules is used instead.
			// returns: Object map of rules which can be parameterized.
			var parmRules = {};
			if(!map) map = this.queryRules;
			for(var key in map) {
				if(this.isParameterizable(key)) {
					parmRules[key] = map[key];
				}
			}
			return parmRules;
		},
		
		getQuery: function(/*String*/ queryKey, /*Object?*/map, /*Boolean?*/ multiValued) {
			// summary: Takes a queryKey string to key into the map argument, if provided, or 
			//		this.queryRules if map is null, and then determines if it is a parameterized key 
			// 		indicated by the last character in the key being a '$'.  If it's not a 
			// 		parameterized key, it simply returns the queryRule matching the key without 
			// 		any substitution.  If it is parameterized,	this finds the parameterizable 
			// 		queryRule whose	key also ends with a '$', and parameterizes that query rule it 
			// 		finds with values for the comma-separated parameters found within the square 
			// 		brackets (Ex. '[param0,param1,param2]') part of the queryKey.
			// 		Example - A queryKey of 'localizedName[edward, tj]$' would lookup
			// 			the key 'localizedName$' in this.queryRules.
			// 			The resulting query rule would be used as a template for parameterization.
			//			The parameter values used to parameterize the template would
			//			be ['edward', 'tj'] and would match the ${0} and ${1} parameters
			//			in the query rule respectively.  Each property in the query rule that
			//			is a string is parameterized by the parameter values in the input queryKey
			//			and then the parameterized query rule object is returned.  Note that 
			//			the parameter values are white-space-trimmed on the beginning and end of
			//			the strings.
			//		
			//		These additional properties get set on the returned rule as well (and therefore
			// 		should be considered reserved properties in rule definitions) -
			//			{
			//				ruleKey: String input key with parameters in place
			// 				parameters: Array of parameters found in the input key, or null
			// 					if none existed
			// 				multiValued: Boolean multiValued property as specified by the argument
			//			}
			// queryKey: String key to lookup the query rule and perform any parameterization substitutions
			//		that are valid.  String in one of two forms.
			// 		1) "key" for a non-parameterized key.  MUST NOT end with a '$' character.
			//		2) "key[param0,param1,...]$" for a parameterized key.  MUST end with a '$' character.
			// 			Parameters are extracted out via comma-splitting and returned in an array as 
			// 			strings in the order they appear.  Returns 
			// 			{key: "key$", parameters: ["param0", "param1", ...]}
			// map: Object map to use to lookup query rules.  If null, this.queryRules is used instead.
			// multiValued: Boolean value set on the rule's multiValued property.  Rules and query engines
			//		can use this property to perform queries that generate arrays of results instead of
			//		single values (if the attribute supports that).
			// returns: The query rule specified by this queryKey after all (if any) parameters 
			// 			have been bound to their values as specified by the caller.
			var obj = this.extractKeyParts(queryKey);
			if(!map) map = this.queryRules;
			var rule = map[obj.key];
			
			if(!rule) return rule;
			// if this is not a parameterized map rule, just return the map rule as-is
			if(!obj.parameters) {
				rule.parameters = null;
				rule.ruleKey = obj.key;
				rule.multiValued = !!multiValued;
				return rule;
			}
			
			// otherwise, parameterize the map by using the array of parameters passed into the attribute
			// like this ->  attribute[parm1, parm2, parm3]$
			// parameterized attributes always end with $
			var expRule = {};
			for(var prop in rule) {
				if(dojo.isString(rule[prop])) {
					expRule[prop] = dojo.string.substitute(rule[prop], obj.parameters, function(part){
						return dojo.string.trim(part);
					});
				}
				else {
					expRule[prop] = rule[prop];
				}
			}
			expRule.parameters = obj.parameters;
			expRule.ruleKey = obj.key;
			expRule.multiValued = !!multiValued;
			return expRule;
		},
		extractKeyParts: function( /*String*/ keyInput ) {
			// summary: Expands a keyInput and determines if it is a key to a parameterizable query rule.
			//		If it is, extract the parameters from the argument and return the base key name
			//		used to look up the correct query rule in this object's queryRules without the parameter
			// 		values, and also return the parameters as an array of strings.  
			// 		If the keyInput is not the key of a parameterizable map rule, just return the key and a 
			// 		null parameter array.
			// keyInput: String in one of two forms.
			// 		1) "key" for a non-parameterized key.  Must not end with a '$' character.  Returns
			//			{key: "key", parameters: null}
			//		2) "key[param0,param1,...]$" for a parameterized key.  Parameters are extracted out
			//			and returned in an array as strings in the order they appear.  Returns 
			// 			{key: "key$", parameters: ["param0", "param1", ...]}
			// returns: Object {key: queryKeyWithoutParameters, parameters: arrayOfParameterValues}
			if(this.isParameterizable(keyInput)) {
				var start = keyInput.indexOf('[');
				var end = keyInput.lastIndexOf(']');
				if(start > -1 && end > start) {
					var baseName = keyInput.substring(0, start) + "$";
					var parms = keyInput.substring(start + 1, end).split(',');
					return {key: baseName, parameters: parms};
				}
			}
			return {key: keyInput, parameters: null};
		},
		
		getByRule: function(/*Object*/ data, /*Object*/ rule, /*Boolean?*/ multiValued) {
			return data;
		},
		setByRule: function(/*Object*/ data, /*Object*/ rule, /*Object*/ value, /*Boolean?*/ multiValued) {
			if(this.isReadOnly(rule)) return false;
			return true;
		},
		unsetByRule: function(/*Object*/ data, /*Object*/ rule, /*Boolean?*/ multiValued) {
			if(this.isReadOnly(rule)) return false;
			return true;
		}
	}
);

}
