/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.widgets.StatusBar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.widgets.StatusBar"] = true;
dojo.provide( "com.ibm.widgets.StatusBar" );

dojo.require( "dojox.data.dom" );

dojo.require("dojo.i18n");

dojo.declare( "com.ibm.widgets.StatusType", 
    null,
    {
	    constructor: function(/*int*/id,  /*String*/styleClass, /*String*/ iconPath, /*String*/ colorStr) {
	        this._id = id;
	        this._styleClass = styleClass;
	        this._iconPath = iconPath;
	        this._color = colorStr;  // status color fader start color for this class
	    }
    }
);

dojo.declare( "com.ibm.widgets.StatusMessage", 
    null,
    {
    	constructor: function(/*int*/type, /*String*/message, /*String*/details) {
	        this._type = type;
	        this._message = message;
	        this._details = details;
    	}	
    }
);


dojo.declare( "com.ibm.widgets.StatusColorFader", 
    null,
    {
    	constructor: function(/*String*/containerId, /*String*/endColorStr) {
	        this._containerId = containerId;
	        this._beginColorStr = "";
	        this._endColorStr = "";
	
	        this._redCtr = 0x00;
	        this._greenCtr = 0x00;
	        this._blueCtr = 0x00;
	
	        this._endRed = 0x00;
	        this._endGreen = 0x00;
	        this._endBlue = 0x00;
	
	        this.setEndColor(endColorStr);
	
	        this._fadeCallbackId = null;
	
	        this._enable = true;
	    },
        fade: function() {

             // adjust r, g, b counters and see if we have reached end color
             var shouldStop = true;

             // looping here rather than incrementing/decrementing by 10
             // this makes sure we approach the end value without going past it
             // and we can still support any arbitrary start and end color values
             for (var ctr = 0; ctr < 10; ctr++) {
                 if (this._redCtr < this._endRed) {
                     this._redCtr++;
                     shouldStop = false;
                 }
                 else if (this._redCtr > this._endRed) {
                     this._redCtr--;
                     shouldStop = false;
                 }
    
                 if (this._greenCtr < this._endGreen) {
                     this._greenCtr++;
                     shouldStop = false;
                 }
                 else if (this._greenCtr > this._endGreen) {
                     this._greenCtr--;
                     shouldStop = false;
                 }
    
                 if (this._blueCtr < this._endBlue) {
                     this._blueCtr++;
                     shouldStop = false;
                 }
                 else if (this._blueCtr > this._endBlue) {
                     this._blueCtr--;
                     shouldStop = false;
                 }
             }

             if (shouldStop) {
                 this.stop();
             }
             else {
                 var updateColorStr = "#" + this.getHexString(this._redCtr) + this.getHexString(this._greenCtr) + this.getHexString(this._blueCtr);
                 var header = document.getElementById(this._containerId);
                 if (header != null) {
                     header.style.backgroundColor = updateColorStr;
                     
                 }
                 else {
                     //alert("problem - header was null");
                     this.stop();
                 }
             }

        },
        start: function() {

             if (this._enable) {
                 var header = document.getElementById(this._containerId);
                 header.style.backgroundColor = this._beginColorStr;
                 var me = this;
                 this._fadeCallbackId = self.setInterval(function () { me.fade() }, 20);
             }

        },
        stop: function() {

             if (this._fadeCallbackId != null) {
                 clearInterval(this._fadeCallbackId);
             }

        },
        setBeginColor: function(/*string*/ hexString) {

             this._beginColorStr = hexString;

             this._redCtr = this.getColorValue(hexString, "r");
             this._greenCtr = this.getColorValue(hexString, "g");
             this._blueCtr = this.getColorValue(hexString, "b");

        },
        setEndColor: function(/*string*/ hexString) {

             this._endColorStr = hexString;

             this._endRed = this.getColorValue(hexString, "r");
             this._endGreen = this.getColorValue(hexString, "g");
             this._endBlue = this.getColorValue(hexString, "b");

        },
        getColorValue: function(/*string*/ hexString, /*string*/ rgb) {

             var subStr = "";
             if (rgb == "r") {
                 subStr = hexString.slice(1, 3);
             }
             else if (rgb == "g") {
                 subStr = hexString.slice(3, 5);
             }
             else if (rgb == "b") {
                 subStr = hexString.slice(5);
             }
             var rv = parseInt(subStr, 16);

             return rv;
        },
        getHexString: function(/*int*/ num) {

             var rv = 0;
             if (num >= 0x10) {
                 rv = num.toString(16);
             }
             else {
                 rv = "0" + num.toString(16);
             }
             return rv;
        },
        setEnable: function(/*boolean*/ value) {

             this._enable = value;

        }
    }
);



dojo.declare( "com.ibm.widgets.StatusBar", 
    [dijit._Widget, dijit._Templated],
    // end constructor
    {
    	uid: "statusBar",
    	locale: "en",
    	errorGifPath: "",
    	warningGifPath: "",
    	infoGifPath: "",
    	multipleMessagesGifPath: "",
    	showDetails: false,
    	
    	templateString:"<div class=\"statusBar\"\r\n\t><div dojoAttachEvent=\"\"\r\n\t\t\twaiRole=\"region\" \r\n\t\t\tclass=\"ibmPortalStatusBar\" \r\n\t\t\tdojoAttachPoint=\"containerNode,focusNode\"\r\n\t></div\r\n></div>\r\n",
    	
    	postCreate: function() {  
	        this._messages = new Array();
	        this._numMessages = 0;
	        this._baseURL = window.location.protocol+"//"+window.location.host;
	        this._statusTypesMap = { 
	            "error" : new com.ibm.widgets.StatusType(0, "ibmPortalError", this._baseURL + this.errorGifPath, "#FF0000"),
	            "warning" : new com.ibm.widgets.StatusType(1, "ibmPortalWarning", this._baseURL + this.warningGifPath, "FFFF00"),
	            "info" : new com.ibm.widgets.StatusType(2, "ibmPortalInfo", this._baseURL + this.infoGifPath, "#0077FF"),
	            "multiple" : new com.ibm.widgets.StatusType(3, "ibmPortalMultiple",this._baseURL + this.multipleMessagesGifPath, "#E9CAF9")
	        };
	        //Subscribe to status topics.
	        this._statusHandler = dojo.subscribe("/portal/status", this, this.addMessage);
	
	        // create new message status bar color fader
	        this._summaryHeaderId = "ibmPortalStatusBarHeader";
	        var endBackgroundColorStr = this._statusTypesMap["multiple"]._color;
	        this.colorFader = new com.ibm.widgets.StatusColorFader( this._summaryHeaderId, endBackgroundColorStr);
	
	    }, 
	    startup: function() {
			this.inherited(arguments);
		},
        _getStatusContainer: function () {
            return this.containerNode;   
        },
        setShowDetails: function (/*boolean*/showDetails) {
            this.showDetails = showDetails;    
        },
        toggleShowDetails: function() {
            this.setShowDetails(!this.showDetails);
            //var statusContainer = this._getStatusContainer();
            //dojox.data.dom.removeChildren( statusContainer );
            this.colorFader.setEnable(false);  // disable the fader when toggling details
            this.render();
            this.colorFader.setEnable(true);
        },
        clear: function () {
            //summary: clears all contents of the statusBar
            var statusContainer = this._getStatusContainer();
            dojox.data.dom.removeChildren( statusContainer );
            this._messages.length = 0;
        },
        print: function() {
            //create new window
            var printWindow=window.open( '', 'status', 'resizable=yes,scrollbars=yes,menubar=no,toolbar=yes,status=no,width=800px,height=600px,screenX=10px,screenY=10px,top=10px,left=10px');
            var printDoc = printWindow.document;
            printDoc.title="Status";
            // set base url so relative image paths will resolve correctly
            var baseElem = printWindow.document.createElement("base");
            baseElem.setAttribute("href",window.location.protocol+"//"+window.location.host);
            printDoc.documentElement.childNodes[0].appendChild(baseElem);
            
            //copy in portal styles - This way doesn't work
            //var portalStylesLink = document.getElementById("portalStyles");
            //printDoc.documentElement.childNodes[0].appendChild(portalStylesLink.cloneNode(true));
            
            // this way doesn't work either
            //printWindow.document.head.appendChild(portalStylesLink.cloneNode());
            //if(document.createStyleSheet) { 
            //    document.createStyleSheet("javascript:'"+styles+"'"); 
            //} else { 
            //    newSS=document.createElement('link'); 
            //    newSS.rel='stylesheet'; 
            //    newSS.href='data:text/css,'+escape(styles); 
            //    document.documentElement.childNodes[0].appendChild(newSS); 
            //}  
            
            
            //neither does this one at the moment          
            /*
            var portalStyles = printDoc.createElement("style");
            for(var i = 0; i < document.styleSheets.length; i++ ){
                var ii = 0;
                do{
                    var cssRule = document.styleSheets[i].cssRules[ii];
                    
                    if(cssRule){
                        portalStyles.appendChild(printDoc.createTextNode(cssRule.cssText));
                    }
                    
                    ii++;
                }while(cssRule)  
            }
            printDoc.body.appendChild(portalStyles);         
             */

            // for(var i = 0; i < document.styleSheets.length; i++ ){
                //printWindow.stylesheets.appendChild(document.stylesheets[1].cloneNode());
                //var ii = 0;
                //do{
                //  var cssRule = document.styleSheets[i].cssRules[ii];
                //  
                //  if(cssRule){
                //      win.document.write(cssRule.cssText);
                //  }
                //  
                //  ii++;
                //}while(cssRule)  
            //}         
            // write in status
            //var statusHTML = this._getStatusContainer().innerHTML;
            //printWindow.document.body.innerHTML = "<div id='printStatusBar'>"+statusHTML+"</div>";
            
            //add container div
            printDoc.body.innerHTML = "<div id='statusBar' class='statusBar'></div>";
//          this.render(printWindow.document.getElementById("statusBar"));
            
            for (x in this._messages){
                var statusType = this._statusTypesMap[this._messages[x]._type];
            
            // status entry base container
                var entry = printDoc.createElement( "div" );
                entry.className = statusType._styleClass;
            
            // summary header - the colored area of each status message
                var header = printDoc.createElement("h3");
            // Lefthandside (LHS) contains icon, message, and show/hide details
                var header_LHS = printDoc.createElement("div");
                header_LHS.className = "ibmPortalstatusHeaderLHS";
                var icon = printDoc.createElement("img");
                icon.src = statusType._iconPath;
                header_LHS.appendChild(icon);
                header_LHS.appendChild(printDoc.createTextNode(this._messages[x]._message));
                header.appendChild(header_LHS);
                entry.appendChild(header);
                
                var detailsDiv = printDoc.createElement("div");
                detailsDiv.className = "ibmPortalDetails";
                detailsDiv.appendChild(printDoc.createTextNode(this._messages[x]._details));
                entry.appendChild(detailsDiv);
                printWindow.document.getElementById("statusBar").appendChild(entry);
            }

            printDoc.close();
            printWindow.focus();
        },
        addMessage: function(/*Object*/args) {
            //summary: adds a message to the message array
            if(args.uid && args.uid != this.uid) {
            	return;
            }
            var message = args.message;
            this._messages.push(message);
            // temp limit to handle infinite loops if there is an error raised in this code
            if (this._messages.length < 100){
               this.render(this._getStatusContainer());
            }
  
        },
        render: function(/*Node*/ targetContainer) {
            //summary: render the message(s) in the statusBar  
            if (targetContainer == null){
            targetContainer = this._getStatusContainer();
            }
            dojox.data.dom.removeChildren( targetContainer );
            // only render multiple header if there is more than one message.
            // in renderMessage, if their is only one message, it will render as the header
            if (this._messages.length > 1) {
                this.renderHeader(targetContainer);
            }
            else if (this._messages.length == 1) {
                this.renderMessage(targetContainer,this._messages[0], this.showDetails,0);
            }
            
            // if showing details, render all the messages
            if(this.showDetails && this._messages.length > 1){
                for (x in this._messages){
                    this.renderMessage(targetContainer, this._messages[x], this.showDetails, x);
                }
            }
            // if not showing details, only show the last one
            //else {
            //    this.renderMessage(this._messages[this._messages.length-1], this._showDetails, this._messages.length-1 );
            //}


            // if we have more than one message and we are not showing details,
            // then start a color fade based on the last message received
            if(!this.showDetails && this._messages.length > 1){
                // in case its already running for a previous message
                this.colorFader.stop();

                // get the last message type that came in to get its color
                var end = this._messages.length - 1;
                var startBackgroundColorStr = this._statusTypesMap[this._messages[end]._type]._color;
                this.colorFader.setBeginColor(startBackgroundColorStr);
                this.colorFader.start();
            }
        
   
        },
        renderHeader: function(/*Node*/ targetContainer) {
            var v3theme_Properties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", this.locale);
            
            //summary: render the main statusBar 
            //description: this is the main statusBar, which includes the main controls, and either the single status message,
            // or an indicator of the number of messages contained within the details view. 
            //
            if(this._messages.length > 1) {
                var headerMessage = new com.ibm.widgets.StatusMessage( "multiple", v3theme_Properties["itemsAvailable"], "" );
                this.renderMessage(targetContainer, headerMessage, this.showDetails, -1);
            }
            // if only one message, render it as the header
            else if(this._messages.length == 1){
               // this.renderMessage(targetContainer, this._messages[0], this._showDetails, 0);
            }
 
        },
        renderMessage: function (/*Node*/ targetContainer, /*com.ibm.widgets.StatusMessage*/message, /*boolean*/ showDetails, /*int*/index ) {
            //summary: renders a single message in the statusBar
            //description: renders the following markup:
            //<div class="ibmPortalError">
            //<h3><img src="./error.gif"/> An Error Occurred <a href="#"> 3 more...</a></h3>
            //<div class="ibmPortalDetails">Its a really really bad one</div>
            //</div>
            //var statusContainer = this._getStatusContainer();
            var statusType = this._statusTypesMap[message._type];
            //dojox.data.dom.removeChildren( statusContainer );
            
            // status entry base container
            var entry = document.createElement( "div" );
            entry.className = statusType._styleClass;
            
            // summary header - the colored area of each status message
            var header = document.createElement("h3");
            header.id = this._summaryHeaderId;   // set id for color fader
            // Lefthandside (LHS) contains icon, message, and show/hide details
            var header_LHS = document.createElement("div");
            if(!ibmPortalConfig.isRTL){
                header_LHS.className = "ibmPortalStatusHeaderLHS";
            }
            else{
                header_LHS.className = "ibmPortalStatusHeaderRHS";
            }               

            var icon = document.createElement("img");
            icon.src = statusType._iconPath;
            header_LHS.appendChild(icon);
            
            var messageText = null;
            var v3theme_Properties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", this.locale);
            
            // is this the top header status summary and we have more than one message
            if(this._messages.length > 1 && index < 0){
//                header_LHS.appendChild(document.createTextNode(''+this._messages.length+' messages are available for review.'));
                // then format the message
                if(!ibmPortalConfig.isRTL)
                    messageText = ''+this._messages.length+' '+v3theme_Properties["messagesAvailable"];
                else
                    messageText = v3theme_Properties["messagesAvailable"]+' '+this._messages.length+'';
            }
            // do we have only one message
            else if(this._messages.length == 1){
                messageText = message._message;
                // adjust the left margin for a single message which will have a style class that is normally indented
                entry.style.marginLeft="0px";
            }
            else{
                header_LHS.appendChild(document.createTextNode(message._message));
            }

            header.appendChild(header_LHS);

            // render top level multiple messages status bar header
            if((this._messages.length > 1 && index < 0)||(this._messages.length == 1)){
                var me = this;
                var detailsAnchor = document.createElement("a");
                detailsAnchor.href="javascript:void(0)";
                detailsAnchor.onclick=function (e) { me.toggleShowDetails(); return false; };

                var header_RHS = document.createElement("div");

                var clearAnchor = document.createElement("a");
                clearAnchor.href="javascript:void(0);";
                clearAnchor.onclick=function (e) { me.clear(); return false; };
                clearAnchor.appendChild(document.createTextNode(v3theme_Properties["clear"]));

                var printAnchor = document.createElement("a");
                printAnchor.href="javascript:void(0);";
                printAnchor.onclick=function(e){ me.print(); };
                printAnchor.appendChild(document.createTextNode(v3theme_Properties["print"]));

                var showDetailsText = v3theme_Properties["showDetails"];
                var hideDetailsText = v3theme_Properties["hideDetails"];
                
                if(ibmPortalConfig.isRTL){
                	header_RHS.className = "ibmPortalStatusHeaderLHS";
                }
                else {
                	header_RHS.className = "ibmPortalStatusHeaderRHS";
                }
                
                header_LHS.appendChild(document.createTextNode(messageText));
                if (showDetails){
                    detailsAnchor.appendChild(document.createTextNode(hideDetailsText));
                }
                else {
                    detailsAnchor.appendChild(document.createTextNode(showDetailsText));
                }
                header_LHS.appendChild(detailsAnchor);
                // clear link
                header_RHS.appendChild(clearAnchor);
                // print link
                header_RHS.appendChild(printAnchor);
                header.appendChild(header_RHS);
            }

            var clearingDiv = document.createElement("div");
            clearingDiv.className="wptheme-clearing ";
            header.appendChild(clearingDiv);

            entry.appendChild(header);
    
            // details
            if (showDetails && index >=0){
                var detailsDiv = document.createElement("div");
                detailsDiv.className = "ibmPortalDetails";
                detailsDiv.appendChild(document.createTextNode(message._details));
                entry.appendChild(detailsDiv);
            }
            targetContainer.appendChild(entry);
    
        }
    }
); // end declare StatusBar

}
